@ECHO OFF
 rem ==========================================================================
 rem === Obtain User Manager Info for Each User Account
 rem ==========================================================================
 rem
 rem Created On: 07 Jul 2001
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Oct 2023 / 30 Nov 2020              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: NT4 or later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem ==========================================================================

 :::  This script uses enumerates the user accounts on the local machine or
 :::  within the domain to provide detailed information about the user account.
 :::  Unlike the "Home Directory" scripts this script can handle User Accounts
 :::  which contain spaces, because it parses the entire output of the NET VIEW
 :::  command.
 :::
 :::  (Jul 2018) v3.0 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::  -- Added new Filtered CMD Title and Syntax Help processing routines;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com
 :::
 :::  (Oct 2018) v3.1 -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Oct 2019) v3.2 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (Dec 2019) v3.3 -- Developed a drop-in replacement for Microsoft's
 :::  NOW.EXE utility, taking back that function from DATEINFO.EXE;
 :::
 :::  (Nov 2020) v3.4 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Oct 2023) v3.5 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 30 Nov 2020 / 26 Jul 2018              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 30 Nov 2020 / 26 May 2020              
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v3.5.0.550

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- Main Variables
 SET @SKIP=4
 SET @TARGET=on %COMPUTERNAME%

 FOR %%d IN (- /) DO FOR %%P IN (D DOMAIN) DO IF /I "%~1"=="%%~d%%~P" (
   SET @SKIP=6
   SET @DOMAIN=/DOMAIN
   SET @TARGET=for the %USERDOMAIN% Domain
 )

 rem -- Logging Variables
 SET @LOGPATH=%@STORAGE%\Logs\Maint
 SET @LOGFILE=%@LOGPATH%\%~n0.%@FILESTAMP%


 rem ==========================================================================
 rem === Cycle Through Account List And Obtain User Info
 rem === Updated On: 10 Dec 2019 / 26 Jul 2018              
 rem ==========================================================================
:GetAccounts
 IF NOT EXIST "%@LOGPATH%" MD "%@LOGPATH%"

 NOW Obtaining Detailed Account Information %@TARGET%... >"%@LOGFILE%"
 NOW Obtaining Detailed Account Information %@TARGET%...
 ECHO:
 FOR /F "TOKENS=* SKIP=%@SKIP% DELIMS=/" %%U IN ('NET USER %@DOMAIN%') DO CALL :ParseAccts %%U
 ECHO Finished Viewing Account Info %@TARGET%...
 ECHO:


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019              
 rem ==========================================================================
:ExitBatch
 DIR "%@LOGFILE%" | FIND /I "%~n0.%@FILESTAMP%"
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Parse Account Names From NET USER Output
 rem === Updated On: 07 Jul 2001                     
 rem ==========================================================================
:ParseAccts
 rem %* - Current Line of Accounts To Be Parsed

 SET @PARAM=%*
 SET @ACCT1=%@PARAM:~00,25%
 SET @ACCT2=%@PARAM:~25,25%
 SET @ACCT3=%@PARAM:~50,25%
 CALL :ViewInfo %@ACCT1%
 CALL :ViewInfo %@ACCT2%
 CALL :ViewInfo %@ACCT3%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: View Detailed User Manager Account Info
 rem === Updated On: 26 Jul 2018 / 07 Jul 2001              
 rem ==========================================================================
:ViewInfo
 rem %* - Current Account For Which To Provide Detailed Info

 SET @ACCT=%*
 IF /I "%@ACCT%"=="The command completed suc" GOTO :EOF
 IF /I "%@ACCT%"=="cessfully." GOTO :EOF
 IF /I NOT "%@ACCT%"=="" (
	 NET USER "%@ACCT%" %@DOMAIN%
	 NET USER "%@ACCT%" %@DOMAIN% >>"%@LOGFILE%"
	 ECHO %@DIVIDER-%
	 ECHO:
 )
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET @SCRIPT_BEG#="%%~d"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 30 Nov 2020 / 26 Jul 2018              
 rem ==========================================================================
:HelpMessage
 ECHO Obtain User Manager Info for Each User Account
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  [/D ^| /DOMAIN]
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:
 ECHO -------------------
 ECHO OPTION DEFINITIONS:
 ECHO -------------------
 ECHO  -?,  /H ..... Display This Help Message  (also --HELP)
 ECHO  -??, /H2 .... Display Extended Help Message
 ECHO       /D  .... Get User Account Info from current domain
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0
	 ECHO  %~n0 /D
	 ECHO  %~n0 /DOMAIN
	 ECHO  %~n0 -?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
	 ECHO:
	 ECHO  * Options are case insensitive, and can be prefaced with "-" or "/".
 )
 CALL :ShowStatus "FINISHED"
                                           