@ECHO OFF
 rem ==========================================================================
 rem === Add/Manage Windows Library Entries
 rem ==========================================================================
 rem
 rem Created On: 24 Sep 2013
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Oct 2023 / 30 Nov 2020              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows Vista and later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    SLUTIL ................. http://www.grimadmin.com/filemgmt/index.php?id=5
 rem
 rem  Input Files Used By Script (stored in %SystemDrive%\Scripts\Bat\Input):
 rem    WinLibraryList.TXT ..... List of Windows Library Settings
 rem
 rem  Required Scripts:
 rem    SetDrive.BAT ........... Set Global Variables for this Environment
 rem ==========================================================================

 :::  This script uses the SLUTIL utility (replacing ShLib) to manage Windows
 :::  Libraries in Windows 7/8/+.  The configuration information is stored in
 :::  WinLibraryList.TXT in the INPUT folder, and allows library entries to be
 :::  created, modified or deleted.
 :::
 :::  (Jul 2018) v2.4 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::  -- Added new Filtered CMD Title and Syntax Help processing routines;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com
 :::
 :::  (Oct 2018) v2.5 -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Oct 2019) v2.6 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (May 2020) v2.7 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::
 :::  (Nov 2020) v2.8 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Oct 2023) v2.9 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 30 Nov 2020 / 26 Jul 2018              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 30 Nov 2020 / 26 May 2020              
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v2.9.0.290

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 (
	 ECHO Add/Manage Windows Library Entries
	 ECHO:
	 ECHO    YOU TYPED: %0 %*
	 ECHO:
	 ECHO   CMD SYNTAX: %~n0
	 ECHO               %~n0  [-h^|-?]
	 ECHO:
	 ECHO  MAIN CONFIG: %~dp0Input\WinLibraryList.TXT
	 ECHO:
	 ECHO  USAGE NOTES:
	 ECHO    * Parameters surrounded by ^<^> are mandatory.
	 ECHO    * Parameters surrounded by [] are optional.
	 ECHO    * Options are case-insensitive, and can be prefaced by "-" or "/".
	 GOTO :ExitBatch
 )

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- Verify Script Running on Windows 7 or Later
 IF %@OSBUILD% LSS 7600 (
	 ECHO ERROR -- Sorry, this script requires Windows 7 or later
	 GOTO :ExitBatch
 )

 rem -- Set Library Entries
 SET @COUNT=100
 SET @FILESERVER=\\!@FILESERVER:\=!
 SET @LIBSOURCE=%USERPROFILE%\AppData\Roaming\Microsoft\Windows\Libraries

 rem -- List of Windows Library Settings -- Grab from WinLibraryList.TXT -OR- Set Manually
 IF EXIST "%@LIBRARYLIST%" (
	 FOR /F "USEBACKQ SKIP=2 EOL=; TOKENS=*" %%L IN ("%@LIBRARYLIST%") DO (
		 SET /A @COUNT+=1
		 SET @LIBRARY-!@COUNT:~-2!=%%L
	 )
 ) ELSE (
	 rem @LIBRARY-##="Default/Custom Library Container",  "Library Folder Location"
	 SET @LIBRARY-01="FOLDERID_DocumentsLibrary",         "%SystemDrive%\Storage\Docs"
	 SET @LIBRARY-02="FOLDERID_DocumentsLibrary",         "%@STORAGE%\Logs"
	 SET @LIBRARY-03="FOLDERID_DocumentsLibrary",         "%@FILESERVER%\Storage\Files\Bible\Bible Study"
	 SET @LIBRARY-04="FOLDERID_PicturesLibrary",          "%@FILESERVER%\Storage\Pictures"
	 SET @LIBRARY-05="FOLDERID_PicturesLibrary",          "%@FILESERVER%\Storage\ClipArt"
	 SET @LIBRARY-06="FOLDERID_MusicLibrary",             "%@STORAGE%\MediaServer"
	 SET @LIBRARY-07="FOLDERID_MusicLibrary",             "%SystemDrive%\MediaServer"
	 SET @LIBRARY-08="CUSTOM_WindowsThemes",              "%@FILESERVER%\Storage\Files\Downloads\Themes\Windows 8 Themes"
	 SET @LIBRARY-09="CUSTOM_WindowsThemes",              "%@STORAGE%\Themes\WinThemes.%COMPUTERNAME%"
 )


 rem ==========================================================================
 rem -- Cycle Through Library Entries
 rem === Updated On: 01 Jan 2018 / 01 Nov 2013              
 rem ==========================================================================
:Main
 FOR /F "TOKENS=2-4 DELIMS==,;" %%L IN ('SET @LIBRARY-') DO CALL :ManageOneLibrary %%L %%M %%N


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019              
 rem ==========================================================================
:ExitBatch
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Create/Manage Library Entries
 rem === Updated On: 27 Nov 2013 / 23 Oct 2013              
 rem ==========================================================================
:ManageOneLibrary
 rem %1 = Name of Default or Custom Library Container
 rem %2 = New Library Folder Location
 rem %3 = Delete entry, if "DELETE" keyword is supplied

 SET @LIBRARY_EXISTS=FALSE
 SET @FOLDER_EXISTS=FALSE
 SET @LIBRARY=%~1
 SET @LIBRARY=%@LIBRARY:FOLDERID_=%
 SET @LIBRARY=%@LIBRARY:Library=%
 SET @DELETE=%~3

 IF /I "%@LIBRARY:~0,7%"=="CUSTOM_" (
	 SET @ISCUSTOM=TRUE
	 SET @LIBRARY=!@LIBRARY:~7,999!
 )

 rem -- Determine Existence of Library and/or Folder
 FOR /F "TOKENS=*" %%E IN ('SLUTIL ListFolders "%@LIBRARY%" ^| FIND /V "ListFolders: Can't load library."') DO (
	 SET @LIBRARY_EXISTS=TRUE
	 IF /I "%~2"=="%%~E" SET @FOLDER_EXISTS=TRUE
 )

 IF DEFINED @DELETE (
	 rem -- Remove Folder from Library if Folder Exists
	 IF /I "%~3"=="REMOVE" IF /I "%@FOLDER_EXISTS%"=="TRUE" (
		 ECHO:
		 ECHO Removing Library Entry from ... %@LIBRARY%
		 ECHO             Folder Location ... %~2
		 SLUTIL RemoveFolder "%@LIBRARY%" "%~2"
	 )

	 rem -- Delete Library if it Exists
	 IF /I "%~3"=="DELETE" IF /I "%@LIBRARY_EXISTS%"=="TRUE" (
		 ECHO:
		 ECHO Deleting Entire Library ....... %@LIBRARY%
		 SLUTIL Delete "%@LIBRARY%"
	 )
 ) ELSE (
	 rem -- Add Folder to Library if Folder Exists [Possibly Create Library]
	 IF EXIST "%~2" (
		 ECHO:
		 ECHO Creating Library Entry for .... %@LIBRARY%
		 ECHO            Folder Location .... %~2
		 IF /I "!@ISCUSTOM!"=="TRUE" IF /I "%@LIBRARY_EXISTS%"=="FALSE" SLUTIL Create "!@LIBRARY!"
		 SLUTIL AddFolder "%@LIBRARY%" "%~2"
	 )
 )
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET @SCRIPT_BEG#="%%~d"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF
                                                                                          