@ECHO OFF
 rem ==========================================================================
 rem === Create a Backup of TextPad Configuration Settings
 rem ==========================================================================
 rem
 rem Created On: 25 Apr 2011
 rem         By: Beverly Howard
 rem
 rem Updated On: 04 Feb 2012
 rem         By: Mike Good
 rem
 rem Updated On: 13 Jan 2024 / 16 Oct 2023              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2003 and later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem ==========================================================================

 :::  This script has been modified to address some errors in handling folder
 :::  names containing spaces. It also appends the current date and time to  
 :::  each backup, allowing you to maintain more than one set of backups.
 :::  Only the ten (10) most recent backups are retained.
 :::
 :::  ORIGINAL SUMMARY FOLLOWS:
 :::  -------------------------------------------
 :::  BackupTextPadSettings.cmd
 :::
 :::  Saves current Textpad backup settings to two folders in the TextPad program files folder:
 :::  - TextPadConfig.reg   has all config settings, run this on target machine to copy settings
 :::  - TextPadSystem.cab   has all add-ons and syntax files, unzip entire folder contents to
 :::                        the target machine's TextPad program files folder to copy these files.
 :::
 :::  ZIP is more convenient/ubiquitous archive format, but CAB is only one we
 :::  employ without having to go to non-native tools (e.g. 7-zip).  Windows
 :::  Explorer can be used to open resulting CAB files.
 :::
 :::  2012-02-04 Mike Good, Tampa  adapted from Beverly Howard's 2004 add-on
 :::  -------------------------------------------
 :::
 :::  (May 2018) v4.0 -- Added filtered CMD Title Routine;
 :::  -- Added the "..\Helios\TextPad\8\ConfigState.xml" file to the backups
 :::  -- Changed Script Documentation to www.BrainWaveCC.com
 :::
 :::  (Aug 2018) v4.1 -- Added SYNTAX HELP option using CheckParams.exe;
 :::  -- Added the "..\Helios\TextPad\8\GUIState.xml" file to the backups
 :::
 :::  (Oct 2018) v4.2 -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Feb 2019) v4.3 -- Expanded filtering for the CHECKPARAMS command;
 ::   -- Use DATEINFO --Script-Variables to pull in a richer set of variables;
 :::
 :::  (Oct 2019) v4.4 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (May 2020) v4.5 -- Updated :DebugMode routine to support the STEP option;
 :::
 :::  (Nov 2020) v4.6 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Jan 2023) v5.0 -- Added more error checking of file locations;
 :::  -- Added :CheckEssentials routine to ensure the existence of critical files;
 :::
 :::  (Aug 2023) v5.1 -- Change the way the @SCRIPT_BEG# variable is calculated
 :::  in the :ShowStatus subroutine;
 :::
 :::  (Jan 2024) v5.2 -- Place :CheckEssentials routine after :SyntaxHelp routine;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 30 Nov 2020 / 31 May 2018              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Enable Debug Mode If 'Debug' Variable is Set to "TRUE"
 rem === Updated On: 13 May 2020 / 28 Oct 2018              
 rem ==========================================================================
:DebugMode
 SET #DEBUG=& FOR %%d IN (LOG STEP TRUE VERBOSE) DO IF /I "%Debug%"=="%%~d" SET #DEBUG=TRUE
 IF DEFINED #DEBUG (
	 SET @DIAGMODE=TRUE
	 SET @DEBUG_MODE=[DEBUG MODE ON]
	 SET @PAUSE_IF_DEBUG=TIMEOUT 5
	 SET @PRINT_IF_DEBUG=ECHO:^& SET @^& ECHO:^& TIMEOUT 5
	 SET @END_DEBUG_MODE=ECHO:^& ECHO *** DEBUG MODE FINISHED ***^& ECHO:^& SET @^& TIMEOUT 30
	 IF /I "%Debug%"=="LOG" (SET @PAUSE_IF_DEBUG=& SET @PRINT_IF_DEBUG=ECHO:^& SET @^& ECHO:)
	 IF /I "%Debug%"=="STEP" (SET @PAUSE_IF_DEBUG=ECHO ##PAUSE##& SET @PRINT_IF_DEBUG=ECHO ##PRINT##)
	 IF /I "%Debug%"=="VERBOSE" (SET @VERBOSE_OFF=ECHO ON) ELSE (FOR %%t IN (ON OFF) DO SET @VERBOSE_%%t=ECHO %%t)
	 ECHO *** DEBUG MODE = ON ***
	 ECHO ON
 ) ELSE ( FOR %%d IN (DEBUG_MODE PAUSE_IF_DEBUG PRINT_IF_DEBUG END_DEBUG_MODE VERBOSE_ON VERBOSE_OFF) DO SET @%%d=)


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 12 Jan 2024 / 15 Jan 2023              
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v5.2.0.420

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 (
	 ECHO Create a Backup of TextPad Configuration Settings
	 ECHO:
	 ECHO    YOU TYPED: %0 %*
	 ECHO:
	 ECHO   CMD SYNTAX: %~n0
	 ECHO               %~n0  [-h^|-?]
	 ECHO:
	 ECHO  USAGE NOTES:
	 ECHO    * Parameters surrounded by ^<^> are mandatory.
	 ECHO    * Parameters surrounded by [] are optional.
	 ECHO    * Options are case-insensitive, and can be prefaced by "-" or "/".
	 GOTO :ExitBatch
 )

 rem -- Check for Essential Utilities and/or Scripts
 PATH %PATH%;%~dp0
 FINDFILES -w -m -f "NOW.EXE" "DATEINFO.EXE" "CHECKPARAMS.EXE" "MAKESTRING.EXE" "REG.EXE" "MAKECAB.EXE"
 IF ERRORLEVEL 64 GOTO :ExitBatch

 rem -- Obtain Date/Time and Supporting Core Variables
 FOR /F "TOKENS=1-2 DELIMS=;" %%P IN ('DATEINFO --Script-Variables 2^>NUL') DO SET %%P=%%~Q

 rem -- Main Variables
 SET @KEEP_HISTORICAL=10
 SET @DIRNAME=TextPad*
 SET @TP_BACKUP_DIR=
 SET @TP_REG_KEY=
 SET @HELIOS_REG_KEYS="HKCU\Software\Helios" "HKLM\Software\Helios" "HKLM\Software\Wow6432Node\Helios"
 SET @HELIOS_CLASS_KEYS=HKEY_CURRENT_USER\Software\Classes
 SET @CONFIG_ROOT=%USERPROFILE%\AppData\Roaming\Helios\TextPad\
 SET @CAB_DDF=%TEMP%\BackupTextPadSettings.DDF
 SET @TEMP_EXPORT=%TEMP%\BackupTextPadSettings.TEMP

 rem -- Determine newest TextPad registry key (relies on default REG alpha sort)
 FOR %%K IN (%@HELIOS_REG_KEYS%) DO FOR /F "TOKENS=*" %%R in ('REG QUERY "%%~K" 2^>NUL') DO SET @TP_REG_KEY=%%R

 rem -- Get TextPad Folder Info
 SET @PROGFILES="%ProgramFiles%" "%ProgramFiles(x86)%" "%SystemDrive%\Programs"
 FOR %%P IN (%@PROGFILES%) DO IF EXIST "%%~P\%@DIRNAME%" (
 	 FOR /F "TOKENS=*" %%F IN ('DIR "%%~P\%@DIRNAME%" /AD /B /ON 2^>NUL') DO (
 		 SET @TP_ROOT_DIR=%%~P\%%~F
 		 SET @TP_BACKUP_ROOT=%%~P\%%~F\Backup
 		 SET @TP_BACKUP_DIR=!@TP_BACKUP_ROOT!\%@DIRSTAMP%
 	)
 )

 rem -- Backup Files
 SET @REGFILE=%@TP_BACKUP_DIR%\TextPadConfig.REG
 SET @CABFILE=%@TP_BACKUP_DIR%\TextPadSystem.CAB



 rem ==========================================================================
 rem === Verify TextPad Folders
 rem === Updated On: 15 Jan 2023 / 23 Oct 2012              
 rem ==========================================================================
:VerifyFolder
 IF NOT DEFINED @TP_BACKUP_DIR (
 	 ECHO TextPad does not appear to be installed on this system...
	 ECHO No Edition of TextPad is installed on this machine...
 	 GOTO :ExitBatch
 )

 IF NOT EXIST "%@TP_ROOT_DIR%\TextPad.exe" (
 	 ECHO TextPad does not appear to be installed in "%@TP_ROOT_DIR%"
 	 GOTO :ExitBatch
 )

 IF NOT EXIST "%@TP_BACKUP_DIR%" MD "%@TP_BACKUP_DIR%"
 IF NOT EXIST "%@TP_BACKUP_DIR%" (
 	 ECHO You do not have enough permissions to create "%@TP_BACKUP_DIR%"
 	 GOTO :ExitBatch
 )
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Create File and Registry Backups
 rem === Updated On: 31 May 2018 / 24 Oct 2016              
 rem ==========================================================================
:CreateBackup
 PUSHD "%@TP_BACKUP_DIR%"
 ECHO Exporting Various TextPad Registry Settings...
 MAKESTRING "-" 75

 rem -- Backup the registry settings, then use TYPE to convert from Unicode
 REG EXPORT "%@TP_REG_KEY%" "%@TEMP_EXPORT%" /Y
 TYPE "%@TEMP_EXPORT%" >"%@REGFILE%"

 rem -- Find TextPad-related keys under CLASSES hive
 FOR /F "TOKENS=*" %%C IN ('REG QUERY "%@HELIOS_CLASS_KEYS%" ^| FIND /I "TextPad"') DO (
	 SET @CURRENT_CLASS=%%~C
	 SET @CURRENT_CLASS=!@CURRENT_CLASS:%@HELIOS_CLASS_KEYS%\=!
	 REG EXPORT "%@HELIOS_CLASS_KEYS%\!@CURRENT_CLASS!" "%TEMP%\!@CURRENT_CLASS!.REG" /Y
	 TYPE "%TEMP%\!@CURRENT_CLASS!.REG" | FIND /V "Windows Registry Editor Version 5.00" >>"%@REGFILE%"
 )
 MAKESTRING "-" 75
 ECHO:


 rem -- Backup All Available *.XML files (usually ConfigState.XML and GUIState.XML)
:SaveConfigState
 FOR /F "TOKENS=*" %%F IN ('DIR /B /S "%@CONFIG_ROOT%*.xml" ') DO (
	 SET @NEWNAME=%%~F
	 SET @NEWNAME=!@NEWNAME:%@CONFIG_ROOT%=!
	 SET @NEWNAME=!@NEWNAME::\=_!
	 SET @NEWNAME=!@NEWNAME:\=_!
	 FOR /F "TOKENS=*" %%Z IN ('COPY "%%~F" "!@NEWNAME!" /Y') DO ECHO Saving "%%~F" -- %%Z
 )

 rem -- Create a [temporary] directive file for MAKECAB
 ( rem -- Write to Log
	 ECHO: >CON
	 ECHO .set DiskDirectory1="%@TP_BACKUP_DIR%"
	 ECHO .set CabinetNameTemplate=TextPadSystem.cab
	 ECHO .set UniqueFiles="OFF"
	 ECHO .set Cabinet=on
	 ECHO .set Compress=on
	 ECHO .set CompressionType=MSZIP
	 FOR %%F IN ("%@TP_ROOT_DIR%\System\*.*") DO ECHO "%%~pnxF"
 ) >"%@CAB_DDF%"


 rem -- Use MAKECAB to backup the system folder; occasionally this fails for unknown reasons, added crude retry for this
 IF EXIST "%@CABFILE%" DEL /Q "%@CABFILE%"
:RetryMAKECAB
 FOR %%L IN (1 2 3) DO (
 	MAKECAB /F "%@CAB_DDF%"
 	IF ERRORLEVEL 0 GOTO :MAKECAB_Complete
 )

:MAKECAB_Complete
 ECHO:
 ECHO TextPad settings have been backed up to "%@TP_BACKUP_DIR%"
 ECHO -- REG file: !@REGFILE:%@TP_BACKUP_DIR%\=!
 ECHO -- CAB file: !@CABFILE:%@TP_BACKUP_DIR%\=!
 ECHO:
 POPD
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Keep Only The Last %@KEEP_HISTORICAL% TextPad Backups (Default=10)
 rem === Updated On: 05 Feb 2015 / 23 Oct 2012              
 rem ==========================================================================
:PruneOldBackups
 ( rem -- Write to Log
	 ECHO:
	 ECHO:
	 ECHO Pruning Old TextPad Backups [Keep only the last %@KEEP_HISTORICAL% copies]
	 FOR /F "SKIP=%@KEEP_HISTORICAL% TOKENS=*" %%V IN ('DIR "%@TP_BACKUP_ROOT%" /AD /O-N /B') DO (
		 ECHO  - Deleting "%%V" from "%@TP_BACKUP_ROOT%"
		 RD /Q /S "%@TP_BACKUP_ROOT%\%%~V"
	 )
	 ECHO:
 )
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 13 Jan 2024 / 04 Oct 2019              
 rem ==========================================================================
:ExitBatch
 IF NOT DEFINED DEBUG FOR %%V IN ("%@TEMP_EXPORT%" "%@CAB_DDF%" "%TEMP%\TextPad*.REG") DO IF EXIST "%%~V" DEL /Q "%%~V"
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 28 Aug 2023 / 28 Aug 2023              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" FOR /F "TOKENS=*" %%d IN ('DATEINFO -S -F "mm/dd/yyyy hh:nn:ss.zzz" -Q 2^>NUL') DO SET @SCRIPT_BEG#="%%~d"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF
         