@ECHO OFF
 rem ==========================================================================
 rem === Create a Rotating Schedule of SystemState Backups
 rem ==========================================================================
 rem
 rem Created On: 06 Mar 2003
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Dec 2007  (Prior Update: 09 Jan 2007)
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2000 and later
 rem
 rem  Non-Native Utilities Required:
 rem  	Resource Kit ........... http://KB.UltraTech-llc.com/?File=ResKit.TXT
 rem
 rem  Other Scripts Called:
 rem    SetDrive.BAT ........... Primary Script for Setting Common Variables
 rem ==========================================================================

 :::  This script creates a schedule of daily SystemState backups on Windows
 :::  2000/XP/2003 system using the native NTBACKUP utility.  It will generate
 :::  a new backup for each day of the week. This script could be easily
 :::  modified to create a new backup file for each week of the month, as well.
 :::
 :::  You can control the location of the backup file by changing the
 :::  environment variables or adding your desired destination as a
 :::  command-line parameter to the script.
 :::
 :::  Remember to use quotes to surround long file/folder names.
 :::
 :::  The filename which is generated for the backup will include the name of
 :::  the day (Sun, Mon, Tue, etc) and will be overwritten every week.  This
 :::  will provide up to 6 days worth of backup, but requires 7 times the space
 :::  of a single backup.  Minimum SystemState backup is approx 200MB for
 :::  systems running Windows 2000, and 350MB for Windows 2003. Domain
 :::  Controllers will use even more space, as much as 600MB per day, and
 :::  Exchange 2000/2003 servers will also backup the Exchange Store.
 :::
 :::  All task scheduling is handled by SCHTASKS (from XP/2003) you can edit
 :::  this executable to make it run under 2000.
 :::  See: http://www.jsiinc.com/SUBJ/tip4800/rh4806.htm
 :::
 :::  The NOW.EXE utility from the Resource Kit is actually optional.  It's
 :::  benefit is in ensuring that the script can obtain the DATE in a format
 :::  that is friendly even outside US-ENGLISH region.
 :::
 :::  The script now supports the use of marker files to prevent the creation
 :::  of either entire SystemState backup, or of the Exchange portion for
 :::  servers also running Exchange.  Placing the followng files in the
 :::  C:\Scripts\Bat\Input folder will prevent the creation of the files:
 :::
 :::  	- NoExchangeData.DAT: Create only a SystemState backup; No Exchange
 :::  	- NoSystemState.DAT:  Don't create any backups at all.
 :::
 :::
 :::  For More Scripting Assistance, see the following:
 :::  http://KB.UltraTech-llc.com/?File=Scripting.TXT


 rem ==========================================================================
 rem === Enable Debug Mode If 'Debug' Variable is Set to "TRUE"
 rem === Last Modified On: 25 Oct 2005
 rem ==========================================================================
:DebugMode

 :::  This is one of my default subroutines. It allows easy debugging without
 :::  making changes to the script.  All you need to do is set the variable
 :::  DEBUG to be true (SET DEBUG=TRUE) at a CMD prompt, and then run any
 :::  script with this section, and wherever you find the following variable,
 :::  %@PAUSE_IF_DEBUG%, the script will execute a timeout, or whatever else
 :::  you've designated when defining the variable.  Use the ampersand to
 :::  create compound statements, as evidenced by the @END_DEBUG_MODE variable.

 IF /I "%Debug%"=="TRUE" (
	 SET @PAUSE_IF_DEBUG=TIMEOUT 5
	 SET @END_DEBUG_MODE=ECHO:^& ECHO *** DEBUG MODE FINISHED ***^& ECHO:^& SET @^& TIMEOUT 30
	 ECHO *** DEBUG MODE = ON ***
	 ECHO ON
 ) ELSE (
	 SET @PAUSE_IF_DEBUG=
	 SET @END_DEBUG_MODE=
 )


 rem ==========================================================================
 rem === Display Syntax Help if Required/Requested
 rem === Last Modified On: 09 Mar 2003
 rem ==========================================================================
:NeedHelp
 SET @VER=%~nx0 v2.3.0

 :::  Here we parse the command-line to see if /H or /HELP was entered so that
 :::  we can jump right to the Syntax Help. The second line checks for "/?"
 :::  which is also a request for assistance, but we can't use a ? in the FOR
 :::  loop, as it has a different purpose there.

 FOR /F "TOKENS=2" %%H IN ('CHECKPARAMS -q -c "H HELP ?" -s %* 2^>NUL') DO GOTO :HelpMessage


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Last Modified On: 09 Jan 2007  (Prior Update: 20 Mar 2006)
 rem ==========================================================================
:Variables

 :::  These are the variables we want to use in the rest of the script. We will
 :::  attempt to assign the first parameter to the variable @BACKUPFILE. Using
 :::  "%~1" rather than "%1" means that any quotation marks which were included
 :::  will be removed. Very useful if you plan any string manipulation, and any
 :::  input might include spaces, thus requiring quotes.  The good thing is
 :::  that even if quotes are not used, this works properly.
 :::
 :::  BTW, I like prefixing all my variables with "@" because it makes it
 :::  easier to find them with a SET statement (SET @).

 SETLOCAL


 :::  Here we manipulate the DATE and TIME so that we can get variables to use
 :::  when renaming our logs.  In this case, we're really interested in the
 :::  DAY, but I usually copy all the lines in so that if I want some other
 :::  aspect of the date or time, I don't have to remember the syntax.
 :::
 :::  The truth is the this line is made unnecessary by the one that follows,
 :::  but in the event that someone runs this script in isolation, they'll at
 :::  least have the date functions working properly.

 rem -- Calculate Date/Time (Very Short Procedure)
 FOR /F "TOKENS=1-4 DELIMS=/ " %%D IN ('DATE /T') DO SET @DAY=%%D& SET @DATE=%%G%%E%%F
 SET @TIMESTAMP=%DATE% at %TIME%

 :::  The SetDrive.BAT file is a special file for us that sets a number of
 :::  crucial variables, including the date and time.  Why do we need this
 :::  when it's been done above?  Because the date variables need to be
 :::  parsed differently in Non-US Regions, and that manipulation requires
 :::  different utilities to obtain the date (NOW.EXE from the ResKit).
 :::  This is one of the key reasons why the Resource Kit is needed/useful
 :::  for this edition of the script, whereas it wasn't necessary before.
 :::
 :::  One of the other critical variables that gets set in SetDrive.BAT is
 :::  the location of the \Storage folder structure, where logs will be
 :::  stored.  We like to keep our scripts and their output in consistent
 :::  locations.

 rem -- Get Official Drive Letters and Calculate Date/Time for NON-US Regions
 SET @STORAGE=%SystemDrive%\STORAGE& IF EXIST "%~dp0SetDrive.BAT" CALL "%~dp0SetDrive.BAT"
 SET @FILESTAMP=%@TODAY%-%@NOW%.BKF


 :::  These variables are for scheduling the Backup job

 rem -- Job Scheduling Variables
 SET @JOBNAME=SystemState Backup
 SET @JOBPATH=%~f0
 SET @JOBFREQ=DAILY
 SET @JOBTIME=01:15:00
 SET @JOBUSER=SYSTEM
 SET @JOBDATE=

 rem -- Standard Variables
 SET @NOBACKUP=%~dp0Input\NoSystemState.DAT
 SET @NOEXCHANGE=%~dp0Input\NoExchangeData.DAT
 SET @LOGPATH=%USERPROFILE%\Local Settings\Application Data\Microsoft\Windows NT\NTBackup\Data
 SET @BACKUPEXE=%SystemRoot%\System32\ntbackup.exe
 SET @BACKUPFILE=%~1& IF "%~1"=="" SET @BACKUPFILE=%@STORAGE%\Backups\SystemState.bkf


 :::  We're going to call a subroutine in order to manipulate the backup file
 :::  that we were asked to use, and add the Day of the Week to the name. There
 :::  are several ways to do it, but the subroutine is the coolest, IMO.
 :::  Details will be provided in the subroutine itself

 CALL :SetBackupName "%@BACKUPFILE%"


 rem ==========================================================================
 rem === Create Scheduled Job using SCHTASKS (XP/2003/Vista/+) or AT (NT/2000)
 rem === Last Modified On: 09 Jan 2007  (Prior Update: 26 May 2006)
 rem ==========================================================================
:ScheduleJobs
 ECHO:
 ECHO Scheduling "%@JOBNAME%" on %COMPUTERNAME%...
 ECHO:

 :::  We want to schedule the job with the TaskScheduler if it exists, or with
 :::  the AT command if it doesn't, but ONLY if the command isn't already
 :::  scheduled.  So, we search for a substring that will exist after the job
 :::  is properly scheduled.  We previously defined @JOBNAME as the full path
 :::  of the name of the executed script.  That way, if someone renames the
 :::  script, they won't have to change the information in this section.

 IF EXIST "%SystemRoot%\System32\SCHTASKS.EXE" (
	 SCHTASKS /QUERY /V /FO LIST | FIND /I "%@JOBPATH%" >NUL
	 IF ERRORLEVEL 1 (
		 SCHTASKS /CREATE /TN "%@JOBNAME%" /TR "%@JOBPATH%" /RU "%@JOBUSER%" /SC %@JOBFREQ% %@JOBDATE% /ST %@JOBTIME%  %@FORCE%
	 )
 ) ELSE (
	 AT | FIND /I "%@JOBPATH%" >NUL
	 IF ERRORLEVEL 1 AT %@JOBTIME% /EVERY:M,T,W,TH,F,SA,SU  "%@JOBPATH%"
 )

 rem -- If ANY of the parameters is "." then update the scheduled job and exit the script
 FOR %%P IN (%*) DO IF "%%~P"=="." (
	 SCHTASKS /CREATE /TN "%@JOBNAME%" /TR "%@JOBPATH%" /RU "%@JOBUSER%" /SC %@JOBFREQ% %@JOBDATE% /ST %@JOBTIME%  %@FORCE%
	 GOTO :EOF
 )


 rem ==========================================================================
 rem === Create Backup Set Config File (in Unicode) If It Doesn't Already Exist
 rem === Last Modified On: 16 Dec 2007  (Prior Update: 09 Jan 2007)
 rem ==========================================================================
:NewBackupSet

 :::  Here's a fun one...  We know how to backup the SystemState, but we might
 :::  want to backup other things -- like Exchange -- if we find it on the
 :::  system, or have already set something up manually.  So, we're going to
 :::  try and create a BackupSet file, which has to be written in Unicode, to
 :::  automatically handle the SystemState as well as Exchange if it exists.
 :::  If, however, there's  a reason that the BackupSet cannot be autocreated,
 :::  or we don't want to use one, then we create a file (SystemStateOnly.DAT)
 :::  in the Input folder which will allow us to skip this section entirely.
 :::  By default, the Input folder is: C:\Scripts\Bat\Input

 IF NOT EXIST "%@NOBACKUP%" ( rem -- Drop This Marker File To Prevent Creation of Backup Set
	 IF NOT EXIST "%@BKUPSET%" (
		 ECHO Creating Backup Configuration File...
		 IF NOT EXIST "%@NOEXCHANGE%" ( rem -- Drop This Marker File To Prevent Creation of Exchange Backup Set
		 	 ECHO Adding Exchange Backup Info to the Configuration File...
		 	 FOR /F "TOKENS=1* DELIMS==" %%B IN ('SET EXCHBACKUP') DO CMD /U /C ECHO JET %%C >>"%@BKUPSET%"
		 )
		 CMD /U /C ECHO SystemState >>"%@BKUPSET%"
	 )
 )


 rem ==========================================================================
 rem === Initiate Backup Based on Backup Set or Generic SystemState Backup
 rem === Last Modified On: 08 Mar 2006
 rem ==========================================================================
:CreateBackup

 :::  Since we allow the destination backup file to be provided at the command
 :::  line, we need to make sure that the parent folder exists, so there won't
 :::  be any problems.

 IF NOT EXIST "%@BACKUPDIR%" MD "%@BACKUPDIR%"


 :::  The purpose behind the manipulation of the archive bit (using ATTRIB) is
 :::  that we will want to view the log that NTBACKUP creates when it is
 :::  complete. Since we can't be sure what the filename will be, we'll just
 :::  clear the archive bit on all the log files, and then when the script is
 :::  complete, the file with the archive bit set will be the file we want to
 :::  view.

 ECHO Creating SystemState Backup for %COMPUTERNAME%...
 ECHO:
 ECHO Please wait (up to 5 minutes)...
 ATTRIB -A "%@LOGPATH%\*.LOG"


 :::  Now, we're going to use a whole lot of variables so that the NTBACKUP
 :::  parameters don't make it any harder to read than it is.  I ran NTBACKUP
 :::  manually, and saved the job to run later, so that I could get all the
 :::  parameters.   Also, run "NTBACKUP /?" to get a list of most of the
 :::  available parameters in the syntax.  If a BackupSet file exists, then
 :::  we'll use that, otherwise we'll just backup the SystemState.

 rem -- Backup SystemState ONLY, or SystemState plus whatever is in the BackupSet file
 IF NOT EXIST "%@BKUPSET%" (
	 %@BACKUPEXE% backup SystemState /n %@ARCHIVE% /d "%@JOBNAME%" /v:yes /r:no /rs:no /hc:off /m normal /j "%@JOBFREQ%" /l:s /f "%@BACKUPFILE%"
 ) ELSE (
	 %@BACKUPEXE% backup "@%@BKUPSET%" /n %@ARCHIVE% /d "%@JOBNAME%" /v:yes /r:no /rs:no /hc:off /m normal /j "%@JOBFREQ%" /l:s /f "%@BACKUPFILE%"
 )


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Last Modified On: 15 Oct 2005
 rem ==========================================================================
:ExitBatch

 :::  When it's done, we want to see the folder and get the output of the
 :::  current log.  The FOR command is really cool in NT4 and later versions of
 :::  Windows (2000/XP/2003/etc) because it will allow you to process the
 :::  output of a command (surrounded by single quotes and using /F ) without
 :::  having to first create a temp file with the desired output. So, we can
 :::  immediately act on the results of our DIR command (looking for .LOG files
 :::  with the archive bit set) and send the result to our TYPE command.  No
 :::  need to delete any temp files after the fact. Also, we want to output all
 :::  the backup files in chronological order, without listing any directories...

 DIR "%@BACKUPDIR%\*.BKF" /OD /A-D
 FOR /F %%V IN ('DIR "%@LOGPATH%\*.LOG" /AA /B') DO TYPE "%@LOGPATH%\%%V"


 :::  If Debug Mode is enabled, as discussed in the beginning of the script,
 :::  then we need to close it out with the final output of information. And
 :::  we need clear out all the environment variables we have used, and then
 :::  skip to the end to avoid any subroutines.

 %@END_DEBUG_MODE%
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set Name of Backup File Based On Day of Week
 rem === Last Modified On: 10 Mar 2003
 rem ==========================================================================
:SetBackupName

 :::  The fun part. Shell Scripting in Windows NT and beyond allow you to use
 :::  the CALL statement to execute a subroutine within the script, as well as
 :::  the normal use of calling another script and returning execution to the
 :::  calling program.  When CALL is used, whether internally or externally,
 :::  the parameters of the CALL statement are mapped to %1 through %9.
 :::
 :::  The reason that this is cool, is that we can do more fun stuff with the
 :::  %0-%9 and with the single character FOR variables than we can with our
 :::  user-created environment variables. This allows us to reconstruct the
 :::  file and folder names for use with NTBACKUP.
 :::
 :::  To see what options are available to you, run FOR /? AT a command prompt,
 :::  and look at the last 20 or so lines of the syntax options.

 rem %1 = Current Filename
 SET @BACKUPFILE=%~dp1%~n1-%@DAY%.BKF
 SET @BACKUPDIR=%~dp1
 SET @ARCHIVE="%~n1-%@DAY%.BKF created on %@DATE%"
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help For "/?"
 rem === Last Modified On: 19 Mar 2006
 rem ==========================================================================
:HelpMessage

 :::  Here is where we display the syntax for our script.  We use environment
 :::  variables to process the current filename, so we don't have to edit that
 :::  part if you simply choose the change the script name. Any parameters that
 :::  are surrounded by square brackets [] are optional, and those with the
 :::  curly braces {} are mandatory.

 ECHO:
 ECHO Syntax Help for %@VER%
 ECHO:
 ECHO === %~n0 [destination_file]
 ECHO:
 ECHO Example: %~n0
 ECHO          %~n0 C:\Backups\MyBackup.bkf
 ECHO          %~n0 "C:\My Backups\MyBackup.bkf"
 ECHO:
 ECHO ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 ECHO NOTE: Any parameters which contain spaces should be surrounded by quotes.
 ECHO       Shortnames *can* be surrounded by quotes if you choose, but it is
 ECHO       not mandatory by any means.
 ECHO ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 ECHO:
