@ECHO OFF
 rem ==========================================================================
 rem === Obtain Basics Forensics Info from Unknown Systems (for Darren)
 rem ==========================================================================
 rem
 rem Created On: 10 May 2005
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Oct 2023 / 30 Nov 2020              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2000 or later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    Resource Kit ........... https://en.wikipedia.org/wiki/Resource_Kit
 rem    PSTOOLS ................ https://technet.microsoft.com/en-us/sysinternals/bb896649
 rem
 rem    Blat ................... http://www.blat.net/
 rem    NMAP (Optional) ........ https://nmap.org/download.html
 rem    WHOSIP (Optional) ...... http://www.nirsoft.net/utils/whosip.html
 rem
 rem  Input Files Used By Script (stored in %SystemDrive%\Scripts\Bat\Input):
 rem    BasicInfo.TXT .......... List of IP Addresses to Check, if None Provided
 rem
 rem  Required Scripts:
 rem    SetDrive.BAT ........... Set Global Variables for this Environment
 rem ==========================================================================

 :::  This script was simply stripped down from GetInfo.BAT.  It only uses
 :::  RESOLVE.EXE and cycles through a list of IPs in the BasicInfo.TXT file.
 :::  The full documentation for GetInfo remains below:
 :::
 :::  This script uses various utilites from the SysInternals site to capture
 :::  running diagnostics info such as the list of running processes, active
 :::  DLLs and open file handles on a remote machine, in order to identify it.
 :::
 :::  This data is obtained via several of the utilities from the PSTOOLS
 :::  suite.  It also uses SRVINFO which can be downloaded as part of the free
 :::  Windows 2003 Resource Kit (only runs on XP and 2003).
 :::
 :::  NOTE: The Resource Kit has to be installed after being downloaded...
 :::
 :::  This script also supports NMAP.  To run it, the NMAP folder must be in
 :::  your PATH.  As it pertains to this script, the default supported PATH
 :::  for NMAP is "C:\Programs\NMap"
 :::
 :::  Much of the detailed info is only possible with admin access of the
 :::  remote system.  Some information might not be possible depending on the
 :::  configuration of the remote system (e.g. firewall enabled). Additionally,
 :::  the further away the system in question is, the longer the script will
 :::  take to obtain some of the details.
 :::
 :::  (Nov 2010) Changed the default log extension of the reports from .LOG to
 :::  .TXT so that the reports that are mailed to mobile devices can be read
 :::  automatically. (Not too many devices can natively handle .LOG files)
 :::
 :::  (Jan 2014) v2.0 -- Expanded the @NMAP variable to include multiple folder
 :::  paths. Moved the BASIC.TXT input file to the INPUT folder, rather than
 :::  using the current folder of the script.
 :::
 :::  (Jun 2018) v3.0 -- Changed the HELP routine to use FINDSTR;
 :::  -- Added new Filtered CMD Title and Syntax Help processing routines;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com
 :::  -- Added TRACERT and WHOIS to the list of IP lookups
 :::
 :::  (Jul 2018) v3.1 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::
 :::  (Oct 2018) v3.2 -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Oct 2019) v3.3 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (May 2020) v3.4 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::
 :::  (Nov 2020) v3.5 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Oct 2023) v3.6 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 30 Nov 2020 / 11 Jun 2018              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 30 Nov 2020 / 26 May 2020              
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v3.6.0.360

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- Main Variables
 SET @SOURCE=%~1&  IF "%~1"=="" SET @SOURCE=%~dp0Input\BasicInfo.TXT
 SET @NMAPDIRS="%SystemDrive%\Nmap";"%SystemDrive%\Programs\NMap";"%ProgramFiles%\Nmap";"%ProgramFiles(x86)%\Nmap"
 SET @NMAP=& FOR %%F IN (%@NMAPDIRS%) DO IF EXIST "%%~F" (SET @NMAP=%%~F& PATH !PATH!;%%~F)
 PATH %PATH%;%SystemDrive%\Scripts\Utils

 rem -- Standard Log Variables
 SET @LOGHEADER=Remote Forensics Log -- [Generated by %@VER% on %@TIMESTAMP%]
 SET @LOGPATH=%@STORAGE%\Logs\Forensics
 SET @LOGFILE=%@LOGPATH%\%~n0.%@FILESTAMP%


 rem ==========================================================================
 rem === Obtain As Much Forensics Info As Possible
 rem === Updated On: 05 Oct 2019 / 11 Jun 2018              
 rem ==========================================================================
:GetForensics
 PUSHD %@NMAP%
 CALL :GetTime @STIME

 CLS
 ECHO Attempting To Obtain Basic Forensics Info... (%@VER%)
 ECHO:
 ECHO Please Standby...
 ECHO:
 CALL :SpecialInfo

 rem -- Create Initial Log Headers...
 IF NOT EXIST "%@LOGPATH%" MD "%@LOGPATH%"
 ( rem -- Write to Log
	 ECHO %@LOGHEADER%
	 ECHO:
 ) >"%@LOGFILE%"

 rem -- Cycle Through List and Obtain Basic IP/NAME Info...
 FOR /F %%S IN (%@SOURCE%) DO (
	 SET @UNKNOWN=%%S
	 SET @UNKNOWN=!@UNKNOWN:\\=!
	 FOR /F "TOKENS=*" %%V IN ('RESOLVE !@UNKNOWN!') DO (
		 ECHO %%S	= %%V
		 ECHO %%S	= %%V >>"%@LOGFILE%"
	 )
 )
 CALL :Separator

 rem -- Cycle Through List and Attempt To Capture Basic Forensics Data...
 FOR /F %%S IN (%@SOURCE%) DO (
	 SET @UNKNOWN=%%S
	 SET @UNKNOWN=!@UNKNOWN:\\=!
	 CALL :LogInfo "RESOLVE           !@UNKNOWN!"
	 CALL :LogInfo "WHOSIP            !@UNKNOWN!"
	 CALL :LogInfo "PING              !@UNKNOWN!"
	 CALL :LogInfo "TRACERT -d -w 100 !@UNKNOWN!"
	 CALL :LogInfo "NMAP -O -P0       !@UNKNOWN!"
	 CALL :LogInfo "NET VIEW        \\!@UNKNOWN!"
	 CALL :LogInfo "PSLOGGEDON      \\!@UNKNOWN! %@PSPARAMS%" -L
	 CALL :LogInfo "PSLIST          \\!@UNKNOWN! %@PSPARAMS%"
	 CALL :Separator #
 )


 rem ==========================================================================
 rem === Close Out Forensic Log Files
 rem === Updated On: 17 Jan 2016 / 05 Feb 2015              
 rem ==========================================================================
:FinishLog
 CALL :GetTime @ETIME
 ( rem -- Write to Log
	 ECHO:
	 ECHO %@DIVIDER-%
	 ECHO  Forensics Started On .......................... %@STIME%
	 ECHO  Forensics Ended On ............................ %@ETIME%
	 ECHO:
 ) >>"%@LOGFILE%"


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019              
 rem ==========================================================================
:ExitBatch
 POPD
 DIR "%@LOGFILE%" & START "Basic Forensics Info" "%@LOGFILE%"
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Store Current Date and Time in Variable
 rem === Updated On: 16 Oct 2023 / 11 Jun 2018              
 rem ==========================================================================
:GetTime
 rem %1 = Variable to Store Date/Time

 SET %~1=!DATE! at !TIME: =0!
 FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET %~1#="%%~d"
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Store Current 12-Hour Time in Variable
 rem === Updated On: 11 Jun 2018                     
 rem ==========================================================================
:GetTime12
 rem %1 = Variable to Store Time

 FOR /F "TOKENS=*" %%T IN ('TIME /T') DO SET %~1=%%~T
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Convert Multiple Spaces To Single Space
 rem === Updated On: 16 May 2005                     
 rem ==========================================================================
:SingleSpace
 rem %1 = New Variable Name
 rem %2 = Variable to Reduce To Single Spaces

 SET @Z=%~2
 SET @Z=%@Z:'="%& FOR /L %%S IN (1,1,100) DO SET @Z=!@Z:  = !
 SET %~1=%@Z%
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Execute Process And Log Output
 rem === Updated On: 11 Jun 2018 / 17 Jan 2016              
 rem ==========================================================================
:LogInfo
 rem %1 = Diagnostics Command/Utility with parameters

 SET @CMD=%~1
 SET @CMD=%@CMD:'="%
 CALL :GetTime12 @CTIME
 CALL :SingleSpace @MSG %1

 ECHO  [%@CTIME%]: %@CMD%
 ( rem -- Write to Log
	 ECHO:
	 ECHO %@DIVIDER-%
	 ECHO --- [!TIME!]: %@MSG%
	 ECHO %@DIVIDER-%
	 %@CMD%
	 ECHO:
 ) >>"%@LOGFILE%" 2>&1
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Output Warning Message
 rem === Updated On: 17 Jan 2016 / 10 Oct 2013              
 rem ==========================================================================
:SpecialInfo
 ECHO:
 ECHO %@DIVIDER-%
 ECHO  P N:  This script may break if all the necessary files have not been
 ECHO  L O:  downloaded and placed somewhere in your PATH.  Additionally, many
 ECHO  E T:  of the commands will only provide details if you have sufficient
 ECHO  A E:  ADMINISTRATIVE ACCESS to the remote system.  This script is to be
 ECHO  S  :  used for diagnostics or forensics purposes only -- and only where
 ECHO  E  :  you are the authorized user or admin of the system or network.
 ECHO %@DIVIDER-%
 ECHO:
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Generate Output Separator
 rem === Updated On: 11 Jun 2018 / 17 Jan 2016              
 rem ==========================================================================
:Separator
 rem %1 = If Non-Blank, Display Thick Divider

 ECHO %@DIVIDER-%
 ECHO:
 ( rem -- Write to Log
	 IF "%~1"=="" (
		 ECHO %@DIVIDER*%
		 ECHO:
	 ) ELSE (
		 FOR /L %%S IN (1,1,3) DO ECHO %@DIVIDER*%
		 FOR /L %%S IN (1,1,3) DO ECHO:
	 )
 ) >>"%@LOGFILE%"
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" CALL :GetTime @SCRIPT_BEG
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 30 Nov 2020 / 11 Jun 2018              
 rem ==========================================================================
:HelpMessage
 ECHO Obtain Basics Forensics Info from Unknown Systems
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  [List_Of_Systems ^| "'Command_Expression'"]
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0
	 ECHO  %~n0 C:\Temp\Systems.TXT
	 ECHO  %~n0 "%~dp0Input\ServerList.TXT"
	 ECHO  %~n0 "'ECHO \\single_machine'"
	 ECHO  %~n0 "'ECHO \\%COMPUTERNAME%'"
	 ECHO  %~n0 "'ECHO 123.45.67.89'"
	 ECHO  %~n0 -?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
 )
 CALL :ShowStatus "FINISHED"
                                       