@ECHO OFF
 rem ==========================================================================
 rem === Perform Bulk Rename of Characters in a Group Of Files from XXX to YYY
 rem ==========================================================================
 rem
 rem Created On: 28 Nov 2005
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Oct 2023 / 15 Mar 2021              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2000 or later
 rem
 rem  Non-Native Utilities Used/Required: NONE
 rem ==========================================================================

 :::  This script will enumerate all files in a given folder and change the
 :::  specified characters in the file names from whatever you supply as the
 :::  source to whatever you supply as the destination characters.
 :::
 :::  (Apr 2018) v1.10 -- Added Filtered CMD TITLE Routine;
 :::  -- The SYNTAX HELP routine has been updated;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com
 :::
 :::  (Jul 2018) v1.11 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::
 :::  (Oct 2018) v1.12 -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Feb 2019) v2.0 -- Updated to use DATEINFO --Script-Variables;
 :::
 :::  (Oct 2019) v2.1 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (May 2020) v3.0 -- Updated :DebugMode routine to support the STEP option;
 :::  -- Added PREFIX and SUFFIX functions to the rename options;
 :::  -- Added a list-only mode that shows which file would be renamed;
 :::  -- Added a dir mode to rename folders/directories only;
 :::  -- Added Verification before proceeding with the renaming operation;
 :::  -- Refactored the :GetParams routine using the CheckParams.exe utility;
 :::
 :::  (Jun 2020) v3.1 -- Streamlined the file selection/enumeration processes;
 :::
 :::  (Nov 2020) v3.2 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Mar 2021) v3.3 -- All the use of /B: to set a base folder for operation;
 :::
 :::  (Oct 2023) v3.4 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 23 Jun 2020 / 19 Apr 2018              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Enable Debug Mode If 'Debug' Variable is Set to TRUE/LOG/STEP/VERBOSE
 rem === Updated On: 13 May 2020 / 28 Oct 2018              
 rem ==========================================================================
:DebugMode
 SET #DEBUG=& FOR %%d IN (LOG STEP TRUE VERBOSE) DO IF /I "%Debug%"=="%%~d" SET #DEBUG=TRUE
 IF DEFINED #DEBUG (
	 SET @DIAGMODE=TRUE
	 SET @DEBUG_MODE=[DEBUG MODE ON]
	 SET @PAUSE_IF_DEBUG=TIMEOUT 5
	 SET @PRINT_IF_DEBUG=ECHO:^& SET @^& ECHO:^& TIMEOUT 5
	 SET @END_DEBUG_MODE=ECHO:^& ECHO *** DEBUG MODE FINISHED ***^& ECHO:^& SET @^& TIMEOUT 30
	 IF /I "%Debug%"=="LOG" (SET @PAUSE_IF_DEBUG=& SET @PRINT_IF_DEBUG=ECHO:^& SET @^& ECHO:)
	 IF /I "%Debug%"=="STEP" (SET @PAUSE_IF_DEBUG=ECHO ##PAUSE##& SET @PRINT_IF_DEBUG=ECHO ##PRINT##)
	 IF /I "%Debug%"=="VERBOSE" (SET @VERBOSE_OFF=ECHO ON) ELSE (FOR %%t IN (ON OFF) DO SET @VERBOSE_%%t=ECHO %%t)
	 ECHO *** DEBUG MODE = ON ***
	 ECHO ON
 ) ELSE ( FOR %%d IN (DEBUG_MODE PAUSE_IF_DEBUG PRINT_IF_DEBUG END_DEBUG_MODE VERBOSE_ON VERBOSE_OFF) DO SET @%%d=)


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 15 Mar 2021 / 30 Nov 2020              
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v3.4.0.340

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc --mp=2 -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Obtain Date/Time and Supporting Core Variables
 FOR /F "TOKENS=1-2 DELIMS=;" %%P IN ('DATEINFO --Script-Variables 2^>NUL') DO SET %%P=%%~Q

 rem -- Main Variables
 SET @MODE=Non-Recursively
 SET @RECURSIVE=FALSE
 SET @PREFIX=FALSE
 SET @SUFFIX=FALSE
 SET @LISTONLY=
 SET @DIR=
 SET @DIRONLY=FALSE
 SET @COUNT=0
 SET @OLDNAME="%~1"
 SET @TMPNAME="%~2"& IF "%~2"==":" SET @TMPNAME=

 rem -- Process Special Command-line Parameters
 SET @ALL_PARAMS=%*
 CALL :GetParams "F FILESPEC" @FILESPEC & IF NOT DEFINED @FILESPEC (SET @FILESPEC=*.*)
 CALL :GetParams "B BASEDIR"  @BASEDIR  & IF NOT DEFINED @BASEDIR  (SET @BASEDIR=%CD%)
 CALL :GetParams "R S"    & IF DEFINED @OK (SET @RECURSIVE=TRUE& SET @MODE=Recursively)
 CALL :GetParams "PRE"    & IF DEFINED @OK (SET @PREFIX=TRUE)
 CALL :GetParams "SUF"    & IF DEFINED @OK (SET @SUFFIX=TRUE)
 CALL :GetParams "D DIR"  & IF DEFINED @OK (SET @DIRONLY=TRUE& SET @DIR=/AD)
 CALL :GetParams "L LIST" & IF DEFINED @OK (SET @LISTONLY=ECHO)
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Select Files To Rename (Recursively or Non-Recursively)
 rem === Updated On: 15 Mar 2021 / 23 Jun 2020              
 rem ==========================================================================
:SelectFiles
 ECHO Preparing to Rename Files...
 ECHO:
 ECHO %@DIVIDER-%
 FOR %%C IN (PREFIX SUFFIX) DO IF /I "!@%%C!"=="TRUE" SET @OLDNAME=*** Ignored Because of %%C ***
 ECHO  Current Folder ..... %CD%
 ECHO  Starting Folder .... %@BASEDIR%
 ECHO  Old Name ........... %@OLDNAME%
 ECHO  New Name ........... %@TMPNAME%
 ECHO  Filespec ........... %@FILESPEC%
 ECHO  Rename Mode ........ %@MODE%
 ECHO  Add Prefix? ........ %@PREFIX%
 ECHO  Add Suffix? ........ %@SUFFIX%
 ECHO  Folders Only? ...... %@DIRONLY%
 IF DEFINED @LISTONLY ECHO  Rename Mode .... LIST ONLY; NO RENAME
 ECHO %@DIVIDER-%

 rem -- Verify Destination
 IF DEFINED @BASEDIR SET @BASEDIR=!@BASEDIR:"=!
 IF NOT EXIST "%@BASEDIR%" (
	 ECHO:
	 ECHO *** ERROR: INVALID DESTINATION -- "%@BASEDIR%"
	 GOTO :ExitBatch
 )

 rem -- Verify Operation
 CHOICE /C YNQEX /T 30 /D Y /N /M "Continue running this script? (Y/N/Q) "
 IF ERRORLEVEL 2 GOTO :ExitBatch

 rem -- Begin Operations
 PUSHD "%@BASEDIR%"
 NOW \n\n\v -- Searching for files to rename...\n\-

 rem -- Non-Recursive Rename
 IF /I "%@RECURSIVE%"=="FALSE" (
	 FOR /F "TOKENS=*" %%F IN ('DIR /B %@DIR% %@FILESPEC% 2^>NUL') DO (
		 SET @FILENAME="%%~nxF"
		 CALL :RenameOneFile !@FILENAME! !@OLDNAME! !@TMPNAME!
	 )
	 POPD
	 GOTO :ExitBatch
 )

 rem -- Recursive Rename
 FOR /R %%D IN (.) DO (
	 PUSHD "%%D"
	 FOR /F "TOKENS=*" %%F IN ('DIR /B %@DIR% %@FILESPEC% 2^>NUL') DO (
		 SET @FILENAME="%%~nxF"
		 CALL :RenameOneFile !@FILENAME! !@OLDNAME! !@TMPNAME!
	 )
	 POPD
 )
 POPD
 %@PAUSE_IF_DEBUG%
 GOTO :ExitBatch


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 19 May 2020 / 04 Oct 2019              
 rem ==========================================================================
:ExitBatch
 ECHO:
 ECHO Number of Files Renamed: %@COUNT%
 ECHO %@DIVIDER-%
 IF /I "%Debug%"=="TRUE" DIR %@DIR%
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Rename One File
 rem === Updated On: 23 Jun 2020 / 26 May 2020              
 rem ==========================================================================
:RenameOneFile
 rem %1 = Name of File
 rem %2 = Old Text
 rem %3 = New Text

 SET @NEWNAME=%1
 SET @NEWNAME=!@NEWNAME:%~2=%~3!
 IF /I "%@PREFIX%"=="TRUE" SET @NEWNAME="%~3%~1"
 IF /I "%@SUFFIX%"=="TRUE" SET @NEWNAME="%~1%~3"

 IF NOT !@NEWNAME!=="%~1" (
	 IF DEFINED @LISTONLY (
		 ECHO  ***NOT*** Renaming: "%~1" to !@NEWNAME!
	 ) ELSE (
		 ECHO  Renaming: "%~1" to !@NEWNAME!
		 REN "%~1" !@NEWNAME!
	 )
	 SET /A @COUNT+=1
 )
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Use CheckParams.exe to Capture Selected Parameters
 rem === Updated On: 15 Mar 2021 / 21 May 2020              
 rem ==========================================================================
:GetParams
 rem %1 = Parameters to Search For
 rem %2 = Variable to Set

 SET @OK=& IF "%~1"=="" GOTO :EOF
 FOR /F "TOKENS=2-3*" %%v IN ('CHECKPARAMS -q -a -c "%~1" -s %@ALL_PARAMS% 2^>NUL') DO IF /I "%%~v"=="TRUE" (
	 SET @OK=T
	 SET @REQ=T
	 IF NOT "%~2"=="" SET %~2=%%~x
	 IF "%%~w"=="" GOTO :EOF
	 SET @USED=%%~w
	 SET @USED=!@USED:[=!
	 SET @USED=!@USED:]=!
	 SET @SELECTED=!@SELECTED! !@USED!
 )
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET @SCRIPT_BEG#="%%~d"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 15 Mar 2021 / 30 Nov 2020              
 rem ==========================================================================
:HelpMessage
 ECHO Perform Bulk Rename of Characters in a Group Of Files from XXX to YYY
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  ^<old_string^> ^<new_string^> [OPTIONS]
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:
 ECHO -------------------
 ECHO OPTION DEFINITIONS:
 ECHO -------------------
 ECHO  -?,  /H ..... Display This Help Message  (also --HELP)
 ECHO  -??, /H2 .... Display Extended Help Message
 ECHO       /B: .... Set Base Folder  (DEFAULT = current folder)
 ECHO       /F: .... Provide Search Spec to rename  (DEFAULT = *.*)
 ECHO  /S,  /R ..... Recursive: Search subfolders
 ECHO       /D ..... Rename Dirs/Folders Only -- Not Files
 ECHO       /L ..... List Only Mode; Files Not Renamed
 ECHO       /PRE ... Add the new text as a prefix
 ECHO       /SUF ... Add the new text as a suffix
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0 Bad Good
	 ECHO  %~n0 123 4567 /F:"*.DOC"
	 ECHO  %~n0 "Version 5" "Edition 6" /F:"*.XLS" /R
	 ECHO  %~n0 "Obsolete" ":" /F:"*.TXT" /S
	 ECHO  %~n0 Silly Funny
	 ECHO  %~n0 ABCD XYZ123 *.* /S /B:"%SystemDrive%\MyFolder"
	 ECHO  %~n0 -?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
	 ECHO:
	 ECHO  * To delete the old string, use ":" as the replacement string.
	 ECHO:
	 ECHO  * If /PRE or /SUF is selected, then the OLD STRING is ignored.
 )
 CALL :ShowStatus "FINISHED"
                                                                 