@ECHO OFF
 rem ==========================================================================
 rem === Change Any Local Account Password on a Group of Systems
 rem ==========================================================================
 rem
 rem Created On: 21 Feb 2001
 rem         By: Andrew S. Baker (adapted from a script by Danny Cahoon)
 rem
 rem Updated On: 12 Jan 2024 / 16 Oct 2023              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: NT4 or later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    ADFind ................. http://www.joeware.net/freetools/tools/adfind/
 rem    ChangePW ............... http://www.joeware.net/freetools/tools/changepw/
 rem
 rem  Input Files Used By Script (stored in %SystemDrive%\Scripts\Bat\Input):
 rem    CustomVariables.TXT .... Centralized Configuration Settings for Scripts
 rem
 rem  Required Scripts:
 rem    SetDrive.BAT ........... Set Global Variables for this Environment
 rem ==========================================================================

 :::  This script uses CUSRMGR from the Win2K ResKit to change the password of
 :::  any account on a group of systems. You need to supply the name of the
 :::  account and the new password at the command line.  Optionally, a file
 :::  containing a list of computer names can be passed as the 3rd parameter.
 :::
 :::  A list of systems can be provided at the command prompt, or you can
 :::  select a dynamically generated list of servers or workstations.
 :::
 :::  If an asterisk is provided for the password, you will be prompted to
 :::  enter the desired password.
 :::
 :::  (Nov 2010) Changed the default log extension of the reports from .LOG to
 :::  .TXT so that the reports that are mailed to mobile devices can be read
 :::  automatically. (Not too many devices can natively handle .LOG files)
 :::
 :::  (Jul 2018) v2.0 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::  -- Added new Filtered CMD Title and Syntax Help processing routines;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com
 :::
 :::  (Oct 2018) v2.1 -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (May 2019) v2.2 -- The new BrainWave GetInput utility is now used to
 :::  obtain password input without displaying it to the console.  Inpired by
 :::  Bill Stewart's nifty READLINE utility (www.westmesatech.com/?page_id=49);
 :::
 :::  (Oct 2019) v2.3 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (May 2020) v2.4 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::
 :::  (Nov 2020) v2.5 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Feb 2022) v3.0 -- Added routine for dynamic system list generation;
 :::  -- Added :CheckEssentials routine to ensure the existence of critical files;
 :::
 :::  (Mar 2022) v3.1 -- Migrated to using ChangePW.exe from JoeWare.net,
 :::  since CUSRMGR does not appear to work properly in newer Windows versions;
 :::
 :::  (Oct 2023) v3.2 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::  (Jan 2024) v3.3 -- Place :CheckEssentials routine after :SyntaxHelp routine;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 22 Mar 2022 / 30 Nov 2020              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %1  %3 %4
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 23 Mar 2022 / 20 Feb 2022              
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v3.3.0.330

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc --mp=3 -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Check for Essential Utilities and/or Scripts
 PATH %PATH%;%~dp0
 FINDFILES -w -m -f "NOW.EXE" "DATEINFO.EXE" "CHECKPARAMS.EXE" "GETINPUT.EXE" "ADFIND.EXE" "CHANGEPW.EXE"
 IF ERRORLEVEL 64 GOTO :ExitBatch

 rem -- Set Global Variables from Centralized Script
 SET @ADFIND=ADFIND
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- Main Variables
 SET @TOTAL=0
 SET @COUNT=0
 SET @GROUP=Custom List
 SET @ACCT=%~1
 SET @NEWPWD=%~2& IF "%~2"=="*" FOR /F "TOKENS=*" %%P IN ('GetInput -p "Please Enter New Password: " -q') DO SET @NEWPWD="%%P"
 SET @NEWPWD="!@NEWPWD:"=!"

 rem -- Standard Log Variables
 SET @LOGPATH=%@STORAGE%\Logs\Maint
 SET @LOGFILE=%@LOGPATH%\%~n0.%@FILESTAMP_R%
 SET @DYNAMIC=%TEMP%\SourceList.%@FILESTAMP_R%

 rem -- Determine Which Source File to Use for Input
 SET @COMPLIST=*%~3*
	 IF "%~3"=="***" SET @COMPLIST=%@DYNAMIC%& SET @GROUP=All Systems via NET VIEW& NET VIEW | FIND "\\" >"%@DYNAMIC%"
	 IF "%~3"=="**"  SET @COMPLIST=%@DYNAMIC%& SET @GROUP=All Systems via ADFIND&   %@ADQUERY_C:$$$=^!%  >"%@DYNAMIC%"
	 IF "%~3"=="*"   SET @COMPLIST=%@DYNAMIC%& SET @GROUP=All Systems [Minus DCs]&  %@ADQUERY_N:$$$=^!%  >"%@DYNAMIC%"
	 IF "%~3"=="+"   SET @COMPLIST=%@DYNAMIC%& SET @GROUP=All NON-Server Systems&   %@ADQUERY_W:$$$=^!%  >"%@DYNAMIC%"
	 IF "%~3"=="$"   SET @COMPLIST=%@DYNAMIC%& SET @GROUP=All Servers [with DCs]&   %@ADQUERY_S:$$$=^!%  >"%@DYNAMIC%"
	 IF "%~3"=="#"   SET @COMPLIST=%@DYNAMIC%& SET @GROUP=All Servers [Minus DCs]&  %@ADQUERY_O:$$$=^!%  >"%@DYNAMIC%"
	 IF "%~3"=="@"   SET @COMPLIST=%@DYNAMIC%& SET @GROUP=Domain Controllers Only&  %@ADQUERY_D:$$$=^!%  >"%@DYNAMIC%"
	 IF "%~3"=="."   SET @COMPLIST=%@INPUT%ServerList
	 SET @FINDLIST="%@COMPLIST%" "%~3" "%~3*" "%~dp0%@COMPLIST%" "%~dp0Input\%@COMPLIST%" "%@INPUT%%@COMPLIST%" "%@INPUT%ServerList"
	 FOR %%F IN (%@FINDLIST%) DO FOR %%E IN ("" ".%COMPUTERNAME%.TXT" ".TXT") DO IF EXIST "%%~F%%~E" IF NOT DEFINED #ASKED_%%~F%%~E (
		 SET @COMPLIST=%%~F%%~E
		 SET #ASKED_%%~F%%~E=TRUE
		 IF /I "!@COMPLIST:~-4!"==".TXT" (
			 IF /I "%@CONSOLE_SESSION%"=="TRUE" (
				 CHOICE /C YNQEX /T 10 /D Y /N /M "Use the following system list? - '!@COMPLIST!' "
			 ) ELSE (
				 CHOICE /C Y /T 0 /D Y /N
			 )
		 )
		 IF ERRORLEVEL 3 GOTO :ExitBatch
		 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 2 (
			 FOR /F %%S IN ('TYPE "!@COMPLIST!"') DO (
				 SET /A @TOTAL+=1
				 SET @SYS=%%~S
				 IF "!@SYS!"=="!@SYS:;=!" (
					 CALL :MakeHostList %%~S
					 SET /A @COUNT+=1
				 )
			 )
			 GOTO :ProcessJob
		 )
	 )


 rem ==========================================================================
 rem === Abort script if no system list has been provided/accepted
 rem === Updated On: 23 Mar 2022                     
 rem ==========================================================================
:AbortExecution
 ECHO:
 ECHO ERROR: No Valid System List Was Provided -- Script Execution Aborted...
 ECHO:
 GOTO :HelpMessage


 rem ==========================================================================
 rem === Execute a Command Against Selected Systems
 rem === Updated On: 23 Mar 2022                     
 rem ==========================================================================
:ProcessJob
 ECHO Preparing to Execute a Command Job Against Selected Systems...
 ECHO %@DIVIDER-%
 ECHO  List of Systems .......: "!@COMPLIST:%TEMP%=%%TEMP%%!"
 ECHO  Type of Systems .......: %@GROUP%
 ECHO  Target System Count ...: %@COUNT% of %@TOTAL%
 ECHO  Target System Names ...:%@HOSTLIST%
 ECHO %@DIVIDER-%
 ECHO:
 IF /I "%@CONSOLE_SESSION%"=="TRUE" (
	 CHOICE /C YNQEX /T 20 /D Y /N /M "Continue running this script? (Y/N/Q) "
	 IF ERRORLEVEL 2 GOTO :ExitBatch
 )


 rem ==========================================================================
 rem === Cycle Through Domain (or File List) And Update Passwords
 rem === Updated On: 20 Feb 2022 / 09 Feb 2004              
 rem ==========================================================================
:GetComputers
 IF NOT EXIST "%@LOGPATH%" MD "%@LOGPATH%"

 NOW \n\v -- Updating the Password for the %@ACCT% Account\n\= >"%@LOGFILE%"
 ECHO Getting a List of Computers via "%@COMPLIST%"...
 ECHO:
 FOR /F %%V IN (%@COMPLIST%) DO CALL :NewPWD %%V
 ECHO Finished Updating Passwords...
 ECHO:
 DIR "%@LOGFILE%" & READABLE "%@LOGFILE%" --DL 1


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019              
 rem ==========================================================================
:ExitBatch
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Change Password on Each Machine and Log Errors
 rem === Updated On: 22 Mar 2022 / 20 Feb 2022              
 rem ==========================================================================
:NewPWD
 rem %1 = Current System To Be Updated

 IF "%~1"=="" GOTO :EOF
 SET @SYS=%1
 SET @SYS=%@SYS:\=%
 SET @CNAME=\\%@SYS% %@DOTS%
 ECHO:
 ECHO Updating password for %@ACCT% on "%@SYS%" ...
 CHANGEPW /s:%@SYS% /u:%@ACCT% /p:%@NEWPWD% >NUL
 IF ERRORLEVEL 1 (
	 ECHO There were problems updating "%1"
	 ECHO  %@CNAME:~0,24% Could Not Update >>"%@LOGFILE%"
 ) ELSE (
	 ECHO  %@CNAME:~0,24% Successfully Updated >>"%@LOGFILE%"
 )
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Generate Concatenated List of UNC or non-UNC Hosts
 rem === Updated On: 23 Mar 2022                     
 rem ==========================================================================
:MakeHostList
 rem %1 = System to Execute the Command on

 SET @UNAME=%~1
 SET @HNAME=%@UNAME:\=%
 SET @UNAME=\\%@HNAME%

 IF /I "%@UNC%"=="TRUE" (SET @CNAME=!@UNAME!) ELSE (SET @CNAME=!@HNAME!)
 SET @HOSTLIST=!@HOSTLIST! !@CNAME!
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET @SCRIPT_BEG#="%%~d"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 23 Mar 2022 / 20 Feb 2022              
 rem ==========================================================================
:HelpMessage
 ECHO Change Any Local Account Password on a Group of Systems
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  ^<account^> ^<new_password ^| *^> ^<OPTION ^| list_of_servers.txt^>
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:
 ECHO -------------------
 ECHO OPTION DEFINITIONS:
 ECHO -------------------
 ECHO  -?,  /H ..... Display This Help Message  (also --HELP)
 ECHO  -??, /H2 .... Display Extended Help Message
 ECHO       .  ..... Use the default systems list  **DEFAULT**
 ECHO       @  ..... Generate a dynamic list of ALL domain controllers using the ADFIND command
 ECHO       #  ..... Generate a dynamic list of domain servers (minus DCs) using the ADFIND command
 ECHO       $  ..... Generate a dynamic list of ALL domain servers using the ADFIND command
 ECHO       +  ..... Generate a dynamic list of domain computers (minus ALL servers) using the ADFIND command
 ECHO       *  ..... Generate a dynamic list of domain computers (minus DCs) using the ADFIND command
 ECHO      **  ..... Generate a dynamic list of ALL domain computers (everything) using the ADFIND command
 ECHO     ***  ..... Generate a dynamic list of domain/workgroup computers using the NET VIEW command
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0 LOCALACCOUNT newpasswd .
	 ECHO  %~n0 ADMINISTRATOR "This is a long Password" $
	 ECHO  %~n0 MYACCOUNT hard2guess C:\TEMP\SERVERS.TXT
	 ECHO  %~n0 THISACCOUNT "ptu0w#3vBQEz" %~dp0Input\ServerList.TXT
	 ECHO  %~n0 ANYACCT mynewpwd "'NET VIEW /DOMAIN:thatdomain'"
	 ECHO  %~n0 -?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
	 ECHO:
	 ECHO  * Use this script to change local account passwords on multiple systems.
	 ECHO:
	 ECHO  * This script cannot process passwords containing the following:
	 ECHO    ? * \ / ^" %% ^( ^) ^< ^> ^| or the exclamation point
	 ECHO:
	 ECHO  * If the name of the List of Servers contains spaces, it should be
	 ECHO    surrounded by quotation marks.
 )
 CALL :ShowStatus "FINISHED"
                                                                                