@ECHO OFF
 rem ==========================================================================
 rem === Check Domain Password Expiration and Send Email (if possible)
 rem ==========================================================================
 rem
 rem Created On: 17 Dec 2006
 rem         By: Jerold Schulman (www.itprotoday.com)
 rem
 rem Updated On: 16 Oct 2023 / 22 Jan 2022              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2008 and later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    ADFind ................. http://www.joeware.net/freetools/tools/adfind/
 rem    Blat ................... http://www.blat.net/
 rem
 rem  Input Files Used By Script (stored in %SystemDrive%\Scripts\Bat\Input):
 rem    CustomVariables.TXT .... Centralized Configuration Settings for Scripts
 rem
 rem  Required Scripts:
 rem    SetDrive.BAT ........... Set Global Variables for this Environment
 rem ==========================================================================

 :::  This script checks a domain to find users whose domain password is within
 :::  the warning period, and sends them an email.
 :::
 :::  Originally: http://www.itprotoday.com/windows-8/q-how-can-i-email-password-expiration-warning-my-domain-users
 :::
 :::  (Aug 2018) v2.2 -- Removed AT command support;
 :::
 :::  (Oct 2018) v2.3 -- Corrected and expanded the DEBUG logging routine;
 :::  -- Expanded the logging of both expiring and non-expiring accounts;
 :::
 :::  (Feb 2019) v2.4 -- Streamlined variables from SetDrive.BAT;
 :::
 :::  (Mar 2019) v2.5 -- Enabled Code Page 1252 Support;
 :::
 :::  (Oct 2019) v2.6 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (Jan 2020) v3.0 -- Use CHECKPARAMS to evaluate parameters besides /? /H;
 :::  -- Added a routine for validating integer parameters;
 :::  -- Added date sanitization for recent Windows 10 builds which add random
 :::  characters;
 :::
 :::  (May 2020) v3.1 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::
 :::  (Nov 2020) v3.2 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Jan 2022) v3.3 -- Modify the @NAMECHK verification to include \ prefix;
 :::
 :::  (Oct 2023) v3.4 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 30 Nov 2020 / 28 Mar 2018              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 30 Nov 2020 / 26 May 2020              
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v3.4.0.340

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Job Scheduling Variables
 SET @JOBNAME=Check for Expiring Passwords
 SET @JOBUSER=SYSTEM
 SET @JOBFREQ=DAILY
 SET @JOBTIME=00:45:00
 SET @JOBDATE=
 SET @JOBPATH="%~f0" %~1
 SET @JOBVRFY=%~1
 SET @JOBTYPE=

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- Main Variables
 SET @COUNT=
 SET @MAILIT=TRUE
 SET @ADFIND=ADFIND -default -nodn -csv -nocsvheader -f "&(objectcategory=person)(objectClass=user)" sAMAccountName mail

 rem -- Get Valid Warning Time
 CALL :GetInteger @WARN "%~1" 14 1 90

 rem -- Set Date Variables
 SET @DATEFORMAT="mm/dd/yyyy"
 SET @COMPFORMAT="yyyymmdd"
 CALL :ReturnDate @CURRENTDATE "" %@DATEFORMAT%
 FOR /F %%D IN ('DATEINFO -a !@WARN! -f %@DATEFORMAT% -q 2^>NUL') DO SET @EXPIRES_BY=%%D
 SET @MSG=Checking for domain users with passwords expiring within %@WARN% days [%@EXPIRES_BY%]

 rem -- Set Mail (Blat) Variables
 SET @MAILSENDER=DomainAdmin@BrainWave.home
 SET @SUBJECT=Password Expiration Notice

 rem -- Standard Log Variables
 SET @LOGPATH=%@STORAGE%\Logs\Maint
 SET @MAINLOG=%@LOGPATH%\%~n0.TXT
 SET @LOGFILE=%@LOGPATH%\%~n0.%@FILESTAMP_R%
 SET @WARNING=%@LOGPATH%\Password-Expiration-%@FILESTAMP_R%

 rem -- Process Special Command-line Parameters
 SET @ALL_PARAMS=%*
 CALL :GetParams "@" & IF DEFINED @OK (SET @NOSCHED=TRUE)
 CALL :GetParams "#" & IF DEFINED @OK (SET @NOSCHED=TRUE&      SET @MAILIT=FALSE)
 CALL :GetParams "-" & IF DEFINED @OK (SET @SCHEDULEONLY=TRUE& SET @UNSCHED=TRUE)
 CALL :GetParams "." & IF DEFINED @OK (SET @SCHEDULEONLY=TRUE& SET @UPDATESCHED=TRUE)
 %@PRINT_IF_DEBUG%


 rem ==========================================================================
 rem === Create Scheduled Job using SCHTASKS (XP/2003/Vista/+)
 rem === Updated On: 22 Jan 2022 / 09 Mar 2019              
 rem ==========================================================================
:ScheduleJobs
 IF /I "%@NOSCHED%"=="TRUE" IF /I NOT "%@UNSCHED%"=="TRUE" (
	 SET @JOBTYPE= // ONE-TIME
	 ECHO:
	 ECHO *** Job Scheduling Disabled ***
	 ECHO:
	 GOTO :ScheduleJobs_END
 )

 ECHO:
 ECHO Scheduling "%@JOBNAME%" on %COMPUTERNAME%...
 ECHO:

 IF %@OSBUILD% LSS 6000 SET @TASKFOLDER=
 SET @NAMECHK=%@JOBNAME:[=\[%
 SET @NAMECHK=!@NAMECHK:]=\]!
 SET @CHK_SCHED=SCHTASKS /QUERY /V /FO LIST
 SET @SET_SCHED=SCHTASKS /CREATE /TN "!@TASKFOLDER!%@JOBNAME%" /TR "%@JOBPATH%" /RU "%@JOBUSER%" /SC %@JOBFREQ% %@JOBDATE% /ST %@JOBTIME% %@JOBLEVEL% %@FORCE%

 rem -- Delete Scheduled Task from Old Custom Folders on Vista/2008+
 IF %@OSBUILD% GEQ 6000 (
	 FOR /F "TOKENS=1* DELIMS=\ " %%s IN ('%@CHK_SCHED% 2^>NUL ^| FIND "TaskName:" ^| FINDSTR /I /R /C:"\\%@NAMECHK%$"') DO (
		 IF /I NOT "%@TASKFOLDER%%@JOBNAME%"=="%%t" SCHTASKS /DELETE /TN "%%t" /F >NUL 2>&1
	 )
 )

 rem -- Check to Remove Scheduled Job
 IF /I "%@UNSCHED%"=="TRUE" (
	 SCHTASKS /DELETE /TN "!@TASKFOLDER!%@JOBNAME%" %@FORCE% 2>NUL
	 ECHO Task has been UNscheduled.  Now exiting script...
	 GOTO :ExitBatch
 )

 rem -- Check for Schedule Update Parameters
 IF /I "%@UPDATESCHED%"=="TRUE" (!@SET_SCHED!) ELSE (CALL :CheckSched "%~f0" " %@JOBVRFY%$")
 ECHO %@DIVIDER-%
 ECHO:
 IF /I "%@SCHEDULEONLY%"=="TRUE" (
	 ECHO Task has been scheduled.  Now exiting script...
	 IF %@OSBUILD% LSS 6000 (SCHTASKS) ELSE (%@CHK_SCHED% /TN "!@TASKFOLDER!%@JOBNAME%")
	 GOTO :ExitBatch
 )
:ScheduleJobs_END
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Check for Users Whose Accounts Will Expire Soon
 rem === Updated On: 24 Mar 2020 / 21 Jan 2020              
 rem ==========================================================================
:CheckForExpiration
 IF NOT EXIST "%@LOGPATH%" MD "%@LOGPATH%">NUL

 rem -- Generate Header
 FOR %%O IN (CON "%@LOGFILE%") DO NOW \= \v -- Generated By %@VER%\n\=\n\* %@MSG%\n\*\n >%%O

 rem -- Cycle Through All Accounts
 FOR /F "TOKENS=1-2 DELIMS=;, " %%U IN ('%@ADFIND%') DO (
	 SET @WARNUSER=TRUE

	 rem -- Get Expiration and Normalize Date Value
	 FOR /F "TOKENS=3" %%A IN ('NET USER "%%~U" /DOMAIN ^| FIND /I "Password expires"') DO SET @EXPIRES_ON=%%A
	 IF /I "!@EXPIRES_ON!"=="Never" (
		 SET @WARNUSER=FALSE
	 ) ELSE (
		 CALL :ReturnDate @EXPIRES_ON "!@EXPIRES_ON!" %@DATEFORMAT%
		 CALL :CompareDates "!@EXPIRES_BY!" "!@EXPIRES_ON!"  %@COMPFORMAT%
		 CALL :CompareDates "!@EXPIRES_BY!" "!@CURRENTDATE!" %@COMPFORMAT%
	 )

	 IF "!@WARNUSER!"=="TRUE" (
		 SET /A @COUNT+=1
		 SET @SUBJECT=%%~U: Your account expires on !@EXPIRES_ON!
		 ECHO !@SUBJECT! >"%@WARNING%"

		 rem -- Send Mail if Email Account Found
		 IF "%%~V"=="" (
			 ECHO *** !@SUBJECT!. No Email configured.
		 ) ELSE (
			 ECHO   + !@SUBJECT!.
			 IF /I "%@MAILIT%"=="TRUE" (
			 	 ECHO %@DIVIDER~%
				 BLAT -install %@MAILSERVER% %@MAILSENDER% 2 %@MAILPORT%
				 BLAT "%@WARNING%" -f %@MAILSENDER% -to %%~V -s "!@SUBJECT!"
			 	 ECHO %@DIVIDER~%
			 )
		 )
		 ECHO  Account Expiration Warning **ATTEMPTED** for %%~U [Expiration Date: !@EXPIRES_ON!]
	 ) ELSE (
		 ECHO  Account Expiration Warning *UNNECESSARY* for %%~U [Expiration Date: !@EXPIRES_ON!]
	 )

	 ECHO %@DIVIDER-%
	 ECHO:
 ) >>"%@LOGFILE%" 2>>&1

 NOW \n\= \v -- TOTAL EXPIRED ACCOUNTS = !@COUNT!\n\=\n >>"%@LOGFILE%" 2>>&1
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Generate Final Report
 rem === Updated On: 24 Mar 2020 / 28 Oct 2018              
 rem ==========================================================================
:FinalReport
 IF EXIST "%@WARNING%" DEL "%@WARNING%"
 IF EXIST "%@LOGFILE%" (
	 TYPE "%@LOGFILE%"
	 COPY "%@LOGFILE%" "%@MAINLOG%" /Y >NUL
	 IF DEFINED @COUNT BLAT "%@LOGFILE%" -f %@MAILSENDER% %@RCPT-TO% -s "Expiring Password Report - !DATE!"
 )
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019              
 rem ==========================================================================
:ExitBatch
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Synchronize Formats then Compare Two Dates
 rem === Updated On: 21 Jan 2020 / 28 Oct 2018              
 rem ==========================================================================
:CompareDates
 rem %1 = Date #1
 rem %2 = Date #2
 rem %3 = Date Format

 IF "%~1"=="" GOTO :EOF
 IF "%~2"=="" GOTO :EOF
 SET @FORMAT="%~3"& IF "%~3"=="" SET @FORMAT=%@COMPFORMAT%
 CALL :ReturnDate @DATE1 "%~1" !@FORMAT!
 CALL :ReturnDate @DATE2 "%~2" !@FORMAT!

 IF "!@DATE1!" LSS "!@DATE2!" SET @WARNUSER=FALSE
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Return Date According to Specific Format
 rem === Updated On: 24 Mar 2020 / 21 Jan 2020              
 rem ==========================================================================
:ReturnDate
 rem %1 = Variable to Store Date
 rem %2 = Date
 rem %3 = Date Format

 SET @ZZ=%~2& IF DEFINED @ZZ SET @ZZ=!@ZZ:?=!
 SET %~1=& FOR /F %%d IN ('DATEINFO -s "!@ZZ!" -f "%~3" -q 2^>NUL') DO SET %~1=%%d
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Determine if Parameter is a Valid Integer, Or Set to Zero
 rem === Updated On: 21 Jan 2020                     
 rem ==========================================================================
:GetInteger
 rem %1 = Variable to Store Integer
 rem %2 = Parameter to Evaluate
 rem %3 = Value to Default to, If Zero
 rem %4 = Optional: Lowest Value
 rem %5 = Optional: Highest Value

 SET @V1=%2& IF DEFINED @V1 SET @V1=!@V1:"=!
 SET @V2=%3& IF DEFINED @V2 SET @V2=!@V2:"=!
 SET @V3=%4& IF DEFINED @V3 SET @V3=!@V3:"=!
 SET @V4=%5& IF DEFINED @V4 SET @V4=!@V4:"=!
 FOR /F "TOKENS=3" %%i IN ('CHECKPARAMS -o --min "!@V3!" --max "!@V4!" -n "!@V1!" "!@V2!" 2^>NUL') DO SET %~1=%%i
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Use CheckParams.exe to Capture Selected Parameters
 rem === Updated On: 21 Jan 2020                     
 rem ==========================================================================
:GetParams
 rem %1 = Parameters to Search For
 rem %2 = Variable to Set

 SET @OK=& IF "%~1"=="" GOTO :EOF
 FOR /F "TOKENS=2-3*" %%v IN ('CHECKPARAMS -q -a -c "%~1" -s %@ALL_PARAMS% 2^>NUL') DO IF /I "%%~v"=="TRUE" (
	 SET @OK=T
	 SET @REQ=T
	 IF NOT "%~2"=="" SET %~2=%%~x
 )
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Check if Scheduled Job is Already Running
 rem === Updated On: 15 May 2018 / 28 Mar 2018              
 rem ==========================================================================
:CheckSched
 rem %1 = Standard Search Criteria
 rem %2 = Advanced Search Criteria

 SET @CHK=%2
 IF DEFINED @CHK (
	 %@CHK_SCHED% 2>NUL | FIND /I "%~1" | FINDSTR /I /R /C:%2 >NUL
 ) ELSE (
	 %@CHK_SCHED% 2>NUL | FIND /I "%~1" >NUL
 )
 IF ERRORLEVEL 1 %@SET_SCHED%
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET @SCRIPT_BEG#="%%~d"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 30 Nov 2020 / 14 May 2018              
 rem ==========================================================================
:HelpMessage
 ECHO Check Domain Password Expiration and Send Email (if possible)...
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  [days_before_expiration] [OPTIONS]
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:
 ECHO -------------------
 ECHO OPTION DEFINITIONS:
 ECHO -------------------
 ECHO  -?,  /H ..... Display This Help Message  (also --HELP)
 ECHO  -??, /H2 .... Display Extended Help Message
 ECHO       .   .... Schedule the Job Only
 ECHO       -   .... Unschedule the Job Only, Without Executing It First
 ECHO       @   .... Execute the Job, But Do Not Schedule It
 ECHO       #   .... Test, But Don't Send Email
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0
	 ECHO  %~n0  10
	 ECHO  %~n0  20 .
	 ECHO  %~n0  30 #
	 ECHO  %~n0  45 @
	 ECHO  %~n0  60
	 ECHO  %~n0 -?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
	 ECHO:
	 ECHO  * If no days are selected, the default is to check for passwords which
	 ECHO    will expire within 14 days.
 )
 CALL :ShowStatus "FINISHED"
                                                                                              