@ECHO OFF
 rem ==========================================================================
 rem === List Directories of Multi-Instance Backups and/or Compressed Archives
 rem ==========================================================================
 rem
 rem Created On: 04 Sep 2011
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Oct 2023 / 30 Nov 2020              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2003 and later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    CHOICE ................. Native in Windows 2003+ / Reskit in NT/2000
 rem    Resource Kit ........... https://en.wikipedia.org/wiki/Resource_Kit
 rem
 rem  Input Files Used By Script (stored in %SystemDrive%\Scripts\Bat\Input):
 rem    ArchiveInfo.TXT ........ List of source and destination folder(s)
 rem    CustomVariables.TXT .... Centralized Configuration Settings for Scripts
 rem
 rem  Required Scripts:
 rem    SetDrive.BAT ........... Set Global Variables for this Environment
 rem ==========================================================================

 :::  This script was modified from CheckTestBackup.BAT and is a companion to
 :::  the CreateArchive.BAT script (formerly BackupFolders.BAT).  It displays
 :::  the folder structure and disk consumption of any set of backups created
 :::  with the CreateArchive script.
 :::
 :::  (Sep 2011) v2.5 -- The script reads the ArchiveInfo.TXT file rather than
 :::  the ReplicationInfo.TXT text.  Although these two files serve a similar
 :::  purpose, it is easy to get them mixed up and use the incorrect parameters
 :::  because they serve different types of scripts. This change should make it
 :::  easier to manage both sets of input files.
 :::
 :::  (Dec 2011) v3.0 -- Added the ability to create daily, monthly and annual
 :::  archives, in addition to the existing folder backups and timed archive.
 :::  This functionality was previously available in the precursor to this script
 :::  (Freelancer-Archive.BAT).
 :::
 :::  (Oct 2013) v3.6 -- The delimiter for all parameters can be either the
 :::  "-" or the "/" character.  This includes the help parameters.
 :::
 :::  (Nov 2013) v3.10 -- This script now captures info about the enhanced
 :::  file comparison options (FCIV, FSUM, MD5SUM, FileHash).  The option to
 :::  bypass obtaining directory statistics has also been simplified.
 :::
 :::  (Jan 2014) v4.0 -- It is now possible to identify the individual file
 :::  and/or directory exemptions that apply to a specific SOURCE/DESTINATION
 :::  archive pair within a larger archive job.
 :::
 :::  (Mar 2014) v4.1 -- Upgraded the DebugMode options to add another
 :::  variable (@PRINT_IF_DEBUG). Also added verification of Syslog usage
 :::  in scheduled jobs.
 :::
 :::  (Apr 2014) v4.2 -- Updated to support the new @HIDE_FULL_PATH option
 :::  available to CreateArchive.BAT jobs.
 :::
 :::  (Oct 2014) v4.5 -- Fastmode processing is now identified at the global
 :::  and local job levels.
 :::
 :::  (Dec 2014) v4.6 -- Now supports the multiple-source-folders config option
 :::  of CreateArchive.BAT v9.5.  However, unlike CreateArchive.BAT, this script
 :::  displays all SOURCE/DESTINATIONS combinations, rather than just those
 :::  where the source folders currently exist.
 :::
 :::  (Mar 2017) v4.9 -- Now supports the hash options of CreateArchive.BAT v10.0
 :::
 :::  (Dec 2017) v5.0 -- Synched up with CreateArchive.BAT v11.0;
 :::  All potential source and destination folders are displayed for each job,
 :::  even though the actual running script will select only primary folders.
 :::
 :::  (Mar 2018) v5.1 -- Changed the HELP routine to use FINDSTR;
 :::  -- Added new Filtered CMD Title and Syntax Help processing routines;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com
 :::
 :::  (Jul 2018) v5.2 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::
 :::  (Oct 2018) v5.3 -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Feb 2019) v5.4 -- Removed unnecessary variables provided by SetDrive.BAT
 :::  -- Corrected folder parsing routine; Made minor aesthetic changes;
 :::
 :::  (Jul 2019) v6.0 -- Synched up functionality with CreateArchive.BAT v16.0;
 :::
 :::  (Oct 2019) v6.1 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (Dec 2019) v6.2 -- New method to determine if 64-bit FILEHASH is running;
 :::
 :::  (May 2020) v6.3 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::  -- Replaced FINDSTR with BrainWave ReadConfig.exe for reading config file;
 :::  -- Added support for more hash algorithms including BLAKE3 and XXHash64;
 :::  -- Changed fallback hash, if none supplied, from MD5 to XXHASH64;
 :::  -- Added support for a MINI edition of the ARCHIVE-INFO config file;
 :::
 :::  (Nov 2020) v6.4 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Oct 2023) v6.5 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 30 Nov 2020 / 25 Feb 2018              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 30 Nov 2020 / 26 May 2020              
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v6.4.0.640

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc --mp=1 -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Set Initial Duration Values
 SET @CYCLE=FILES TIMED DAILY MONTHLY YEARLY
 FOR %%T IN (%@CYCLE%) DO (
	 SET @MAKE_%%T=FALSE
	 SET @KEEP_%%T=FALSE
 )

 rem -- Main Variables
 SET @JOBID=SOURCE
 SET @THISJOB=%~1
 SET @SHOWJOBINFO=TRUE
 SET @REMOTESYS=%~2& IF "%~2"=="" SET @REMOTESYS=%COMPUTERNAME%
 SET @JOBUSER=SYSTEM

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- Grab Appropriate Custom Variables from ArchiveInfo.*.TXT
 FOR %%C IN ("%@ARCHIVE-INFO%" "%@ARCH-INFO-MINI%") DO IF EXIST "%%~C" (
	 FOR /F "TOKENS=*" %%R IN ('READCONFIG "%%~C" -F "%~1" 2^>NUL') DO CALL :RunCMD %%R
	 FOR /F "TOKENS=1 DELIMS=;" %%J IN ('READCONFIG "%%~C" -N -S "#CREATEARCHIVES" -P 2^>NUL') DO SET @VALIDJOB-%%~J=%%~J& SET @JOBLIST=TRUE
 )

 rem -- Set Variables for File Comparison Mode
 SET @GOODHASH=& FOR %%H IN ("" %@HASHUTILS%) DO IF /I "%@HASHUTIL%"=="%%~H" SET @GOODHASH=TRUE
 IF /I "%@GOODHASH%/%@EXTENDED_COMP%"=="TRUE/TRUE" (
	 SET @XCOMPARE=TRUE
	 SET @COMPMODE=%@THISHASH% Hash via %@HASHUTIL%
 ) ELSE (
	 SET @XCOMPARE=FALSE
	 SET @COMPMODE=Time/Date Stamp via DIR
	 SET @HASHUTIL=DIR
 )

 rem -- Determine Valid Hash
 SET @ACCEPTABLE=MD5 SHA1 SHA2 SHA256 SHA512 512 SHA3 SHA384 BLAKE3 XXHASH64
 SET @VALID=& FOR %%H IN (%@ACCEPTABLE%) DO IF /I "%@THISHASH%"=="%%~H" SET @VALID=TRUE
 IF NOT DEFINED @VALID SET @THISHASH=MD5

 rem -- Determine if Running in x64 Mode
 FOR /F "TOKENS=3" %%X IN ('FILEHASH --Version-Only 2^>^&1') DO IF /I "%%~X"=="x64" SET @x64_MSG=64-bit Mode

 rem -- Set Comparison Info Variables
 SET @COPYUTIL=XCOPY& IF DEFINED @ROBOCOPY IF /I "%@USE_ROBOCOPY%"=="TRUE" SET @COPYUTIL=ROBOCOPY

 rem -- Set Zip Compression Variables
 SET @ZIPUTIL=ZIP& IF DEFINED @USE_7z SET @ZIPUTIL=7z

 rem -- Set Variables for Special Modes
 SET @NOSCHED_MSG=Don't Schedule
 SET @UNSCHED_MSG=Unschedule
 SET @UPDATESCHED_MSG=Update Schedule
 SET @SCHEDULEONLY_MSG=Schedule ONLY
 SET @DIAGMODE_MSG=Diagnostics Logs
 SET @TESTMODE_MSG=Testing Mode
 SET @HIDE_BASE_PATH_MSG=Hide Base Path
 SET @HIDE_FULL_PATH_MSG=Filenames Only
 SET @XCOMPARE_MSG=Extended File Comparison
 SET @USE_SYSLOGS_MSG=Forward to Syslog
 SET @USE_LOCAL_ZIP_MSG=Local Zip Mode
 SET @FASTMODE_MSG=Fast Compare Mode
 SET @FORCE_BACKUP_MSG=Forced Backup Mode
 SET @SORT_LIST_MSG=File List Sorted
 SET @USE_VSS_MSG=Shadow Copy Mode
 SET @ITEMIZED_INC_MSG=Itemized Include List
 SET @SAMESOURCE_MSG=Jobs Share Same Source
 SET @MODES=& FOR %%M IN (DIAGMODE TESTMODE HIDE_BASE_PATH HIDE_FULL_PATH USE_LOCAL_ZIP XCOMPARE USE_SYSLOGS FASTMODE x64 SORT_LIST USE_VSS ITEMIZED_INC SAMESOURCE FORCE_BACKUP NOSCHED UNSCHED UPDATESCHED SCHEDULEONLY) DO IF /I "!@%%~M!"=="TRUE" SET @MODES=!@%%~M_MSG! * !@MODES!
 IF "%@MODES%"=="" SET @MODES=NONE *

 rem -- Key Variable Verification
 IF "%~1"=="#" GOTO :HelpMessage
 SET @SOURCEDEST=
 FOR %%C IN ("" %@BASE10%) DO FOR %%X IN ("" %@ALPHA11%) DO IF DEFINED @SOURCE%%~C%%~X (
	 FOR %%L IN ("" %@ALPHA11%) DO IF DEFINED @DEST%%~C%%~L SET @SOURCEDEST=PRESENT
 )
 FOR %%V IN (GOODHASH COPYMODE SOURCEDEST INFO MAKE_FILES KEEP_FILES MAKE_TIMED KEEP_TIMED ALPHA11 BASE10) DO IF NOT DEFINED @%%~V (
	 ECHO:
	 ECHO *** WARNING: The @%%~V variable is undefined.
	 ECHO:
	 ECHO  Please ensure that all configuration info is provided correctly
	 ECHO  within the file: "%@ARCHIVE-INFO%"
	 ECHO:
	 ECHO  Alternatively, this information can be entered directly into this script,
	 ECHO  although this is not recommended because it undermines portability.
	 ECHO %@DIVIDER*%
	 ECHO:
	 GOTO :HelpMessage
 )
 %@PRINT_IF_DEBUG%


 rem ==========================================================================
 rem === Validate Values Used for Keeping Zip and Backup History (0 = Skip)
 rem === Updated On: 03 Feb 2020 / 30 Jul 2019              
 rem ==========================================================================
:ValidateHistory
 FOR %%V IN (%@CYCLE%) DO (
	 FOR %%Y IN ("" "FALSE") DO IF /I "!@MAKE_%%V!"=="%%~Y" SET @KEEP_%%V=0
	 SET /A @CHECK_%%V=@KEEP_%%V

	 IF "!@CHECK_%%V!"=="0" (
		 SET @SKIP_%%V=
		 SET @MAKE_%%V=FALSE
	 ) ELSE (
		 SET @SKIP_%%V=SKIP=!@KEEP_%%V!
		 SET @MAKE_%%V=TRUE
	 )
 )
 %@PRINT_IF_DEBUG%


 rem ==========================================================================
 rem === Select Valid Source and Destination for each Job from the Job Config
 rem === Updated On: 30 Jul 2019 / 31 Dec 2017              
 rem ==========================================================================
:GetValidJobFolders
 ECHO === Obtaining Source/Destination Info from "%@ARCHIVE-INFO%" ===
 ECHO:
 FOR %%C IN ("" %@BASE10%) DO (
	 FOR %%X IN ("" %@ALPHA11%) DO (
		 IF DEFINED @SOURCE%%~C%%~X (
			 CALL :FixDirectoryPath @SOURCE%%~C%%~X
			 SET @SOURCES%%~C=!@SOURCES%%~C! "!@SOURCE%%~C%%~X!"
			 IF NOT "%%~C"=="" SET @JOBID%%~C=SOURCE%%~C
		 )

		 IF DEFINED @DEST%%~C%%~X (
			 CALL :FixDirectoryPath @DEST%%~C%%~X
			 SET @DESTINATIONS%%~C=!@DESTINATIONS%%~C! "!@DEST%%~C%%~X!"
		 )
	 )
 )
 %@PRINT_IF_DEBUG%


 rem ==========================================================================
 rem === Display Contents of Appropriate Archive Folder
 rem === Updated On: 31 Dec 2017 / 26 Jun 2017              
 rem ==========================================================================
:ListBackupFolder
 SET @FIRSTRUN=TRUE

 FOR %%C IN ("" %@BASE10%) DO FOR %%X IN ("" %@ALPHA11%) DO IF DEFINED @SOURCE%%~C%%~X (
	 FOR %%L IN ("" %@ALPHA11%) DO IF DEFINED @DEST%%~C%%~L (
		 CALL :FixDirectoryPath @SOURCE%%~C%%~X
		 CALL :FixDirectoryPath @DEST%%~C%%~L
		 SET @JOBFOLDER=\%@THISJOB%.%@REMOTESYS%& IF /I "!@SIMPLEDEST!"=="TRUE" SET @JOBFOLDER=
		 SET @SOURCENAME=!@SOURCE%%~C%%~X:\\=!
		 SET @SOURCENAME=!@SOURCENAME::\=_!
		 SET @SOURCENAME=!@SOURCENAME::=_!
		 SET @SOURCENAME=!@SOURCENAME:\=_!
		 SET @SOURCENAME=!@SOURCENAME:__=_!& IF "!@SOURCENAME:~-1!"=="_" SET @SOURCENAME=!@SOURCENAME:~0,-1!
		 SET @BACKUP_ROOT=!@DEST%%~C%%~L!!@JOBFOLDER!
		 SET @FILES_FORMAT=!@BACKUP_ROOT!\!@SOURCENAME!.????-??-?? [??.??.??]
		 SET @TIMED_FORMAT=!@BACKUP_ROOT!\%@REMOTESYS%_!@SOURCENAME!.????????-??????.ZIP&  SET @TIMED_FORMAT=!@TIMED_FORMAT:%@REMOTESYS%_%@REMOTESYS%_=%@REMOTESYS%_!
		 SET @DAILY_FORMAT=!@BACKUP_ROOT!\%@REMOTESYS%_!@SOURCENAME!.????-??-??.ZIP&       SET @DAILY_FORMAT=!@DAILY_FORMAT:%@REMOTESYS%_%@REMOTESYS%_=%@REMOTESYS%_!
		 SET @MONTHLY_FORMAT=!@BACKUP_ROOT!\%@REMOTESYS%_!@SOURCENAME!.????-??.ZIP&        SET @MONTHLY_FORMAT=!@MONTHLY_FORMAT:%@REMOTESYS%_%@REMOTESYS%_=%@REMOTESYS%_!
		 SET @YEARLY_FORMAT=!@BACKUP_ROOT!\%@REMOTESYS%_!@SOURCENAME!.????.ZIP&            SET @YEARLY_FORMAT=!@YEARLY_FORMAT:%@REMOTESYS%_%@REMOTESYS%_=%@REMOTESYS%_!

		 rem -- Display Job Header Info on First Pass Only
		 IF /I "!@FIRSTRUN!"=="TRUE" CALL :DisplayJobInfo

		 IF /I NOT "%@REMOTESYS%"=="%COMPUTERNAME%" ECHO *** PLEASE NOTE: Remote Settings May Not Be Accurately Viewable from This System ***
		 ECHO:

		 IF EXIST "!@BACKUP_ROOT!" (
			 CHOICE /C YNQEX /T 5 /D Y /N /M "Continue? (Y/N/Q) "
			 IF !ERRORLEVEL! GTR 1 GOTO :ExitBatch

			 SET @FOUNDANY=FALSE
			 FOR %%V IN (FILES TIMED DAILY MONTHLY YEARLY) DO (
				 IF EXIST "!@%%~V_FORMAT!" (
					 SET @FOUNDANY=TRUE
					 ECHO:
					 ECHO Saved Backups from %%V Job !@JOBID%%~C!: "!@BACKUP_ROOT!"
					 ECHO:
					 DIR "!@%%~V_FORMAT!" | FIND /I "!@SOURCENAME!"
					 ECHO:
				 )
			 )

			 rem -- Check for Existence of Valid Backups for THIS Source
			 IF /I "!@FOUNDANY!"=="FALSE" (
				 ECHO:
				 ECHO Saved Backups from Job !@JOBID%%~C!: "!@BACKUP_ROOT!"
				 ECHO:
				 ECHO No backups of "!@SOURCE%%~C%%~X!" were found here.
				 ECHO --------
				 ECHO:
			 ) ELSE (

				 rem -- Allow a Bypass for Gathering Disk Statistics
				 SET @ANSWER=Y
				 FOR /F %%w IN ('ECHO !@DEST%%~C%%~L! ^| FIND "\\"') DO SET @ANSWER=N
				 CHOICE /C YNQEX /T 5 /D !@ANSWER! /N /M "Get Disk Statistics? (Y/N) "
				 IF "!ERRORLEVEL!"=="1" (
					 ECHO:
					 ECHO Getting Disk Statistics... Please Wait...
					 ECHO:
					 DU -Q -accepteula "!@BACKUP_ROOT!"
					 ECHO --------
					 ECHO:
					 ECHO:
				 )
			 )
		 ) ELSE (
			 ECHO No backups were found at "!@BACKUP_ROOT!"
			 ECHO --------
			 ECHO:
			 SET @SHOWJOBINFO=FALSE
		 )
	 )
 )


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019              
 rem ==========================================================================
:ExitBatch
 CALL :DisplayJobInfo
 ECHO: & ECHO To See All Jobs, type: %~n0 #
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Information About Backup Job
 rem === Updated On: 30 Jul 2019 / 27 Feb 2019              
 rem ==========================================================================
:DisplayJobInfo
 SET @JOBCOUNT=0
 SET @FIRSTRUN=FALSE

 IF /I "%@SHOWJOBINFO%"=="TRUE" (
	 ECHO %@DIVIDER-%
	 ECHO --- Job Name .......... %@INFO%
	 ECHO --- Job Schedule ...... %@JOBFREQ% at %@JOBTIME%  %@JOBDATE%
	 ECHO --- Job User .......... %@JOBUSER%
	 ECHO --- File Comparison ... %@COMPMODE%
	 ECHO --- Backup Utilities .. %@COPYUTIL% and %@ZIPUTIL%
	 ECHO --- Global Modes ...... %@MODES:~0,-2%
	 IF DEFINED @JOBDELAY ECHO --- Initial Delay ..... %@JOBDELAY% Seconds
	 ECHO %@DIVIDER-%
	 ECHO:
	 FOR %%C IN ("" %@BASE10%) DO (
		 SET #SOURCE%%~C=
		 FOR %%X IN ("" %@ALPHA11%) DO IF DEFINED @SOURCE%%~C%%~X (
			 IF NOT DEFINED #SOURCE%%~C (
				 CALL :CreateExceptionLists %%~C
				 IF NOT "!@EXEMPT-DIRS!"==""  SET @EXEMPT-D%%~C=!@EXEMPT-DIRS!
				 IF NOT "!@EXEMPT-FILES!"=="" SET @EXEMPT-F%%~C=!@EXEMPT-FILES!
				 SET #SOURCE%%~C=TRUE
				 SET /A @JOBCOUNT+=1
				 ECHO --- Archive Job ID:     !@JOBID%%~C!
				 FOR %%S IN (!@SOURCES%%~C!) DO      ECHO --- Source Location[s]: %%~S
				 FOR %%D IN (!@DESTINATIONS%%~C!) DO ECHO --- Backup Location[s]: %%~D!@JOBFOLDER!
				 IF DEFINED @CURRENT_IN              ECHO --- Current Filespecs:  !@CURRENT_IN!
				 IF /I "!@CURRENT_FM!"=="TRUE"       ECHO --- Special Processing: Fastmode is enabled for this job...
				 IF DEFINED @EXEMPT-D%%~C            ECHO --- Ignore Directories: !@EXEMPT-D%%~C!
				 IF DEFINED @EXEMPT-F%%~C            ECHO --- Ignore Files:       !@EXEMPT-F%%~C!
				 ECHO:
			 )
		 )
	 )
	 ECHO %@DIVIDER-%
	 ECHO --- TOTAL ARCHIVE JOBS: !@JOBCOUNT!
	 ECHO %@DIVIDER-%
	 ECHO --- Make Folder Backup: %@MAKE_FILES%  	[Retain %@KEEP_FILES% Copies]
	 ECHO --- Make Timed Archive: %@MAKE_TIMED%  	[Retain %@KEEP_TIMED% Archives]
	 ECHO --- Make DAILY Archive: %@MAKE_DAILY%  	[Retain %@KEEP_DAILY% Archives]
	 ECHO ---  " MONTHLY Archive: %@MAKE_MONTHLY%	[Retain %@KEEP_MONTHLY% Archives]
	 ECHO ---  "  YEARLY Archive: %@MAKE_YEARLY% 	[Retain %@KEEP_YEARLY% Archives]
	 ECHO %@DIVIDER%
	 ECHO:
 )
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Create Exception List for ROBOCOPY, XCOPY and ZIP commands
 rem === Updated On: 30 Jul 2019 / 12 Oct 2014              
 rem ==========================================================================
:CreateExceptionLists
 rem %1 = Current Configuration Suffix to Evaluate

 SET @EXEMPT-DIRS=
 SET @EXEMPT-FILES=
 SET @CURRENT_VS=%@USE_VSS%&   IF DEFINED @USE_VSS%~1    SET @CURRENT_VS=!@USE_VSS%~1!
 SET @CURRENT_SD=%@SKIPDIR%&   IF DEFINED @SKIPDIR%~1    SET @CURRENT_SD=!@SKIPDIR%~1!
 SET @CURRENT_SF=%@SKIPFILE%&  IF DEFINED @SKIPFILE%~1   SET @CURRENT_SF=!@SKIPFILE%~1!
 SET @CURRENT_IN=%@FILESPEC%&  IF DEFINED @FILESPEC%~1   SET @CURRENT_IN=!@FILESPEC%~1!
 SET @CURRENT_EX=%@EXEMPTION%& IF DEFINED @EXEMPTION%~1  SET @CURRENT_EX=!@EXEMPTION%~1!
 SET @CURRENT_FM=%@FASTMODE%&  IF DEFINED @FASTMODE%~1   SET @CURRENT_FM=!@FASTMODE%~1!
 SET @SKIP-DIR=&  IF NOT "%@CURRENT_SD%"=="" SET @SKIP-DIR=/XD %@CURRENT_SD%
 SET @SKIP-FILE=& IF NOT "%@CURRENT_SF%"=="" SET @SKIP-FILE=/XF %@CURRENT_SF%

 CALL :ChangeChars1 @CURRENT_IN ""
 CALL :ChangeChars1 @CURRENT_EX ""

 FOR %%e IN (!@CURRENT_EX!) DO (
	 FOR /F %%f IN ('ECHO %%~e ^| FIND "\"') DO SET @EXEMPT-DIRS=%%~f !@EXEMPT-DIRS!
	 FOR /F %%f IN ('ECHO %%~e ^| FIND /V "\"') DO SET @EXEMPT-FILES=%%~f !@EXEMPT-FILES!
 )

 IF DEFINED @EXEMPT-FILES SET @EXEMPT-FILES=!@EXEMPT-FILES:=*!
 IF DEFINED @EXEMPT-DIRS (
	 SET @EXEMPT-DIRS=!@EXEMPT-DIRS:\=!
	 SET @EXEMPT-DIRS=!@EXEMPT-DIRS:\=!
	 SET @EXEMPT-DIRS=!@EXEMPT-DIRS:=!
 )

 CALL :ChangeChars2 @CURRENT_IN ""
 CALL :ChangeChars2 @CURRENT_EX ""
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Remove Trailing \ From Supplied Folder Path
 rem === Updated On: 25 Feb 2019 / 19 Mar 2018              
 rem ==========================================================================
:FixDirectoryPath
 rem %1 = Folder Variable

 IF NOT DEFINED %1 GOTO :EOF
 IF "!%~1:~-1!"=="\" (
	 SET %~1=!%~1:~0,-1!
	 GOTO :FixDirectoryPath
 )
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Execute Dynamic Command From Variable
 rem === Updated On: 12 Jan 2014 / 04 Sep 2011              
 rem ==========================================================================
:RunCMD
 rem %* = Command To Execute

 %*
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Store Current Date and Time in Variable
 rem === Updated On: 16 Oct 2023 / 21 Apr 2015              
 rem ==========================================================================
:GetTime
 rem %1 = Variable to Store Date/Time

 SET %~1=!DATE! at !TIME: =0!
 FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET %~1#="%%~d"
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Reverse Parameters in a Variable
 rem === Updated On: 30 Dec 2017                     
 rem ==========================================================================
:Reverse
 rem %1 = Source Variable
 rem %2 = Destination Variable

 SET %~2=& FOR %%v IN (!%~1!) DO SET %~2=%%v !%~2!
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Use SubString to Manage "*" Characters in Inc/Exc Lists
 rem === Updated On: 30 Jul 2019                     
 rem ==========================================================================
:ChangeChars1
:ChangeChars2
 rem %1 = Name of Resulting Variable
 rem %2 = Content to Perform Substring Operation

 IF "%~1"=="" GOTO :EOF

 SET @DATA=%~2& IF "%~2"=="" SET @DATA=!%~1!
 SET @SS_P=+

 IF /I "%~0"==":ChangeChars1" (
	 SET @SS_F=*
	 SET @SS_R=
 ) ELSE (
	 SET @SS_F=
	 SET @SS_R=*
 )

 FOR /F "TOKENS=*" %%s IN ('SUBSTRING "!@DATA!" "!@SS_P!" "!@SS_R!" 2^>NUL') DO SET @DATA=%%s
 FOR /F "TOKENS=*" %%s IN ('SUBSTRING "!@DATA!" "!@SS_F!" "!@SS_R!" 2^>NUL') DO SET @DATA=%%s
 SET %~1=!@DATA!

 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" CALL :GetTime @SCRIPT_BEG
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 30 Nov 2020 / 30 Jul 2019              
 rem ==========================================================================
:HelpMessage
 ECHO List Directories of Multi-Instance Backups and/or Compressed Archives...
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  ^<name_of_archive_job^>  [remote_computer]
 ECHO  %~n0  [-h^|-?^|#]
 ECHO:
 ECHO -------------------
 ECHO OPTION DEFINITIONS:
 ECHO -------------------
 ECHO  -?,  /H ..... Display This Help Message  (also --HELP)
 ECHO  -??, /H2 .... Display Extended Help Message
 ECHO       #  ..... Enumerate all possible backup jobs which are valid for this computer
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0 DailyData
	 ECHO  %~n0 Main-Files %COMPUTERNAME%
	 ECHO  %~n0 My_Stuff
	 ECHO  %~n0 CriticalStuff  SERVER01
	 ECHO  %~n0 #
	 ECHO  %~n0 ?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
	 ECHO:
	 ECHO  * The name of the archive job cannot contain ANY spaces. Spaces are
	 ECHO    permissable in the job description, however. Shortnames *can* be
	 ECHO    surrounded by quotes if you choose, but it is not mandatory.
	 ECHO:
	 ECHO  * An elevated CMD promt is required in Windows 2008+ in order to display
	 ECHO    the list of scheduled archive tasks.
 )

 IF "%~1"=="#" IF DEFINED @JOBLIST (
	 ECHO:
	 ECHO:
	 ECHO The following are actual job names for this system [%@REMOTESYS%]:
	 ECHO %@DIVIDER%
	 FOR /F "TOKENS=1* DELIMS==" %%B IN ('SET @VALIDJOB-') DO ECHO  %~n0  %%~C
	 ECHO:
	 ECHO:
	 ECHO The following archive jobs are scheduled on this system [%@REMOTESYS%]:
	 ECHO %@DIVIDER%
	 FOR /F "TOKENS=3*" %%B IN ('SCHTASKS /S %@REMOTESYS% ^| FIND /I "Backup Data -"') DO ECHO              %%C
	 ECHO:
	 ECHO:
 )
 CALL :ShowStatus "FINISHED"
                                                                                                     