@ECHO OFF
 rem ==========================================================================
 rem === Set Clean Timestamps of Files in Selected Folder and SubFolders
 rem ==========================================================================
 rem
 rem Created On: 05 Mar 2017
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 10 Apr 2024 / 16 Oct 2023 / 04 Jan 2021
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2000 and later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    DIFF ................... http://gnuwin32.sourceforge.net/packages/diffutils.htm
 rem    FileDate ............... http://www.ABFsoftware.com/
 rem    Resource Kit ........... https://en.wikipedia.org/wiki/Resource_Kit
 rem ==========================================================================

 :::  This script synchronizes the time on files within the current folder
 :::  and any subfolders, so long as the time does not end in 00:00.  This
 :::  script was derived from the TimeStamp.BAT script.
 :::
 :::  (Mar 2017) v1.0 -- Initial Version.
 :::
 :::  (Apr 2018) v1.2 -- Changed the HELP routine to use FINDSTR;
 :::  -- Added new Filtered CMD Title and Syntax Help processing routines;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com
 :::
 :::  (Jul 2018) v1.3 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::
 :::  (Oct 2018) v1.4 -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Feb 2019) v1.5 -- Enabled Code Page 1252 Support;
 :::
 :::  (Mar 2019) v2.0 -- Use DATEINFO to calculate a +1 hour offset, in
 :::  addition to the default of rounding down to the current hour;
 :::  -- Added more parameters, including file selection and diagnostics mode;
 :::
 :::  (May 2019) v2.1 -- The additional parameters must now be used prior to
 :::  the folder parameter;
 :::
 :::  (Oct 2019) v2.2 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (Oct 2019) v2.3 -- Confirm destination before continuing;
 :::
 :::  (Jan 2020) v2.4 -- Use CHECKPARAMS to evaluate parameters besides /? /H;
 :::
 :::  (May 2020) v2.5 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::
 :::  (Jul 2020) v2.6 -- Update Parameters and Provide More Output Status;
 :::
 :::  (Nov 2020) v2.7 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Dec 2021) v2.8 -- Cleanup Old Before/After Temp Files, unless in Debug mode;
 :::
 :::  (Oct 2023) v2.9 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::  (Apr 2024) v3.0 -- Check for essential utilities with FINDFILES;
 :::  -- Add the ability to make a +/- change of a day, not just of an hour;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 20 Jul 2020 / 24 Apr 2018
 rem ==========================================================================
:DisplayTitle
 SET @TTT=%*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] -  %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 10 Apr 2024 / 30 Nov 2020 / 20 Jul 2020
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v3.0.0.300

 rem -- Default Variables (Override via CustomVariables.*.TXT)
 SET @DEFAULT_FOLDER=%SystemDrive%\Scripts
 SET @DEFAULT_FILES=*.*

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Check for Essential Utilities and/or Scripts
 SET @ESSENTIALS="%~dp0SetDrive.BAT" CHECKPARAMS.EXE DATEINFO.EXE NOW.EXE READCONFIG.EXE CHCP.COM FIND.EXE
 SET @MSUTILS=CHOICE.EXE
 PATH %PATH%;%~dp0
 FINDFILES -w -m -f %@ESSENTIALS% %@MSUTILS% FILEDATE.EXE
 IF ERRORLEVEL 64 GOTO :ExitBatch

 rem -- Set Global Variables from Centralized Script
 CALL "%~dp0SetDrive.BAT" "%~n0"

 rem -- Get Basic Parameters
 SET @CMD=
 SET @MODE=
 SET @SET_ALL=FALSE
 SET @INFO=[Existing Hour]
 SET @OFFSET=-z "00:00:00"
 SET @RESTRICTED="%SystemRoot%\System32" "%SystemRoot%\System" "%SystemRoot%" "%ProgramFiles%" "%ProgramFiles(x86)%" "%ProgramData%"

 rem -- Standard Log Variables
 SET @LOGPATH=%@STORAGE%\Logs\Maint
 SET @LOGFILE="%@LOGPATH%\%~n0.%@FILESTAMP_R%"
 FOR %%F IN (BEFORE AFTER COMPARE TEMPLOG) DO SET @%%F=!@LOGFILE:.TXT=.%%F!

 rem -- Process Special Command-line Parameters
 SET @ALL_PARAMS=%*
 CALL :GetParams "+ N NEXT" &      IF DEFINED @OK (SHIFT & SET @OFFSET=-z "01:00:00"& SET @INFO=[+Hour])
 CALL :GetParams "- P PREVIOUS" &  IF DEFINED @OK (SHIFT & SET @OFFSET=-x "01:00:00"& SET @INFO=[-Hour])
 CALL :GetParams "  T TOMORROW" &  IF DEFINED @OK (SHIFT & SET @OFFSET=-z "24:00:00"& SET @INFO=[+Day])
 CALL :GetParams "  Y YESTERDAY" & IF DEFINED @OK (SHIFT & SET @OFFSET=-x "24:00:00"& SET @INFO=[-Day])
 CALL :GetParams "$ A ALL" &       IF DEFINED @OK (SHIFT & SET @SET_ALL=TRUE)
 CALL :GetParams "@ D DEBUG" &     IF DEFINED @OK (SHIFT & SET @MODE=**DEBUG MODE**& SET @CMD=ECHO)

 SET @SOURCE=%~1& FOR %%V IN ("" %@RESTRICTED%) DO IF /I "%~1"=="%%~V" SET @SOURCE=%@DEFAULT_FOLDER%
 SET @SELECT=%~2& IF "%~2"=="" SET @SELECT=%@DEFAULT_FILES%
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Verify Folder and File Selection
 rem === Updated On: 20 Jul 2020 / 24 Jan 2020
 rem ==========================================================================
:VerifySelection
 SET @FOUND=0& FOR /F %%C IN ('DIR "%@SOURCE%\%@SELECT%" /S /B 2^>NUL ^| FIND "\" /C') DO SET @FOUND=%%C
 IF "%@FOUND%"=="0" (
 	 ECHO *** NO FILES FOUND: "%@SOURCE%\%@SELECT%"
	 GOTO :ExitBatch
 )

 IF /I "%@CONSOLE_SESSION%"=="TRUE" (
	 ECHO %@DIVIDER-%
	 ECHO --- Preparing to Standardize the Time of Selected FILES
	 ECHO %@DIVIDER-%
	 ECHO --- FOLDER ......... "%@SOURCE%"
	 ECHO --- FILES .......... %@SELECT%
	 ECHO --- TIME STAMP ..... %@INFO%
	 ECHO --- UPDATE ALL? .... %@SET_ALL%
	 ECHO --- DEBUG MODE ..... %@MODE%
	 ECHO %@DIVIDER-%
	 ECHO:
	 CHOICE /C YNQEX /T 20 /D Y /N /M "Continue running this script? (Y/N/Q) "
	 IF !ERRORLEVEL! EQU 1 GOTO :UpdateTimeStamps
	 IF !ERRORLEVEL! GTR 1 GOTO :ExitBatch
 ) ELSE ( GOTO :UpdateTimeStamps )
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Update All TimeStamps in the Current Tree to hh:00:00 or plus 1 hour
 rem === Updated On: 04 Jan 2021 / 12 Mar 2019
 rem ==========================================================================
:UpdateTimeStamps
 IF NOT EXIST "%@LOGPATH%" MD "%@LOGPATH%"
 CALL :GetTime @STIME
 CALL :ShowFiles BEFORE
 CALL :DisplayHeader UPDATING TIMESTAMPS %@INFO% %@MODE%

 FOR /R "%@SOURCE%" %%F IN (%@SELECT%) DO (
	 FOR /F "TOKENS=1-6 DELIMS=/-: " %%a IN ('ECHO %%~tF') DO (
		 IF /I "!@SET_ALL!"=="TRUE" (SET @SET_TIME=TRUE) ELSE (SET @SET_TIME=FALSE)
		 IF NOT "%%~e"=="00" SET @SET_TIME=TRUE
		 IF /I "!@SET_TIME!"=="TRUE" (
			 FOR /F "TOKENS=1-3" %%T IN ('DATEINFO -t "%%~a/%%~b/%%~c %%~d:%%~e %%~f" %@OFFSET% -f "mm/dd/yyyy hh-00-00 nnss" -q 2^>NUL') DO (
				 IF /I "!@SET_ALL!%%~V"=="FALSE0000" SET @SET_TIME=FALSE
				 SET @NEWDATE=%%~T
				 SET @NEWTIME=%%~U
			 )
		 )
	 )
	 SET @FILENAME="%%~F"
	 SET @FILENAME=!@FILENAME:%%=%%!
	 IF /I "!@SET_TIME!"=="TRUE" %@CMD%  FILEDATE !@FILENAME! "!@NEWDATE!" "!@NEWTIME!" | FIND /I """" | TEE -a %@TEMPLOG%
 )

 IF NOT EXIST %@TEMPLOG% (
	 ECHO  *** NO TIMESTAMPS CHANGED ***
	 ECHO:
	 GOTO :CleanUp
 ) ELSE (
	 TYPE %@TEMPLOG% >>%@LOGFILE%
	 CALL :ShowFiles AFTER
	 CALL :GetTime @ETIME
 )
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Generate Summary Info for Time Stamping Job
 rem === Updated On: 04 Jan 2021 / 12 Mar 2019
 rem ==========================================================================
:FinishLog
 ( rem -- Write to Log
	 CALL :DisplayHeader SUMMARY: %@SELECT% Files With Updated TimeStamps %@INFO% %@MODE%
	 DIFF %@BEFORE% %@AFTER% | FIND ">  "
	 ECHO:
	 ECHO:
	 ECHO  NOTE: Timestamps Cannot Be Changed On Locked Files...
 ) >%@COMPARE% 2>&1
 FOR %%O IN (CON %@LOGFILE%) DO TYPE %@COMPARE% >>%%O

 ( rem -- Write to Log
	 ECHO:
	 ECHO %@DIVIDER-%
	 ECHO  TimeStamp Update Started On ................. %@STIME%
	 ECHO  TimeStamp Update Ended On ................... %@ETIME%
	 DATEINFO -t !@STIME#! -n !@ETIME#! -e "dys hrs mins secs.ms" -o " TimeStamp Duration Was ...................... " 2>NUL
	 ECHO %@DIVIDER-%
 ) >%@TEMPLOG% 2>&1
 FOR %%O IN (CON %@LOGFILE%) DO TYPE %@TEMPLOG% >>%%O

 rem -- Process Temp Logs
:CleanUp
 FOR %%F IN (BEFORE AFTER COMPARE TEMPLOG) DO IF EXIST !@%%~F! DEL !@%%~F!
 ECHO:
 ECHO  LOGGED TO: %@LOGFILE%
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019
 rem ==========================================================================
:ExitBatch
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Store Current Date and Time in Variable
 rem === Updated On: 16 Oct 2023 / 05 Mar 2017
 rem ==========================================================================
:GetTime
 rem %1 = Variable to Store Date/Time

 SET %~1=!DATE! at !TIME: =0!
 FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET %~1#="%%~d"
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Output Header
 rem === Updated On: 06 Mar 2017
 rem ==========================================================================
:DisplayHeader
 rem %* = Header Info

 ECHO:
 ECHO:
 ECHO %@DIVIDER-%
 ECHO --- %*
 ECHO %@DIVIDER-%
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Output for Selected Files
 rem === Updated On: 11 Mar 2019 / 06 Mar 2017
 rem ==========================================================================
:ShowFiles
 rem %1 = File/Folder Status (BEFORE, AFTER, COMPARE)

 IF /I NOT "%~1"=="BEFORE" IF /I NOT "%~1"=="AFTER" (
	 ECHO *** INVALID HEADER ***
	 GOTO :EOF
 )

 ( rem -- Write to Log
	 CALL :DisplayHeader %~1: TimeStamp Update of "%@SOURCE%\%@SELECT%" %@INFO% %@MODE%
	 FOR /R "%@SOURCE%" %%F IN (%@SELECT%) DO IF "%%~tF"=="" (ECHO  *** File Locked *** -- %%~F) ELSE (ECHO  %%~tF -- %%~F)
	 ECHO %@DIVIDER-%
 ) >!@%~1! 2>&1

 FOR %%O IN (CON %@LOGFILE%) DO TYPE !@%~1! >>%%O
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Use CheckParams.exe to Capture Selected Parameters
 rem === Updated On: 21 Jan 2020
 rem ==========================================================================
:GetParams
 rem %1 = Parameters to Search For
 rem %2 = Variable to Set

 SET @OK=& IF "%~1"=="" GOTO :EOF
 FOR /F "TOKENS=2-3*" %%v IN ('CHECKPARAMS -q -a -c "%~1" -s %@ALL_PARAMS% 2^>NUL') DO IF /I "%%~v"=="TRUE" (
	 SET @OK=T
	 SET @REQ=T
	 IF NOT "%~2"=="" SET %~2=%%~x
 )
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" CALL :GetTime @SCRIPT_BEG
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 30 Nov 2020 / 20 Jul 2020
 rem ==========================================================================
:HelpMessage
 ECHO Set Clean Timestamps of Files in Selected Folder and SubFolders
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  [OPTIONS] [root path for files] [file selection]
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:
 IF DEFINED @DEFAULT_FOLDER (
	 ECHO  * Default Root Folder ......: %@DEFAULT_FOLDER%
	 ECHO  * Default File Selection ...: %@DEFAULT_FILES%
	 ECHO:
 )
 ECHO -------------------
 ECHO OPTION DEFINITIONS:
 ECHO -------------------
 ECHO  -?,  /H ..... Display This Help Message  (also --HELP)
 ECHO  -??, /H2 .... Display Extended Help Message
 ECHO   + , /N ..... Set the time and date to the next hour
 ECHO   - , /P ..... Set the time and date to the previous hour
 ECHO       /T ..... Set the timestamp to the same time of the following day
 ECHO       /Y ..... Set the timestamp to the same time of the previous day
 ECHO   $ , /A ..... For the time to be set for all files
 ECHO   @ , /D ..... Enabled Debug Mode -- Don't Make Actual Changes
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0
	 ECHO  %~n0 %SystemDrive%\Scripts
	 ECHO  %~n0 + "D:\My Files"
	 ECHO  %~n0 /M /F "E:\FileStore"
	 ECHO  %~n0 F:\Repository\Stuff *.config
	 ECHO  %~n0 -D G:\Somewhere *.exe
	 ECHO  %~n0 -?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * The default timestamp is set to the current hour (hh:00:00)
	 ECHO:
	 ECHO  * Timestamps Cannot Be Changed On Locked Files...
	 ECHO  * All files containing an exclamation point "^!" in their names will be ignored.
 )
 CALL :ShowStatus "FINISHED"
                                                                                             