@ECHO OFF
 rem ==========================================================================
 rem === Duplicate Shares From One System To Another
 rem ==========================================================================
 rem
 rem Created On: 16 May 2001
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 27 Feb 2024 / 16 Oct 2023              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2003 OR later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    Resource Kit ........... https://en.wikipedia.org/wiki/Resource_Kit
 rem
 rem  Required Scripts:
 rem    SetDrive.BAT ........... Set Global Variables for this Environment
 rem ==========================================================================

 :::  This script copies all the shares (except the administrative ones) from
 :::  one machine to another. You can enter all of the parameters into the
 :::  script itself, or at the command-line.  The directory structure must be
 :::  the same on the destination as it is on the source, if the script is
 :::  going to work. Use ROBOCOPY, SCOPY or XCOPY (in Win2K) to copy the files
 :::  and folders before using this script.
 :::
 :::  (Nov 2010) Changed the default log extension of the reports from .LOG to
 :::  .TXT so that the reports that are mailed to mobile devices can be read
 :::  automatically. (Not too many devices can natively handle .LOG files)
 :::
 :::  (Jul 2018) v2.0 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::  -- Added new Filtered CMD Title and Syntax Help processing routines;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com
 :::
 :::  (Oct 2018) v2.1 -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Oct 2019) v2.2 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (Nov 2020) v2.3 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Oct 2023) v2.4 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::  (Feb 2024) v3.0 -- Added several parameters for more control and flexibility;
 :::  -- Read the shares from the Registry and support shares with embeded spaces
 ::: in the name;
 :::  -- Added essential file verification via FINDFILES;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 30 Nov 2020 / 30 Jul 2018              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 27 Feb 2024 / 30 Nov 2020              
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v3.0.0.300

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc --mp=2 -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Check for Essential Utilities and/or Scripts
 PATH %PATH%;%~dp0
 FINDFILES -w -m -f "NOW.EXE" "DATEINFO.EXE" "CHECKPARAMS.EXE" "REG.EXE" "RMTSHARE.EXE" "PERMCOPY.EXE"
 IF ERRORLEVEL 64 GOTO :ExitBatch

 rem -- Default Variables (Override via CustomVariables.*.TXT)
 SET @SKIP=ADMIN$ IPC$ Backups$ Downloads$ Installs$ Logs$ Scripts$ Storage$ Scripts_Source$ WSUSTemp WsusContent UpdateServicesPackages
 SET @TEST_SHARES="Test Share #1" "Test Share #2" "Test Share #3$"

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- Process Special Command-line Parameters
 SET @ALL_PARAMS=%*
 CALL :GetParams "A ALL"     & IF DEFINED @OK (SET @ALL=TRUE)                      ELSE (SET @ALL=)
 CALL :GetParams "T TEST"    & IF DEFINED @OK (SET @TEST=TRUE)                     ELSE (SET @TEST=)
 CALL :GetParams "E EXEMPT"  & IF DEFINED @OK (SET @EXEMPT=TRUE)                   ELSE (SET @EXEMPT=)
 CALL :GetParams "C CLEANUP" & IF DEFINED @OK (SET @CLEANUP=TRUE)                  ELSE (SET @CLEANUP=)
 CALL :GetParams "D DIAG"    & IF DEFINED @OK (SET @DIAGMODE=TRUE& SET @ECHO=ECHO) ELSE (SET @DIAGMODE=& SET @ECHO=)

 rem -- Main Variables
 SET @SOURCE=%~1& SET @SOURCE=!@SOURCE:\=!
 SET @DEST=%~2&   SET @DEST=!@DEST:\=!
 SET @LOGPATH=%@STORAGE%\Logs\Maint
 SET @LOGFILE=%@LOGPATH%\%~n0.%@FILESTAMP%

 rem -- Registry Variables
 (SET @DELIM=    REG_MULTI_SZ    )
  SET @REQ_QUERY=REG QUERY "\\%@SOURCE%\HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\LanmanServer\Shares"


 rem -- Show Config Options and Verify Operation
:VerifyOperation
 ECHO Preparing to Copy Share Configurion from One Server to Another...
 ECHO %@DIVIDER-%
 ECHO  Source Server .............: %@SOURCE%
 ECHO  Destination Server ........: %@DEST%
 ECHO  Exemption List ............: %@SPECIALSHARES%
 ECHO  Create Test Shares ........: %@TEST%
 ECHO  Skip Exempt List Only .....: %@EXEMPT%
 ECHO  Copy All Shares Test ......: %@ALL%
 ECHO  Cleanup of Test Shares ....: %@CLEANUP%
 ECHO  Diagnostics Mode ..........: %@DIAGMODE%
 ECHO:
 ECHO  Current List of Shares on Source Server (please wait)
 FOR /F "TOKENS=*" %%L IN ('!@REQ_QUERY! ^| FIND /I "!@DELIM!"') DO (
	 SET @OLD_LINE=%%~L
	 SET @NEW_LINE=!@OLD_LINE:%@DELIM%=\!
	 FOR /F "TOKENS=1 DELIMS=\" %%S IN ('ECHO !@NEW_LINE!') DO ECHO  ---------------------------- "%%~S"
 )
 ECHO %@DIVIDER-%
 ECHO:
 CHOICE /C YNQEX /T 20 /D Y /N /M "Continue running this script? (Y/N/Q) "
 IF ERRORLEVEL 2 GOTO :ExitBatch


 rem ==========================================================================
 rem === Creating Test Shares
 rem === Updated On: 27 Feb 2024                     
 rem ==========================================================================
:CreateTestShares
 IF NOT DEFINED @TEST GOTO :GetShares
 ECHO:
 ECHO Creating Test Shares...
 FOR %%V IN (%@TEST_SHARES%) DO (
	 MD "\\%@SOURCE%\C$\TEMP\%%~V" >NUL 2>&1
	 RMTSHARE \\%@SOURCE%\"%%~V"="C:\TEMP\%%~V" /REMARK:"Test Folder - %%~V"
 )
 %@PRINT_IF_DEBUG%


 rem ==========================================================================
 rem === Obtain Share Info From Source Server And Parse
 rem === Updated On: 27 Feb 2024 / 04 Oct 2010              
 rem ==========================================================================
:GetShares
 IF NOT EXIST "%@LOGPATH%" MD "%@LOGPATH%"

 rem -- Skip Special Shares Unless @ALL=TRUE
 IF EXIST "%@SPECIALSHARES%" FOR /F %%V IN ('TYPE "%@SPECIALSHARES%"') DO SET @SKIP=!@SKIP! %%~V
 IF DEFINED @ALL SET @SKIP=

 rem -- Get Source Shares
 NOW Copying Shares From "%@SOURCE%" To "%@DEST%" >"%@LOGFILE%"
 ECHO:
 ECHO Copying Shares From "%@SOURCE%" To "%@DEST%"...
 ECHO:
 FOR /F "TOKENS=*" %%L IN ('!@REQ_QUERY! ^| FIND /I "!@DELIM!"') DO (
	 SET @OLD_LINE=%%~L
	 SET @NEW_LINE=!@OLD_LINE:%@DELIM%=\!
	 FOR /F "TOKENS=1 DELIMS=\" %%S IN ('ECHO !@NEW_LINE!') DO CALL :MigrateShares "%%~S"
 )

 ECHO:
 ECHO Finished Copying Shares... [%@VER%]
 ECHO:
 RMTSHARE \\%@DEST%
 %@PRINT_IF_DEBUG%


 rem ==========================================================================
 rem === Cleanup of Test Shares
 rem === Updated On: 27 Feb 2024                     
 rem ==========================================================================
:TestCleanup
 IF NOT DEFINED @CLEANUP GOTO :ExitBatch
 ECHO:
 ECHO Cleaning Up Test Shares...
 ECHO:
 FOR %%V IN (%@TEST_SHARES%) DO (
	 RMTSHARE \\%@SOURCE%\"%%~V" /DELETE
	 RD "\\%@SOURCE%\C$\TEMP\%%~V"
 )
 ECHO:
 %@PRINT_IF_DEBUG%


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 27 Feb 2024 / 04 Oct 2019              
 rem ==========================================================================
:ExitBatch
 IF EXIST "%@LOGFILE%" (TYPE "%@LOGFILE%" & DIR "%@LOGFILE%")
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Copy Shares From Source To Destination Server
 rem === Updated On: 27 Feb 2024 / 04 Oct 2010              
 rem ==========================================================================
:MigrateShares
 rem %1 = Current Share Name

 SET @SHARE=%1
 IF NOT DEFINED @ALL IF /I %@SHARE:~-2,1%==$ GOTO :EOF
 FOR %%N IN (%@SKIP% The) DO  IF /I "%~1"=="%%~N" GOTO :EOF
 FOR /F "TOKENS=1*" %%S IN ('RMTSHARE \\%@SOURCE%\%1 ^| FINDSTR /I "PATH"') DO SET @SPATH=%%T
 FOR /F "TOKENS=1*" %%C IN ('RMTSHARE \\%@SOURCE%\%1 ^| FINDSTR /I "REMARK"') DO SET @COMMENT=%%D

 IF /I NOT "%@SPATH"=="" (
	 ECHO:
	 ECHO:
	 ECHO Migrating the %1 Share...
	 %@ECHO% RMTSHARE \\%@DEST%\%1="%@SPATH%" /REMARK:"%@COMMENT%"
	 %@ECHO% PERMCOPY \\%@SOURCE% %1 \\%@DEST% %1
 )
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Use CheckParams.exe to Capture Selected Parameters
 rem === Updated On: 27 Feb 2024                     
 rem ==========================================================================
:GetParams
 rem %1 = Parameters to Search For
 rem %2 = Variable to Set

 SET @OK=& IF "%~1"=="" GOTO :EOF
 FOR /F "TOKENS=2-3*" %%v IN ('CHECKPARAMS -q -a -c "%~1" -s %@ALL_PARAMS% 2^>NUL') DO IF /I "%%~v"=="TRUE" (
	 SET @OK=T
	 SET @REQ=T
	 IF NOT "%~2"=="" SET %~2=%%~x
 )
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET @SCRIPT_BEG#="%%~d"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 27 Feb 2024 / 30 Nov 2020              
 rem ==========================================================================
:HelpMessage
 ECHO Duplicate Shares From One System To Another
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  ^<source_machine^> ^<destination_machine^> [OPTIONS]
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:
 ECHO -------------------
 ECHO OPTION DEFINITIONS:
 ECHO -------------------
 ECHO  -?,  /H ...... Display This Help Message  (also --HELP)
 ECHO  -??, /H2 ..... Display Extended Help Message
 ECHO       /A ...... Copy All Shares, Including Hidden($) Ones [And Ignore the Exemption List]
 ECHO       /E ...... Copy All Shares, Including Hidden($) Ones [But Still Honor Exemption List]
 ECHO       /T ...... Copy TEST Shares Only
 ECHO       /C ...... Automatically Clean Up Test Shares from Source
 ECHO       /DIAG ... Only display, but don't execute, the final PERMCOPY and RMTSHARE commands
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~nx0 OLDSERVER NEWSERVER
	 ECHO  %~nx0 MYCOMPUTER YOURLAPTOP /A
	 ECHO  %~nx0 %COMPUTERNAME% NEWCOMPUTER /T
	 ECHO  %~nx0 THATSERVER THISSERVER /T /C /DIAG
	 ECHO  %~n0 -?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
	 ECHO:
	 ECHO  * By default, all hidden shares ^(those ending with $^) are skipped
	 ECHO    in normal copying operation. Using /E will skip only those shares
	 ECHO    that are found in the exemption list.
 )
 CALL :ShowStatus "FINISHED"
                   