@ECHO OFF
 rem ==========================================================================
 rem === Copy Special Folders (Containing Specific Prefix) using ROBOCOPY
 rem ==========================================================================
 rem
 rem Created On: 19 May 2020
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Oct 2023 / 30 Nov 2020              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: 2008 or later (not tested on anything below 2008-R2)
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem  Required Scripts:
 rem    SetDrive.BAT ........... Set Global Variables for this Environment
 rem ==========================================================================

 :::  This script uses ROBOCOPY, which has been a native Windows utility since
 :::  Vista and Server 2008. It facilitates the copying of all top level
 :::  folders from a source location which contain a specific prefix character.
 :::
 :::  The default prefix, if none is provided, is "@"
 :::
 :::  These folders are copied to a destination, with options available to
 :::  maintain or eliminate the prefix at the destination.
 :::
 :::  (May 2020) v1.0 -- Initial implementation using XCOPY;
 :::
 :::  (May 2020) v1.1 -- Revamped implementation using ROBOCOPY;
 :::
 :::  (May 2020) v1.2 -- Added the option to include specific folders;
 :::  -- Added the option to limit folder inclusions to 1st subfolder layer;
 :::  -- Use NETUSE or SUBST to map source and destination drives befoe copying;
 :::
 :::  (May 2020) v1.3 -- Added performance logging for each section of operation;
 :::  -- Updated to use the new logfile maintenance routine (:LogMaint);
 :::
 :::  (May 2020) v1.4 -- Added a new option (/FL = @FOLDERLIST) to provide a way
 :::  to select the 1st level folders that will be copied (rather than by prefix);
 :::
 :::  (Jun 2020) v1.5 -- The default for source tree generation has been changed
 :::  from TRUE to FALSE.  A new option (/ST or /SAVE_TREE) has been provided to
 :::  be able to enable it.  /NT will still work to turn it off, if it has been
 :::  enabled via the @NOTREE variable in one of the config files;
 :::
 :::  (Nov 2020) v1.6 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Oct 2023) v1.7 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 29 May 2020 / 19 May 2020              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 30 Nov 2020 / 14 Jun 2020              
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v1.7.0.220

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc --mp=2 -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- Main Variables
 SET @PARAMS=/NJH /NDL /NS /NC /NP /XJ /XN /SEC /J
 SET @KEEP_PREFIX=TRUE
 SET @PREFIX_MSG=KEEP PREFIX AT DESTINATION
 SET @LIMIT_SEARCH=FALSE
 SET @NOTREE=DON'T
 SET @INCLUDE_LIST=
 SET @FOLDERS2INCLUDE=
 SET @COPY_TYPE=BASIC

 rem -- Process Special Command-line Parameters
 SET @ALL_PARAMS=%*& CALL :SwapCharacters @ALL_PARAMS "=" ":"
 SET @SOURCE=%~1& CALL :FixDirectoryPath @SOURCE
 SET @DEST=%~2&   CALL :FixDirectoryPath @DEST
 CALL :GetParams "P   PREFIX"      @PREFIX     & IF NOT DEFINED @PREFIX     (SET @PREFIX=@)
 CALL :GetParams "F   FILESPEC"    @FILESPEC   & IF NOT DEFINED @FILESPEC   (SET @FILESPEC=*.*)
 CALL :GetParams "SD  SKIP_DIRS"   @SKIP_DIRS  & IF     DEFINED @SKIP_DIRS  (SET @SKIP_DIRS=/XD !@SKIP_DIRS!)
 CALL :GetParams "SF  SKIP_FILES"  @SKIP_FILES & IF     DEFINED @SKIP_FILES (SET @SKIP_FILES=/XF !@SKIP_FILES!)
 CALL :GetParams "FL  FOLDER_LIST" @FOLDERLIST & IF     DEFINED @FOLDERLIST (SET @COPY_TYPE=LIST& CALL :StripQuotes @FOLDERLIST)
 CALL :GetParams "INC INCLUDE"     @INC_DIRS   & IF     DEFINED @INC_DIRS (
	 SET @COPY_TYPE=SEARCH
	 SET @LIMIT_SEARCH=TRUE

	 SET @FOLDERS2INCLUDE=!@INC_DIRS:"=!
	 SET @FOLDERS2INCLUDE=!@FOLDERS2INCLUDE: =.!
	 SET @FOLDERS2INCLUDE=!@FOLDERS2INCLUDE:;= !

	 SET @INC_TEMP=!@INC_DIRS:"=!
	 SET @INC_TEMP=!@INC_TEMP:,=!
	 SET @INC_TEMP=!@INC_TEMP: =!
	 SET @INC_TEMP=!@INC_TEMP:;= !
	 FOR %%F IN (!@INC_TEMP!) DO (SET @INCLUDE_LIST=!@INCLUDE_LIST!"%%~F" )
	 SET @INCLUDE_LIST=!@INCLUDE_LIST:= !
	 SET @INCLUDE_LIST=!@INCLUDE_LIST:=,!
	 SET @INC_TEMP=
 )
 CALL :GetParams "A  ALL_SEARCH"    & IF DEFINED @OK (SET @LIMIT_SEARCH=FALSE)
 CALL :GetParams "L  LIMIT"         & IF DEFINED @OK (SET @LIMIT_SEARCH=TRUE)
 CALL :GetParams "K  KEEP_PREFIX"   & IF DEFINED @OK (SET @KEEP_PREFIX=TRUE)
 CALL :GetParams "R  REMOVE_PREFIX" & IF DEFINED @OK (SET @KEEP_PREFIX=FALSE& SET @PREFIX_MSG=REMOVE PREFIX AT DESTINATION)
 CALL :GetParams "ST SAVE_TREE"     & IF DEFINED @OK (SET @NOTREE=)
 CALL :GetParams "NT NO_TREE"       & IF DEFINED @OK (SET @NOTREE=DON'T )
 CALL :GetParams "T  TEST"          & IF DEFINED @OK (SET @TESTMODE=TRUE& SET @RCLIST=/L) ELSE (SET @TESTMODE=FALSE& SET @RCLIST=)
 CALL :SearchLimitMessage !@LIMIT_SEARCH!

 rem -- Standard Log Variables
 SET @LOGPATH=%@STORAGE%\Logs\Replication
 SET @LFORMAT=%@LOGPATH%\%~n0
 SET @LOGFILE=%@LFORMAT%.%@FILESTAMP_R%
 FOR %%L IN (SUMMARY TEMPLOG ERRORS) DO SET @%%L=%@LFORMAT%.%%L.%@FILESTAMP_R%
 SET @LOGLIST="%@LOGFILE%" "%@ERRORS%"
 SET @CLEANUP="%@TEMPLOG%" "%@ERRORS%" "%@SUMMARY%"
 %@PRINT_IF_DEBUG%


 rem ==========================================================================
 rem === Validate the Existence of the Critical Files and Folders
 rem === Updated On: 28 May 2020 / 21 May 2020              
 rem ==========================================================================
:ValidateCriticalItems
 FOR %%D IN ("%@DEST%" "%@LOGPATH%") DO IF NOT EXIST "%%~D" MD "%%~D" >NUL
 FOR %%I IN ("%@SOURCE%" "%@DEST%" "%@LOGPATH%" %@FOLDERLIST%) DO IF NOT EXIST "%%~I" (
	 ECHO *** Unable to Find or Create the Following ***
	 ECHO  -- "%%~I"
	 GOTO :ExitBatch
 )
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Allow User to Abort Operation (Default = Continue in 30 Seconds)
 rem === Updated On: 29 May 2020 / 27 May 2020              
 rem ==========================================================================
:DblCheck
 CALL :GetTime @STIME

 ( rem -- Write to Log
	 CALL :ShowHeader PREPARING TO COPY SPECIAL FOLDERS
	 ECHO            FROM: "%@SOURCE%"
	 ECHO              TO: "%@DEST%"
	 ECHO  SELECTION MODE:  %@COPY_TYPE%

	 IF DEFINED @FOLDERLIST (
		 ECHO     FOLDER LIST: %@FOLDERLIST%
	 ) ELSE (
		 ECHO          PREFIX: "%@PREFIX%" ... %@PREFIX_MSG%
	 )

	 ECHO        FILESPEC:  %@FILESPEC%
	 ECHO      EXCLUSIONS:  %@SKIP_FILES%  %@SKIP_DIRS%
	 ECHO      INCLUSIONS:  %@INCLUDE_LIST%
	 ECHO    SEARCH LIMIT:  %@LIMIT_MSG%
	 ECHO     SOURCE TREE:  !@NOTREE!GENERATE SOURCE TREE
	 ECHO       TEST MODE:  %@TESTMODE%
	 ECHO %@DIVIDER*%
	 ECHO:
 ) >"%@SUMMARY%" 2>&1

 rem -- Verify Operation
 TYPE "%@SUMMARY%"
 CHOICE /C YNQEX /T 30 /D Y /N /M "Continue running this script? (Y/N/Q) "
 IF ERRORLEVEL 2 GOTO :ExitBatch
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Get Source Folder Info
 rem === Updated On: 27 May 2020 / 26 May 2020              
 rem ==========================================================================
:GetSourceTree
 CALL :GetTime @TTIME
 CALL :ShowHeader GENERATING CURRENT SOURCE TREE

 ( rem -- Write to Log
	 TYPE "%@SUMMARY%"
	 IF NOT DEFINED @NOTREE (
		 CALL :ShowHeader GENERATING SOURCE TREE
		 TREE "%@SOURCE%" /A
		 NOW \s \n\#\#\#
	 )
	 ECHO:
 ) >"%@LOGFILE%" 2>&1
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Search for Special Folders to Copy
 rem === Updated On: 30 May 2020 / 27 May 2020              
 rem ==========================================================================
:CopySpecial
 CALL :GetTime @CTIME
 CALL :ShowHeader STARTING THE COPY PROCESS

 CALL :MapDrive @D_DRIVE "%@DEST%" "DESTINATION"
 CALL :MapDrive @S_DRIVE "%@SOURCE%" "SOURCE"

 rem -- Define Prefix Search vs Folder List Search
 IF DEFINED @FOLDERLIST (SET @FINDSTR=/G:%@FOLDERLIST%) ELSE (SET @FINDSTR=/R "^^^^^\!@PREFIX!")

 rem -- Obtain and Enumerate the 1st Level Folders
 FOR /F "TOKENS=*" %%F IN ('DIR /B /AD /OGN "!@S_DRIVE!" 2^>NUL ^| FINDSTR /I %@FINDSTR%') DO (
	 SET @CURRENT_SOURCE="!@S_DRIVE!\%%~F"
	 SET @CURRENT_DEST="!@D_DRIVE!\%%~F"

	 rem -- Proceed with the Copying of the Folder
	 CALL :RemovePrefix @CURRENT_DEST !@D_DRIVE!
	 CALL :ShowFolders @CURRENT_SOURCE @CURRENT_DEST @TEMPLOG

	 rem -- BASIC ROBOCOPY of All Prefixed Folders with All Subfolders [with File/Folder Exclusions]
	 IF /I "%@COPY_TYPE%"=="BASIC" (
		 ROBOCOPY !@CURRENT_SOURCE! !@CURRENT_DEST! %@FILESPEC% %@THREADS% %@RETRIES% %@RCLIST% /S /XO /NP %@PARAMS% %@SKIP_DIRS% %@SKIP_FILES% /LOG+:"%@TEMPLOG%"
	 )

	 rem -- LIMITED ROBOCOPY of Prefixed or Listed Folders with Specific Subfolders [with Inclusions/Exclusions]
	 IF /I NOT "%@COPY_TYPE%"=="BASIC" (
		 IF /I "%@LIMIT_SEARCH%"=="TRUE" (

			 rem -- Search 1st Level Subfolders ONLY, and Copy All Their Subfolders
			 FOR /F "TOKENS=*" %%D IN ('DIR /B /AD /OGN !@CURRENT_SOURCE! 2^>NUL ^| FINDSTR /I /R "!@FOLDERS2INCLUDE!"') DO (
				 SET @S_FOLDER=!@CURRENT_SOURCE!\%%~D
				 SET @S_FOLDER="!@S_FOLDER:"=!"
				 SET @D_FOLDER=!@S_FOLDER:%@S_DRIVE%=%@D_DRIVE%!

				 rem -- Proceed with the Copying of Included Folders
				 CALL :RemovePrefix @D_FOLDER !@D_DRIVE!
				 CALL :ShowFolders @S_FOLDER @D_FOLDER @TEMPLOG
				 ROBOCOPY !@S_FOLDER! !@D_FOLDER! %@FILESPEC% %@THREADS% %@RETRIES% %@RCLIST% /S /XO /NP %@PARAMS% %@SKIP_DIRS% %@SKIP_FILES% /LOG+:"%@TEMPLOG%"
			 )

		 ) ELSE (

			 rem -- Search Through All Folders for the Inclusions, and Copy Those Only
			 FOR /F "TOKENS=*" %%D IN ('DIR /B /AD /S /OGN !@CURRENT_SOURCE! 2^>NUL ^| FINDSTR /I /R "!@FOLDERS2INCLUDE!"') DO (
				 SET @S_FOLDER="%%~D"
				 SET @D_FOLDER=!@S_FOLDER:%@S_DRIVE%=%@D_DRIVE%!

				 rem -- Proceed with the Copying of Included Folders
				 CALL :RemovePrefix @D_FOLDER !@D_DRIVE!
				 CALL :ShowFolders @S_FOLDER @D_FOLDER @TEMPLOG
				 ROBOCOPY !@S_FOLDER! !@D_FOLDER! %@FILESPEC% %@THREADS% %@RETRIES% %@RCLIST% /XO /NP %@PARAMS% %@SKIP_DIRS% %@SKIP_FILES% /LOG+:"%@TEMPLOG%"
			 )
		 )
	 )
 )

 rem -- Disconnect from Previously Defined NET USE and SUBST Drives
 ECHO:
 CALL :UnMapDrive @S_DRIVE "SOURCE"
 CALL :UnMapDrive @D_DRIVE "DESTINATION"
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Generate Summary Job Reports
 rem === Updated On: 28 May 2020 / 27 May 2020              
 rem ==========================================================================
:SummaryReport
 CALL :GetTime @DTIME

 ( rem -- Write to Log
	 IF /I "%@TESTMODE%"=="TRUE" (
		 ECHO:
		 ECHO TEST MODE ENABLED -- NO FILES WERE ACTUALLY COPIED
		 ECHO:
	 )
	 IF EXIST "%@TEMPLOG%" (
		 TYPE "%@TEMPLOG%"
		 CALL :ShowHeader GENERATING DESTINATION TREE
		 TREE "%@DEST%" /A
		 NOW \s \n\#\#\#
	 ) ELSE (
		 ECHO:
		 ECHO NO FILES OR FOLDERS WERE FOUND MATCHING THESE REQUIREMENTS
		 ECHO:
	 )

	 CALL :GetTime @ETIME
	 CALL :ShowHeader JOB TIMING REPORT
	 ECHO  Special Copy Process Started On ........... !@STIME!
	 ECHO   Source Tree Generation Started On ........ !@TTIME!
	 ECHO   All Copy Jobs Started On ................. !@CTIME!
	 ECHO   Destination Tree Generation Started On ... !@DTIME!
	 ECHO  Special Copy Process Ended On ............. !@ETIME!
	 ECHO %@DIVIDER-%
	 DATEINFO -t !@STIME#! -n !@TTIME#! -e "dys hrs mins secs.ms" -o " Preliminary Job Processing Time Was ....... " 2>NUL
	 DATEINFO -t !@TTIME#! -n !@CTIME#! -e "dys hrs mins secs.ms" -o " Source Tree Generation Time Was ........... " 2>NUL
	 DATEINFO -t !@CTIME#! -n !@DTIME#! -e "dys hrs mins secs.ms" -o " Data Copying Processes Time Was ........... " 2>NUL
	 DATEINFO -t !@DTIME#! -n !@ETIME#! -e "dys hrs mins secs.ms" -o " Destination Tree Generation Time Was ...... " 2>NUL
	 DATEINFO -t !@STIME#! -n !@ETIME#! -e "dys hrs mins secs.ms" -o " TOTAL Job Processing Time Was ............. " 2>NUL
	 ECHO %@DIVIDER*%
 ) >>"%@LOGFILE%" 2>&1
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Perform Log Management/Maintenance (Or Copy Them for Debug Purposes)
 rem === Updated On: 27 May 2020                     
 rem ==========================================================================
:LogMaint
 ECHO PERFORMING FINAL LOG MAINTENANCE...

 IF /I "%DEBUG%"=="COPY" FOR %%L IN (%@LOGLIST% %@CLEANUP%) DO IF EXIST %%L COPY %%L "%TEMP%" /Y

 FOR %%L IN (%@LOGLIST%) DO IF EXIST %%L (
	 SET @THISLOG=%%L
	 COPY %%L !@THISLOG:%@FILESTAMP_R%=TXT! /Y
	 IF %%L=="%@LOGFILE%" (
		 ECHO:
		 DIR %%L | FIND /V " Volume "
		 START "Log for Special Copy Job" %%L
		 TAIL -16 %%L
	 )
 )
 FOR %%L IN (%@CLEANUP%) DO IF EXIST %%L DEL %%L >NUL
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 28 May 2020 / 19 May 2020              
 rem ==========================================================================
:ExitBatch
 RD "%@DEST%" >NUL 2>NUL
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Execute Dynamic Command From Variable
 rem === Updated On: 26 May 2020                     
 rem ==========================================================================
:RunCMD
 rem %* = Command To Execute

 %*
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Store Current Date and Time in Variable
 rem === Updated On: 16 Oct 2023 / 27 May 2020              
 rem ==========================================================================
:GetTime
 rem %1 = Variable to Store Date/Time

 SET %~1=!DATE! at !TIME: =0!
 FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET %~1#="%%~d"
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Section Header
 rem === Updated On: 27 May 2020                     
 rem ==========================================================================
:ShowHeader
 rem %* = Message to Display

 IF "%~1"=="" GOTO :EOF
 ECHO:
 ECHO %@DIVIDER*%
 ECHO *** %* -- !DATE! at !TIME: =0!
 ECHO %@DIVIDER*%
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Show Folders To Be Copied
 rem === Updated On: 26 May 2020                     
 rem ==========================================================================
:ShowFolders
 rem %1 = Source Folder Variable
 rem %2 = Destination Folder Variable
 rem %3 = Log File Variable

 ( FOR %%O IN (CON "!%~3!") DO (
	 NOW \n\n**** PREPARING TO COPY FILES @ \v ****\n\-
	 ECHO  SOURCE:  %@S_DRIVE% = %@SOURCE%
	 ECHO    DEST:  %@D_DRIVE% = %@DEST%
	 ECHO    FROM: !%~1!
	 ECHO      TO: !%~2!
	 ECHO %@DIVIDER-%
 ) >>%%O )
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Use CheckParams.exe to Capture Selected Parameters
 rem === Updated On: 19 May 2020                     
 rem ==========================================================================
:GetParams
 rem %1 = Parameters to Search For
 rem %2 = Variable to Set

 SET @OK=& IF "%~1"=="" GOTO :EOF
 FOR /F "TOKENS=2-3*" %%v IN ('CHECKPARAMS -q -a -c "%~1" -s %@ALL_PARAMS% 2^>NUL') DO IF /I "%%~v"=="TRUE" (
	 SET @OK=T
	 SET @REQ=T
	 IF NOT "%~2"=="" SET %~2=%%~x
 )
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Process the Removal of the Prefix, If Requested
 rem === Updated On: 29 May 2020 / 26 May 2020              
 rem ==========================================================================
:RemovePrefix
 rem %1 = Folder Variable
 rem %2 = Current Destination

 IF DEFINED @FOLDERLIST GOTO :EOF
 IF NOT DEFINED %1 GOTO :EOF
 IF NOT DEFINED %2 GOTO :EOF
 SET @VAR=!%~1!
 IF /I "%@KEEP_PREFIX%"=="FALSE" SET @VAR=!@VAR:%~2\%@PREFIX%=%~2\!
 SET %~1=!@VAR!
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Strip Quotation Marks from a Value
 rem === Updated On: 28 May 2020                     
 rem ==========================================================================
:StripQuotes
 rem %1 = Name of Variable to CleanUp

 IF NOT DEFINED %1 SET %~1=""
 SET @VAL=!%~1!
 SET @VAL=!@VAL:"=!
 SET %~1="!@VAL!"
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Remove Trailing \ From Supplied Folder Path
 rem === Updated On: 19 May 2020                     
 rem ==========================================================================
:FixDirectoryPath
 rem %1 = Folder Variable

 IF NOT DEFINED %1 GOTO :EOF
 FOR /L %%X IN (1,1,9) DO IF "!%~1:~-1!"=="\" SET %~1=!%~1:~0,-1!
 IF "!%~1:~-1!"==":" SET %~1=!%~1!\
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Swap Characters in Variable's Content
 rem === Updated On: 26 May 2020                     
 rem ==========================================================================
:SwapCharacters
 rem %1 = Variable to Make the Swap
 rem %2 = Original Character(s)
 rem %3 = New Character(s)

 IF "%~1"=="" GOTO :EOF
 SET @VAR=!%~1!
 SET @VAR=!@VAR:"=!
 FOR /F "TOKENS=*" %%a IN ('SUBSTRING "!@VAR!" "%~2" "%~3" 2^>NUL') DO SET @VAR=%%a
 SET @VAR=!@VAR:="!
 SET %~1=!@VAR!
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Create a Mapped or SUBST Drive and Obtain Drive Letter
 rem === Updated On: 26 May 2020                     
 rem ==========================================================================
:MapDrive
 rem %1 = Variable to Assign the Drive Letter to
 rem %2 = UNC Path to Connect
 rem %3 = Description

 SET @DESCRIPTION=[%~3] & IF "%~3"=="" SET @DESCRIPTION=
 SET @DRV=
 SET @DPATH=%~2
 IF "%@DPATH:~0,2%"=="\\" (
	 FOR /F "TOKENS=2" %%D IN ('NET USE * "%~2" /PERSISTENT:NO ^| FIND ":"') DO (
		 SET @DRV=%%~D
		 SET %~1=%%~D
		 SET %~1-PATH=%~2
		 SET %~1-METHOD=NETUSE
	 )
 ) ELSE (
	 FOR %%a IN (%@ALPHABET%) DO IF NOT DEFINED @DRV (
		 SUBST %%a: "%~2" >NUL
		 FOR /F "TOKENS=1,3* DELIMS=\ " %%D IN ('SUBST ^| FIND /I "%~2"') DO (
			 SET @DRV=%%~D
			 SET %~1=%%~D
			 SET %~1-PATH=%~2
			 SET %~1-METHOD=SUBST
		 )
	 )
 )
 ECHO Mapping Drive %@DRV% to %@DESCRIPTION%"%@DPATH%"
 ECHO:
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Delete an Existing Mapped or SUBST Drive
 rem === Updated On: 26 May 2020                     
 rem ==========================================================================
:UnMapDrive
 rem %1 = Current Variable
 rem %2 = Description

 SET @DESCRIPTION=[%~2] & IF "%~2"=="" SET @DESCRIPTION=
 CALL :RunCMD SET @DRV=%%%~1%%
 CALL :RunCMD SET @DPATH=%%%~1-PATH%%
 CALL :RunCMD SET @METHOD=%%%~1-METHOD%%
 IF /I "%@METHOD%"=="NETUSE" (
	 NET USE %@DRV% /D /Y >NUL 2>&1
 ) ELSE (
	 SUBST %@DRV% /D
 )
 ECHO Unmapping Drive %@DRV% from %@DESCRIPTION%"%@DPATH%"
 ECHO:
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set Appropriate Search Limit Message
 rem === Updated On: 28 May 2020                     
 rem ==========================================================================
:SearchLimitMessage
 rem %1 = Search Limit Status

 IF /I "%~1"=="TRUE" (
	 SET @LIMIT_MSG=LIMIT SEARCH TO 1ST SUBFOLDER LEVEL
 ) ELSE (
	 SET @LIMIT_MSG=SEARCH ALL SUBFOLDERS
 )
 %@PRINT_IF_DEBUG%


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 19 May 2020              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET @SCRIPT_BEG#="%%~d"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 30 Nov 2020 / 14 Jun 2020              
 rem ==========================================================================
:HelpMessage
 ECHO Copy Special Folders (Containing Specific Prefix) using ROBOCOPY
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  ^<source^> ^<dest^> [options]
 ECHO  %~n0  ^<source^> ^<dest^> [-t] [-f] [-sd] [-sf] [-inc= [-l^|-a]] [-nt^|-st] [-p=] [-k^|-r]
 ECHO  %~n0  ^<source^> ^<dest^> [-t] [-f] [-sd] [-sf] [-inc= [-l^|-a]] [-nt^|-st] [-fl=]
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:
 ECHO -------------------
 ECHO OPTION DEFINITIONS:
 ECHO -------------------
 ECHO  -?,   /H .............. Display This Help Message  (also --HELP)
 ECHO  -??,  /H2 ............. Display Extended Help Message
 ECHO  -T,   /TEST ........... Only ECHO the Output instead of performing the copies
 ECHO  -F,   /FILESPEC ....... Provide specific file spec to copy   (DEFAULT = *.*)
 ECHO  -SD,  /SKIP_DIRS ...... Exclusion spec for dirs/folders
 ECHO  -SF,  /SKIP_FILES ..... Exclusion spec for files
 ECHO  -INC, /INCLUDE ........ Semicolon delimited list of folders to include
 ECHO  -L,   /LIMIT_SEARCH ... Limit folder inclusion search to 1st subfolder level
 ECHO  -A,   /ALL_SEARCH ..... Search for folder inclusions through entire source tree
 ECHO  -FL,  /FOLDER_LIST .... Provide a list of 1st level folder names to copy
 ECHO  -P,   /PREFIX ......... Set the folder prefix to search for  (DEFAULT = @)
 ECHO  -K,   /KEEP_PREFIX .... Keep the prefix in the destination copy  *** DEFAULT ***
 ECHO  -R,   /REMOVE_PREFIX .. Eliminate the prefix in the destination copy
 ECHO  -NT,  /NO_TREE ........ Don't Generate the Source Tree  *** DEFAULT ***
 ECHO  -ST,  /SAVE_TREE ...... Generate the Source Tree for the logs (can be lengthy)
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0  C:\Source\Folder D:\Destination\Directory
	 ECHO  %~n0 "C:\Source Folder" "D:\Destination Directory" /F:"*.DOC*"
	 ECHO  %~n0 "D:\Local Drive" \\REMOTE\SHARE\Directory -P:#
	 ECHO  %~n0  E:\Easy \\SERVER1\Repository /SD:"\OLD" /SF:"*.OLD *.BAK"
	 ECHO  %~n0  F:\MuchData \\SERVER2\Data /INC:"DontCopy;SomeOtherFolder;A Long Folder Name;AndThisToo;\Dir\With\Subfolder"
	 ECHO  %~n0 "G:\This Folder" "H:\That Folder" /KEEP_PREFIX /TEST /NO_TREE /INCLUDE="Bin;Docs;Notes"
	 ECHO  %~n0 "J:\Source Location" "K:\New Destination" /FL="C:\MyListOfFolders.TXT" /INC:"Scripts;Automation" /SF=TEST*.*
	 ECHO  %~n0 -?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
	 ECHO  * Options with additional parameters can use /OPTION: or /OPTION=
	 ECHO:
	 ECHO  * If the following options are used, an additional value needs to be
	 ECHO    supplied: /P, /F, /SD, /SF, /INC, and /FL.   If this extra value is
	 ECHO    not provided, or set to blank^(""^), then the following will occur:
	 ECHO    ---  /P, /F ........... Default values are used instead
	 ECHO    ---  /SD, /SF, /INC ... The inclusion or exclusion will be ignored
	 ECHO    ---  /FL .............. The initial folders will be determined by prefix
	 ECHO:
	 ECHO  * If /INC is used, then ONLY the files in the folders specified will be copied.
	 ECHO    File and folder exclusions will still be applied against the include list, so
	 ECHO    plan accordingly.  The use of /INC automatically implies the use of /L unless
	 ECHO    explicitly overridden by /A.
	 ECHO:
	 ECHO  * The search criteria for /INC can only include "\" if the /A option is used.
	 ECHO    The default of /L will only search the first subfolder under the prefixed
	 ECHO    folder.
	 ECHO:
	 ECHO  * If /FL is used, then /P, /K and /R will all be ignored.  Only the listed
	 ECHO    folders will be copied, and the names will be retained at the destination.
	 ECHO:
	 ECHO  * If the file associated with /FL does not exist, the job will error and exit.
	 ECHO:
	 ECHO  * Any parameters which contain spaces should be surrounded by quotes.
	 ECHO    Shortnames *can* be surrounded by quotes if you choose, but it is not
	 ECHO    mandatory by any means.
 )
 CALL :ShowStatus "FINISHED"
                                                       