@ECHO OFF
 rem ==========================================================================
 rem === Capture Various Date/Time Elements Into Variables
 rem ==========================================================================
 rem
 rem Created On: 24 May 1999
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Oct 2023 / 30 Nov 2020              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: NT4 or Higher
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem    Resource Kit ........... https://en.wikipedia.org/wiki/Resource_Kit
 rem
 rem    DOFF ................... http://www.jfitz.com/dos/index.html#DOFF
 rem ==========================================================================

 :::  These scripts obtain various elements of the date and time for use in
 :::  naming files -- particularly logs.
 :::
 :::  A number of different approaches are taken, some with the native only
 :::  tools, and some with the use of Resource Kit tools.
 :::
 :::  (Nov 2010) Changed the default log extension of the reports from .LOG to
 :::  .TXT so that the reports that are mailed to mobile devices can be read
 :::  automatically. (Not too many devices can natively handle .LOG files)
 :::
 :::  (Apr 2015) v7.0 -- Major updates to the time/date routines, taking
 :::  advantage of the updated DATEINFO.EXE utility. The :ShowStatus routine
 :::  was also updated to display script runtime duration.
 :::
 :::  (Apr 2017) v8.0 -- Added a new test subroutine that relies ONLY on
 :::  native utilities and an updated version of the DATEINFO.EXE utility.
 :::
 :::  (May 2018) v8.1 -- Changed the HELP routine to use FINDSTR;
 :::  -- Added new Filtered CMD Title and Syntax Help processing routines;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com;
 :::  -- Added 12-Hour time via DATEINFO;
 :::
 :::  (Feb 2019) v9.0 -- Consolidate redundant sections into a new routine;
 :::  -- Take advantage of the 4.x version of DATEINFO;
 :::
 :::  (Oct 2019) v9.2 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (Dec 2019) v9.3 -- Developed a drop-in replacement for Microsoft's
 :::  NOW.EXE utility; Needs the @MSNOW_COMPATIBILITY variable set to TRUE;
 :::
 :::  (Nov 2020) v9.4 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Oct 2023) v9.5 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::  NOTE: The parsing of date variables in all of the date-related scripts in
 :::        this archive is dependent on the Regional Settings in the Control
 :::        Panel.  These scripts were all tested with the default English (US)
 :::        option enabled, only.
 :::
 :::        Options #4-8 are the only methods that work independently of the
 :::        Regional Settings, including the default English (US)
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 30 Nov 2020 / 14 May 2018              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Display Various Date/Time Calculations
 rem === Updated On: 10 Dec 2019 / 20 Dec 2018              
 rem ==========================================================================
:DisplayAll
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v9.5.0.850

 rem -- Main Variables
 SET @MSNOW_COMPATIBILITY=TRUE

 rem -- Main Routine
 FOR /L %%i IN (1,1,8) DO CALL :GetDate%%i
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display the Variables for Each Section
 rem === Updated On: 05 Apr 2017 / 21 Apr 2015              
 rem ==========================================================================
:Display
 rem %1 = Long Name of Section
 rem %2 = Short Name of Section

 ECHO:
 ECHO Output for: %1
 ECHO ---------------------------------------------------------------------------
 ECHO %%DATE%%   = %DATE%
 ECHO %%TIME%%   = %TIME%
 ECHO:
 FOR /F "TOKENS=*" %%D IN ('DATE /T') DO ECHO DATE /T  = %%D
 FOR /F "TOKENS=*" %%T IN ('TIME /T') DO ECHO TIME /T  = %%T
 ECHO:
 FOR /F "TOKENS=*" %%D IN ('NOW') DO ECHO NOW      = %%D
 ECHO:
 FOR /F "TOKENS=*" %%D IN ('DOFF "mm/dd/yyyy hh:mi:ss"') DO ECHO DOFF     = %%D
 ECHO:
 FOR /F "TOKENS=*" %%D IN ('DATEINFO -s -f "mm/dd/yyyy hh:nn:ss.zz" -q') DO ECHO DATEINFO = %%D
 ECHO:
 ECHO:
 SET @
 ECHO:
 ECHO Sample File #1: "%TEMP%\My File %@TODAY%.TXT"
 ECHO Sample File #2: "%SystemRoot%\System32\LogFiles\%2.%@FILESTAMP%"
 ECHO ---------------------------------------------------------------------------
 ECHO:
 TIMEOUT 20
 ECHO:
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Get Current Date and Time                      (OPTION #1)
 rem === Updated On: 07 Nov 2010 / 25 May 2005              
 rem ==========================================================================
:GetDate1
 rem ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 rem NT4 or Higher -- No Resource Kit Required
 rem ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 SETLOCAL
 FOR /F "DELIMS=" %%T IN ('TIME /T') DO SET @TIME=%%T
 FOR /F "TOKENS=1" %%D IN ('DATE /T') DO SET @DAY=%%D
 FOR /F "TOKENS=2" %%D IN ('DATE /T') DO SET @DATE=%%D
 SET @DD=%@DATE:~3,2%
 SET @MM=%@DATE:~0,2%
 SET @YYYY=%@DATE:~6,4%
 SET @HOUR=%@TIME:~0,2%
 SET @SUFFIX=%@TIME:~6,1%
 IF /I "%@SUFFIX%"=="A" IF %@HOUR% EQU 12 SET @HOUR=00
 IF /I "%@SUFFIX%"=="P" IF %@HOUR% LSS 12 SET /A @HOUR+=12
 SET @NOW=%@HOUR%%@TIME:~3,2%
 SET @NOW=%@NOW: =0%
 SET @TODAY=%@YYYY%%@MM%%@DD%
 SET @FILESTAMP=%@TODAY%-%@NOW%.TXT

 CALL :Display "Option #1 - No ResKit" Option1
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Get Current Date and Time                      (OPTION #2)
 rem === Updated On: 07 Nov 2010 / 07 Dec 2003              
 rem ==========================================================================
:GetDate2
 rem ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 rem NT4 or Higher -- No Resource Kit Required
 rem ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 SETLOCAL
 FOR /F "TOKENS=2-5 DELIMS=/ " %%D IN ('DATE /T') DO SET @TODAY=%%F%%D%%E
 FOR /F "TOKENS=1-3 DELIMS=:. " %%T IN ('ECHO %TIME: =0%') DO SET @NOW=%%T%%U%%V
 SET @FILESTAMP=%@TODAY%-%@NOW%.TXT

 CALL :Display "Option #2 - No ResKit" Option2
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Get Current Date and Time                      (OPTION #3)
 rem === Updated On: 07 Nov 2010 / 25 May 2005              
 rem ==========================================================================
:GetDate3
 rem ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 rem NT4 or Higher -- No Resource Kit Required
 rem ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 SETLOCAL
 FOR /F "DELIMS=" %%T IN ('TIME /T') DO SET @TIME=%%T
 FOR /F "TOKENS=2" %%D IN ('DATE /T') DO SET @DATE=%%D
 FOR /F "TOKENS=1-4 DELIMS=-/ " %%D IN ('DATE /T') DO (
	 SET @DAY=%%D
	 SET @DD=%%F
	 SET @MM=%%E
	 SET @YYYY=%%G
 )
 SET @HOUR=%@TIME:~0,2%
 SET @SUFFIX=%@TIME:~6,1%
 IF /I "%@SUFFIX%"=="A" IF %@HOUR% EQU 12 SET @HOUR=00
 IF /I "%@SUFFIX%"=="P" IF %@HOUR% LSS 12 SET /A @HOUR+=12
 SET @NOW=%@HOUR%%@TIME:~3,2%
 SET @NOW=%@NOW: =0%
 SET @TODAY=%@YYYY%%@MM%%@DD%
 SET @FILESTAMP=%@TODAY%-%@NOW%.TXT

 CALL :Display "Option #3 - No ResKit" Option3
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Get Current Date and Time (Long Method)        (OPTION #4)
 rem === Updated On: 07 Nov 2010 / 08 Jan 2007              
 rem ==========================================================================
:GetDate4
 rem ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 rem Requires NT4 or Higher, Plus Appropriate Server Resource Kit
 rem ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 SETLOCAL
 FOR /F "TOKENS=2" %%D IN ('DATE /T') DO SET @DATE=%%D
 FOR /F "TOKENS=*" %%T IN ('TIME /T') DO SET @TIME12=%%T
 FOR /F "TOKENS=1-5" %%D IN ('NOW') DO (
	 SET @DAY=%%D
	 SET @DD=%%F
	 SET @MM=%%E
	 SET @MMM=%%E
	 SET @YYYY=%%H
	 SET @TIME=%%G
	 SET @TIME24=%%G
 )

 rem -- Change Long Month (MMM) to Short Month (MM)
 SET @MM=%@MM:Jan=01%
 SET @MM=%@MM:Feb=02%
 SET @MM=%@MM:Mar=03%
 SET @MM=%@MM:Apr=04%
 SET @MM=%@MM:May=05%
 SET @MM=%@MM:Jun=06%
 SET @MM=%@MM:Jul=07%
 SET @MM=%@MM:Aug=08%
 SET @MM=%@MM:Sep=09%
 SET @MM=%@MM:Oct=10%
 SET @MM=%@MM:Nov=11%
 SET @MM=%@MM:Dec=12%

 SET @BEGMONTH=%@MM%01%@YYYY%
 SET @MMDDYYYY=%@MM%%@DD%%@YYYY%
 SET @TODAY=%@YYYY%%@MM%%@DD%
 SET @TODAY_EXP=%@YYYY%-%@MM%-%@DD%
 SET @NOW=%@TIME::=%
 SET @TIMESTAMP=%DATE% at %@TIME12%
 SET @FILESTAMP=%@TODAY%-%@NOW%.TXT

 CALL :Display "Option #4 - NT4 ResKit or Higher" Option4
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Get Current Date and Time (Short Method)       (OPTION #5)
 rem === Updated On: 07 Nov 2010 / 01 Dec 2003              
 rem ==========================================================================
:GetDate5
 rem ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 rem Requires NT4 or Higher, Plus Appropriate Server Resource Kit
 rem ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 SETLOCAL
 FOR /F "TOKENS=2-5 DELIMS=/ " %%D IN ('DATE /T') DO SET @TODAY=%%F%%D%%E
 FOR /F "TOKENS=4-6 DELIMS=: " %%T IN ('NOW') DO SET @NOW=%%T%%U%%V
 SET @FILESTAMP=%@TODAY%-%@NOW%.TXT

 CALL :Display "Option #5 - NT4 ResKit or Higher" Option5
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Get Current Date and Time (Long Method)       (OPTION #6)
 rem === Updated On: 03 Feb 2020 / 07 Feb 2016              
 rem ==========================================================================
:GetDate6
 rem ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 rem Requires Windows 2000 or Higher, Plus Resource Kit and 3rd Party Utilities
 rem ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 SETLOCAL ENABLEDELAYEDEXPANSION

 rem -- Calculate Current Date/Time (Long Procedure -- Using ResKit [Includes Day/Month names])
 rem -- Updated On: 16 Mar 2008 / 07 Jan 2007              
 FOR /F "TOKENS=2" %%D IN ('DATE /T') DO SET @DATE=%%D
 FOR /F "TOKENS=*" %%T IN ('TIME /T') DO SET @TIME12=%%T
 FOR /F "TOKENS=1-5" %%D IN ('NOW') DO (
	 SET @DAY=%%D
	 SET @MMM=%%E
	 SET @TIME=%%G
	 SET @TIME24=%%G
 )

 rem -- Calculate Current Date/Time (Long Procedure -- Using Doff.exe)
 rem -- Updated On: 27 May 2010 / 16 Mar 2008              
 FOR /F "TOKENS=1-7" %%D IN ('DOFF "yyyy yy mm dd hh mi ss"') DO (
	 SET @YYYY=%%D
	 SET @YY=%%E
	 SET @MM=%%F
	 SET @DD=%%G
	 SET @HR=%%H
	 SET @MIN=%%I
	 SET @SEC=%%J
	 SET @NOW=%%H%%I%%J
	 SET @NOW_EXP=%%H.%%I.%%J
 )

 rem -- Determine Current Day of Year (Using DateInfo.exe)
 rem -- Updated On: 07 Feb 2016 / 15 Feb 2015              
 SET @PATH=%PATH:(=[%
 SET @PATH=%@PATH:)=]%
 SET @DATEINFO$=& FOR %%V IN (%@PATH: =_%) DO IF EXIST "%%~V\DateInfo.EXE" SET @DATEINFO$="%%~V\DateInfo.EXE"
 IF DEFINED @DATEINFO$ (
	 FOR /F "TOKENS=1-2" %%D IN ('DATEINFO -s -q -f "dddd mmmm"') DO (SET @DDDD=%%D& SET @MMMM=%%E)
	 FOR /F "TOKENS=1"   %%L IN ('DATEINFO -l -q') DO SET @LEAPYEAR=%%L
	 FOR /F "TOKENS=1"   %%N IN ('DATEINFO -w -q') DO SET @DAYOFWEEK=%%N
	 FOR /F "TOKENS=1"   %%D IN ('DATEINFO -y -q') DO SET @DAYOFYEAR=%%D
 ) ELSE (
	 FOR /L %%C IN (0,1,366) DO FOR /F "TOKENS=1" %%D IN ('DOFF "yyyymmdd" -%%Cd') DO IF /I "%%D"=="%@YYYY%0101" SET @DAYOFYEAR=%%C
	 IF /I "%@DAY%"=="Sun" (SET @DAYOFWEEK=1& SET @DDDD=Sunday)
	 IF /I "%@DAY%"=="Mon" (SET @DAYOFWEEK=2& SET @DDDD=Monday)
	 IF /I "%@DAY%"=="Tue" (SET @DAYOFWEEK=3& SET @DDDD=Tuesday)
	 IF /I "%@DAY%"=="Wed" (SET @DAYOFWEEK=4& SET @DDDD=Wednesday)
	 IF /I "%@DAY%"=="Thu" (SET @DAYOFWEEK=5& SET @DDDD=Thursday)
	 IF /I "%@DAY%"=="Fri" (SET @DAYOFWEEK=6& SET @DDDD=Friday)
	 IF /I "%@DAY%"=="Sat" (SET @DAYOFWEEK=7& SET @DDDD=Saturday)
 )

 rem -- Determine EVEN or ODD Day
 rem -- Updated On: 08 Dec 2010 / 11 Nov 2010              
 SET /A @DD_DIV_2=%@DD:~-1% %% 2
 IF %@DD_DIV_2% GEQ 1 (
	 SET @DATE_IS_ODD=TRUE
	 SET @DATE_IS_EVEN=FALSE
	 SET @DATE_IS=ODD
 ) ELSE (
	 SET @DATE_IS_ODD=FALSE
	 SET @DATE_IS_EVEN=TRUE
	 SET @DATE_IS=EVEN
 )

 rem -- Calculate Future Dates (Long Procedure -- Using DOff.exe)
 rem -- Updated On: 03 Feb 2020 / 10 Apr 2017              
 SET /A @NM=32 - @DD
 FOR /F "TOKENS=1"   %%Y IN ('DOFF "yyyy"') DO SET /A @NEXTYEAR=%%Y + 1
 FOR /F "TOKENS=1-2" %%M IN ('DOFF "yyyy mm" +%@NM%') DO (SET @NEXTMONTH=%%N& SET @NEXTMONTH_EXP=%%M-%%N)
 FOR /F "TOKENS=1-3" %%W IN ('DOFF "yyyy mm dd" +7d') DO (SET @NEXTWEEK=%%W%%X%%Y& SET @NEXTWEEK_EXP=%%W-%%X-%%Y)
 FOR /F "TOKENS=1-3" %%T IN ('DOFF "yyyy mm dd" +1d') DO (SET @TOMORROW=%%T%%U%%V& SET @TOMORROW_EXP=%%T-%%U-%%V)

 rem -- Calculate Previous Dates (Long Procedure -- Using DOff.exe)
 rem -- Updated On: 06 Apr 2015 / 13 Mar 2012              
 FOR /F "TOKENS=1"   %%Y IN ('DOFF "yyyy"') DO SET /A @LASTYEAR=%%Y - 1
 FOR /F "TOKENS=1-2" %%L IN ('DOFF "yyyy mm" -%@DD%') DO (SET @LASTMONTH=%%M& SET @LASTMONTH_EXP=%%L-%%M)
 FOR /F "TOKENS=1-3" %%W IN ('DOFF "yyyy mm dd" -7d') DO (SET @LASTWEEK=%%W%%X%%Y& SET @LASTWEEK_EXP=%%W-%%X-%%Y)
 FOR /F "TOKENS=1-3" %%D IN ('DOFF "yyyy mm dd" -1d') DO (SET @YESTERDAY=%%D%%E%%F& SET @YESTERDAY_EXP=%%D-%%E-%%F)

 SET @BEGMONTH=%@MM%01%@YYYY%
 SET @MMDDYYYY=%@MM%%@DD%%@YYYY%
 SET @TODAY=%@YYYY%%@MM%%@DD%
 SET @TODAY_EXP=%@YYYY%-%@MM%-%@DD%
 SET @ZIPSTAMP=%@TODAY%-%@NOW%.ZIP
 SET @FILESTAMP=%@TODAY%-%@NOW%.TXT
 SET @TIMESTAMP=%@DATE:/=-% at %@TIME12::=.%
 SET @TIMESTAMP24=%@DATE:/=-% at %TIME: =0%

 CALL :Display "Option #6 - Windows 2000+, Plus Resource Kit / 3rd Party Utilities" Option6
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Get Complete Date/Time Info (Using DateInfo)   (OPTION #7)
 rem === Updated On: 15 Feb 2019 / 14 May 2018              
 rem ==========================================================================
:GetDate7
 rem ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 rem Requires Windows 2003 or Higher with BrainWave DateInfo.exe Utility
 rem ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 SETLOCAL ENABLEDELAYEDEXPANSION

 CALL :GetDateInfoValues

 CALL :Display "Option #7 - Windows 2003+ with BrainWave DateInfo Utility" Option7
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Single Pass DateIinfo (v4.0+)                  (OPTION #8)
 rem === Updated On: 15 Feb 2019 / 21 Dec 2018              
 rem ==========================================================================
:GetDate8
 rem ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 rem Requires Windows 2003 or Higher with BrainWave DateInfo.exe v4.0+
 rem ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 SETLOCAL ENABLEDELAYEDEXPANSION

 SET @APP_ARCH=
 FOR /F "TOKENS=1-2 DELIMS=;" %%P IN ('DATEINFO --Script-Variables 2^>NUL') DO SET %%P=%%~Q
 IF DEFINED @APP_ARCH (
	 CALL :Display "Option #8 - Windows with BrainWave DateInfo v4.1+ [--Script-Variables]" Option8a
 ) ELSE (
	 CALL :GetDateInfoValues
	 CALL :Display "Option #8 - Windows with BrainWave DateInfo Utility [Old Method]" Option8b
 )
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Gather DATEINFO calculations, individually
 rem === Updated On: 03 Feb 2020 / 15 Feb 2019              
 rem ==========================================================================
:GetDateInfoValues

 rem -- Generate Unique 5-Digit Random Number via DATEINFO.exe (or %RANDOM%)
 rem -- Updated On: 14 May 2018 / 22 Feb 2017              
 SET @RANDOM=123%RANDOM%%RANDOM%& FOR /F %%R IN ('DATEINFO -R -Q') DO SET @RANDOM=%%R
 IF DEFINED @RANDOM (SET @RANDOM=%@RANDOM:~-5%) ELSE (SET @RANDOM=%RANDOM%)

 rem -- Obtain Current Date/Time Info via DateInfo.exe (12-Hour Format)
 rem -- Updated On: 14 May 2018                     
 FOR /F "TOKENS=1*" %%D IN ('DATEINFO -s -f "mm/dd/yyyy hh:nn:ss am/pm" -q') DO (
	 SET @DATE=%%D
	 SET @TIME12=%%E
 )

 rem -- Obtain Current Date/Time Info via DateInfo.exe (24-Hour Format)
 rem -- Updated On: 14 May 2018 / 05 Apr 2017              
 FOR /F "TOKENS=1-12" %%D IN ('DATEINFO -s -f "yyyy yy mmmm mmm mm dddd ddd dd hh nn ss zz" -q') DO (
	 SET @YYYY=%%D
	 SET @YY=%%E
	 SET @MMMM=%%F
	 SET @MMM=%%G
	 SET @MM=%%H
	 SET @DDDD=%%I
	 SET @DDD=%%J
	 SET @DAY=%%J
	 SET @DD=%%K
	 SET @HR=%%L
	 SET @MIN=%%M
	 SET @SEC=%%N
	 SET @MSEC=%%O
	 SET @TIME=%%L:%%M:%%N
	 SET @TIME24=%%L:%%M:%%N
	 SET @NOW=%%L%%M%%N
	 SET @NOW_EXP=%%L.%%M.%%N
 )

 rem -- Calculate Special Date/Time Info (Using DateInfo.exe)
 rem -- Updated On: 10 Apr 2017 / 05 Apr 2017/02 Feb 2016              
 FOR /F "TOKENS=1" %%L IN ('DATEINFO -l -q') DO SET @LEAPYEAR=%%L
 FOR /F "TOKENS=1" %%N IN ('DATEINFO -w -q') DO SET @DAYOFWEEK=%%N
 FOR /F "TOKENS=1" %%D IN ('DATEINFO -y -q') DO SET @DAYOFYEAR=%%D
 FOR /F "TOKENS=1" %%E IN ('DATEINFO -m -q') DO SET @ENDOFMONTH=%%E

 rem -- Calculate Past/Future Date/Time Info (Using DateInfo.exe)
 rem -- Updated On: 03 Feb 2020 / 10 Apr 2017              
 SET /A @NM=32 - @DD
 SET /A @LASTYEAR=@YYYY - 1
 SET /A @NEXTYEAR=@YYYY + 1
 FOR /F "TOKENS=1-2" %%L IN ('DATEINFO -s -p %@DD% -f "yyyy mm" -q') DO (SET @LASTMONTH=%%M&       SET @LASTMONTH_EXP=%%L-%%M)
 FOR /F "TOKENS=1-2" %%M IN ('DATEINFO -s -a %@NM% -f "yyyy mm" -q') DO (SET @NEXTMONTH=%%N&       SET @NEXTMONTH_EXP=%%M-%%N)
 FOR /F "TOKENS=1-3" %%W IN ('DATEINFO -s -p 7 -f "yyyy mm dd" -q') DO  (SET @LASTWEEK=%%W%%X%%Y&  SET @LASTWEEK_EXP=%%W-%%X-%%Y)
 FOR /F "TOKENS=1-3" %%W IN ('DATEINFO -s -a 7 -f "yyyy mm dd" -q') DO  (SET @NEXTWEEK=%%W%%X%%Y&  SET @NEXTWEEK_EXP=%%W-%%X-%%Y)
 FOR /F "TOKENS=1-3" %%T IN ('DATEINFO -s -p 1 -f "yyyy mm dd" -q') DO  (SET @YESTERDAY=%%T%%U%%V& SET @YESTERDAY_EXP=%%T-%%U-%%V)
 FOR /F "TOKENS=1-3" %%T IN ('DATEINFO -s -a 1 -f "yyyy mm dd" -q') DO  (SET @TOMORROW=%%T%%U%%V&  SET @TOMORROW_EXP=%%T-%%U-%%V)

 rem -- Determine EVEN or ODD Day
 rem -- Updated On: 08 Dec 2010 / 11 Nov 2010              
 SET /A @DD_DIV_2=%@DD:~-1% %% 2
 IF %@DD_DIV_2% GEQ 1 (
	 SET @DATE_IS_ODD=TRUE
	 SET @DATE_IS_EVEN=FALSE
	 SET @DATE_IS=ODD
 ) ELSE (
	 SET @DATE_IS_ODD=FALSE
	 SET @DATE_IS_EVEN=TRUE
	 SET @DATE_IS=EVEN
 )

 rem -- Generate TimeStamp Variables
 rem -- Updated On: 14 May 2018 / 05 Apr 2017 / 22 Feb 2017              
 SET @BEGMONTH=%@MM%01%@YYYY%
 SET @MMDDYYYY=%@MM%%@DD%%@YYYY%
 SET @TODAY=%@YYYY%%@MM%%@DD%
 SET @TODAY_EXP=%@YYYY%-%@MM%-%@DD%
 SET @ZIPSTAMP=%@TODAY%-%@NOW%.ZIP
 SET @FILESTAMP=%@TODAY%-%@NOW%.TXT
 SET @TIMESTAMP=%@DATE:/=-% at %@TIME12::=.%
 SET @TIMESTAMP24=%@DATE:/=-% at %TIME: =0%
 SET @ZIPSTAMP_R=%@TODAY%-%@NOW%#%@RANDOM%.ZIP
 SET @FILESTAMP_R=%@TODAY%-%@NOW%#%@RANDOM%.TXT
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET @SCRIPT_BEG#="%%~d"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF
                      