@ECHO OFF
 rem ==========================================================================
 rem === Delete Old Files (Win2K/FORFILES Version)
 rem ==========================================================================
 rem
 rem Created On: 10 Oct 1998
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Oct 2023 / 08 Dec 2021              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2000 or later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    Resource Kit ........... https://en.wikipedia.org/wiki/Resource_Kit
 rem
 rem  Required Scripts:
 rem    SetDrive.BAT ........... Set Global Variables for this Environment
 rem ==========================================================================

 :::  This version will work on Win2K and later OSes only.  The use of FORFILES
 :::  from the ResKit makes it much smaller and faster.
 :::
 :::  Folder names containing spaces should be entered at the command line
 :::  surrounded by Quotation Marks. This is only supported in Win2K, not NT4.
 :::  Thanks to Gerald Gaston for the idea and the code. This will require less
 :::  grief (than versions A - D of DELOLD) for anyone who has folders which
 :::  contain weird characters (such as '+').
 :::
 :::  (Nov 2010) Changed the default log extension of the reports from .LOG to
 :::  .TXT so that the reports that are mailed to mobile devices can be read
 :::  automatically. (Not too many devices can natively handle .LOG files)
 :::
 :::  (Nov 2012) v1.8 -- Added the ability to use the newer, native version of
 :::  FORFILES found in the SYSTEM32 folder of Windows 2003+ systems, making
 :::  the necessary accomodations for their slightly different syntax.
 :::
 :::  (Oct 2018) v3.0 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::  -- Added new Filtered CMD Title and Syntax Help processing routines;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com;
 :::  -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Feb 2019) v3.1 -- Enabled Code Page 1252 Support;
 :::
 :::  (Oct 2019) v3.2 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (May 2020) v3.3 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::
 :::  (Nov 2020) v3.4 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Dec 2021) v3.5 -- Remove trailing \ from provided folder names;
 :::
 :::  (Oct 2023) v3.6 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 30 Nov 2020 / 25 Oct 2018              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 08 Dec 2021 / 30 Nov 2020              
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v3.6.0.370

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc --mp=1 -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- Main Variables
 SET @FOLDER=%~1& CALL :FixDirectoryPath @FOLDER
 SET @FILESPEC=%~2& IF "%~2"=="" SET @FILESPEC=*.*
 SET @AGE=%~3& IF "%~3"=="" SET @AGE=30
 SET @LOGPATH=%@STORAGE%\Logs\Maint
 SET @LOGFILE=%@LOGPATH%\%~n0.%@FILESTAMP%


 rem ==========================================================================
 rem === Recurse Through Subfolders And Delete Old Files
 rem === Updated On: 14 Nov 2012 / 20 May 2005              
 rem ==========================================================================
:SetFolders
 IF NOT EXIST "%@LOGPATH%" MD "%@LOGPATH%"
 %@PAUSE_IF_DEBUG%

 SET @DELETE_COUNT=0
 SET @FORFILES=FORFILES -p"%@FOLDER%" -m%@FILESPEC% -d-%@AGE% -s -c"CMD /C IF @ISDIR==FALSE ECHO @PATH\@FILE"
 IF EXIST "%SystemRoot%\System32\FORFILES.EXE" (
	 SET @FORFILES=%SystemRoot%\System32\FORFILES -p "%@FOLDER%" -m %@FILESPEC% -d -%@AGE% -s -c "CMD /C IF @ISDIR==FALSE ECHO @PATH"
 )

 ( rem -- Create Header
	 NOW Hunting Through "%@FOLDER%" And Subfolders For Old %@FILESPEC% Files...
	 ECHO:
 ) >"%@LOGFILE%" 2>&1

 TYPE "%@LOGFILE%"
 FOR /F "TOKENS=*" %%D IN ('%@FORFILES%') DO CALL :RemoveFiles "%%~D"

 ( rem -- Create Footer
	 ECHO:
	 NOW TOTAL %@FILESPEC% FILES DELETED: %@DELETE_COUNT%
	 ECHO:
 ) >>"%@LOGFILE%" 2>&1

 IF @DELETE_COUNT GTR 0 TYPE "%@LOGFILE%" & DIR "%@LOGFILE%"
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 08 Dec 2021 / 04 Oct 2019              
 rem ==========================================================================
:ExitBatch
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Remove Files That Are Over XX Days Old
 rem === Updated On: 14 Nov 2012 / 20 May 2005              
 rem ==========================================================================
:RemoveFiles
 rem %* = Current Filename

 SET /A @DELETE_COUNT+=1
 ECHO:                 >>"%@LOGFILE%"
 ECHO -- Purging %*...
 ECHO -- Purging %*... >>"%@LOGFILE%"
 ATTRIB -R -S -H %*
 DEL %*                >>"%@LOGFILE%"
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Remove Surrounding Quotes From Variable
 rem === Updated On: 01 Jul 2001                     
 rem ==========================================================================
:NoQuotes
 rem %1 = Current Filename

 SET %1=%~2
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Remove Trailing \ From Supplied Folder Path
 rem === Updated On: 08 Dec 2021                     
 rem ==========================================================================
:FixDirectoryPath
 rem %1 = Folder Variable

 IF NOT DEFINED %1 GOTO :EOF
 IF "!%~1:~-1!"=="\" (
	 SET %~1=!%~1:~0,-1!
	 GOTO :FixDirectoryPath
 )
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET @SCRIPT_BEG#="%%~d"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 30 Nov 2020 / 25 Oct 2018              
 rem ==========================================================================
:HelpMessage
 ECHO Delete Old Files (Win2K/FORFILES Version)
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  ^<initial_dir^> [file_spec] [age_of_files]
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0 C:\STORAGE *.LOG 10
	 ECHO  %~n0 D:\HOME *.* 90
	 ECHO  %~n0 E:\ARCHIVES OLD*.*
	 ECHO  %~n0 "F:\OLD ARCHIVES"
	 ECHO  %~n0 "G:\MISCELLANEOUS STUFF" *.OLD 45
	 ECHO  %~n0 -?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
	 ECHO:
	 ECHO  * Folder names which contain spaces should be surrounded by quotes.
	 ECHO  * Shortnames *can* be surrounded by quotes if you choose, but it is
	 ECHO    not mandatory by any means.
 )
 CALL :ShowStatus "FINISHED"
        