@ECHO OFF
 rem ==========================================================================
 rem === Disable a Service on Remote Systems
 rem ==========================================================================
 rem
 rem Created On: 28 Feb 1998
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Oct 2023 / 30 Nov 2020              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2003 and later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    Resource Kit ........... https://en.wikipedia.org/wiki/Resource_Kit
 rem
 rem  Required Scripts:
 rem    SetDrive.BAT ........... Set Global Variables for this Environment
 rem ==========================================================================

 :::  This script uses SC from the ResKit to set the startup parameters of a
 :::  service (supplied at the command line) to Disabled, and also stops it if
 :::  it's running. You must use the shortname of the service (use SCLIST to
 :::  determine the shortname of any installed service).
 :::
 :::  If no list of systems is provided at the command line, then a list will
 :::  be created of all of the machines in the domain (as determined via the
 :::  NET VIEW command).
 :::
 :::  (Nov 2010) Changed the default log extension of the reports from .LOG to
 :::  .TXT so that the reports that are mailed to mobile devices can be read
 :::  automatically. (Not too many devices can natively handle .LOG files)
 :::
 :::  (Mar 2019) v2.0 -- Enabled Code Page 1252 Support;
 :::  -- Added new Filtered CMD Title and Syntax Help processing routines;
 :::  -- Replaced FIND/FINDSTR with CHECKPARAMS for /? processing;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com;
 :::
 :::  (Oct 2019) v2.1 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (May 2020) v2.2 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::
 :::  (Nov 2020) v2.3 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Oct 2023) v2.4 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 30 Nov 2020 / 08 Mar 2019              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 30 Nov 2020 / 26 May 2020              
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v2.4.0.240

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc --mp=1 -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- Main Variables
 SET @SERVICE=%1
 SET @COMPLIST=%~2& IF /I "%~2"=="" SET @COMPLIST='NET VIEW'
 SET @LOGPATH=%@STORAGE%\Logs\Maint
 SET @LOGFILE=%@LOGPATH%\%~n0.%@FILESTAMP%
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Cycle Through Domain (or File List) And Disable Service
 rem === Updated On: 04 May 2011 / 05 Dec 2003              
 rem ==========================================================================
:GetComputers
 IF NOT EXIST "%@LOGPATH%" MD "%@LOGPATH%"

 NOW Disabling The "%@SERVICE%" Service >"%@LOGFILE%"
 ECHO Getting a List of Computers via "%@COMPLIST%"...
 ECHO:
 FOR /F %%V IN (%@COMPLIST%) DO CALL :Disable %%~V
 ECHO Finished Disabling The "%@SERVICE%" Service...
 ECHO:
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019              
 rem ==========================================================================
:ExitBatch
 TYPE "%@LOGFILE%" & DIR "%@LOGFILE%"
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Disable Designated Service and Stop It
 rem === Updated On: 08 Mar 2019 / 05 Feb 2015              
 rem ==========================================================================
:Disable
 rem %1 = Current System To Be Updated

 SET @SYS=%~1
 SET @SYS=\\%@SYS:\=%

 ( rem -- Write to Log
	 ECHO Disabling the "%@SERVICE%" service on "%@SYS%" >CON
	 ECHO Disabling the "%@SERVICE%" service on "%@SYS%"
	 SC %@SYS% QC %@SERVICE%                     >NUL
	 SC %@SYS% CONFIG %@SERVICE% START= disabled >NUL
	 SC %@SYS% STOP %@SERVICE%                   >NUL
	 SC %@SYS% QC %@SERVICE%                     >NUL
	 SC %@SYS% QC %@SERVICE%
	 ECHO %@DIVIDER*%
	 ECHO:
 ) >>"%@LOGFILE%" 2>&1
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET @SCRIPT_BEG#="%%~d"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 30 Nov 2020 / 08 Mar 2019              
 rem ==========================================================================
:HelpMessage
 ECHO Disable a Service on Remote Systems...
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  ^<service^> [list_of_servers.txt]
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:
 ECHO -------------------
 ECHO OPTION DEFINITIONS:
 ECHO -------------------
 ECHO  -?,  /H ..... Display This Help Message  (also --HELP)
 ECHO  -??, /H2 .... Display Extended Help Message
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0 Browser
	 ECHO  %~n0 LicenseService C:\TEMP\SERVERS.TXT
	 ECHO  %~n0 LicenseService "%~dp0Input\ServerList.TXT"
	 ECHO  %~n0 Schedule "'NET VIEW /DOMAIN:my_domain'"
	 ECHO  %~n0 -h
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
	 ECHO:
	 ECHO  * Any parameters which contain spaces should be surrounded by quotes.
	 ECHO    Shortnames *can* be surrounded by quotes if you choose, but it is not
	 ECHO    mandatory by any means.
	 ECHO:
	 ECHO  * Computer names in the list should ideally be prefaced by \\
 )
 CALL :ShowStatus "FINISHED"
                                                                                 