@ECHO OFF
 rem ==========================================================================
 rem === Obtain Local Disk Information using PSINFO, SRVINFO and COMPINFO
 rem ==========================================================================
 rem
 rem Created On: 27 Oct 2003
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Oct 2023 / 03 Jan 2021              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: NT4 or later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    BeyondExecv2 ........... http://www.beyondlogic.org/solutions/remoteprocess/BeyondExec.htm (no longer available)
 rem    PAEXEC ................. https://www.poweradmin.com/paexec/
 rem    PSTOOLS ................ https://technet.microsoft.com/en-us/sysinternals/bb896649
 rem    COMPINFO ............... http://www.beyondlogic.org/solutions/compinfo/compinfo.htm (no longer available)
 rem    SRVINFO ................ http://www.microsoft.com/en-us/download/details.aspx?id=17657
 rem
 rem  Required Scripts:
 rem    SetDrive.BAT ........... Set Global Variables for this Environment
 rem ==========================================================================

 :::  This script obtains the disk information from the target server, with or
 :::  without the \\ prefix.  It displays total, free and used space, and also
 :::  displays a message if any volume is below 10 percent free space.
 :::
 :::  (Oct 2013) v3.0 -- Added disk capture metrics via COMPINFO, and provided
 :::  options for skipping over SRVINFO or PSINFO output.
 :::
 :::  (Apr 2015) v3.3 -- Updated :ShowStatus routine to track script duration.
 :::
 :::  (Feb 2018) v4.0 -- Added proper Randomization function from DATEINFO.exe
 :::  -- Updated Script Layout, Error Checking, and Version Info
 :::  -- Added filtered CMD Title Routine.
 :::
 :::  (Apr 2018) v4.1 -- The SYNTAX HELP routine has been updated.
 :::
 :::  (Jul 2018) v4.2 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::
 :::  (Oct 2018) v4.3 -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Oct 2019) v4.4 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::  -- Replace the use of PSEXEC with PAEXEC;
 :::
 :::  (Oct 2019) v4.5 -- Update :PadLeft and :PadRight routines;
 :::
 :::  (Jan 2020) v4.6 -- Consolidate :PadLeft and :PadRight routines;
 :::
 :::  (May 2020) v4.7 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::
 :::  (Nov 2020) v4.8 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Jan 2021) v5.0 -- Allow the enabling or disabling of specific displays
 :::  such as SRVINFO, PSINFO and COMPINFO;
 :::
 :::  (Oct 2023) v5.1 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 30 Nov 2020 / 25 Feb 2018              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 03 Jan 2021 / 30 Nov 2020              
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v5.1.0.320

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- Get Low Value
 SET /A @LOWDISK=@LOWDISK
 IF 1!@LOWDISK! LSS 110 SET @LOWDISK=10

 rem -- Process Special Command-line Parameters
 SET @ALL_PARAMS=%*
 CALL :GetParams "C+ C" & IF DEFINED @OK (SHIFT& SET @USE_COMPINFO=TRUE)
 CALL :GetParams "S+ S" & IF DEFINED @OK (SHIFT& SET @USE_SRVINFO=TRUE)
 CALL :GetParams "P+ P" & IF DEFINED @OK (SHIFT& SET @USE_PSINFO=TRUE)
 CALL :GetParams "C-  " & IF DEFINED @OK (SHIFT& SET @USE_COMPINFO=FALSE)
 CALL :GetParams "S-  " & IF DEFINED @OK (SHIFT& SET @USE_SRVINFO=FALSE)
 CALL :GetParams "P-  " & IF DEFINED @OK (SHIFT& SET @USE_PSINFO=FALSE)

 rem -- Main Variables
 SET @CONNECT=%~1& IF "%~1"=="" SET @CONNECT=%COMPUTERNAME%
 SET @CONNECT=%@CONNECT:\=%
 SET @LOGTEMP=%TEMP%\%@CONNECT%.#%@RANDOM%.TMP
 SET @RMTCONSOLE=%@REXEC_UTIL% \\%@CONNECT%& IF "%~1"=="" SET @RMTCONSOLE=
 SET @DRIVEINFO=& FOR %%E IN (PS SRV COMP) DO IF /I NOT "!@USE_%%EINFO!"=="FALSE" SET @DRIVEINFO=%%EINFO !@DRIVEINFO!


 rem ==========================================================================
 rem === Get Drive Information
 rem === Updated On: 10 Oct 2013                     
 rem ==========================================================================
:GetDriveInfo
 ECHO Getting Drive Info via %@DRIVEINFO%
 IF /I NOT "%@USE_COMPINFO%"=="FALSE" CALL :FindDrives-C
 IF /I NOT "%@USE_SRVINFO%"=="FALSE"  CALL :FindDrives-S
 IF /I NOT "%@USE_PSINFO%"=="FALSE"   CALL :FindDrives-P


 rem ==========================================================================
 rem === Generate Summary Info for Drive Info Job
 rem === Updated On: 21 Apr 2015 / 10 Oct 2013              
 rem ==========================================================================
:FinishLog
 CALL :GetTime @ETIME
 ECHO:
 ECHO %@MINORDIV%
 ECHO  *** DISK SPACE INFORMATION for \\%@CONNECT% via %@VER% ***
 ECHO %@MINORDIV%
 IF DEFINED @CTIME ECHO  COMPINFO Query Started On ..................... %@CTIME%
 IF DEFINED @STIME ECHO  SRVINFO Query Started On ...................... %@STIME%
 IF DEFINED @PTIME ECHO  PSINFO Query Started On ....................... %@PTIME%
 ECHO %@MINORDIV%


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019              
 rem ==========================================================================
:ExitBatch
 IF EXIST "%@LOGTEMP%" DEL "%@LOGTEMP%" >NUL
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Obtain Drive Info via COMPINFO
 rem === Updated On: 22 Feb 2018 / 25 Nov 2014              
 rem ==========================================================================
:FindDrives-C
 CALL :GetTime @CTIME
 ECHO:
 ECHO  %@CONNECT% - COMPINFO Disk Stats
 ECHO %@MINORDIV%
 ECHO  Drv Type     Format Label                      Total Size     Free Space  Used
 ECHO %@MINORDIV%

 %@RMTCONSOLE% COMPINFO 2>NUL | FIND ":\ " >"%@LOGTEMP%" 2>&1
 FOR /F "TOKENS=1-2* DELIMS=\ " %%C IN ('TYPE "%@LOGTEMP%"') DO (
	 SET @VOL=
	 SET @ALL=%%D %%E
	 SET @INFO=%%E
	 FOR /L %%n IN (0,1,100) DO IF NOT DEFINED @VOL IF "!@INFO:~%%n,4!"=="    " (
		 SET @VOL=!@INFO:"=!
		 SET @VOL=!@VOL:~0,%%n!
		 FOR /F "TOKENS=1-5 DELIMS= " %%O IN ('ECHO !@INFO:~%%n!') DO (
			 CALL :PadRight @dTY "%%D"    " " 7
			 CALL :PadRight @dLB "!@VOL!" " " 21
			 CALL :PadRight @dFM "%%O"    " " 5
			 CALL :PadRight @dSN "%%P"    " " 10
			 CALL :PadLeft  @dFS "%%Q"    " " 14
			 CALL :PadLeft  @dTS "%%R"    " " 14
			 CALL :PadLeft  @dUS "%%S"    " " 4
			 SET @PCT=%%& IF "%%~S"=="" SET @PCT=
			 SET @ALL=!@dTY!  !@dFM!  !@dLB!  !@dTS! !@dFS! !@dUS!!@PCT!
		 )
	 )
	 ECHO   %%C !@ALL!
 )
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Obtain Drive Info via SRVINFO -ns
 rem === Updated On: 10 Oct 2013 / 30 May 2006              
 rem ==========================================================================
:FindDrives-S
 CALL :GetTime @STIME
 ECHO:
 ECHO:
 ECHO  %@CONNECT% - SRVINFO Disk Stats
 ECHO %@MINORDIV%
 ECHO [Drive] [FileSys]  [ Size ]  [ Free ]  [ Used ]  [Resource Type]
 ECHO %@MINORDIV%

 SRVINFO -NS \\%@CONNECT% | FIND "$" >"%@LOGTEMP%" 2>&1
 FOR /F "TOKENS=* DELIMS=" %%S IN ('TYPE "%@LOGTEMP%" ^| FIND "$" ^| FIND /V "\"') DO ECHO %%S
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Obtain Drive Info via PSINFO -d
 rem === Updated On: 22 Feb 2018 / 10 Oct 2013              
 rem ==========================================================================
:FindDrives-P
 CALL :GetTime @PTIME
 ECHO:
 ECHO:
 ECHO  %@CONNECT% - PSINFO Disk Stats
 ECHO %@MINORDIV%
 ECHO Volume Type       Format     Label Name            Disk Size  Disk Free  %%Free
 ECHO %@MINORDIV%

 PSINFO -accepteula -D \\%@CONNECT% : >"%@LOGTEMP%" 2>&1
 FOR /F "TOKENS=* DELIMS=" %%P IN ('TYPE "%@LOGTEMP%" ^| FIND "%%" ^| FINDSTR "NTFS FAT CDFS"') DO ECHO %%P

 rem -- Report Low Disk Space (less than 10% Free [default])
:ReportLowSpace
 FOR /F "TOKENS=* DELIMS=" %%D IN ('TYPE "%@LOGTEMP%" ^| FIND /I "Fixed" ^| FINDSTR "NTFS FAT CDFS"') DO (
	 SET @CHECKIT=%%D
	 SET @CHECKIT=!@CHECKIT:"=!
	 CALL :GetFreeSpace %@CONNECT% "!@CHECKIT!"
 )
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Generate Error If Free Space Less Than 10%
 rem === Updated On: 22 Feb 2018 / 30 May 2006              
 rem ==========================================================================
:GetFreeSpace
 rem %1 = Current Server For Which Stats Will Be Obtained
 rem %2 = Current Drive Stats, Include Drive Letter and Free Space

 SET @STATS=%~2
 IF NOT DEFINED @STATS GOTO :EOF
 SET @DRIVE=%@STATS:~4,2%
 FOR /F "TOKENS=1 DELIMS=. " %%I IN ('ECHO !@STATS:~-6!') DO SET /A @FREE=%%I

 IF %@FREE% LSS %@LOWDISK% ECHO:& ECHO *** DISK SPACE ON %1\%@DRIVE% HAS FALLEN BELOW !@LOWDISK!%% (~%@FREE%%%) ***
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Store Current Date and Time in Variable
 rem === Updated On: 16 Oct 2023 / 21 Apr 2015              
 rem ==========================================================================
:GetTime
 rem %1 = Variable to Store Date/Time

 SET %~1=!DATE! at !TIME: =0!
 FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET %~1#="%%~d"
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Use CheckParams.exe to Capture Selected Parameters
 rem === Updated On: 03 Jan 2021                     
 rem ==========================================================================
:GetParams
 rem %1 = Parameters to Search For
 rem %2 = Variable to Set

 SET @OK=& IF "%~1"=="" GOTO :EOF
 FOR /F "TOKENS=2-3*" %%v IN ('CHECKPARAMS -q -a -c "%~1" -s %@ALL_PARAMS% 2^>NUL') DO IF /I "%%~v"=="TRUE" (
	 SET @OK=T
	 SET @REQ=T
	 IF NOT "%~2"=="" SET %~2=%%~x
 )
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Pad Variable Content to the Left/Right for Display in Logs
 rem === Updated On: 22 Jan 2020 / 14 Oct 2019              
 rem ==========================================================================
:PadLeft
:PadRight
 rem %1 = Variable to Store New Content
 rem %2 = Existing Value to Pad
 rem %3 = Character to use as padding [typically " ", "." or "0"]
 rem %4 = New String Length

 SET @STR=%~2
 SET @LEN=& FOR /L %%c IN (0,1,199) DO IF "!@STR:~%%c,1!"=="" IF NOT DEFINED @LEN SET @LEN=%%c
 IF 1%@LEN% GEQ 1%~4 (
	 SET %~1=%~2
 ) ELSE (
	 IF /I "%~0"==":PadLeft" (
		 SET @TEMPVAR=& FOR /L %%c IN (1,1,%~4) DO SET @TEMPVAR=!@TEMPVAR!%~3
		 SET @TEMPVAR=!@TEMPVAR!%~2
		 SET %~1=!@TEMPVAR:~-%~4!
	 )
	 IF /I "%~0"==":PadRight" (
		 SET @TEMPVAR=%~2& FOR /L %%c IN (1,1,%~4) DO SET @TEMPVAR=!@TEMPVAR!%~3
		 SET %~1=!@TEMPVAR:~0,%~4!
	 )
 )
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" CALL :GetTime @SCRIPT_BEG
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 03 Jan 2021 / 30 Nov 2020              
 rem ==========================================================================
:HelpMessage
 ECHO Obtain Local Disk Information using PSINFO, SRVINFO and COMPINFO
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  ^<machine_name^>
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:
 ECHO -------------------
 ECHO OPTION DEFINITIONS:
 ECHO -------------------
 ECHO  -?,  /H ........ Display This Help Message  (also --HELP)
 ECHO  -??, /H2 ....... Display Extended Help Message
 ECHO       /C+ ....... Enable Info from COMPINFO
 ECHO       /C- ....... Disable Info from COMPINFO
 ECHO       /P+ ....... Enable Info from PSINFO
 ECHO       /P- ....... Disable Info from PSINFO
 ECHO       /S+ ....... Enable Info from SRVINFO
 ECHO       /S- ....... Disable Info from SRVINFO
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0 \\MyPC
	 ECHO  %~n0 YourPC
	 ECHO  %~n0 \\Machine1 /P-
	 ECHO  %~n0 TheServer /S- /P-
	 ECHO  %~n0 -?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
 )
 CALL :ShowStatus "FINISHED"
                               