@ECHO OFF
 rem ==========================================================================
 rem === Scan Domain Machines For Specified Running Process
 rem ==========================================================================
 rem
 rem Created On: 28 Feb 2001
 rem         By: Andrew S. Baker (adapted from a script by Danny Cahoon)
 rem
 rem Updated On: 16 Oct 2023 / 07 Jul 2021              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: NT4 or later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    ADFind ................. http://www.joeware.net/freetools/tools/adfind/
 rem    PSTOOLS ................ https://technet.microsoft.com/en-us/sysinternals/bb896649
 rem
 rem  Input Files Used By Script (stored in %SystemDrive%\Scripts\Bat\Input):
 rem    CustomVariables.TXT .... Centralized Configuration Settings for Scripts
 rem
 rem  Required Scripts:
 rem    SetDrive.BAT ........... Set Global Variables for this Environment
 rem ==========================================================================

 :::  This version uses NOW from the ResKit and PSTOOLS from the SysInternals
 :::  website (http://www.sysinternals.com/). It can be configured to run with
 :::  any other remote process enumeration tool.
 :::
 :::  (Nov 2010) Changed the default log extension of the reports from .LOG to
 :::  .TXT so that the reports that are mailed to mobile devices can be read
 :::  automatically. (Not too many devices can natively handle .LOG files)
 :::
 :::  (Apr 2018) v2.0 -- Replaced NOW.EXE with DATEINFO.EXE
 :::  -- Replaced NET VIEW with ADFIND.EXE;
 :::  -- Added new Filtered CMD Title and Syntax Help processing routines;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com
 :::
 :::  (Jul 2018) v2.1 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::
 :::  (Oct 2018) v2.2 -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Feb 2019) v2.3 -- Minor streamlining of variables related to ADFIND;
 :::
 :::  (Aug 2019) v3.0 -- Added /S to search servers only, rather than all
 :::  systems in the domain;
 :::
 :::  (Oct 2019) v3.1 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (Dec 2019) v3.2 -- Developed a drop-in replacement for Microsoft's
 :::  NOW.EXE utility, taking back that function from DATEINFO.EXE;
 :::
 :::  (Nov 2020) v3.3 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Oct 2023) v3.4 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 30 Nov 2020 / 25 Feb 2018              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 30 Nov 2020 / 26 May 2020              
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v3.4.0.340

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc --mp=1 -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- Main Variables
 SET @SERVERSONLY=
 SET @MESSAGE=
 SET @xPARAMS=-nodn -s SUB -maxe 5000 -nocsvheader -csv attr name -excldn "OBSOLETE;RETIRED;DECOMMISSIONED"
 SET @PROCESS=%~1
 SET @LOGPATH=%@STORAGE%\Logs\Maint
 SET @LOGFILE=%@LOGPATH%\%~n0.%@FILESTAMP%

 rem -- Process Special Command-line Parameters
 SET @ALL_PARAMS=%*
 CALL :GetParams "S" & IF DEFINED @OK (SET @SERVERSONLY=TRUE& SET @MESSAGE={Servers Only})

 rem -- Determine Whether to Search for All Computers or Just Servers
 IF /I "%@SERVERSONLY%"=="TRUE" (
	 SET @ADQUERY=ADFind -f "(&(objectcategory=computer)(|(operatingSystem=Windows*Server*)(operatingSystem=Windows*Storage*)))" %@xPARAMS%
 ) ELSE (
	 SET @ADQUERY=ADFind -f "(&(objectcategory=computer)(operatingSystem=Windows*))" %@xPARAMS%
 )


 rem ==========================================================================
 rem === Scan Domain Machines For Specified Running Process
 rem === Updated On: 07 Jul 2021 / 13 Dec 2019              
 rem ==========================================================================
:GetProcess
 IF NOT EXIST "%@LOGPATH%" MD "%@LOGPATH%"
 ECHO Searching Domain for '%@PROCESS%' Process...    Please Wait...    %@MESSAGE%
 ECHO:

 ( rem -- Write to Log
	 FOR /F "DELIMS=," %%C IN ('%@ADQUERY%') DO (
		 SET @MACHINE=\\%%~C
		 SET @SYSTEM= %%~C %@DOTS%

		 rem -- Start "Remote Registry" service on !@MACHINE! if not running
		 FOR /F "TOKENS=4" %%S IN ('SC !@MACHINE! QUERY "RemoteRegistry" ^| FIND "STATE"') DO (
			 IF NOT "%%~S"=="RUNNING" (
				 ECHO Starting Remote Registry Service on !@MACHINE!...
				 SC !@MACHINE! CONFIG RemoteRegistry START= auto
				 SC !@MACHINE! START RemoteRegistry
				 ECHO:
			 ) >CON
		 )

		 FOR /F "TOKENS=*" %%P IN ('PSLIST !@MACHINE! "%@PROCESS%" ^| FIND /I "%@PROCESS%"') DO ECHO !@SYSTEM:~0,20!.. %%P
		 ECHO:
	 )

	 NOW "\n\v -- Domain Search for '%@PROCESS%' Process Completed.\n"
 ) >"%@LOGFILE%"

 rem -- Final Report
 TYPE "%@LOGFILE%" & DIR "%@LOGFILE%"


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019              
 rem ==========================================================================
:ExitBatch
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Use CheckParams.exe to Capture Selected Parameters
 rem === Updated On: 21 Jan 2020 / 24 Sep 2019              
 rem ==========================================================================
:GetParams
 rem %1 = Parameters to Search For
 rem %2 = Variable to Set

 SET @OK=& IF "%~1"=="" GOTO :EOF
 FOR /F "TOKENS=2-3*" %%v IN ('CHECKPARAMS -q -a -c "%~1" -s %@ALL_PARAMS% 2^>NUL') DO IF /I "%%~v"=="TRUE" (
	 SET @OK=T
	 SET @REQ=T
	 IF NOT "%~2"=="" SET %~2=%%~x
 )
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET @SCRIPT_BEG#="%%~d"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 30 Nov 2020 / 29 Aug 2019              
 rem ==========================================================================
:HelpMessage
 ECHO Scan Domain Machines For Specified Running Process
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  ^<process_name^>  [OPTIONS]
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:
 ECHO -------------------
 ECHO OPTION DEFINITIONS:
 ECHO -------------------
 ECHO  -?,  /H ..... Display This Help Message  (also --HELP)
 ECHO  -??, /H2 .... Display Extended Help Message
 ECHO       /S ..... Search on Servers Only on the Process
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0 OUTLOOK
	 ECHO  %~n0 MSFTPSVC /S
	 ECHO  %~n0 "LONG NAME"
	 ECHO  %~n0 -?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * Process Names which contain spaces must be surrounded by quotation
	 ECHO    marks.
 )
 CALL :ShowStatus "FINISHED"
            