@ECHO OFF
 rem ==========================================================================
 rem === Configure Computer Browser Service Against Remote Systems
 rem ==========================================================================
 rem
 rem Created On: 28 Feb 1998
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Oct 2023 / 30 Nov 2020              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2003 and later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    Resource Kit ........... https://en.wikipedia.org/wiki/Resource_Kit
 rem
 rem  Required Scripts:
 rem    SetDrive.BAT ........... Set Global Variables for this Environment
 rem ==========================================================================

 :::  This script uses SC from the ResKit to set the startup parameters of a
 :::  service (supplied at the command line) to Disabled, and also stops it if
 :::  it's running. You must use the shortname of the service (use SCLIST to
 :::  determine the shortname of any installed service).
 :::
 :::  If no list of systems is provided at the command line, then a list will
 :::  be created of all of the machines in the domain (as determined via the
 :::  NET VIEW command).
 :::
 :::  (Mar 2019) v3.0 -- Added filtered CMD Title Routine;
 :::  -- Replaced FIND/FINDSTR with CHECKPARAMS for /? processing;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com;
 :::  -- The SYNTAX HELP routine has been updated;
 :::  -- Enabled Code Page 1252 Support;
 :::
 :::  (Oct 2019) v3.1 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (May 2020) v3.2 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::
 :::  (Nov 2020) v3.3 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Oct 2023) v3.4 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 30 Nov 2020 / 08 Mar 2019              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 30 Nov 2020 / 26 May 2020              
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v3.4.0.340

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc --mp=1 -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- Main Variables
 SET @SERVICE=BROWSER
 FOR %%P IN (%*) DO CALL :ParseParams %%P
 IF /I "%@ENABLE%"==""  SET @ENABLE="%~dp0Input\EnableBrowseList.TXT"
 IF /I "%@DISABLE%"=="" SET @DISABLE="%~dp0Input\DisableBrowseList.TXT"

 rem -- Logging Variables
 SET @LOGPATH=%@STORAGE%\Logs\Maint
 SET @LOGFILE=%@LOGPATH%\%~n0.%@FILESTAMP%
 SET @SKIP-EM=%@LOGPATH%\EXEMPT-%~n0.%@FILESTAMP%
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Cycle Through List of Systems to Enable the Browser Service
 rem === Updated On: 24 Aug 2004                     
 rem ==========================================================================
:MakeExemptions
 IF NOT EXIST "%@LOGPATH%" MD "%@LOGPATH%"

 ECHO:>"%@SKIP-EM%"
 ECHO Creating List of Exemptions...
 ECHO  - %@ENABLE%
 ECHO:
 FOR /F %%V IN ('TYPE %@ENABLE%') DO ECHO %%V >>"%@SKIP-EM%"
 ECHO Finished Creating Exemption List...
 ECHO:
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Cycle Through Domain (or File List) And Disable Service
 rem === Updated On: 24 Aug 2004                     
 rem ==========================================================================
:GetComputers
 NOW Disabling The "%@SERVICE%" Service in %@DOMAIN%... >"%@LOGFILE%"
 ECHO Getting a List of Computers in %@DOMAIN%...
 ECHO  - %@DISABLE%
 ECHO:
 FOR /F %%V IN ('TYPE %@DISABLE%') DO CALL :Disable %%V

 NOW Enabling The "%@SERVICE%" Service in %@DOMAIN%... >>"%@LOGFILE%"
 FOR /F %%V IN ('TYPE %@SKIP-EM%') DO CALL :Enable %%V
 ECHO Finished Disabling The "%@SERVICE%" Service...
 ECHO:
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019              
 rem ==========================================================================
:ExitBatch
 TYPE "%@LOGFILE%" & DIR "%@LOGFILE%"
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Disable Designated Service and Stop It
 rem === Updated On: 08 Mar 2019 / 05 Feb 2015              
 rem ==========================================================================
:Disable
 rem %1 = Current System To Be Updated

 SET @SYS=%~1
 SET @SYS=\\%@SYS:\=%
 FOR /F %%E IN (%@SKIP-EM%) DO IF /I "%@SYS%"=="%%~E" GOTO :EOF
 FOR /F %%E IN (%@SKIP-EM%) DO IF /I "%@SYS%"=="\\%%~E" GOTO :EOF

 ( rem -- Write to Log
	 ECHO Disabling the "%@SERVICE%" service on "%@SYS%" >CON
	 ECHO Disabling the "%@SERVICE%" service on "%@SYS%"
	 SC %@SYS% QC %@SERVICE%                     >NUL
	 SC %@SYS% CONFIG %@SERVICE% START= disabled >NUL
	 SC %@SYS% STOP %@SERVICE%                   >NUL
	 SC %@SYS% QC %@SERVICE%                     >NUL
	 SC %@SYS% QC %@SERVICE%
	 ECHO %@DIVIDER*%
	 ECHO:
 ) >>"%@LOGFILE%" 2>&1
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Enable Designated Service and Start It
 rem === Updated On: 05 Feb 2015 / 23 Oct 2013              
 rem ==========================================================================
:Enable
 rem %1 = Current System To Be Updated

 SET @SYS=%~1
 SET @SYS=\\%@SYS:\=%
 FOR %%S IN (The List) DO IF "%@SYS%"=="\\%%~S" GOTO :EOF
 FOR /F %%E IN (%@SKIP-EM%) DO IF /I "%@SYS%"=="%%~E" GOTO :EOF

 ( rem -- Write to Log
	 ECHO Enabling the "%@SERVICE%" service on "%@SYS%" >CON
	 ECHO Enabling the "%@SERVICE%" service on "%@SYS%"
	 SC %@SYS% QC %@SERVICE%                 >NUL
	 SC %@SYS% CONFIG %@SERVICE% START= auto >NUL
	 SC %@SYS% START %@SERVICE%              >NUL
	 SC %@SYS% QC %@SERVICE%                 >NUL
	 SC %@SYS% QC %@SERVICE%
	 ECHO %@MAJORDIV%
	 ECHO:
 ) >>"%@LOGFILE%" 2>&1
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Parse Commandline Parameters
 rem === Updated On: 08 Mar 2019 / 23 Oct 2013              
 rem ==========================================================================
:ParseParams
 rem %1 = Current Command Line Parameter

 IF "%~1"=="" GOTO :EOF
 SET @PARAM=%~1
 FOR %%d IN (- /) DO (
	 IF /I "%@PARAM:~0,3%"=="%%~dD:" SET @DISABLE=%@PARAM:~3,99%
	 IF /I "%@PARAM:~0,3%"=="%%~dE:" SET @ENABLE=%@PARAM:~3,99%
 )
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET @SCRIPT_BEG#="%%~d"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 30 Nov 2020 / 08 Mar 2019              
 rem ==========================================================================
:HelpMessage
 ECHO Configure Computer Browser Service Against Remote Systems
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  ^</D:list_to_disable.txt^> [/E:list_to_enable.txt]
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:
 ECHO -------------------
 ECHO OPTION DEFINITIONS:
 ECHO -------------------
 ECHO  -?,  /H ..... Display This Help Message  (also --HELP)
 ECHO  -??, /H2 .... Display Extended Help Message
 ECHO       /D ..... List of Systems to have the service disabled
 ECHO       /E ..... List of Systems to have the service enabled
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0 /D:C:\TEMP\WORKSTATIONS.TXT /E:C:\TEMP\SERVERS.TXT
	 ECHO  %~n0 /D:%~dp0Input\ServerList.TXT
	 ECHO  %~n0 -?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
	 ECHO:
	 ECHO  * Any parameters which contain spaces should be surrounded by quotes.
	 ECHO    Shortnames *can* be surrounded by quotes if you choose, but it is not
	 ECHO    mandatory by any means.
	 ECHO:
	 ECHO  * Computer names in the list should ideally be prefaced by \\
 )
 CALL :ShowStatus "FINISHED"
                                                                  