@ECHO OFF
 rem ==========================================================================
 rem === Generate Updated INPUT Lists of Windows Servers from Active Directory
 rem ==========================================================================
 rem
 rem Created On: 12 Mar 2017 - 01 May 2017
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Oct 2023 / 30 Nov 2020              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2008 and later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    ADFind ................. http://www.joeware.net/freetools/tools/adfind/
 rem    CMSort ................. http://www.chmaas.handshake.de/delphi/freeware/cmsort/cmsort.htm
 rem
 rem  Input Files Used By Script (stored in %SystemDrive%\Scripts\Bat\Input):
 rem    CustomVariables.TXT .... Centralized Configuration Settings for Scripts
 rem
 rem  Required Scripts:
 rem    SetDrive.BAT ........... Set Global Variables for this Environment
 rem ==========================================================================

 :::  This script grabs the most current server list from Active Directory and
 :::  checks to see which systems are live, and what their configuration stats
 :::  are, so that it can populate the server INPUT list files for the script
 :::  repository.  This script is derived from several existing scripts,
 :::  including: GetServerIPs.BAT and MakeSystemsList.BAT.
 :::
 :::  (May 2017) v1.0 -- Initial Deployment
 :::
 :::  (Oct 2018) v2.0 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::  -- Added new Filtered CMD Title and Syntax Help processing routines;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com;
 :::  -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Feb 2019) v2.1 -- Enabled Code Page 1252 Support;
 :::
 :::  (Oct 2019) v2.2 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (Oct 2019) v2.3 -- Update :PadLeft and :PadRight routines;
 :::
 :::  (Jan 2020) v2.4 -- Consolidate :PadLeft and :PadRight routines;
 :::
 :::  (May 2020) v2.5 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::
 :::  (Nov 2020) v2.6 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Oct 2023) v2.7 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 30 Nov 2020 / 25 Oct 2018              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 30 Nov 2020 / 26 May 2020              
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v2.7.0.280

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- Default Script Variables
 SET @OBSOLETE=OBSOLETE;RETIRED;DECOMMISSIONED
 SET @SEARCHDOMAIN=%USERDNSDOMAIN%
 SET @BASEDN=& CALL :GetDomainName %~1
 SET @RESOLVE=%~dp0..\Utils\Resolve.EXE
 SET @ADFIND=%~dp0..\Utils\ADFind.EXE
 SET @QUERY1="(&(objectcategory=computer)(|(operatingSystem=Windows*Server*)(operatingSystem=Windows*Storage*)))"
 SET @QUERY2="(&(objectcategory=computer)(operatingSystem=Windows*))"
 SET @MYQUERY=%@QUERY1%
 SET @NOIP={No-IP-Found}
 SET @NOVAL=
 SET @L_TOTAL=0
 SET @D_TOTAL=0
 SET @TOTAL=0

 rem -- Get Options
 SET @SYS_ENV=%USERDOMAIN%& IF /I NOT "%~2"=="" SET @SYS_ENV=%~2
 SET @ALL_SYS=FALSE& FOR %%P IN (   TRUE YES Y) DO IF /I "%~3"=="%%~P" (SET @ALL_SYS=TRUE& SET @MYQUERY=%@QUERY2%)
 SET @COMMENT=FALSE& FOR %%P IN ("" TRUE YES Y) DO IF /I "%~4"=="%%~P" SET @COMMENT=TRUE
 SET @ADQUERY=%@ADFIND% %@BASEDN% -f %@MYQUERY% -nodn -s SUB -maxe 5000 -csv attr name operatingSystem description -excldn "%@OBSOLETE%"

 rem -- Standard Log Variables
 SET @VERINFO=Generated on %@TIMESTAMP:.=:% by %@VER%
 SET @LOGPATH=%@STORAGE%\Logs\RepositoryInfo
 SET @LOGFILE="%@LOGPATH%\%~n0.%@FILESTAMP%"
 SET @CSVLIST="%@LOGPATH%\SystemList.%@FILESTAMP%"
 SET @DEFLIST="%@LOGPATH%\FolderDefaults.%@FILESTAMP%"
 SET @SVRLIST="%@LOGPATH%\ServerList.%@FILESTAMP%"
 SET @SECLIST="%@LOGPATH%\SecurityList.%@FILESTAMP%"
 SET @MONLIST="%@LOGPATH%\UptimeList.%@FILESTAMP%"
 FOR %%F IN (TEMPLOG CSVTEMP DEFTEMP SVRTEMP LIVELOG DEADLOG ERRORS) DO SET @%%F="%@LOGPATH%\%~n0.%%F.%@FILESTAMP%"
 SET @LOGLIST=%@LOGFILE% %@CSVLIST% %@DEFLIST% %@SVRLIST% %@SECLIST% %@MONLIST% %@ERRORS%
 SET @CLEANUP=%@TEMPLOG% %@CSVTEMP% %@DEFTEMP% %@SVRTEMP% %@LIVELOG% %@DEADLOG% %@ERRORS%
 %@PRINT_IF_DEBUG%


 rem ==========================================================================
 rem === Verify Steps Before Execution
 rem === Updated On: 01 May 2017                     
 rem ==========================================================================
:VerifyExecution
 ( rem -- Write to Log
	 ECHO LOGFILE -- %@VERINFO%
	 ECHO %@DIVIDER-%
	 ECHO  Domain Name [Base DN] ............ %@SEARCHDOMAIN% [%@BASEDN:~3,999%]
	 ECHO  System Environment Label ......... %@SYS_ENV%
	 ECHO  Capture Info on Workstations ..... %@ALL_SYS%
	 ECHO  Filter Out Unreachable Systems ... %@COMMENT%
	 ECHO %@DIVIDER-%
	 ECHO:
 ) >>%@LOGFILE% 2>&1

 TYPE %@LOGFILE%
 CHOICE /C YNQEX /T 20 /D Y /N /M "Continue running this script? (Y/N/Q) "
 IF !ERRORLEVEL! GTR 1 GOTO :ExitBatch
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Initialize Log Files
 rem === Updated On: 04 May 2017 / 01 May 2017              
 rem ==========================================================================
:GetServerInfo
 ECHO:& ECHO GETTING CURRENT SYSTEM INFORMATION...

 IF NOT EXIST "%@LOGPATH%" MD "%@LOGPATH%"
 FOR %%F IN (DEADLOG LIVELOG) DO ECHO %%F -- %@VERINFO% >!@%%F! 2>&1

 ( rem -- Write to Log
	 ECHO SERVER_NAME, IP_ADDRESS, ENVIRONMENT, LOGS, ROOT, BACKUPS, MACHINE_CLASS, MACHINE_TYPE, OPERATING_SYSTEM, ARCHITECTURE, HARDWARE, DEVICE_PURPOSE,PATCH_GROUP, FOLDER_DEF, DC_LIST, MGMT_LIST, SERVER_LIST, UPTIME_LIST, SEC_LIST, SMALL_LIST, EXCH_LIST, EXCH_TEST, LOW_DISK, SUS_SERVER, SUS_CLIENT, IN_DEFAULT, CUSTOM_LISTS, MASTERLIST -- %@VERINFO%
 ) >%@CSVTEMP% 2>&1

 ( rem -- Write to Log
	 ECHO SERVER_NAME             LOGS    ROOT    BACKUPS   MACHINE_CLASS   MACHINE_TYPE    OPERATING_SYSTEM        DESCRIPTION 	[!DATE! at !TIME: =0!]
	 MAKESTRING - 175
 ) >%@DEFTEMP% 2>&1

 ( rem -- Write to Log
	 ECHO ;LIST OF SERVERS -- %@VERINFO%
	 ECHO ;%@DIVIDER-%
 ) >%@SVRTEMP% 2>&1
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Get Server Names and Editions from Active Directory
 rem === Updated On: 04 May 2017 / 01 May 2017              
 rem ==========================================================================
:GetServerInfo
 FOR /F "SKIP=1 TOKENS=1-2* DELIMS=," %%S IN ('%@ADQUERY%') DO (
	 SET @WMIC=WMIC /NODE:"%%~S"
	 SET @GETDRIVES= !@WMIC! /OUTPUT:"%@TEMPLOG%" LOGICALDISK Get Description,Name^,FileSystem^,FreeSpace
	 SET @FREESPACE= !@WMIC! LOGICALDISK Get FileSystem^,DriveType^,FreeSpace
	 SET @SYSTEMINFO=!@WMIC! COMPUTERSYSTEM Get SystemType
	 SET @MODELINFO= !@WMIC! COMPUTERSYSTEM LIST FULL
	 SET @RMTSHARE=  !@WMIC! SHARE Get Name^,Path
	 SET @HARDWARE=  !@WMIC! BASEBOARD Get Product

	 CALL :PadRight @SERVER %%~S " " 20
	 SET @IPAnswer=%@NOIP%	!@SERVER!
	 SET @MACHINE=Workstation:& FOR /F "TOKENS=*" %%A IN ('ECHO %%~T ^| FIND /I "SERVER"') DO SET @MACHINE=Server:
	 SET @OS=@@@@%%~T& SET @OS=!@OS: =@!& IF NOT DEFINED !@OS! (SET !@OS!=1) ELSE (SET /A !@OS!+=1)
	 SET /A @TOTAL+=1
	 SET @SYS_BKUPS=-
	 SET @CHAR=;

	 rem -- Show Current IP or IP Placeholder
	 SET @IP=%@NOIP%
	 IF EXIST "%@RESOLVE%" (
		 FOR /F "TOKENS=1" %%I IN ('%@RESOLVE% %%~S ^| FIND /V "ERROR:"') DO (SET @IPAnswer=%%I	!@SERVER!& SET @IP=%%I)
	 ) ELSE (
		 FOR /F "TOKENS=2-3 DELIMS=[] " %%I IN ('PING -n 1 %%~S ^| FIND "["') DO (SET @IPAnswer=%%J	%%I& SET @IP=%%J)
	 )

   rem -- Record Reachable vs Unreachable Systems
	 ECHO !@IPAnswer!	%%T | FIND    "%@NOIP%" >>%@DEADLOG%
	 ECHO !@IPAnswer!	%%T | FIND /V "%@NOIP%" >>%@LIVELOG%

	 rem -- If IP Address is Live, AND Destination is Reachable, Then Move Forward
	 CALL :VerifyTarget "\\%%~S\ADMIN$"
	 IF NOT DEFINED @ALIVE (
		 ECHO  -- The following machine cannot be accessed at this time: %%~S
		 CALL :GetWinVer %%~T

		 rem -- Search for Existing Server Entry, and Use if Found
		 SET @CSVINFO=& FOR /F "TOKENS=* EOL=" %%E IN ('TYPE "%@MASTERLIST%"  ^| FINDSTR /I /R "^[ ]*%%~S" ') DO SET @CSVINFO=%%E
		 SET @DEFINFO=& FOR /F "TOKENS=* EOL=" %%E IN ('TYPE "%@STORAGELIST%" ^| FINDSTR /I /R "^.%%~S"    ') DO SET @DEFINFO=%%E
		 SET @SVRINFO=& FOR /F "TOKENS=* EOL=" %%E IN ('TYPE "%@SERVERLIST%"  ^| FINDSTR /I /R "^.\\\\%%~S"') DO SET @SVRINFO=%%E

		 rem -- Generate Estimated Data
		 IF /I "!@COMMENT!"=="TRUE" (SET @CHAR=;) ELSE (SET @CHAR= )
		 SET @SYS_LOGS=C:
		 SET @SYS_ROOT=C$
		 FOR %%U IN (SYS_CLASS STYPE SYS_TYPE MODEL SYS_DESC) DO SET @%%~U=%@NOVAL%

		 rem -- Obtain Previous Content for INPUT Files (Unreachable Systems)
		 CALL :PadVariables %%~S TrimOnly
		 IF "!@CSVINFO!"=="" (
			 ECHO !@CHAR!%%~S,!@IP!,!@SYS_ENV!,!@SYS_LOGS!,!@SYS_ROOT!,!@SYS_BKUPS!,!@SYS_CLASS!,!@SYS_TYPE!,!@WINVER!,!@STYPE!,!@MODEL!,!@SYS_DESC!,0,C,-,-,Y,Y,Y,-,-,-,Y,Y,-,Y, >>%@CSVTEMP%
		 ) ELSE (
			 ECHO !@CSVINFO:	= ! >>%@CSVTEMP%
		 )

		 CALL :PadVariables %%~S
		 IF "!@DEFINFO!"=="" (
			 ECHO !@CHAR!!@SERVER! !@SYS_LOGS! !@SYS_ROOT! !@SYS_BKUPS! !@SYS_CLASS! !@SYS_TYPE! !@WINVER! !@DESC! >>%@DEFTEMP%
		 ) ELSE (
			 ECHO !@DEFINFO:	= ! >>%@DEFTEMP%
		 )

		 IF "!@SVRINFO!"=="" (
			 ECHO !@CHAR!\\!@SERVER! !@SYS_CLASS! !@SYS_TYPE! !@WINVER! !@DESC! >>%@SVRTEMP%
		 ) ELSE (
			 ECHO !@SVRINFO:	= ! >>%@SVRTEMP%
		 )

	 ) ELSE (

		 rem -- Determine Full OS Version Info [plus i386/x64]
		 FOR /F "TOKENS=1 DELIMS=- " %%I IN ('!@SYSTEMINFO! ^| FIND /I "-based PC"') DO CALL :GetWinVer %%~T %%~I

		 rem -- Determine System Hardware Type [Physical/Virtual]
		 SET @SYS_TYPE=Physical:& FOR /F "TOKENS=1" %%I IN ('!@HARDWARE! ^| FIND /I "Virtual Machine"') DO SET @SYS_TYPE=Virtual:

		 rem -- Obtain Default System Description
		 SET @SYS_DESC=%%~U
		 SET @SYS_DESC=!@SYS_DESC:^&=+!
		 SET @SYS_DESC=!@SYS_DESC:^(=[!
		 SET @SYS_DESC=!@SYS_DESC:^)=]!
		 FOR /F "TOKENS=1-2 DELIMS=:()[]" %%I IN ('ECHO !@SYS_DESC!') DO (
			 SET @SYS_CLASS=%%~I:
			 SET @SYS_DESC=%%~J
			 IF "%%~J"==""     (SET @SYS_CLASS=!@MACHINE!& SET @SYS_DESC=%%~I)
			 IF "%%~I"=="attr" (SET @SYS_CLASS=!@MACHINE!& SET @SYS_DESC=[Not Identified])
		 )

		 rem -- Determine Computer Hardware Model
		 SET @STYPE=& FOR /F "TOKENS=2 DELIMS==" %%V IN ('!@MODELINFO! ^| FINDSTR /I "SystemType"') DO SET @STYPE=%%V
		 SET @MODEL=& FOR /F "TOKENS=2 DELIMS==" %%V IN ('!@MODELINFO! ^| FINDSTR /I "Manufacturer Model"') DO SET @MODEL=!@MODEL! %%V

		 rem -- Determine Root Folder for Scripts
		 SET @SYS_ROOT=& FOR /F "TOKENS=2 DELIMS=:\ " %%V IN ('!@RMTSHARE! ^| FIND /I "ADMIN$"') DO SET @SYS_ROOT=%%V$
		 IF "!@SYS_ROOT!"=="" FOR %%D IN (F E D C) DO FOR /F "TOKENS=1 DELIMS=:" %%I IN ('!@GETDRIVES! ^| FIND /I "Fixed Disk" ^| FIND /I "%%~D:"') DO SET @SYS_ROOT=%%~I$

		 rem -- Determine Root Folder for Logs/Logs$
		 SET @SYS_LOGS=& FOR /F "TOKENS=2 DELIMS=\ "  %%V IN ('!@RMTSHARE! ^| FIND /I "STORAGE$"') DO SET @SYS_LOGS=%%V
		 IF "!@SYS_LOGS!"=="" FOR %%D IN (C D E F) DO FOR /F "TOKENS=1,3" %%I IN ('!@FREESPACE! ^| FIND /I "Fixed Disk" ^| FIND "NTFS" ^| FIND /I "%%~D:"') DO (
			 IF "!@MOSTSPACE!"=="" SET @MOSTSPACE=%%J
			 IF %%J GEQ !@MOSTSPACE! (
				 SET @MOSTSPACE=%%J
				 SET @SYS_LOGS=%%D:
			 )
		 )

		 rem -- Obtain Dynamic Content for INPUT Files (Reachable Systems)
		 CALL :PadVariables %%~S TrimOnly
		 ECHO  %%~S,!@IP!,!@SYS_ENV!,!@SYS_LOGS!,!@SYS_ROOT!,!@SYS_BKUPS!,!@SYS_CLASS!,!@SYS_TYPE!,!@WINVER!,!@STYPE!,!@MODEL!,!@SYS_DESC!,0,Y,-,-,Y,Y,Y,-,-,-,Y,Y,-,Y, >>%@CSVTEMP%
		 CALL :PadVariables %%~S
		 ECHO  !@SERVER! !@SYS_LOGS! !@SYS_ROOT! !@SYS_BKUPS! !@SYS_CLASS! !@SYS_TYPE! !@WINVER! !@DESC! >>%@DEFTEMP%
		 ECHO  \\!@SERVER! !@SYS_CLASS! !@SYS_TYPE! !@WINVER! !@DESC! >>%@SVRTEMP%
	 )
 )
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Perform File Processing (Sort Server Lists by Environment, Type, OS)
 rem === Updated On: 04 May 2017 / 01 May 2017              
 rem ==========================================================================
:ProcessFiles
 ECHO:& ECHO PROCESSING OUTPUT FILES...

 rem -- Apply Multi-key Sorting
 CMSORT /Q /H=1 /V=$2C,$22 /CV=3,1,0 /CV=7,1,0 /CV=8,1,0 /CV=9,1,0- %@CSVTEMP% %@CSVLIST% >>%@ERRORS% 2>&1
 CMSORT /Q /H=2            /S=107,5  /C=52,4   /C=68,5   /C=83,15-  %@DEFTEMP% %@DEFLIST% >>%@ERRORS% 2>&1
 CMSORT /Q /H=2            /S=83,5   /C=28,4   /C=44,5   /C=59,10-  %@SVRTEMP% %@SVRLIST% >>%@ERRORS% 2>&1

 rem -- Append Static Systems Info, If Available and Not Duplicated
 FOR %%L IN (%@SECLIST% %@MONLIST%) DO COPY %@SVRLIST% %%L /Y
 FOR %%F IN (FolderDefaults ServerList SecurityList UptimeList) DO (
	 SET @STATIC=%@INPUT%\%%F.[Static].TXT
	 IF EXIST "!@STATIC!" TYPE "!@STATIC!" >>"%@LOGPATH%\%%F.%@FILESTAMP%"
 )

 ( rem -- Write to Log
	 FOR %%O IN (%@CSVLIST% %@DEFLIST% %@SVRLIST% %@LIVELOG% %@DEADLOG%) DO IF EXIST %%O (
		 TYPE %%O
		 ECHO %@DIVIDER%
		 ECHO:
	 )

	 FOR /F "TOKENS=1-2 DELIMS==" %%V IN ('SET @@@@') DO (
		 SET @COUNT=     %%W
		 SET @OS=%%V
		 ECHO !@COUNT:~-6! each !@OS:@= !
	 )
	 ECHO %@DIVIDER*%
	 ECHO 	TOTAL SYSTEMS = !@TOTAL!
	 ECHO %@DIVIDER*%
 ) >>%@LOGFILE% 2>&1
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Perform Log Management/Maintenance (Or Copy Them for Debug Purposes)
 rem === Updated On: 26 Apr 2017                     
 rem ==========================================================================
:LogMaint
 ECHO:& ECHO PERFORMING FINAL LOGFILE MAINTENANCE...

 IF /I "%DEBUG%"=="COPY" FOR %%L IN (%@LOGLIST% %@CLEANUP%) DO IF EXIST %%L COPY %%L "%TEMP%" /Y

 FOR %%L IN (%@LOGLIST%) DO IF EXIST %%L (
	 SET @THISLOG=%%L
	 COPY %%L !@THISLOG:%@FILESTAMP%=TXT! /Y
	 IF %%L==%@LOGFILE% (ECHO: & TYPE %%L & ECHO: & DIR %%L)
 )
 FOR %%L IN (%@CLEANUP%) DO IF EXIST %%L DEL %%L >NUL
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019              
 rem ==========================================================================
:ExitBatch
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Pad Variable Content to the Left/Right for Display in Logs
 rem === Updated On: 22 Jan 2020 / 14 Oct 2019              
 rem ==========================================================================
:PadLeft
:PadRight
 rem %1 = Variable to Store New Content
 rem %2 = Existing Value to Pad
 rem %3 = Character to use as padding [typically " ", "." or "0"]
 rem %4 = New String Length

 SET @STR=%~2
 SET @LEN=& FOR /L %%c IN (0,1,199) DO IF "!@STR:~%%c,1!"=="" IF NOT DEFINED @LEN SET @LEN=%%c
 IF 1%@LEN% GEQ 1%~4 (
	 SET %~1=%~2
 ) ELSE (
	 IF /I "%~0"==":PadLeft" (
		 SET @TEMPVAR=& FOR /L %%c IN (1,1,%~4) DO SET @TEMPVAR=!@TEMPVAR!%~3
		 SET @TEMPVAR=!@TEMPVAR!%~2
		 SET %~1=!@TEMPVAR:~-%~4!
	 )
	 IF /I "%~0"==":PadRight" (
		 SET @TEMPVAR=%~2& FOR /L %%c IN (1,1,%~4) DO SET @TEMPVAR=!@TEMPVAR!%~3
		 SET %~1=!@TEMPVAR:~0,%~4!
	 )
 )
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Get Windows Version plus Architecture (If Available)
 rem === Updated On: 30 Apr 2017                     
 rem ==========================================================================
:GetWinVer
 rem %* = Current Info for Assigning to @WINVER

 SET @WINVER=%*
 FOR %%z IN (Microsoft Storage Server Standard Enterprise Advanced Datacenter Home Pro Professional Ultimate Insider Preview) DO SET @WINVER=!@WINVER:%%~z=!
 FOR /L %%z IN (1,1,9) DO SET @WINVER=!@WINVER:  = !
 SET @WINVER=!@WINVER: R2=-R2!
 SET @WINVER=!@WINVER:x86=x86!
 SET @WINVER=!@WINVER:x64=x64!
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Trim Starting/Ending Blanks from Variables
 rem === Updated On: 30 Apr 2017                     
 rem ==========================================================================
:TrimIt
 rem %1 = Variable to Trim
 rem wildcard subtitutions *= and ?=

 SET #VAR=
 SET @ZZZ=!%~1!
 SET @ZZZ=!@ZZZ:^?=!
 FOR %%z IN (!@ZZZ!) DO SET #VAR=!#VAR! %%z
 SET #VAR=!#VAR:=?!
 IF "!#VAR!"=="" (SET %~1=%@NOVAL%) ELSE (SET %~1=!#VAR:~1,999!)
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Pad All Variables with Spaces
 rem === Updated On: 04 May 2017 / 30 Apr 2017              
 rem ==========================================================================
:PadVariables
 rem %1 = Current Server Variable

 FOR %%D IN (IP SYS_ENV SYS_LOGS SYS_ROOT SYS_BKUPS STYPE SYS_CLASS SYS_TYPE MODEL WINVER SYS_DESC) DO CALL :TrimIt @%%D
 SET @DESC=!@SYS_ENV!: !@SYS_DESC!

 IF NOT "%~2"=="" GOTO :EOF
 CALL :PadRight @SERVER    "%~1"          " " 22
 CALL :PadRight @IP        "!@IP!"        " " 15
 CALL :PadRight @SYS_ENV   "!@SYS_ENV!"   " " 15
 CALL :PadRight @SYS_LOGS  "!@SYS_LOGS!"  " " 7
 CALL :PadRight @SYS_ROOT  "!@SYS_ROOT!"  " " 7
 CALL :PadRight @SYS_BKUPS "!@SYS_BKUPS!" " " 9
 CALL :PadRight @SYS_CLASS "!@SYS_CLASS!" " " 15
 CALL :PadRight @SYS_TYPE  "!@SYS_TYPE!"  " " 15
 CALL :PadRight @STYPE     "!@STYPE!"     " " 15
 CALL :PadRight @MODEL     "!@MODEL!"     " " 30
 CALL :PadRight @WINVER    "!@WINVER!"    " " 23
 CALL :PadRight @SYS_DESC  "!@SYS_DESC!"  " " 60
 CALL :PadRight @DESC      "!@DESC!"      " " 60
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Validate Availability of Target System for Repository
 rem === Updated On: 26 Apr 2017                     
 rem ==========================================================================
:VerifyTarget
 rem %1 = Path of Destination System to Verify

 IF "%~1"=="" GOTO :EOF
 SET @SYS=
 SET @ALIVE=
 SET @PREFIX=%~1
 SET @PREFIX=%@PREFIX:~0,2%
 FOR /F "TOKENS=1 DELIMS=\" %%S IN ('ECHO %~1') DO SET @SYS=%%~S
 IF NOT "!@PREFIX!"=="\\" SET @SYS=%COMPUTERNAME%
 IF DEFINED #EXISTS-!@SYS! (SET @ALIVE=TRUE& GOTO :EOF)

 FOR /F "TOKENS=*" %%A IN ('NLTEST /QUERY /SERVER:%@SYS% 2^>NUL ^| FIND /I "The command completed successfully"') DO SET @ALIVE=!@SYS!
 IF EXIST "%~1" SET @ALIVE=%~1
 SET #EXISTS-!@SYS!=!@SYS!
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Get Active Directory Domain Name If Provided
 rem === Updated On: 25 Oct 2018 / 12 Mar 2017              
 rem ==========================================================================
:GetDomainName
 rem %1 = Domain Name Input

 SET #DOMAIN=%~1& IF "%~1"=="" SET #DOMAIN=%USERDNSDOMAIN%
 SET #DOMAIN=!#DOMAIN: =!
 SET #DOMAIN=!#DOMAIN:,=.!
 SET #DOMAIN=!#DOMAIN:dc=!
 SET @DOMAIN=& FOR %%D IN (!#DOMAIN!) DO SET @DOMAIN=!@DOMAIN!%%D
 SET #DOMAIN=
 SET @BASEDN=

 FOR /F "TOKENS=1-5 DELIMS=.= " %%D IN ('ECHO !@DOMAIN!') DO (
	 FOR %%z IN ("%%~D" "%%~E" "%%~F" "%%~G" "%%~H") DO IF NOT "%%~z"=="" SET @BASEDN=!@BASEDN!,DC=%%~z
 )

 SET @BASEDN=-b "!@BASEDN:~1,999!"
 SET @SEARCHDOMAIN=!@DOMAIN!
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET @SCRIPT_BEG#="%%~d"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 30 Nov 2020 / 25 Oct 2018              
 rem ==========================================================================
:HelpMessage
 ECHO Generate Updated INPUT Lists of Windows Servers from Active Directory
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  [Fully Qualified AD domain name] [Environment] [TRUE^|FALSE (include workstations)] [TRUE^|FALSE (identify unreachable systems)]
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:
 ECHO -------------------
 ECHO OPTION DEFINITIONS:
 ECHO -------------------
 ECHO  -?,  /H ..... Display This Help Message  (also --HELP)
 ECHO  -??, /H2 .... Display Extended Help Message
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0
	 ECHO  %~n0 mydomain.local
	 ECHO  %~n0 corp.office "Corporate"
	 ECHO  %~n0 thisdomain.home "" TRUE
	 ECHO  %~n0 thatdomain.com  Production FALSE FALSE
	 ECHO  %~n0 -?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
 )
 CALL :ShowStatus "FINISHED"
                                   