@ECHO OFF
 rem ==========================================================================
 rem === Install EventLog to SysLog Service on local box (EvtSys)
 rem ==========================================================================
 rem
 rem Created On: 26 Sep 2005
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Oct 2023 / 30 Nov 2020              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2003 and later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    PSTOOLS ................ https://technet.microsoft.com/en-us/sysinternals/bb896649
 rem    EvtSys ................. http://code.google.com/p/eventlog-to-syslog/
 rem    Resource Kit ........... https://en.wikipedia.org/wiki/Resource_Kit
 rem
 rem  Input Files Used By Script (stored in %SystemDrive%\Scripts\Bat\Input):
 rem    CustomVariables.TXT .... Centralized Configuration Settings for Scripts
 rem
 rem  Required Scripts:
 rem    SetDrive.BAT ........... Set Global Variables for this Environment
 rem ==========================================================================

 :::  This script installs an EventLog-to-SysLog service to send event log
 :::  info to SysLog or other Security Event Consolidation servers.
 :::
 :::  This is a subset of a larger client startup script for AD...
 :::
 :::  (Sep 2011) v2.5 -- Added support for v4.4.2 of the EventLog-to-SysLog
 :::  service, which can now support multiple syslog destinations.
 :::
 :::  (Dec 2011) v2.6 -- The syslog servers must be provided via the
 :::  CustomVariables.TXT file.  No entries at the command prompt will be
 :::  considered other than /O (overwrite) or /U (uninstall).
 :::
 :::  (Jan 2014) v2.7 -- Event-to-Syslog has been upgraded to v4.5, which
 :::  results in changes to the files copied, and the parameters used. There
 :::  is now support for more than two syslog servers (@SYSLOG, @SYSLOG2, etc)
 :::
 :::  (Feb 2014) v2.9 -- The script has been configured to support both the
 :::  Event-to-Syslog v4.4 and v4.5, which provide different parameters for
 :::  supporting log hosts. EvtSys versions prior to 4.5 can only support two
 :::  syslog hosts via the -h and -b parameters (one each).
 :::
 :::  (Apr 2015) v2.11 -- Updated :ShowStatus routine to track script duration.
 :::
 :::  (Feb 2019) v3.0 -- Added filtered CMD Title Routine;
 :::  -- The SYNTAX HELP routine has been updated;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com;
 :::  -- The new BrainWave CheckParams utility is now used to check for
 :::  the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::  -- Corrected and expanded the DEBUG logging routine;
 :::  -- Updated folder locations (from x32 to x86) for legacy executable;
 :::  -- Enabled Code Page 1252 Support;
 :::
 :::  (Oct 2019) v3.1 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (Dec 2019) v3.2 -- Use CHECKPARAMS to evaluate parameters besides /? /H;
 :::  -- Change FC to FILEHASH in File Verification Routine;
 :::
 :::  (May 2020) v3.4 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::
 :::  (Nov 2020) v3.5 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Oct 2023) v3.6 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  https://www.BrainWaveCC.com/knowledge-base/tools-for-managing-the-windows-eventlog/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 08 Jul 2020 / 27 Feb 2019              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 30 Nov 2020 / 08 Jul 2020              
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v3.6.0.660

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Main Variables
 SET @PARAMS=%0 %*
 SET @NOTFOUND=The specified service does not exist as an installed service.
 SET @UNINSTALL=FALSE
 SET @UPTODATE=FALSE
 SET @VIEWONLY=FALSE
 SET @SVCFOUND=TRUE
 SET @SVCSTATE=*****
 SET @SYSLOG=192.168.1.200
 SET @SYSLOG2=
 SET @SYSLOG3=
 SET @SYSLOG4=

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- Identify Location for SysLog Agent (EvtSys)
 SET @SYSLOG_SERVERS=%@SYSLOG%& FOR /L %%X IN (2,1,9) DO IF DEFINED @SYSLOG%%~X SET @SYSLOG_SERVERS=!@SYSLOG_SERVERS!;!@SYSLOG%%~X!
 SET @EVT_KEY=HKLM\Software\ECN\EvtSys\3.0
 SET @EVTSYS_ROOT=%~dp0Services\EvtSys
 SET @SOURCE=%@EVTSYS_ROOT%\%@OS_ARCH%
 SET @DEST=%SystemRoot%\System32
 SET @EDITION=[EvtSys %@OS_ARCH%]

 rem -- Standard Log Variables
 SET @LOGPATH=%@STORAGE%\Logs\Install
 SET @LOGFILE=%@LOGPATH%\%~n0.%@FILESTAMP%
 SET @TEMPLOG=%@LOGPATH%\%~n0.TEMP.%@FILESTAMP%

 rem -- Process Special Command-line Parameters
 SET @ALL_PARAMS=%*
 CALL :GetParams "~ O" & IF DEFINED @OK (SET @OVERRIDE=TRUE) ELSE (SET @OVERRIDE=FALSE)
 CALL :GetParams ". U" & IF DEFINED @OK (SET @UNINSTALL=TRUE)
 CALL :GetParams "# V" & IF DEFINED @OK (SET @VIEWONLY=TRUE)

 rem -- Key Variable Verification
 FOR %%V IN (SOURCE DEST SYSLOG SYSLOG_SERVERS LOGFILE TEMPLOG NOTFOUND NODIFF SVCFOUND PARAMS) DO IF NOT DEFINED @%%~V (
	 ECHO:
	 ECHO *** WARNING: The @%%~V variable is undefined.
	 ECHO:
	 ECHO  Please ensure that all configuration info is provided correctly
	 ECHO  within the file: "%@CUSTOM-VARS%"
	 ECHO:
	 ECHO  Alternatively, this information can be entered directly into this script,
	 ECHO  although this is not recommended because it undermines portability.
	 ECHO %@DIVIDER*%
	 ECHO:
	 ECHO:
	 GOTO :HelpMessage
 )
 %@PRINT_IF_DEBUG%


 rem ==========================================================================
 rem === Verify that EventLog-to-SysLog files are current
 rem === Updated On: 24 Apr 2020 / 29 Dec 2019              
 rem ==========================================================================
:VerifyInstallation
 FOR %%D IN ("%@LOGPATH%") DO IF NOT EXIST %%D MD %%D >NUL

 NOW Cleaning Up Obsolete Folders...
 IF EXIST "%@EVTSYS_ROOT%\x32" (
	 REN "%@EVTSYS_ROOT%\x32" x86 >NUL
	 IF EXIST "%@EVTSYS_ROOT%\x32" RD /S /Q "%@EVTSYS_ROOT%\x32"
 )

 NOW Validating EventLog-to-SysLog Installation...
 FOR /F "TOKENS=1" %%C IN ('FILEHASH --XXHash64 -K "%@SOURCE%\EVTSYS.EXE" "%@DEST%\EVTSYS.EXE" 2^>NUL') DO SET @UPTODATE=%%C
 FOR /F "TOKENS=*" %%S IN ('SC QUERY "EvtSys" ^| FIND /I "%@NOTFOUND%"') DO SET @SVCFOUND=FALSE
 FOR /F "TOKENS=4" %%S IN ('SC QUERY "EvtSys" ^| FIND "STATE"') DO SET @SVCSTATE=%%~S
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Determine Whether to Install/Upgrade or Uninstall EvtSys Service
 rem === Updated On: 08 Jul 2020 / 17 Dec 2019              
 rem ==========================================================================
:ManageService
 IF /I "%@VIEWONLY%"=="TRUE" GOTO :DisplayConfig
 IF /I "%@OVERRIDE%"=="TRUE" SET @UPTODATE=FALSE

 IF /I "%@UNINSTALL%"=="TRUE" (
	 IF "%@SVCFOUND%"=="TRUE" ( rem -- Attempt Uninstall Only If Parameter = "." OR "/U" and Service Exists
		 NOW Uninstalling EventLog-to-SysLog %@EDITION% service...
		 NET STOP EVTSYS /Y
		 %@DEST%\evtsys -u
		 SET @UPTODATE=TRUE
	 ) ELSE (
		 NOW EventLog-to-SysLog %@EDITION% service: *** NOT INSTALLED ***
	 )

 ) ELSE (

	 rem -- Update or Install EventLog-to-SysLog if Files Aren't Current
	 IF /I "%@UPTODATE%%@SVCSTATE%"=="TRUERUNNING" (
		 NOW EventLog-to-SysLog %@EDITION% service: *** UP-TO-DATE ***
	 ) ELSE (
		 NOW Installing/Updating EventLog-to-SysLog %@EDITION% service...
		 IF "%@SVCFOUND%"=="TRUE" NET STOP EVTSYS /Y
		 SLEEP 3
		 IF EXIST "%@DEST%\EvtSys.*" %@DEST%\EvtSys -u
		 SLEEP 5
		 XCOPY "%@SOURCE%\EVTSYS.*" "%@DEST%" /C /I /F /H /K /Y
		 CALL :FindVersion "%@SOURCE%\EVTSYS.EXE"
		 IF DEFINED @FILEVER IF !@FILEVER! GEQ 4.5 (
			 %@DEST%\evtsys -i -h %@SYSLOG_SERVERS% -l 0 -f Local2
		 ) ELSE (
			 %@DEST%\evtsys -i -h %@SYSLOG% -b %@SYSLOG2% -l 0 -f Local2
		 )
	 )

	 rem -- Start EventLog-to-SysLog service if not running
	 FOR /F "TOKENS=4" %%S IN ('SC QUERY "EvtSys" ^| FIND "STATE"') DO IF NOT "%%~S"=="RUNNING" (
		 NOW Starting EventLog-to-SysLog %@EDITION% service...
		 NET START EVTSYS
	 )
 )
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 17 Dec 2019 / 04 Oct 2019              
 rem ==========================================================================
:DisplayConfig
 ( rem -- Write to Log
	 NOW Displaying EventLog-to-SysLog Registry Config...\n\-\nSearching for Registry Key: '%@EVT_KEY%'
	 REG QUERY "%@EVT_KEY%"
	 ECHO %@DIVIDER*%
 ) >%@LOGFILE% 2>&1

 IF EXIST "%@LOGFILE%" TYPE "%@LOGFILE%"
 DIR %@LOGPATH% | FIND "%@FILESTAMP%"
 ECHO %@DIVIDER*%
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019              
 rem ==========================================================================
:ExitBatch
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Execute Dynamic Command From Variable
 rem === Updated On: 27 Nov 2010                     
 rem ==========================================================================
:RunCMD
 rem %* = Command To Execute

 %*
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Determine the version of the EvtSys.exe to be installed
 rem === Updated On: 27 Feb 2019 / 25 Feb 2014              
 rem ==========================================================================
:FindVersion
 rem %1 = File to Check for Version Info

 SET @FILEVER=n/a
 FOR /F %%f IN ('PRINTFILEINFO "%~1" -q -f 2^>NUL') DO SET @FILEVER=%%~f
 IF /I "%@FILEVER%"=="n/a" SET @FILEVER=0.0.0.0
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Use CheckParams.exe to Capture Selected Parameters
 rem === Updated On: 21 Jan 2020 / 17 Dec 2019              
 rem ==========================================================================
:GetParams
 rem %1 = Parameters to Search For
 rem %2 = Variable to Set

 SET @OK=& IF "%~1"=="" GOTO :EOF
 FOR /F "TOKENS=2-3*" %%v IN ('CHECKPARAMS -q -a -c "%~1" -s %@ALL_PARAMS% 2^>NUL') DO IF /I "%%~v"=="TRUE" (
	 SET @OK=T
	 SET @REQ=T
	 IF NOT "%~2"=="" SET %~2=%%~x
 )
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET @SCRIPT_BEG#="%%~d"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 30 Nov 2020 / 08 Jul 2020              
 rem ==========================================================================
:HelpMessage
 ECHO Install EventLog to SysLog Service on local box (EvtSys)
 ECHO:
 ECHO Requires: EventLog-to-SysLog v4.5.0 or higher
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  [OPTIONS]
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:
 ECHO -------------------
 ECHO OPTION DEFINITIONS:
 ECHO -------------------
 ECHO  -?,  /H ..... Display This Help Message  (also --HELP)
 ECHO  -??, /H2 .... Display Extended Help Message
 ECHO       /O ..... Overwrite Existing Config and Executables
 ECHO       /U ..... Uninstall EventLog-to-SysLog service
 ECHO       /V ..... View Registry Config of EventLog-to-SysLog service
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0
	 ECHO  %~n0 /O
	 ECHO  %~n0 /U
	 ECHO  %~n0 /V
	 ECHO  %~n0 /?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
 )
 CALL :ShowStatus "FINISHED"
                                                                                      