@ECHO OFF
 rem ==========================================================================
 rem ===  Log Detailed Info and/or Terminate Long-Running Processes
 rem ==========================================================================
 rem
 rem Created On: 20 Feb 2006
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 28 Jan 2024 / 20 Jan 2024 / 16 Oct 2023
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2003 or later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    SysLogGen .............. http://www.snmpsoft.com/freetools/sysloggen.html
 rem
 rem    PSTOOLS ................ https://technet.microsoft.com/en-us/sysinternals/bb896649
 rem    Resource Kit ........... https://en.wikipedia.org/wiki/Resource_Kit
 rem
 rem  Input Files Used By Script (stored in %SystemDrive%\Scripts\Bat\Input):
 rem    CustomVariables.TXT .... Centralized Configuration Settings for Scripts
 rem    [LongProcesses].TXT .... List of process names and max durations
 rem
 rem  Required Scripts:
 rem    SetDrive.BAT ........... Set Global Variables for this Environment
 rem ==========================================================================

 :::  This script reads the input file LongProcesses.TXT to obtain a list of
 :::  processes which are to be regulated for maximum runtime, or start time
 :::  for generating a diagnostics log.  The processes are logged and/or
 :::  terminated by process ID (PID) rather than process name.  Native tools
 :::  such as TaskList are used to generate the diagnostics info, as well as
 :::  tools from SysInternals, such as psList, psKill, and Handle are used to
 :::  manage the identification and termination of the processes.
 :::
 :::  The input list was introduced in version 2.0, as a way to provide custom
 :::  durations per process, and also allow the execution of the script to be
 :::  modified without having to frequently edit the script itself.  Older
 :::  versions of this script used environment variables for managing the list
 :::  of processes to manage.  Version 2.x continued to fall back to using the
 :::  environment variables if no input file was available, but version 3+
 :::  will only operate if the input file exists.
 :::
 :::  (Jan 2009) v2.6 -- An entry is logged in a daily log file for any
 :::  processes that are killed indicating the date, time, process name and
 :::  process ID.
 :::
 :::  (Jun 2010) v3.0 -- The LongProcess.TXT input file has been modified to
 :::  accommodate the entries necessary for capturing diagnostic logging
 :::  information of processes running beyond a certain duration. This was
 :::  briefly handled by a new script called LogLongProcess.BAT, but has now
 :::  been rolled into this script. Diagnostics info is output to a separate
 :::  daily log file.  Because the check for logging needs to occur more
 :::  frequently, the scheduled interval has been updated from every 2 minutes
 :::  to every minute.
 :::
 :::  (Jul 2010) v3.2 -- It is now possible to set a process so that it will
 :::  never be killed.  (This is good for processes that you only wish to log.)
 :::  Simply set the "Time To Kill" entry to all zeros, and process termination
 :::  will be disabled for that process.  v3.3, the same technique can be used
 :::  with "Time To Log" entry to disabled logging of diagnostic info for a
 :::  specific process.
 :::
 :::  (Jul 2010) v3.4 -- It is now possible to set a threshold for the number
 :::  of processes that must be running concurrently before any logging or
 :::  termination of processes will occur.  The minimum valid value is 1, and
 :::  the number used is valid for both logging and termination.
 :::
 :::  (Oct 2010) v3.5.1 -- It is possible to check for processes with short
 :::  names by adding a space to the end of the name in the input file (before
 :::  the ;)
 :::
 :::  (Nov 2010) Changed the default log extension of the reports from .LOG to
 :::  .TXT so that the reports that are mailed to mobile devices can be read
 :::  automatically. (Not too many devices can natively handle .LOG files)
 :::
 :::  (Nov 2010) v3.6.1 -- The contents of the INPUT file must match the
 :::  process name exactly.  This change has been made so that no processes
 :::  are killed whose names are supersets of the processes listed in the input
 :::  file. A space is added to each process name for searching purposes.
 :::
 :::  (Oct 2011) v3.8 -- Moved the processing of the CustomVariables.TXT
 :::  INPUT file near the end of the variables section to allow almost all of
 :::  the variables to be overridden from that INPUT file (except for the log
 :::  files).  The default schedule was also reset to 2 minutes.
 :::
 :::  (Dec 2011) Simplified the job scheduling section by way of environment
 :::  variables for verifying whether to use SCHTASKS or the AT command.
 :::
 :::  (Mar 2012) The new version of PSLIST supports checking for an exact
 :::  process name, so several routines were modified to improve performance.
 :::
 :::  (Dec 2012) Made it possible for the scheduled tasks to be stored in their
 :::  own folder, rather than the default root folder of the Task Scheduler.
 :::
 :::  (Feb 2014) v4.0 -- Added the capability to send log data to one or more
 :::  Syslog servers using the SyslogGen or NeoLog utilities in conjunction
 :::  with the @SYSLOGx environment variables.
 :::
 :::  (Mar 2014) v4.1 -- Upgraded the DebugMode options to add another
 :::  variable (@PRINT_IF_DEBUG). Also revamped the routines for logging to
 :::  Syslog servers.
 :::
 :::  (Feb 2015) v4.3 -- Added support for a local "LongProcesses-MINI.TXT"
 :::  file to provide customized supplementary updates over the global input
 :::  file: "[LongProcessess].TXT"
 :::
 :::  (Apr 2015) v4.5 -- Added a job history file (*.RECORDING.TXT) to record
 :::  the process killing/logging activities.
 :::
 :::  (Nov 2015) v4.6 -- Removed DNS error output from the Syslog routines.
 :::
 :::  (Oct 2017) v4.7 -- Added -nobanner parameter to all SysInternals utilities
 :::  to strip header content from script output.  (Reduces errors).
 :::
 :::  (May 2018) v4.8 -- Changed the HELP routine to use FINDSTR;
 :::  -- Added new Filtered CMD Title and Syntax Help processing routines;
 :::  -- Support for NeoLogger has been eliminated from this script;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com;
 :::  -- The default schedule interval has been moved to 5 minutes, up from 2 min
 :::
 :::  (Jul 2018) v4.9 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::
 :::  (Aug 2018) v4.10 -- Removed AT command support;
 :::
 :::  (Sep 2018) v5.0 -- Instead of calling for a new process list for each
 :::  process to be evaluated, output a full PSLIST to a file, and perform all
 :::  comparisons from that saved list;
 :::  -- Created/Renamed logs, and added an extended Log Maintenance routine;
 :::  -- Changed :SaveJobHistory to a subroutine for flexibility;
 :::  -- Added new parameters for scheduling/unscheduling/testing;
 :::  -- Processes will not be killed when TESTMODE is enabled;
 :::  -- Moved HELP SYNTAX processing to ensure necessary variables are available
 :::
 :::  (Oct 2018) v5.1 -- Corrected and expanded the DEBUG logging routine;
 :::  -- Adjusted the process evaluation routines to avoid false positives;
 :::
 :::  (Dec 2018) v5.2 -- If TASKKILL exists, it will be used instead of PSKILL;
 :::
 :::  (Mar 2019) v5.3 -- Enabled Code Page 1252 Support;
 :::  -- Record additional attributes in the Job History log;
 :::  -- Prevent DEBUG MODE data from being stored in the .RECORDING.TXT file;
 :::
 :::  (Oct 2019) v5.4 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (Oct 2019) v5.5 -- Update :PadLeft and :PadRight routines;
 :::
 :::  (Jan 2020) v5.6 -- Use CHECKPARAMS to evaluate parameters besides /? /H;
 :::  -- Added a routine for capturing legitimate integer parameters;
 :::  -- Refactored the :Prepare4Syslog routine;
 :::  -- Consolidate :PadLeft and :PadRight routines;
 :::
 :::  (May 2020) v5.7 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::
 :::  (Nov 2020) v5.8 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Jan 2022) v5.9 -- Modify the @NAMECHK verification to include \ prefix;
 :::
 :::  (Oct 2023) v5.10 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::  (Jan 2024) v6.0 -- Add a dated SNAPSHOT file that keeps track of the
 :::  most recent (default=10) full process list captures (called @SNAPSHOTS);
 :::  -- Add the @SAVE_SNAPSHOT variable to control this new behavior;
 :::
 ::: --------
 :::  NOTES:
 ::: --------
 :::  *	Version 2.x was the last version of this script officially tested
 :::    under Windows NT 4.0.
 :::  * Versions 3.x and 4.x have not been tested on anything earlier than XP/2003
 :::  * Versions 5.x has not been tested on anything earlier than 2008-R2.
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 30 Nov 2020 / 30 Apr 2018
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 28 Jan 2024 / 20 Jan 2024 / 30 Nov 2020
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v6.0.0.1200

 rem -- Default Variables (Override via CustomVariables.*.TXT)
 SET @INTERVAL=5
 SET @SNAPSHOTS=10
 SET @SAVE_SNAPSHOT=FALSE

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 CALL :SaveJobHistory ABORTED - NO %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Job Scheduling Variables
 SET @JOBNAME=Kill Long-Running Processes
 SET @JOBUSER=SYSTEM
 SET @JOBFREQ=MINUTE /MO %@INTERVAL%
 SET @JOBTIME=00:00:00
 SET @JOBDATE=
 SET @JOBPATH=%~f0
 SET @JOBTYPE=

 rem -- Process Tracking Variables
 SET @SNAP_STATUS=N
 SET @LOGGED_LIST=
 SET @KILLED_LIST=
 SET @RECORDED=FALSE
 SET @TERMINATED=FALSE
 SET @TOTAL_PROC=0
 SET @TOTAL_EVAL=0
 SET @NUM_LOGGED=0
 SET @NUM_KILLED=0
 SET @DIAG_MSG=No Processes Logged...
 SET @KILL_MSG=No Processes Killed...

 rem -- TaskKill vs PsKill
 IF EXIST "%WINDIR%\System32\TASKKILL.EXE" (
	 SET @PK=TASKKILL
	 SET @KILL_PROCESS=!@PK! /F /T /PID
 ) ELSE (
	 SET @PK=PSKILL
	 SET @KILL_PROCESS=!@PK! -t %@PSPARAMS%
 )

 rem -- Standard Log Variables
 SET @LOGPATH=%@STORAGE%\Logs\Maint
 SET @LFORMAT=%@LOGPATH%\%~n0
 SET @TIMINGS=%@LFORMAT%.RECORDING.TXT
 SET @LOGFILE=%@LFORMAT%.%@TODAY%.TXT
 FOR %%L IN (TEMPLOG LOGINFO ERRORS SNAPSHOT) DO SET @%%L=%@LFORMAT%.%%L.%@FILESTAMP%
 SET @LOGLIST="%@LOGFILE%" "%@LOGINFO%" "%@ERRORS%"
 SET @CLEANUP="%@TEMPLOG%" "%@ERRORS%"

 rem -- Ensure Valid Log Processing Values
 CALL :GetInteger @MINROWS "%@MIN_LOG_ROWS%" 005 001 010
 CALL :GetInteger @MAXROWS "%@MAX_LOG_ROWS%" 999 199 999

 rem -- Process Special Command-line Parameters
 SET @ALL_PARAMS=%*
 CALL :GetParams "@" & IF DEFINED @OK (SET @NOSCHED=TRUE)
 CALL :GetParams "-" & IF DEFINED @OK (SET @UNSCHED=TRUE)
 CALL :GetParams "." & IF DEFINED @OK (SET @SCHEDULEONLY=TRUE& SET @UPDATESCHED=TRUE)
 CALL :GetParams "*" & IF DEFINED @OK (SET @MINROWS=1& SET @MAXROWS=1)
 CALL :GetParams "~" & IF DEFINED @OK (SET @TESTMODE=TRUE& SET @DIAGMODE=TRUE)
 %@PRINT_IF_DEBUG%


 rem ==========================================================================
 rem === Create Scheduled Job using SCHTASKS (XP/2003/Vista/+)
 rem === Updated On: 22 Jan 2022 / 09 Mar 2019
 rem ==========================================================================
:ScheduleJobs
 IF /I "%@NOSCHED%"=="TRUE" IF /I NOT "%@UNSCHED%"=="TRUE" (
	 SET @JOBTYPE= // ONE-TIME
	 ECHO:
	 ECHO *** Job Scheduling Disabled ***
	 ECHO:
	 GOTO :ScheduleJobs_END
 )

 ECHO:
 ECHO Scheduling "%@JOBNAME%" on %COMPUTERNAME%...
 ECHO:

 IF %@OSBUILD% LSS 6000 SET @TASKFOLDER=
 SET @NAMECHK=%@JOBNAME:[=\[%
 SET @NAMECHK=!@NAMECHK:]=\]!
 SET @CHK_SCHED=SCHTASKS /QUERY /V /FO LIST
 SET @SET_SCHED=SCHTASKS /CREATE /TN "!@TASKFOLDER!%@JOBNAME%" /TR "%@JOBPATH%" /RU "%@JOBUSER%" /SC %@JOBFREQ% %@JOBDATE% /ST %@JOBTIME% %@JOBLEVEL% %@FORCE%

 rem -- Delete Scheduled Task from Old Custom Folders on Vista/2008+
 IF %@OSBUILD% GEQ 6000 (
	 FOR /F "TOKENS=1* DELIMS=\ " %%s IN ('%@CHK_SCHED% 2^>NUL ^| FIND "TaskName:" ^| FINDSTR /I /R /C:"\\%@NAMECHK%$"') DO (
		 IF /I NOT "%@TASKFOLDER%%@JOBNAME%"=="%%t" SCHTASKS /DELETE /TN "%%t" /F >NUL 2>&1
	 )
 )

 rem -- Check to Remove Scheduled Job
 IF /I "%@UNSCHED%"=="TRUE" (
	 SCHTASKS /DELETE /TN "!@TASKFOLDER!%@JOBNAME%" %@FORCE% 2>NUL
	 ECHO Task has been UNscheduled.  Now exiting script...
	 GOTO :ExitBatch
 )

 rem -- Check for Schedule Update Parameters
 IF /I "%@UPDATESCHED%"=="TRUE" (!@SET_SCHED!) ELSE (CALL :CheckSched "%~f0")
 ECHO %@DIVIDER-%
 ECHO:
 IF /I "%@SCHEDULEONLY%"=="TRUE" (
	 ECHO Task has been scheduled.  Now exiting script...
	 IF %@OSBUILD% LSS 6000 (SCHTASKS) ELSE (%@CHK_SCHED% /TN "!@TASKFOLDER!%@JOBNAME%")
	 GOTO :ExitBatch
 )
:ScheduleJobs_END
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Search For Processes Which Are Eligible for Logging or Termination
 rem === Updated On: 28 Jan 2024 / 20 Jan 2024 / 04 Sep 2018
 rem ==========================================================================
:EnumerateProcesses
 IF NOT EXIST "%@LOGPATH%" MD "%@LOGPATH%"

 rem -- Generate Task List SNAPSHOT, if snapshot configuration enabled
 IF /I "%@SAVE_SNAPSHOT%"=="TRUE" (
	 SET @SNAP_STATUS=Y
	 TASKLIST /V
	 PSLIST /M %@PSPARAMS%
	 PSLIST /X %@PSPARAMS%
 ) >>"%@SNAPSHOT%" 2>>"%@ERRORS%"

 SET @FILELIST=& FOR %%P IN ("%@PROCESS-INFO%" "%@PROCESS-INFO2%") DO IF EXIST "%%~P" SET @FILELIST=!@FILELIST! "%%~P"
 IF DEFINED @FILELIST (
	 FOR /F "USEBACKQ SKIP=3 TOKENS=1-4 DELIMS=; " %%P IN (%@FILELIST%) DO CALL :EvalutateProcess "%%~P" "%%~Q" "%%~R" "%%~S"
 ) ELSE (
	 ECHO:
	 ECHO No Valid Input File Found. Operation Halted.
	 ECHO  -- Primary Input File ..... "%@PROCESS-INFO%"
	 ECHO  -- Secondary Input File ... "%@PROCESS-INFO2%"
	 GOTO :ExitBatch
 )
 %@PRINT_IF_DEBUG%


 rem ==========================================================================
 rem === Keep Only The Last %@SNAPSHOTS% Process List Snapshots (Default=10)
 rem === Updated On: 20 Jan 2024      
 rem ==========================================================================
:PurgeOldSnapshots
 FOR /F "SKIP=%@SNAPSHOTS% TOKENS=*" %%v IN ('DIR "%@LFORMAT%.SNAPSHOT.*" /O-N /S /B 2^>NUL') DO (
	 ECHO  - Deleting "%%~v"
	 IF /I NOT "%@DIAGMODE%"=="TRUE" DEL "%%~v"
 )
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Perform Log Management/Maintenance (Or Copy Them for Debug Purposes)
 rem === Updated On: 07 Sep 2018      
 rem ==========================================================================
:LogMaint
 CALL :SaveJobHistory
 IF /I "%DEBUG%"=="COPY" FOR %%L IN (%@LOGLIST% %@CLEANUP%) DO IF EXIST %%L COPY %%L "%TEMP%" /Y

 FOR %%L IN (%@LOGLIST%) DO IF EXIST %%L (
	 SET @THISLOG=%%L
	 SET @THISLOG=!@THISLOG:%@TODAY%.TXT=TXT!
	 SET @THISLOG=!@THISLOG:%@FILESTAMP%=TXT!
	 COPY %%L !@THISLOG! /Y
	 IF %%L=="%@LOGFILE%" (ECHO: & TYPE %%L & DIR %%L)
 )
 FOR %%L IN (%@CLEANUP%) DO IF EXIST %%L DEL %%L >NUL
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Generate Final Job Report
 rem === Updated On: 04 Sep 2018 / 30 Apr 2018
 rem ==========================================================================
:FinalReport
 IF /I "%@CONSOLE_SESSION%"=="TRUE" (
	 IF /I "%@RECORDED%/%@TERMINATED%"=="FALSE/FALSE" (
		 ECHO:
		 ECHO No Processes Logged OR Terminated...
		 ECHO:
	 ) ELSE (
		 ECHO:
		 ECHO:
		 ECHO %@DIAG_MSG%
		 ECHO -- %@LOGGED_LIST%
		 ECHO:
		 ECHO %@KILL_MSG%
		 ECHO -- %@KILLED_LIST%
		 ECHO:
	 )
	 TAIL -2 "%@TIMINGS%"
	 SLEEP 5
	 ECHO %@DIVIDER*%
	 PSLIST %@PSPARAMS% 2>NUL
 )


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019
 rem ==========================================================================
:ExitBatch
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Execute Dynamic Command From Variable
 rem === Updated On: 20 Jul 2010      
 rem ==========================================================================
:RunCMD
 rem %* = Command To Execute

 %*
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Store Current Date and Time in Variable
 rem === Updated On: 16 Oct 2023 / 28 Apr 2015
 rem ==========================================================================
:GetTime
 rem %1 = Variable to Store Date/Time

 SET %~1=!DATE! at !TIME: =0!
 FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET %~1#="%%~d"
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Determine If a Process Should Be Logged and/or Terminated
 rem === Updated On: 28 Oct 2018 / 06 Sep 2018
 rem ==========================================================================
:EvalutateProcess
 rem %1 = Process Name
 rem %2 = Maximum Allowed Runtime of Process
 rem %3 = Logging Threshold of Process

 SET @CAN_KILL=TRUE
 SET @WILL_LOG=TRUE
 SET @FIRSTRUN=TRUE
 SET @HOWMANY=0

 SET /A @TOTAL_PROC+=1
 SET @PROCESS=%~1
 SET @PNAME=          %~1
 SET @LIMIT=%~2& IF %~2 LEQ %@ALL_ZEROES% SET @CAN_KILL=FALSE
 SET @LOGIT=%~3& IF %~3 LEQ %@ALL_ZEROES% SET @WILL_LOG=FALSE
 SET @COUNT=%~4& IF 0%~4 LEQ 00 SET @COUNT=1

 IF NOT EXIST "%@TEMPLOG%" PSLIST %@PSPARAMS% >"%@TEMPLOG%" 2>"%@ERRORS%"
 FOR /F %%V IN ('TYPE "%@TEMPLOG%" ^| FINDSTR /I /R /C:"^%@PROCESS% " ^| FIND /C " "') DO SET @HOWMANY=%%~V
 FOR /F "TOKENS=2,8 DELIMS= " %%V IN ('TYPE "%@TEMPLOG%" ^| FINDSTR /I /R /C:"^%@PROCESS% "') DO (
	 SET /A @TOTAL_EVAL+=1

	 rem -- Determine PID and Length of Run-time
	 SET @PID=%%~V
	 SET @RUNTIME=0000%%~W
	 SET @RUNTIME=!@RUNTIME:~-14!

	 rem -- Provide Visible Count for Debug Purposes
	 FOR %%D IN (TRUE SOME PARTIAL) DO IF /I "%Debug%"=="%%~D" (
		 ECHO "%@PROCESS%" [!@PID!] - %@HOWMANY% - %@COUNT% - !@RUNTIME! - %@LOGIT% - %@WILL_LOG% - %@LIMIT% - %@CAN_KILL%
		 SET @
	 )

	 IF %@HOWMANY% GEQ %@COUNT% (
		 IF /I "%@WILL_LOG%"=="TRUE" IF !@RUNTIME! GEQ %@LOGIT% (
			 SET /A @NUM_LOGGED+=1
			 CALL :LogDiagInfo
		 ) ELSE (
			 ECHO %@PNAME:~-10% [pid=!@PID!] 	runtime: !@RUNTIME! -- [Log  @ %@LOGIT:~1,9%]  count: %@HOWMANY% [Log  @ %@COUNT%+] -- NOT LOGGED
		 )

		 IF /I "%@CAN_KILL%"=="TRUE" IF !@RUNTIME! GEQ %@LIMIT% (
			 SET /A @NUM_KILLED+=1
			 CALL :TerminateProcess
		 ) ELSE (
			 ECHO %@PNAME:~-10% [pid=!@PID!] 	runtime: !@RUNTIME! -- [Kill @ %@LIMIT:~1,9%]  count: %@HOWMANY% [Kill @ %@COUNT%+] -- NOT KILLED
		 )
	 )
	 IF /I NOT "!@WILL_LOG!/!@CAN_KILL!"=="FALSE/FALSE" ECHO:
 )
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Log Diagnostics Info for Current Process by PID
 rem === Updated On: 07 Sep 2018 / 30 Apr 2018 / 18 Oct 2017
 rem ==========================================================================
:LogDiagInfo
 rem %* = NO PARAMETERS NEEDED

 SET @LOGGED_LIST=!@LOGGED_LIST! "%@PROCESS%" [%@PID%];
 SET @RECORDED=TRUE
 SET @DIAG_MSG=Logged Detailed Info on all Long-Running Processes...

 ( rem -- Write to Log
	 rem -- Display Version Header Info, Once Per Process Name
	 IF /I "%@FIRSTRUN%"=="TRUE" (
		 ECHO:
		 ECHO %@DIVIDER*%
		 ECHO *** %@VER% on %@TIMESTAMP% for "%@PROCESS%"
		 ECHO %@DIVIDER*%
		 ECHO:
	 )

	 ECHO %DATE% %TIME% -- %@PNAME:~-10% [pid=%@PID%] 	runtime: %@RUNTIME% -- [Log  @ %@LOGIT:~1,9% / Kill @ %@LIMIT:~1,9% / MinCount = %@COUNT%] -- LOGGED
	 CALL :ForwardToSyslog "5" %@PNAME:~-10% [pid=%@PID%] 	runtime: %@RUNTIME% -- [Log  @ %@LOGIT:~1,9%]  count: %@HOWMANY% [Log  @ %@COUNT%+] -- LOGGED >CON

	 rem -- Generate a List of ALL Processes of a given Process Name, Once Per Process Name
	 IF /I "%@FIRSTRUN%"=="TRUE" (
		 ECHO:
		 ECHO *** SHOW ALL RUNNING "%@PROCESS%" PROCESSES ***
		 TASKLIST /V /FI "IMAGENAME eq %@PROCESS%.EXE"
		 ECHO:
		 PSLIST    -e "%@PROCESS%" %@PSPARAMS% 2>NUL
		 ECHO:
		 PSLIST -m -e "%@PROCESS%" %@PSPARAMS% 2>NUL
		 ECHO:
		 ECHO *** SHOW ALL RUNNING PROCESSES ***
		 PSLIST    %@PSPARAMS% 2>NUL
		 PSLIST -m %@PSPARAMS% 2>NUL
		 ECHO %@DIVIDER-%
		 ECHO:
		 ECHO:
		 SET @FIRSTRUN=FALSE
	 )

	 ECHO:
	 ECHO *** BASIC PROCESS INFO ***
	 PSLIST    %@PID% %@PSPARAMS%

	 ECHO:
	 ECHO *** EXTENDED PROCESS INFO ***
	 PSLIST -x %@PID% %@PSPARAMS%

	 ECHO:
	 ECHO *** PROCESS MODULE INFO ***
	 TASKLIST /M /FI "PID eq %@PID%"

	 ECHO:
	 ECHO *** VERBOSE PROCESS DETAIL ***
	 TASKLIST /V /FI "PID eq %@PID%"

	 ECHO:
	 ECHO *** FINDING OPEN HANDLES FOR PID %@PID% ***
	 HANDLE -p %@PID% %@PSPARAMS% | FINDSTR "handles :"
	 ECHO:
	 ECHO:
	 ECHO:
 ) >>"%@LOGINFO%" 2>>"%@ERRORS%"

 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Terminate Current Process by PID
 rem === Updated On: 05 Dec 2018 / 05 Sep 2018
 rem ==========================================================================
:TerminateProcess
 rem %* = NO PARAMETERS NEEDED

 SET @KILLED_LIST=!@KILLED_LIST! "%@PROCESS%" [%@PID%];
 SET @TERMINATED=TRUE
 SET @KILL_MSG=Killed All Long-Running Processes...

 ECHO %DATE% %TIME% -- %@PNAME:~-10% [pid=%@PID%] 	runtime: %@RUNTIME% -- [Kill @ %@LIMIT:~1,9%]  count: %@HOWMANY% [Kill @ %@COUNT%+] -- KILLED >>"%@LOGFILE%" 2>>"%@ERRORS%"
 CALL :ForwardToSyslog "4" %@PNAME:~-10% [pid=%@PID%] 	runtime: %@RUNTIME% -- [Kill @ %@LIMIT:~1,9%]  count: %@HOWMANY% [Kill @ %@COUNT%+]
 IF /I "%@TESTMODE%"=="TRUE" (SET @TESTCMD=ECHO) ELSE (SET @TESTCMD=)
 FOR /F "TOKENS=*" %%K IN ('!@TESTCMD! %@KILL_PROCESS% %@PID% 2^>NUL') DO (
	 CALL :ForwardToSyslog "4" %%K
	 (ECHO %%~K& ECHO:) >>"%@LOGFILE%" 2>>"%@ERRORS%"
 )
 SLEEP 3
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Determine if Parameter is a Valid Integer, Or Set to Zero
 rem === Updated On: 21 Jan 2020      
 rem ==========================================================================
:GetInteger
 rem %1 = Variable to Store Integer
 rem %2 = Parameter to Evaluate
 rem %3 = Value to Default to, If Zero
 rem %4 = Optional: Lowest Value
 rem %5 = Optional: Highest Value

 SET @V1=%2& IF DEFINED @V1 SET @V1=!@V1:"=!
 SET @V2=%3& IF DEFINED @V2 SET @V2=!@V2:"=!
 SET @V3=%4& IF DEFINED @V3 SET @V3=!@V3:"=!
 SET @V4=%5& IF DEFINED @V4 SET @V4=!@V4:"=!
 FOR /F "TOKENS=3" %%i IN ('CHECKPARAMS -o --min "!@V3!" --max "!@V4!" -n "!@V1!" "!@V2!" 2^>NUL') DO SET %~1=%%i
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Use CheckParams.exe to Capture Selected Parameters
 rem === Updated On: 21 Jan 2020      
 rem ==========================================================================
:GetParams
 rem %1 = Parameters to Search For
 rem %2 = Variable to Set

 SET @OK=& IF "%~1"=="" GOTO :EOF
 FOR /F "TOKENS=2-3*" %%v IN ('CHECKPARAMS -q -a -c "%~1" -s %@ALL_PARAMS% 2^>NUL') DO IF /I "%%~v"=="TRUE" (
	 SET @OK=T
	 SET @REQ=T
	 IF NOT "%~2"=="" SET %~2=%%~x
 )
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Pad Variable Content to the Left/Right for Display in Logs
 rem === Updated On: 22 Jan 2020 / 14 Oct 2019
 rem ==========================================================================
:PadLeft
:PadRight
 rem %1 = Variable to Store New Content
 rem %2 = Existing Value to Pad
 rem %3 = Character to use as padding [typically " ", "." or "0"]
 rem %4 = New String Length

 SET @STR=%~2
 SET @LEN=& FOR /L %%c IN (0,1,199) DO IF "!@STR:~%%c,1!"=="" IF NOT DEFINED @LEN SET @LEN=%%c
 IF 1%@LEN% GEQ 1%~4 (
	 SET %~1=%~2
 ) ELSE (
	 IF /I "%~0"==":PadLeft" (
		 SET @TEMPVAR=& FOR /L %%c IN (1,1,%~4) DO SET @TEMPVAR=!@TEMPVAR!%~3
		 SET @TEMPVAR=!@TEMPVAR!%~2
		 SET %~1=!@TEMPVAR:~-%~4!
	 )
	 IF /I "%~0"==":PadRight" (
		 SET @TEMPVAR=%~2& FOR /L %%c IN (1,1,%~4) DO SET @TEMPVAR=!@TEMPVAR!%~3
		 SET %~1=!@TEMPVAR:~0,%~4!
	 )
 )
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Update Job Execution Stats
 rem === Updated On: 28 Jan 2024 / 20 Jan 2024 / 09 Mar 2019
 rem ==========================================================================
:SaveJobHistory
 rem %* = Optional Message to include in Job History

 CALL :GetTime @SCRIPT_END
 CALL :PadLeft @TP "%@TOTAL_PROC%" "." 3
 CALL :PadLeft @EV "%@TOTAL_EVAL%" "." 3
 CALL :PadLeft @NL "%@NUM_LOGGED%" "." 3
 CALL :PadLeft @NK "%@NUM_KILLED%" "." 3
 SET @ROWCOUNT=0
 SET @TRUNCATED=
 SET @ELAPSED=%@NO_TIME%& FOR /F "DELIMS=" %%d IN ('DATEINFO -t !@SCRIPT_BEG#! -n !@SCRIPT_END#! -e "hr:min:sec.ms" -q 2^>NUL') DO SET @ELAPSED=%%d
 SET @SPECIAL=%@JOBTYPE%& IF NOT "%~1"=="" SET @SPECIAL= // %*
 SET @JOBHISTORY=%@JOBNAME% // PROCESS INFO [CHECKED.%@TP% FOUND.%@EV% LOGGED.%@NL% KILLED.%@NK% SNAP..%@SNAP_STATUS%] // DURATION: %@ELAPSED% // START: %@SCRIPT_BEG% // END: !@SCRIPT_END! // %@PK% // %~n0 %@VERSION%%@SPECIAL%

 %@VERBOSE_OFF%
 BREAK ON >>"%@TIMINGS%"
 FOR /F %%c IN ('TYPE "%@TIMINGS%" ^| FIND /C "/"') DO SET @ROWCOUNT=%%c
 IF !@ROWCOUNT! GTR %@MAXROWS% (
	 SET @TRUNCATED= // TRUNCATED
	 SET @OLDTIME=!@TIMINGS:TXT=%@FILESTAMP%!
	 COPY "%@TIMINGS%" "!@OLDTIME!" /V /Y >CON
	 TAIL -%@MINROWS% "!@OLDTIME!" >"%@TIMINGS%"
	 ECHO:
	 CALL :ForwardToSyslog "5" %@JOBNAME%: Job History Logfile Truncated -- [!DATE! at !TIME!]
 ) >>"%@LOGFILE%" 2>>"%@ERRORS%"

 CALL :ForwardToSyslog "6" %@JOBHISTORY%!@RUNMODE!!@TRUNCATED! >>"%@TIMINGS%" 2>>"%@ERRORS%"
 %@VERBOSE_ON%
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Forward Output to Available Syslog Server(s) as LOCAL3
 rem === Updated On: 13 Jan 2020 / 07 Mar 2019
 rem ==========================================================================
:ForwardToSyslog
 rem %1 = Syslog Severity Level (RFC 3164)
 rem %* = Message to Send to Display and Send to Syslog Server

 SET @S_MSG=%*& IF "%~2"=="" SET @S_MSG=%1 - This is a Syslog Test Message from %@VER%
 SET @SEV=6
 FOR %%a IN (0 EMERGENCY) DO IF /I "%~1"=="%%~a" SET @SEV=0
 FOR %%a IN (1 ALERT) DO IF /I "%~1"=="%%~a" SET @SEV=1
 FOR %%a IN (2 CRITICAL) DO IF /I "%~1"=="%%~a" SET @SEV=2
 FOR %%a IN (3 ERROR) DO IF /I "%~1"=="%%~a" SET @SEV=3
 FOR %%a IN (4 WARNING) DO IF /I "%~1"=="%%~a" SET @SEV=4
 FOR %%a IN (5 NOTICE) DO IF /I "%~1"=="%%~a" SET @SEV=5
 FOR %%a IN (6 INFORMATION) DO IF /I "%~1"=="%%~a" SET @SEV=6
 FOR %%a IN (7 DEBUG) DO IF /I "%~1"=="%%~a" SET @SEV=7

 %@VERBOSE_OFF%
 CALL :Prepare4Syslog %@S_MSG%

 ECHO !@S_MSG!
 IF DEFINED @SYSLOG_UTIL IF /I "%@USE_SYSLOGS%"=="TRUE" FOR %%s IN ("" 0 1 2 3 4) DO IF DEFINED @SYSLOG%%~s (
	 SET @S_PORT=514& IF DEFINED @SYSLOG%%s_PORT SET @S_PORT=!@SYSLOG%%~s_PORT!
	 "%@SYSLOGGEN%" -t:!@SYSLOG%%~s! -p:!@S_PORT! -f:19 -s:!@SEV! -tg:"%~n0" -m:"!@S_MSG2!" -q -hn:%@LOCALHOST% | FIND /V "%@NO_SYSLOGGEN%"
 )
 %@VERBOSE_ON%
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Prepare Messages for SyslogGen Syslog Utility
 rem === Updated On: 14 Jan 2020 / 30 Apr 2018
 rem ==========================================================================
:Prepare4Syslog
 rem %* = Syslog Message to be Cleaned Up

 SET @PREFIX=%1
 SET @S_MSG=%*
 SET @S_MSG=!@S_MSG:%@PREFIX% =!

:TrimLeadingSpaces
 IF "!@S_MSG:~0,1!"==" " (
	 SET @S_MSG=!@S_MSG:~1,9999!
	 GOTO :TrimLeadingSpaces
 )

 SET @S_MSG2=!@S_MSG:\=\x5B!
 SET @S_MSG2=!@S_MSG2:"=\x22!
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Check if Scheduled Job is Already Running
 rem === Updated On: 15 May 2018 / 05 Feb 2015
 rem ==========================================================================
:CheckSched
 rem %1 = Standard Search Criteria
 rem %2 = Advanced Search Criteria

 SET @CHK=%2
 IF DEFINED @CHK (
	 %@CHK_SCHED% 2>NUL | FIND /I "%~1" | FINDSTR /I /R /C:%2 >NUL
 ) ELSE (
	 %@CHK_SCHED% 2>NUL | FIND /I "%~1" >NUL
 )
 IF ERRORLEVEL 1 %@SET_SCHED%
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" CALL :GetTime @SCRIPT_BEG
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 30 Nov 2020 / 04 Sep 2018
 rem ==========================================================================
:HelpMessage
 ECHO Logs and/or terminates processes that have been running longer than a specified duration
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  [OPTIONS]
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:
 ECHO -------------------
 ECHO OPTION DEFINITIONS:
 ECHO -------------------
 ECHO  -?,  /H ..... Display This Help Message  (also --HELP)
 ECHO  -??, /H2 .... Display Extended Help Message
 ECHO       .  ..... Schedule the Job, But Don't Execute It
 ECHO       @  ..... Execute the Job without Scheduling It
 ECHO       -  ..... Remove Job from Task Scheduler
 ECHO       *  ..... Truncate Job History Logfile
 ECHO       ~  ..... Test/Diagnostics Mode [don't kill any processes]
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0
	 ECHO  %~n0 .
	 ECHO  %~n0 *
	 ECHO  %~n0 @
	 ECHO  %~n0 -
	 ECHO  %~n0 @ ~
	 ECHO  %~n0 -?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
	 ECHO:
	 ECHO  * The following INPUT files, if they exist, are the only places where
	 ECHO    this script will look for information about processes to be evaluated.
	 ECHO    -- "%@PROCESS-INFO%"
	 ECHO    -- "%@PROCESS-INFO2%"
 )
 CALL :ShowStatus "FINISHED"
                                                       