@ECHO OFF
 rem ==========================================================================
 rem === Logon Script -- Map Drives and Log System/User Diagnostics Info
 rem ==========================================================================
 rem
 rem Created On: 10 Jun 1997
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Oct 2023 / 07 Dec 2022
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2003 and later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    Event-to-SysLog ........ https://code.google.com/archive/p/eventlog-to-syslog/
 rem
 rem    PSTOOLS ................ https://technet.microsoft.com/en-us/sysinternals/bb896649
 rem    Resource Kit ........... https://en.wikipedia.org/wiki/Resource_Kit
 rem ==========================================================================

 :::  This is a generic logon script which maps drives based on domain group
 :::  membership.  You can make it do lots of other things.  Lots of ResKit
 :::  utilities, including Kixtart, are used to compile some system info which
 :::  will allow you to more quickly resolve client-side issues.  This info is
 :::  copied to a centralized network share.
 :::
 :::  This script allows you to update scripts and utilities on your client
 :::  machines, as well as installing an EventLog to SysLog service.
 :::
 :::  Be sure to add all of the files/folders into the appropriate GPO folder
 :::  in order to have them copied to your client systems.
 :::
 :::  Example:
 :::        -- Logon.Settings
 :::        -- NOW.EXE
 :::        -- DateInfo.EXE
 :::        -- CheckParams.EXE
 :::        -- PrintFileInfo.EXE
 :::        -- ReadConfig.EXE
 :::        -- SyslogStatus.BAT
 :::        -- Logon.BAT
 :::        ----- Scripts\Util1.bat
 :::        ----- Scripts\Util2.exe
 :::        ----- Scripts\Util3.exe
 :::        ----- System32\File1.bat
 :::        ----- System32\File2.exe
 :::        ----- System32\File3.sys
 :::
 :::  This script originally supported Win9x/NT and later, but recent versions
 :::  of the script have only been tested with Windows 2003 and later.
 :::
 :::  (Nov 2010) Changed the default log extension of the reports from .LOG to
 :::  .TXT so that the reports that are mailed to mobile devices can be read
 :::  automatically. (Not too many devices know what to do with the .LOG files)
 :::
 :::  (Mar 2014) v6.0 -- Replaced the FINDGRP utility with assorted JoeWare
 :::  utilities such as GetUserInfo and MemberOf due to an "ERROR 234"
 :::  condition on certain combinations of OS + group membership that resulted
 :::  in logon times exceeding 5 minutes.  Also corrected the old Syslog
 :::  settings for Event-to-Syslog configuration.
 :::
 :::  (Jun 2017) v6.2 -- Replaced NOW.EXE with DATEINFO.EXE;
 :::  -- Removed the legacy system inventory component.
 :::
 :::  (Feb 2018) v6.3 -- Changed Script Documentation to www.BrainWaveCC.com
 :::
 :::  (May 2018) v6.4 -- Changed to HTTPS links for Script Documentation;
 :::  -- Consolidated all BrainWave Utilities Links under one heading;
 :::
 :::  (Feb 2019) v7.0 -- Updated DEBUG routine; Changed Logging methodology;
 :::  -- Added new DATEINFO --Script-Variables routine to set default variables;
 :::  -- Replaced FILEVER.EXE with PRINTFILEINFO.EXE;
 :::
 :::  (Oct 2019) v7.1 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::  -- Moved the CMD filtering to its own routine, and added {/?} filtering;
 :::
 :::  (Nov 2019) v7.2 -- Add extra output markers and cleanup logging;
 :::
 :::  (Dec 2019) v7.3 -- Developed a drop-in replacement for Microsoft's
 :::  NOW.EXE utility, taking back that function from DATEINFO.EXE;
 :::
 :::  (Dec 2019) v7.4 -- Change FC to FILEHASH in File Verification Routine;
 :::  -- Corrected some issues with the Syslog Agent Service routines;
 :::
 :::  (Jan 2020) v8.0 -- The logon scripts are being modified to pull custom
 :::  settings from the "Logon.Settings.TXT" file to ensure that the scripts
 :::  themselves are consistent across various deployments;
 :::
 :::  (Mar 2020) v8.2 -- Modified the SysLog variable tracker;
 :::
 :::  (Apr 2020) v8.3 -- Updated FILEHASH to support the BLAKE3 and XXHASH64 ciphers;
 :::
 :::  (May 2020) v8.4 -- Updated :DebugMode routine to support the STEP option;
 :::  -- Use ReadConfig.exe to process the Logon.Settings config file;
 :::
 :::  (Jul 2020) v8.5 -- Add Version Information to the History Log;
 :::
 :::  (Jul 2020) v8.6 -- Added extra version info for the logon/logoff family
 :::  of scripts in the format of yyyy.mm.dd.hh.nn (@VERDATE);
 :::
 :::  (Aug 2020) v8.7 -- Minor update to consolidated status report;
 :::
 :::  (Aug 2020) v9.0 -- Alter sequence of Debug Verification routine;
 :::  -- Read additional debug related variables from Logon.Setting.TXT;
 :::  -- Make Header and Footer entries consistent across all scripts;
 :::
 :::  (Dec 2022) v9.1 -- Add subfolders to repository copy jobs;
 :::
 :::  (Oct 2023) v9.2 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::  NOTE: The logon/logoff/etc scripts are mostly self-contained, and make
 :::        use of their own central config file (Logon.Settings) rather than
 :::        the common config files of all the other scripts.
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/tools-for-managing-the-windows-eventlog/
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 31 May 2020 / 05 Oct 2019
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 17 Aug 2020 / 11 Aug 2020
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v9.2.0.2120  v2023.09.17.1500

 rem -- Obtain Date/Time and Supporting Core Variables
 FOR /F "TOKENS=1-2 DELIMS=;" %%P IN ('DATEINFO --Script-Variables 2^>NUL') DO SET %%P=%%~Q

 rem -- Main Variables (Overridden by Logon.Settings)
 SET @RESERVED=%HOMEDRIVE% V: W: X: Y: Z:
 SET @STRICT=FALSE

 rem -- Search Logon.Settings.*.TXT for Custom Variables
 SET @SETTINGS=%~dp0Logon.Settings.
 SET @CONFIG=& FOR %%F IN ("TXT" "%COMPUTERNAME%.TXT") DO IF EXIST "!@SETTINGS!%%~F" SET @CONFIG="!@SETTINGS!%%~F"
 IF DEFINED @CONFIG (
	 FOR /F "TOKENS=*" %%P IN ('READCONFIG %@CONFIG% 2^>NUL') DO CALL :RunCMD %%P
 ) ELSE (
	 SET @ACCOUNTS=Administrator AdminScripts ServiceAcct
	 SET @SERVERS=SERVER1 SERVER2 SERVER3 SERVER4
	 SET @NODIAG=TESTSYS
	 SET @DOMAIN=%USERDOMAIN%
	 SET @FILESERVER=\\MyFILESERVER
	 SET @WEBSERVER=\\MYWEB
	 SET @UTILITIES=\\MYUTIL

	 rem -- IP Address OR DNS Name for SysLog Server
	 FOR %%n IN ("" 1 2 3) DO SET @SYSLOG%%~n=192.168.255.22%%~n
 )
 SET @SYSLOG_SERVERS=& FOR %%X IN ("" 1 2 3 4 5 6 7 8 9 0) DO IF DEFINED @SYSLOG%%~X SET @SYSLOG_SERVERS=!@SYSLOG_SERVERS!;!@SYSLOG%%~X!

 rem -- Standard Variables
 SET @MARKER=%SystemRoot%\SkipLogon.TXT
 SET @NOTFOUND=The specified service does not exist as an installed service.
 SET @UPTODATE=FALSE
 SET @SVCFOUND=TRUE
 SET @SVCSTATE=*****
 SET @OBSOLETE="Logon Info:1234" "Logon Info.TXT:5678" "LMScript.TXT:2345" "LMScript.LOG:3456"

 rem -- Standard Folder Variables
 SET @FOLDERS=Bat Utils
 SET @SOURCE1=%~dp0Scripts
 SET @SOURCE2=%~dp0System32& IF /I "%@OS_ARCH%"=="x64" SET @SOURCE2=%~dp0System32\x64
 SET @DEST1=%SystemDrive%\Scripts
 SET @DEST2=%SystemRoot%\System32

 rem -- Syslog Agent Info
 SET @AGENT_NAME=EvtSys
 SET @AGENT_TITLE=EventLog-to-SysLog [%@AGENT_NAME% %@OS_ARCH%]
 SET @AGENT_STORE=%@DEST1%\Bat\Services\%@AGENT_NAME%

 rem -- Standard Log Variables
 SET @LOGPATH=%SystemDrive%\Storage\LogonData
 SET @LOGBASE=%@LOGPATH%\Logon Info
 SET @LOGFILE=%@LOGBASE%.%@FILESTAMP_R%
 SET @HISTORY=%@LOGBASE%.RECORDING.TXT
 SET @NETLOG=%@FILESERVER%\Maint\LogonData\%COMPUTERNAME%

 rem -- Version Info Messages
 SET @INITIATION=Logon Script ...... %@LVERSION% ... STARTED on %COMPUTERNAME% by '%USERNAME%'
 SET @COMPLETION=%@INITIATION:STARTED=COMPLETED%


 rem ==========================================================================
 rem === Enable Debug Mode If 'Debug' Variable is Set to TRUE/LOG/STEP/VERBOSE
 rem === Updated On: 16 Aug 2020 / 31 May 2020
 rem ==========================================================================
:DebugMode
 SET #DEBUG=& FOR %%d IN (LOG STEP TRUE VERBOSE) DO IF /I "%Debug%"=="%%~d" SET #DEBUG=TRUE
 FOR %%d IN (%@DEBUG_SYS%) DO IF /I "%COMPUTERNAME%"=="%%~d" (SET #DEBUG=TRUE& SET DEBUG=STEP)
 IF EXIST "%@DEBUG_FLAG%" (SET #DEBUG=TRUE& SET DEBUG=STEP)
 IF DEFINED #DEBUG (
	 SET @DIAGMODE=TRUE
	 SET @DEBUG_MODE=[DEBUG MODE ON]
	 SET @PAUSE_IF_DEBUG=TIMEOUT 5
	 SET @PRINT_IF_DEBUG=ECHO:^& SET @^& ECHO:^& TIMEOUT 5
	 SET @END_DEBUG_MODE=ECHO:^& ECHO *** DEBUG MODE FINISHED ***^& ECHO:^& SET @^& TIMEOUT 30
	 SET @COMPLETION=%@COMPLETION% !@DEBUG_MODE!
	 IF /I "%Debug%"=="LOG" (SET @PAUSE_IF_DEBUG=& SET @PRINT_IF_DEBUG=ECHO:^& SET @^& ECHO:)
	 IF /I "%Debug%"=="STEP" (SET @PAUSE_IF_DEBUG=ECHO ##PAUSE##& SET @PRINT_IF_DEBUG=ECHO ##PRINT##)
	 IF /I "%Debug%"=="VERBOSE" (SET @VERBOSE_OFF=ECHO ON) ELSE (FOR %%t IN (ON OFF) DO SET @VERBOSE_%%t=ECHO %%t)
	 ECHO *** DEBUG MODE = ON ***
	 ECHO ON
 ) ELSE ( FOR %%d IN (DEBUG_MODE PAUSE_IF_DEBUG PRINT_IF_DEBUG END_DEBUG_MODE VERBOSE_ON VERBOSE_OFF) DO SET @%%d=)


 rem ==========================================================================
 rem === Record Logon Info locally, for diagnostics purposes
 rem === Updated On: 17 Aug 2020 / 15 Mar 2020
 rem ==========================================================================
:TrackLogon
 FOR %%D IN ("%@LOGPATH%" "%@NETLOG%") DO IF NOT EXIST "%%~D" MD "%%~D"
 ATTRIB +H "%@LOGPATH%"

 ECHO %@VER% -- Logon Script is Running -- Please Wait...
 ECHO:

 ( rem -- Write Log to File
	 NOW \*\v -- %@INITIATION%\n\*\n
	 CALL :EchoAll Drive Connections at the start of the script...
	 NET USE
	 ECHO %@DIVIDER-%
	 %@PRINT_IF_DEBUG%
 ) >"%@LOGFILE%" 2>&1


 rem ==========================================================================
 rem === Update Local Repository of Scripts and Utilities
 rem === Updated On: 07 Dec 2022 / 24 Apr 2020
 rem ==========================================================================
:UpdateFiles
 ( rem -- Write Log to File
	 ECHO:
	 CALL :EchoAll Updating System Scripts...
	 ECHO:
	 FOR %%V IN (%@FOLDERS%) DO XCOPY "%@SOURCE1%\%%~V" "%@DEST1%\%%~V" /C /I /D /F /H /K /Y /S
	                            XCOPY "%@SOURCE2%"      "%@DEST2%"      /C /I /D /F /H /K /Y /S

	 rem -- Validate Syslog Agent Service and Binaries
	 ECHO:
	 CALL :EchoAll Validating %@AGENT_TITLE% Installation...
	 ECHO:
	 IF EXIST "%@AGENT_STORE%\x32" REN "%@AGENT_STORE%\x32" x86
	 FOR /F "TOKENS=1" %%C IN ('FILEHASH --XXHash64 -K "%@SOURCE2%\%@AGENT_NAME%.EXE" "%@DEST2%\%@AGENT_NAME%.EXE" 2^>NUL') DO SET @UPTODATE=%%C
	 FOR /F "TOKENS=*" %%S IN ('SC QUERY "%@AGENT_NAME%" ^| FIND /I "%@NOTFOUND%"') DO SET @SVCFOUND=FALSE
	 FOR /F "TOKENS=4" %%S IN ('SC QUERY "%@AGENT_NAME%" ^| FIND "STATE"') DO SET @SVCSTATE=%%~S

	 rem -- Update or Install Syslog Agent Binaries if Files Aren't Current
	 IF /I "!@UPTODATE!!@SVCSTATE!"=="TRUERUNNING" (
		 CALL :EchoAll %@AGENT_TITLE% service: *** UP-TO-DATE ***

	 ) ELSE (
		 rem -- Update Syslog Agent Binaries If Necessary
		 IF /I "!@UPTODATE!"=="FALSE" (
			 CALL :EchoAll Updating %@AGENT_TITLE% service...
			 IF "!@SVCFOUND!"=="TRUE" (
				 IF /I "!@SVCSTATE!"=="RUNNING" (
					 NET STOP %@AGENT_NAME% /Y
					 SLEEP 2
				 )
				 %@DEST2%\%@AGENT_NAME% -u
				 SLEEP 2
			 )
			 XCOPY "%@SOURCE2%\%@AGENT_NAME%.*" "%@DEST2%" /C /I /F /H /K /Y
			 SLEEP 2
		 )

		 rem -- Install Syslog Agent Service If Necessary
		 IF /I NOT "!@SVCSTATE!"=="RUNNING" (
			 CALL :EchoAll Installing %@AGENT_TITLE% service...
			 CALL :FindVersion "%@SOURCE2%\%@AGENT_NAME%.EXE"
			 IF DEFINED @FILEVER (
				 IF "!@FILEVER!" GEQ "4.5" (
					 %@DEST2%\%@AGENT_NAME% -i -h %@SYSLOG_SERVERS% -l 0 -f Local2
				 ) ELSE (
					 %@DEST2%\%@AGENT_NAME% -i -h %@SYSLOG% -b %@SYSLOG2% -l 0 -f Local2
				 )
			 ) ELSE (
				 CALL :EchoAll Could Not Determine Version of %@AGENT_TITLE% Binaries
			 )
		 )

		 rem -- Start Syslog Agent Service if not running
		 FOR /F "TOKENS=4" %%S IN ('SC QUERY "%@AGENT_NAME%" ^| FIND "STATE"') DO IF NOT "%%~S"=="RUNNING" (
			 CALL :EchoAll Starting %@AGENT_TITLE% service...
			 NET START %@AGENT_NAME%
		 )
	 )
	 %@PAUSE_IF_DEBUG%
 ) >>"%@LOGFILE%" 2>&1


 rem ==========================================================================
 rem === Map Shares (skip reserved letters: Default = HomeDrive + W: to Z:)
 rem === Updated On: 18 Aug 2020 / 28 Jan 2020
 rem ==========================================================================
:MapShares
 ( rem -- Write Log to File
	 ECHO:
	 CALL :EchoAll Mapping Server Shares...
	 ECHO:

	 rem -- Delete All Non-Reserved Mappings if @STRICT=TRUE
	 IF "%@STRICT%"=="TRUE" FOR /F "TOKENS=2" %%V IN ('NET USE ^| FIND ":"') DO CALL :EraseMappings %%V

	 rem -- Default Mappings for Domain Users
	 FOR /F "TOKENS=1* DELIMS==" %%M IN ('SET @D_MAPPING 2^>NUL') DO CALL :RunCMD %%~N

	 rem -- Skip the Following Section if Marker Found, or if Using a Domain Admin Account, or if on a Server
	 IF EXIST "%@MARKER%" GOTO :FINISHED_MapShares
	 FOR %%V IN (%@ACCOUNTS%) DO IF /I "%USERNAME%"=="%%V" GOTO :FINISHED_MapShares
	 FOR %%V IN (%@SERVERS%) DO IF /I "%COMPUTERNAME%"=="%%V" GOTO :FINISHED_MapShares

	 rem -- Custom Mappings based on Group Membership
	 FOR /F "TOKENS=1* DELIMS==" %%M IN ('SET @C_MAPPING 2^>NUL') DO CALL :RunCMD %%~N

	:FINISHED_MapShares
	 ECHO %@DIVIDER-%
	 ECHO:

	 %@PAUSE_IF_DEBUG%
 ) >>"%@LOGFILE%" 2>&1


 rem ==========================================================================
 rem === Create System Diagnostics Log For Debug Purposes
 rem === Updated On: 17 Aug 2020 / 12 Mar 2020
 rem ==========================================================================
:CreateLogs
 FOR %%V IN (%@NODIAG%) DO IF /I "%COMPUTERNAME%"=="%%~V" GOTO :FinalReport

 rem -- Generate System Logs
:SystemLogs
 ( rem -- Write Log to File
	 ECHO:
	 CALL :EchoAll Generating Logon Diagnostics Log...
	 ECHO:
	 SYSTEMINFO
	 KIX32 "%~dp0Configure.KIX"
	 KIX32 "%~dp0LogonInfo.KIX"
	 ECHO:
	 ECHO %@DIVIDER*%
	 MEMBEROF | FIND /I "=" | SORT
	 ECHO:
	 GETUSERINFO %USERDOMAIN%\%USERNAME% /P
	 ECHO:
	 GETUSERINFO %USERNAME% /P
	 ECHO:
	 SHOWGRPS | FIND "User:" /I
	 SHOWGRPS | FIND "User:" /I /V | FIND "\" | SORT
	 ECHO:
	 ECHO %@DIVIDER-%
	 SRVINFO -ns
	 ECHO:
	 ECHO %@DIVIDER-%
	 PSINFO -d
	 ECHO:
	 ECHO %@DIVIDER-%
	 NET START
	 ECHO:
	 ECHO %@DIVIDER-%
	 NET SHARE
	 ECHO:
	 ECHO %@DIVIDER-%
	 NET USER
	 ECHO:
	 ECHO %@DIVIDER-%
	 DIR "%~dp0" /OGN
	 DIR %@DEST1%\BAT /OGN
	 DIR %@DEST1%\UTILS /OGN
	 DIR %@DEST2%\%@AGENT_NAME%.*
	 DIR %@LOGPATH%
	 ECHO:
	 ECHO:
	 ECHO Current Environment Variables
	 ECHO %@DIVIDER-%
	 SET
	 ECHO %@DIVIDER*%
	 ECHO %@DIVIDER*%
	 ECHO %@DIVIDER*%
	 ECHO:
	 NOW System Diagnostics Log Completed\n\n
 ) >>"%@LOGFILE%" 2>&1
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Generate Final Logon Report
 rem === Updated On: 13 Jul 2020 / 15 Mar 2020
 rem ==========================================================================
:FinalReport
 ( rem -- Write Log to File
	 ECHO:
	 CALL :EchoAll Drive Connections at the end of the script...
	 NET USE
	 NOW \-\v -- %@COMPLETION%\n\-
 ) >>"%@LOGFILE%" 2>&1
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Send Syslog Message When Completed (if available)
 rem === Updated On: 17 Aug 2020 / 13 Jul 2020
 rem ==========================================================================
:SendSyslogMessage
 NOW %@COMPLETION% >>"%@HISTORY%"
 IF EXIST "%~dp0SyslogStatus.BAT" CALL "%~dp0SyslogStatus.BAT" "%@SYSLOG_SERVERS%" . "6" "%~n0" %@COMPLETION%
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Copy Results to Network
 rem === Updated On: 12 Mar 2020
 rem ==========================================================================
:CopyToNetwork
 IF EXIST "%@LOGBASE%*.TXT" (
	 COPY "%@LOGFILE%" "%@NETLOG%.LOG" /Y
	 COPY "%@HISTORY%" "%@NETLOG%.TXT" /Y
	 COPY "%@LOGBASE%*.TXT" "%@NETLOG%" /Y
	 TYPE "%@LOGFILE%"
 )
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Keep Only The Last %@KEEP_HISTORICAL% Logons Entries (Default=30)
 rem === Updated On: 13 Jul 2020 / 29 Jan 2020
 rem ==========================================================================
:PruneOldLogons
 CALL :GetInteger @KEEP_HISTORICAL "%@KEEP_HISTORICAL%" 30 5 199

 ( rem -- Write to Log
	 NOW \n\n \v\n Renaming Obsolete Logs\n\*
	 FOR %%V IN (%@OBSOLETE%) DO (
		 FOR /F "TOKENS=1-2 DELIMS==:" %%X IN ('ECHO %%~V') DO (
			 IF EXIST "%@LOGPATH%\%%~X" REN "%@LOGPATH%\%%~X" "Logon Info.%@TODAY%-%@NOW%#%%~Y.TXT"
		 )
	 )

	 NOW \n\n \v\n Pruning Old Local Logon Records \t[Keep previous %@KEEP_HISTORICAL% copies]\n\=
	 FOR /F "SKIP=%@KEEP_HISTORICAL% TOKENS=*" %%V IN ('DIR "%@LOGPATH%" /AD /O-N /B /A') DO (
		 ECHO  - Deleting "%%~V" from Local Data Folder
		 DEL "%@LOGPATH%\%%~V"
	 )

	 NOW \n\n \v\n Pruning Old Network Logon Records \t[Keep previous %@KEEP_HISTORICAL% copies]\n\=
	 FOR /F "SKIP=%@KEEP_HISTORICAL% TOKENS=*" %%V IN ('DIR "%@NETLOG%" /AD /O-N /B /A') DO (
		 ECHO  - Deleting "%%~V" from Network Data Folder
		 DEL "%@NETLOG%\%%~V"
	 )
	 %@PAUSE_IF_DEBUG%
 ) >>"%@LOGFILE%" 2>&1


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 05 Oct 2019 / 25 Mar 2016
 rem ==========================================================================
:ExitBatch
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Execute Dynamic Command From Variable
 rem === Updated On: 28 Jan 2020
 rem ==========================================================================
:RunCMD
 rem %* = Command To Execute

 %*
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Echo to Console and Standard Output
 rem === Updated On: 06 Nov 2019
 rem ==========================================================================
:EchoAll
 rem %* = Output to Echo

 ECHO %* >CON
 ECHO %*
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Map Shares Based On Group Membership
 rem === Updated On: 16 Aug 2020 / 11 Feb 2019
 rem ==========================================================================
:MapGroup
 rem %1 = Group Membership to Check For
 rem %2 = Drive Letter To Use For Mapping
 rem %3 = Remote Resource To Map [if blank, then drive mapping is deleted, not replaced]

 SET @GROUP=%@DOMAIN%\%~1
 SET @DRIVE=%~2
 SET @SHARE=%~3
 SET @COMPARE=
 SET @MAPERROR=

 rem -- Check for Various Error Conditions
 IF "%~1"==""                              SET @MAPERROR=TRUE& ECHO ERROR: No Valid Domain Group Provided...
 IF NOT DEFINED @DRIVE                     SET @MAPERROR=TRUE& ECHO ERROR: No Drive Letter Provided...
 IF DEFINED @SHARE IF NOT EXIST "%@SHARE%" SET @MAPERROR=TRUE& ECHO ERROR: Cannot connect %@DRIVE% to "%@SHARE%" -- The Share does not exist.
 IF DEFINED @MAPERROR GOTO :EOF

 rem -- Determine if drive is already mapped appropriately
 SET @EXISTING=
 FOR /F "TOKENS=2*" %%C IN ('NET USE %@DRIVE% 2^>NUL ^| FIND "\\"') DO SET @EXISTING=%%D
 IF /I "%@SHARE%"=="!@EXISTING!" (
	 ECHO Drive letter %@DRIVE% is already mapped to "!@EXISTING!"
 ) ELSE (
	 rem -- Set Drive to New Share with a Persistent Mapping
	 IF DEFINED @SHARE (
		 IFMEMBER "%@GROUP%"
		 IF ERRORLEVEL 1 (
			 IF DEFINED @EXISTING (
				 ECHO Clearing off drive letter %@DRIVE% before mapping it to "%@SHARE%"
				 NET USE %@DRIVE% /DELETE /Y
			 ) ELSE (
				 ECHO Currently mapping drive %@DRIVE% to "%@SHARE%"
			 )
			 NET USE %@DRIVE% "%@SHARE%" /PERSISTENT:YES
		 ) ELSE (
		   ECHO Not authorized to have drive %@DRIVE% mapped to "%@SHARE%"
		 )
	 )
 )
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Erase All Existing Mappings Except U-Z
 rem === Updated On: 25 Apr 2011 / 23 Mar 2011
 rem ==========================================================================
:EraseMappings
 rem %1 = Current Drive Letter

 FOR %%D IN (%@RESERVED%) DO IF /I "%1"=="%%~D" GOTO :EOF
 NET USE %1 /D /Y
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Determine if Parameter is a Valid Integer, Or Set to Zero
 rem === Updated On: 28 Jan 2020
 rem ==========================================================================
:GetInteger
 rem %1 = Variable to Store Integer
 rem %2 = Parameter to Evaluate
 rem %3 = Value to Default to, If Zero
 rem %4 = Optional: Lowest Value
 rem %5 = Optional: Highest Value

 SET @V1=%2& IF DEFINED @V1 SET @V1=!@V1:"=!
 SET @V2=%3& IF DEFINED @V2 SET @V2=!@V2:"=!
 SET @V3=%4& IF DEFINED @V3 SET @V3=!@V3:"=!
 SET @V4=%5& IF DEFINED @V4 SET @V4=!@V4:"=!
 FOR /F "TOKENS=3" %%i IN ('CHECKPARAMS -o --min "!@V3!" --max "!@V4!" -n "!@V1!" "!@V2!" 2^>NUL') DO SET %~1=%%i
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Determine the version of the Agent Binary to be installed
 rem === Updated On: 18 Feb 2019 / 14 Apr 2014
 rem ==========================================================================
:FindVersion
 rem %1 = File to Check for Version Info

 SET @FILEVER=n/a
 FOR /F %%f IN ('PRINTFILEINFO "%~1" -q -f 2^>NUL') DO SET @FILEVER=%%~f
 IF /I "%@FILEVER%"=="n/a" SET @FILEVER=0.0.0.0
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 16 Feb 2022
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version
 rem %3 = Date-based Extended Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF NOT "%~3"=="" (SET @VERDATE=%~n0 %~3& SET @LVERSION=%~3)
 IF /I "%~1"=="STARTED" SET @SCRIPT_BEG#="%DATE:~4,10% %TIME: =0%"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF
                                            