@ECHO OFF
 rem ==========================================================================
 rem === Initiate Check Disk of Local Drives
 rem ==========================================================================
 rem
 rem Created On: 21 Apr 1997
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Oct 2023 / 30 Nov 2020              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: NT4 or later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    Resource Kit ........... https://en.wikipedia.org/wiki/Resource_Kit
 rem
 rem  Required Scripts:
 rem    SetDrive.BAT ........... Set Global Variables for this Environment
 rem ==========================================================================

 :::  NT4 or 2000 Resource Kit Required.
 :::
 :::  This script determines all the local drives on a machine and schedules
 :::  each of them for CHKDSK upon next reboot. It requires SRVINFO.EXE,
 :::  NOW.EXE and CHKNTFS.EXE from the ResKit.
 :::
 :::  (Nov 2010) Changed the default log extension of the reports from .LOG to
 :::  .TXT so that the reports that are mailed to mobile devices can be read
 :::  automatically. (Not too many devices can natively handle .LOG files)
 :::
 :::  (Oct 2019) v2.0 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::  -- Corrected and expanded the DEBUG logging routine;
 :::  -- Generate Filtered CMD Title;
 :::
 :::  (May 2020) v2.1 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::
 :::  (Nov 2020) v2.2 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Oct 2023) v2.3 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 30 Nov 2020 / 05 Oct 2019              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 30 Nov 2020 / 26 May 2020              
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v2.2.0.220

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 ECHO:
	 GOTO :ExitBatch
 )

 rem -- Main Variables
 SET @DRV=
 SET @LOGPATH=%@STORAGE%\Logs\Maint
 SET @LOGFILE=%@LOGPATH%\%~n0.%@FILESTAMP%


 rem ==========================================================================
 rem === Schedule CHKDSK on All Local Drives (as determined by SRVINFO)
 rem === Updated On: 05 Feb 2015 / 25 Oct 2005              
 rem ==========================================================================
:Schedule
 IF NOT EXIST "%@LOGPATH%" MD "%@LOGPATH%"

 CALL :GetTime @STIME
 ECHO *** Please Wait ***
 FOR /F "DELIMS=$" %%D IN ('SRVINFO -NS ^| FIND "$"') DO CALL :GetDrives %%D

 ( rem -- Write to Log
	 NOW Scheduling Disk Check for '%COMPUTERNAME%'...
	 SRVINFO -NS | FIND "$"
	 CHKNTFS /T:5
	 FOR %%V IN (%@DRV%) DO (
		 ECHO   CHKDSK scheduled on drive %%V:
		 CHKNTFS /C %%V:
		 CHKNTFS %%V:
	 )
	 CHKNTFS /T
	 ECHO FINISHED Scheduling Disk Check for '%COMPUTERNAME%'!
 ) >"%@LOGFILE%"
 CALL :GetTime @ETIME


 rem ==========================================================================
 rem === Generate Final Job Report
 rem === Updated On: 21 Apr 2015                     
 rem ==========================================================================
:FinalReport
 CLS
 TYPE "%@LOGFILE%" & DIR "%@LOGFILE%"


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019              
 rem ==========================================================================
:ExitBatch
 ECHO:
 ECHO %@MINORDIV%
 ECHO  Chkdsk Started On ............................. %@STIME%
 ECHO  Chkdsk Ended On ............................... %@ETIME%
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Generate List Of All Local Drives
 rem === Updated On: 09 Dec 2002
 rem ==========================================================================
:GetDrives
 rem %1 = current drive letter

 SET @DRV=%@DRV% %1
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Store Current Date and Time in Variable
 rem === Updated On: 16 Oct 2023 / 21 Apr 2015              
 rem ==========================================================================
:GetTime
 rem %1 = Variable to Store Date/Time

 SET %~1=!DATE! at !TIME: =0!
 FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET %~1#="%%~d"
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" CALL :GetTime @SCRIPT_BEG
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF
                                                                