@ECHO OFF
 rem ==========================================================================
 rem === Schedule Important Maintenance Jobs
 rem ==========================================================================
 rem
 rem Created On: 04 Jun 2003
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 04 Apr 2024 / 02 Apr 2024 / 01 Apr 2024 / 17 Mar 2024 / 12 Jan 2024
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2003 and later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem  Required Scripts:
 rem    BackupDNS.BAT .......... In Script Repository Archive
 rem    BackupDHCP.BAT ......... In Script Repository Archive
 rem    BackupGPO.BAT .......... In Script Repository Archive
 rem    BackupSched.BAT ........ In Script Repository Archive
 rem    CheckDisk.BAT .......... In Script Repository Archive
 rem    CheckIntegrity.BAT ..... In Script Repository Archive
 rem    CheckWAN.BAT ........... In Script Repository Archive
 rem    CPU_Benchmark.BAT ...... In Script Repository Archive
 rem    CreateArchive.BAT ...... In Script Repository Archive
 rem    DateMemoryDump.BAT ..... In Script Repository Archive
 rem    Debug.BAT .............. In Script Repository Archive
 rem    DefragSys.BAT .......... In Script Repository Archive
 rem    IPDebug.BAT ............ In Script Repository Archive
 rem    KillLongProcess.BAT .... In Script Repository Archive
 rem    MoveArchives.BAT ....... In Script Repository Archive
 rem    Replicate-Files.BAT .... In Script Repository Archive
 rem    SaveLogs.BAT ........... In Script Repository Archive
 rem    SetDrive.BAT ........... In Script Repository Archive
 rem    StorageInfo.BAT ........ In Script Repository Archive
 rem    ZipFiles.BAT ........... In Script Repository Archive
 rem    ZipLogs.BAT ............ In Script Repository Archive
 rem ==========================================================================

 :::  This script uses SCHTASKS to create default maintenance tasks on a given
 :::  system.  The tasks scheduled are based on other scripts in this archive.
 :::  If the SCHTASKS executable cannot be found on the system, then the job
 :::  will be scheduled using the AT command (which is not nearly as flexible).
 :::
 :::  The version of SCHTASKS provided with Windows 2003 supports a few
 :::  additional parameters, such as /F, that fail under other versions of
 :::  Windows (2000 and XP), so a special version test is executed to
 :::  determine which OS is running, and use the appropriate switches.
 :::
 :::  NOTE: The AT command under NT4 may behave inconsistently when spaces and
 :::  long file names are involved as parameters.  (NOW OBSOLETE)
 :::
 :::  (Dec 2012) Made it possible for the scheduled tasks to be stored in their
 :::  own folder, rather than the default root folder of the Task Scheduler.
 :::
 :::  (Jan 2013) v4.0 -- This script will now ensure that jobs are only stored
 :::  in the current custom folder of the Task Scheduler.  Older jobs with the
 :::  same name found in the root or another custom folder, will be deleted.
 :::
 :::  (Jun 2014) v4.2 -- The Task Scheduler history in Windows 2012 does not
 :::  appreciate the single quote character in the job name, so it has been
 :::  replaced by square brackets [].
 :::
 :::  (Oct 2014) v4.3 -- Added a hash-based full-drive integrity checker
 :::  (originally based on FCIV, later FSUM, but finally BrainWave FileHash).
 :::
 :::  (Mar 2015) v4.6 -- Supports the change of the @ and $ parameters in the
 :::  CheckIntegrity.BAT script.
 :::
 :::  (Apr 2015) v4.8 -- Scheduled jobs on Windows 8/2012 which contain the []
 :::  characters, do not always execute. As a result, these have been removed
 :::  from the script.
 :::
 :::  (May 2015) v4.9 -- More Windows 8/2012 woes: If the executable name is
 :::  in quotes, AND the next parameter is in quotes, the job might fail to
 :::  run successfully, so these have been changed in each relevant script.
 :::
 :::  (May 2018) v5.0 -- Changed the HELP routine to use FINDSTR;
 :::  -- Added new Filtered CMD Title and Syntax Help processing routines;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com
 :::
 :::  (Jul 2018) v5.1 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::
 :::  (Aug 2018) v6.0 -- Eliminated all NT4 and 2000 support
 :::
 :::  (Oct 2018) v6.1 -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Nov 2018) v6.2 -- Moved to having jobs implement their native scheduling
 :::  routines so that any custom schedules can be derived from the config files;
 :::  -- Allow CORE tasks to be scheduled with the "@" parameter;
 :::
 :::  (Feb 2019) v6.3 -- Enabled Code Page 1252 Support;
 :::
 :::  (Jun 2019) v6.4 -- Added Support for CreateArchive SystemState;
 :::  -- Use CheckParams to process extended parameters;  Added more parameter options;
 :::
 :::  (Oct 2019) v6.5 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (May 2020) v6.7 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::
 :::  (Nov 2020) v6.8 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Jan 2022) v6.9 -- Added CPU_Benchmark and SS Backups to the CORE list;
 :::  -- Add Replicate-Files for backing up local UrBackup configuration files;
 :::
 :::  (Feb 2022) v7.0 -- Added verification for enabling UrBackup job;
 :::  -- Added verification for enabling various jobs related to domain controllers;
 :::  -- Added :CheckEssentials routine to ensure the existence of critical files;
 :::
 :::  (Oct 2023) v7.1 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::  (Jan 2024) v7.2 -- Place :CheckEssentials routine after :SyntaxHelp routine;
 :::
 :::  (Mar 2024) v7.3 -- Added CheckWAN to the daily schedule;
 :::
 :::  (Apr 2024) v7.4 -- Added DateMemoryDump to the daily schedule;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 30 Nov 2020 / 08 May 2018
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 04 Apr 2024 / 02 Apr 2024 / 01 Apr 2024 / 15 Mar 2024
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v7.4.0.542

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Check for Essential Utilities and/or Scripts
 SET @ESSENTIALS="%~dp0SetDrive.BAT" CHECKPARAMS.EXE DATEINFO.EXE NOW.EXE READCONFIG.EXE CHCP.COM FIND.EXE
 PATH %PATH%;%~dp0
 FINDFILES -w -m -f %@ESSENTIALS% FINDSTR.EXE SCHTASKS.EXE
 IF ERRORLEVEL 64 GOTO :ExitBatch

 rem -- Process Special Command-line Parameters
 SET @ALL_PARAMS=%*
 SET @ONLYCORE=& CALL :GetParams "@ C" & IF DEFINED @OK SET @ONLYCORE=Y
 SET @ALTJOBS=Y& CALL :GetParams "$ L" & IF DEFINED @OK SET @ALTJOBS=

 rem -- Variables Related to Installed Services
 CALL :Check4Service @AD_RUNNING   NTDS
 CALL :Check4Service @DNS_RUNNING  DNS
 CALL :Check4Service @DHCP_RUNNING DHCPServer
 CALL :Check4Service @URB_CLIENT   UrBackupClientBackend
 CALL :Check4Service @URB_SERVER   UrBackupWinServer
 SET @URBACKUP=%@URB_CLIENT%%@URB_SERVER%

 rem -- Job Scheduling Variables
 SET @BATCH=%~d0\Scripts\BAT
 SET @KEEP_ONLINE=100
 SET @ARCHIVE_AFTER=370

 SET @JOB01NAME=defrag local drives
 SET @JOB01USER=SYSTEM
 SET @JOB01FREQ=WEEKLY
 SET @JOB01TIME=01:45:00
 SET @JOB01DATE=/D SAT
 SET @JOB01PATH="%@BATCH%\DefragSys.BAT"
 SET @JOB01CORE=

 SET @JOB02NAME=Save Event Logs
 SET @JOB02USER=SYSTEM
 SET @JOB02FREQ=DAILY
 SET @JOB02TIME=01:00:00
 SET @JOB02DATE=
 SET @JOB02PATH="%@BATCH%\SaveLogs.BAT"
 SET @JOB02CORE=TRUE

 SET @JOB03NAME=CheckDisk
 SET @JOB03USER=SYSTEM
 SET @JOB03FREQ=MONTHLY
 SET @JOB03TIME=01:30:00
 SET @JOB03DATE=/D 1
 SET @JOB03PATH="%@BATCH%\CheckDisk.BAT"
 SET @JOB03CORE=

 SET @JOB04NAME=SystemState Backup
 SET @JOB04USER=SYSTEM
 SET @JOB04FREQ=DAILY
 SET @JOB04TIME=00:15:00
 SET @JOB04DATE=
 SET @JOB04PATH="%@BATCH%\BackupSched.BAT"
 SET @JOB04CORE=TRUE

 SET @JOB05NAME=Track Disk Space Usage
 SET @JOB05USER=SYSTEM
 SET @JOB05FREQ=DAILY
 SET @JOB05TIME=01:30:00
 SET @JOB05DATE=
 SET @JOB05PATH="%@BATCH%\StorageInfo.BAT"
 SET @JOB05CORE=TRUE

 SET @JOB06NAME=Local diagnostics Report - Networking
 SET @JOB06USER=SYSTEM
 SET @JOB06FREQ=WEEKLY
 SET @JOB06TIME=04:15:00
 SET @JOB06DATE=/D SUN
 SET @JOB06PATH="%@BATCH%\IPDebug.BAT"
 SET @JOB06CORE=TRUE

 SET @JOB07NAME=Local diagnostics Report - System
 SET @JOB07USER=SYSTEM
 SET @JOB07FREQ=WEEKLY
 SET @JOB07TIME=04:30:00
 SET @JOB07DATE=/D SUN
 SET @JOB07PATH="%@BATCH%\Debug.BAT"
 SET @JOB07CORE=

 SET @JOB08NAME=Daily CPU Benchmark
 SET @JOB08USER=SYSTEM
 SET @JOB08FREQ=HOURLY /MO 12
 SET @JOB08TIME=00:03:00
 SET @JOB08DATE=
 SET @JOB08PATH="%@BATCH%\CPU_Benchmark.BAT"
 SET @JOB08CORE=TRUE

 SET @JOB09NAME=Record WAN IPs
 SET @JOB09USER=SYSTEM
 SET @JOB09FREQ=DAILY
 SET @JOB09TIME=00:10:00
 SET @JOB09DATE=
 SET @JOB09PATH="%@BATCH%\CheckWAN.BAT"
 SET @JOB09CORE=TRUE

 SET @JOB10NAME=Date Memory Dump Files
 SET @JOB10USER=SYSTEM
 SET @JOB10FREQ=DAILY
 SET @JOB10TIME=00:50:00
 SET @JOB10DATE=
 SET @JOB10PATH="%@BATCH%\DateMemoryDump.BAT"
 SET @JOB10CORE=TRUE

 rem -- Set Global Variables from Centralized Script
 CALL "%~dp0SetDrive.BAT" "%~n0"

 rem -- List of All Active Jobs
 SET @CHK_SCHED=SCHTASKS /QUERY /V /FO LIST
 SET @JOBLIST=& FOR %%I IN (0 1 2) DO FOR %%J IN (0 1 2 3 4 5 6 7 8 9) DO IF DEFINED @JOB%%I%%JNAME SET @JOBLIST=!@JOBLIST! %%I%%J
 IF %@OSBUILD% LSS 6000 SET @TASKFOLDER=

 rem -- Log Path Variables
 SET @LOGPATH=%@STORAGE%\Logs\Maint
 SET @LOGFILE=%@LOGPATH%\SchedJobs.%@FILESTAMP%


 rem ==========================================================================
 rem === Schedule All Jobs
 rem === Updated On: 15 Mar 2024 / 17 Feb 2022
 rem ==========================================================================
:ScheduleJobs
 IF NOT EXIST "%@LOGPATH%" MD "%@LOGPATH%" >NUL

 rem -- Initiate Task: Kill Long Running Processes
 CALL "%~dp0KillLongProcess.BAT" .

 rem -- Initiate Task: Manually Backup SystemState
 CALL "%~dp0CreateArchive.BAT" SystemState -s

 rem -- Initiate Task: Backup Script Repository
 IF NOT DEFINED @ONLYCORE CALL "%~dp0CreateArchive.BAT" ScriptRepository -s

 rem -- Initiate Task: Archive UrBackup Configuration Files
 IF DEFINED @URBACKUP (
	 CALL "%~dp0CreateArchive.BAT"   UrBackup -s
	 CALL "%~dp0Replicate-Files.BAT" UrBackup-Config .
 )

 rem -- Initiate Task: Backup of DNS, DHCP and GPOs
 IF DEFINED @ALTJOBS (
	IF DEFINED @AD_RUNNING   CALL "%~dp0BackupGPO.BAT" .
	IF DEFINED @DNS_RUNNING  CALL "%~dp0BackupDNS.BAT" .
	IF DEFINED @DHCP_RUNNING CALL "%~dp0BackupDHCP.BAT" .
 )

 rem -- Initiate Task: Check File Integrity
 CALL "%~dp0CheckIntegrity.BAT" $ .

 rem -- Initiate Task: Create Monthly Script Archive
 IF NOT DEFINED @ONLYCORE CALL "%~dp0ZipFiles.BAT" "%SystemDrive%\Scripts" "" "" .

 rem -- Initiate Task: Create Monthly Log Archive
 CALL "%~dp0ZipLogs.BAT" "%@STORAGE%\Logs" "" "" .

 rem -- Initiate Task: Create Monthly Windows Log Archive
 CALL "%~dp0ZipLogs.BAT" "%SystemRoot%\System32\LogFiles" "" "" .

 rem -- Initiate Tasks: Move Zip Archives
 IF DEFINED @ALTJOBS FOR %%J IN (Log Script) DO CALL "%~dp0MoveArchives.BAT" %%JZips .

 rem -- Delete Old Scheduled Tasks as necessary
 FOR %%z IN (%@JOBLIST%) DO (
	 SET @THISTASK=!@JOB%%zNAME!
	 SET @TASKCHK=!@THISTASK:[=\[!
	 SET @TASKCHK=!@TASKCHK:]=\]!

	 IF "%@FORCE%"=="" ( rem - Delete from Windows XP/2003
		 SCHTASKS /DELETE /TN "!@THISTASK!" /F
	 ) ELSE (
		 IF %@OSBUILD% GEQ 6000 ( rem - Delete from Vista/2008+ [in Old Custom Folders]
			 FOR /F "TOKENS=1* DELIMS=\ " %%s IN ('%@CHK_SCHED% 2^>NUL ^| FIND "TaskName:" ^| FINDSTR /I /R /C:"!@TASKCHK!$"') DO (
				 IF /I NOT "%%t"=="%@TASKFOLDER%!@THISTASK!" SCHTASKS /DELETE /TN "%%t" /F >NUL 2>&1
			 )
		 )
	 )
 )

 rem -- Schedule Other Jobs for Windows 2003+
 FOR %%s IN (%@JOBLIST%) DO (
	 SET @NEWJOB=%@TASKFOLDER%!@JOB%%sNAME!
	 SET @OLDJOB=!@NEWJOB:[='!
	 SET @OLDJOB=!@OLDJOB:]='!
	 IF NOT "!@OLDJOB!"=="!@NEWJOB!" (
		 ECHO  Checking for the existence of old jobs: !@OLDJOB!
		 SCHTASKS /DELETE /TN "!@OLDJOB!" /F
	 )
	 IF DEFINED @ONLYCORE (
		 IF DEFINED @JOB%%sCORE SCHTASKS /CREATE /TN "!@NEWJOB!" /TR "!@JOB%%sPATH!" /SC !@JOB%%sFREQ! !@JOB%%sDATE! /ST !@JOB%%sTIME! /RU "!@JOB%%sUSER!"  %@FORCE%
	 ) ELSE (
		 SCHTASKS /CREATE /TN "!@NEWJOB!" /TR "!@JOB%%sPATH!" /SC !@JOB%%sFREQ! !@JOB%%sDATE! /ST !@JOB%%sTIME! /RU "!@JOB%%sUSER!"  %@FORCE%
	 )
 )
 SLEEP 10
 SCHTASKS /QUERY /V /FO LIST >"%@LOGFILE%"


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019
 rem ==========================================================================
:ExitBatch
 IF EXIST "%@LOGFILE%" (TYPE "%@LOGFILE%" & DIR "%@LOGFILE%")
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Use CheckParams.exe to Capture Selected Parameters
 rem === Updated On: 21 Jan 2020 / 24 Sep 2019
 rem ==========================================================================
:GetParams
 rem %1 = Parameters to Search For
 rem %2 = Variable to Set

 SET @OK=& IF "%~1"=="" GOTO :EOF
 FOR /F "TOKENS=2-3*" %%v IN ('CHECKPARAMS -q -a -c "%~1" -s %@ALL_PARAMS% 2^>NUL') DO IF /I "%%~v"=="TRUE" (
	 SET @OK=T
	 SET @REQ=T
	 IF NOT "%~2"=="" SET %~2=%%~x
 )
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Determine if a Service is Installed
 rem === Updated On: 21 Feb 2022 / 17 Feb 2022
 rem ==========================================================================
:Check4Service
 rem %1 = Variable to Set
 rem %2 = Service to Check

 IF "%~1"=="" GOTO :EOF
 IF "%~2"=="" GOTO :EOF

 SET %~1=
 FOR /F "TOKENS=4" %%S IN ('SC QUERY "%~2" ^| FIND "STATE"') DO IF "%%~S"=="RUNNING" SET %~1=TRUE
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET @SCRIPT_BEG#="%%~d"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 30 Nov 2020 / 23 Jun 2019
 rem ==========================================================================
:HelpMessage
 ECHO Schedule Important Maintenance Jobs
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  [OPTIONS]
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:
 ECHO -------------------
 ECHO OPTION DEFINITIONS:
 ECHO -------------------
 ECHO  -?,  /H ..... Display This Help Message  (also --HELP)
 ECHO  -??, /H2 .... Display Extended Help Message
 ECHO   @,  /C ..... Only Schedule Core Jobs
 ECHO   $,  /L ..... Only Schedule Jobs Requiring LOCAL SYSTEM Credentials
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0
	 ECHO  %~n0 $
	 ECHO  %~n0 /L
	 ECHO  %~n0 @
	 ECHO  %~n0 /C
	 ECHO  %~n0 -?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
 )
 CALL :ShowStatus "FINISHED"
                                                                    