@ECHO OFF
 rem ==========================================================================
 rem === Configure EULA Acceptance for SysInternals Utils on Remote Systems
 rem ==========================================================================
 rem
 rem Created On: 10 Nov 2006
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Oct 2023 / 30 Nov 2020              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2000 or later
 rem
 rem  Non-Native Utilities Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    Resource Kit ........... https://en.wikipedia.org/wiki/Resource_Kit
 rem    psEXEC ................. https://technet.microsoft.com/en-us/sysinternals/bb896649
 rem
 rem  Input Files Used By Script (stored in %SystemDrive%\Scripts\Bat\Input):
 rem    SysInternals.TXT ....... List of All SysInternals Executables
 rem    ServerList.TXT ......... List of Machines to Configure for SysInternals EULA
 rem
 rem  Required Scripts:
 rem    SetDrive.BAT ........... Set Global Variables for this Environment
 rem ==========================================================================

 :::  This script uses some Resource Kit utilities to configure the acceptance
 :::  of the EULA agreement for SysInternals tools so that they can be easily
 :::  executed against remote systems via PSEXEC, without generating any
 :::  prompts that it would not be possible to accept by the remote system or
 :::  user.
 :::
 :::  The script will now accept a second parameter to force an update if it
 :::  had previously been run against the same system successfully.  Any
 :::  character will suffice to force the update.   It will also compare the
 :::  timestamp of the SysInternals.TXT file and force an update if the file
 :::  has been modified more recently than the last execution time.
 :::
 :::  If this script is run at the console, it will display the final log file
 :::  upon completion, but if it is run as part of a scheduled task, then it
 :::  simply exit on completion.  The logfile will be generated to the default
 :::  Log folder in either event.
 :::
 :::  (Nov 2010) Changed the default log extension of the reports from .LOG to
 :::  .TXT so that the reports that are mailed to mobile devices can be read
 :::  automatically. (Not too many devices can natively handle .LOG files)
 :::
 :::  (Jul 2018) v2.0 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::  -- Added new Filtered CMD Title and Syntax Help processing routines;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com
 :::
 :::  (Oct 2018) v2.1 -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Feb 2019) v2.2 -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Oct 2019) v2.3 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (May 2020) v2.4 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::
 :::  (Nov 2020) v2.5 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Oct 2023) v2.6 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 30 Nov 2020 / 26 Jul 2018              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 30 Nov 2020 / 26 May 2020              
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v2.6.0.260

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- SysInternals EULA Registry Variables
 FOR %%F IN ("%~dp0Input\SysInternals.TXT") DO SET @SI_TIMESTAMP=%%~tF
 SET @RESET=%~2
 SET @STATUS=New Entry& IF DEFINED @RESET SET @STATUS=Manual Update
 SET @TEST_KEY=HKLM\SOFTWARE\Microsoft\Windows\CurrentVersion\Setup
 SET @TEST_VAL=Bootdir
 SET @EULA_KEY=Software\Sysinternals
 SET @EULA_VAL=DateConfigured
 SET @EULA_NEW=SourceTimeStamp
 SET @UTILNAMES=%~dp0Input\SysInternals.TXT

 rem -- Main Variables
 SET @CONNECT=\\%~1&  IF "%~1"=="" SET @CONNECT=%COMPUTERNAME%
 SET @CONNECT=\\%@CONNECT:\=%
 SET @LOGPATH=%@STORAGE%\Logs\Maint
 SET @LOGFILE="%@LOGPATH%\%~n0-%@CONNECT:\\=%.%@FILESTAMP%"

 rem -- If The User Account is One of the Special Accounts, Log Output to a single file for the day
 IF /I "%USERNAME%"=="" SET @LOGFILE="%@LOGPATH%\%~n0-%@CONNECT:\\=%.%@TODAY%.TXT"


 rem ==========================================================================
 rem === Configure SysInternals EULA Settings using System List
 rem === Updated On: 27 Feb 2019 / 05 Feb 2015              
 rem ==========================================================================
:ConfigureEULA
 IF NOT EXIST "%@LOGPATH%" MD "%@LOGPATH%"

 ( rem -- Write to Log
	 ECHO %@TIMESTAMP%: Configuring SysInternals EULA Acceptance Settings [%@VER%]
	 ECHO %@MINORDIV%
	 ECHO:
 ) >%@LOGFILE%

 ECHO:
 ECHO %@TIMESTAMP%: %@VER%
 ECHO:
 ECHO Configuring SysInternals EULA Acceptance Settings on: "%@CONNECT%"
 ECHO Configuring SysInternals EULA Acceptance Settings on: "%@CONNECT%" >>%@LOGFILE% 2>&1

:VerifyAccess
 SET @REGCHECK=System Unavailable
 FOR /F %%A IN ('REG QUERY "%@CONNECT%\%@TEST_KEY%" /v "%@TEST_VAL%"') DO SET @REGCHECK=%%A

 IF NOT "%@REGCHECK%"=="%@TEST_VAL%" (
	 ECHO !@CONNECT! is not available, or access was denied. [!@REGCHECK!] >>%@LOGFILE%
 ) ELSE (
	 FOR /F %%P IN ('REG QUERY "!@CONNECT!\HKU" ^| FIND "\" ^| FIND /I /V "Classes"') DO (
		 SET @REGCHECK=System Unconfigured

		 rem -- Test to see if the approvals were previously set
		 FOR /F "TOKENS=1,2*" %%R IN ('REG QUERY "!@CONNECT!\%%~P\%@EULA_KEY%" ^| FIND "%@EULA_VAL%"') DO (
			 SET @REGCHECK=%%R%@RESET%
			 SET @EULATIME=%%T
		 )

		 rem -- If Old Date Format Was Used For Approval, Update Registry
		 FOR /F %%O IN ('ECHO !@EULATIME! ^| FIND "-"') DO (
			 SET @STATUS=Automatic Update
			 SET @REGCHECK=FALSE
		 )

		 rem -- If Approval Date is Older Than Source File Timestamp, Update Registry
		 IF "%@SI_TIMESTAMP%" GTR "!@EULATIME!" (
			 SET @STATUS=Automatic Update
			 SET @REGCHECK=FALSE
		 )

		 IF "!@REGCHECK!"=="%@EULA_VAL%" (
			 ECHO  EULA Already Approved for %%~P
			 ECHO  - [Approved on !@EULATIME!]
			 ECHO:
			 ( rem -- Write to Log
				 ECHO  EULA Already Approved for %%~P
				 ECHO  - [Approved on !@EULATIME!]
				 ECHO:
			 ) >>%@LOGFILE% 2>&1
		 ) ELSE (
			 ECHO  Setting Registry for: %%~P
			 ECHO  Setting Registry for: %%~P >>%@LOGFILE% 2>&1

			 ( rem -- Write to Log

				 rem -- Set the TimeStamp and Update Status of the EULA Approvals
				 REG ADD "!@CONNECT!\%%~P\%@EULA_KEY%" /v "%@EULA_NEW%" /t REG_SZ /d "%@SI_TIMESTAMP%" /f
				 REG ADD "!@CONNECT!\%%~P\%@EULA_KEY%" /v "%@EULA_VAL%" /t REG_SZ /d "%@DATE% %@TIME12%" /f
				 REG ADD "!@CONNECT!\%%~P\%@EULA_KEY%" /v "SetByVersion" /t REG_SZ /d "%@VER%" /f
				 REG ADD "!@CONNECT!\%%~P\%@EULA_KEY%" /v "UpdateStatus" /t REG_SZ /d "!@STATUS!" /f
				 REG ADD "!@CONNECT!\%%~P\%@EULA_KEY%" /v "PreviousUpdate" /t REG_SZ /d "!@EULATIME!" /f

				 rem -- Set the EULA Approvals for Each SysInternals Utility
				 FOR /F "TOKENS=*" %%U IN ('TYPE "%@UTILNAMES%"') DO (
					 REG ADD "!@CONNECT!\%%~P\%@EULA_KEY%\%%~U" /v "EulaAccepted" /t REG_DWORD /d 1 /f
				 )

				 rem -- Write to Log: Capture the current registry settings for EULA acceptance
				 REG QUERY "!@CONNECT!\%%~P\%@EULA_KEY%" /s
				 ECHO %@MAJORDIV%
				 ECHO:
				 ECHO:
			 ) >>%@LOGFILE% 2>&1
		 )
	 )
 )

 ( rem -- Write to Log
	 ECHO %@MINORDIV%
	 ECHO:
	 ECHO:
	 ECHO:
 ) >>%@LOGFILE%


 rem ==========================================================================
 rem === Generate Final Report
 rem === Updated On: 27 Feb 2019                     
 rem ==========================================================================
:FinalReport
 IF /I "%@CONSOLE_SESSION%"=="TRUE" START "SysInternals EULA Approvals" %@LOGFILE%
 IF EXIST %@LOGFILE% (
	 ECHO:
	 TYPE %@LOGFILE%
	 DIR %@LOGFILE%
 )


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019              
 rem ==========================================================================
:ExitBatch
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET @SCRIPT_BEG#="%%~d"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 30 Nov 2020 / 27 Feb 2019              
 rem ==========================================================================
:HelpMessage
 ECHO Configure EULA Acceptance for SysInternals Utils on Remote Systems
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  [name of remote system]
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0 \\SERVER1
	 ECHO  %~n0 SERVER002
	 ECHO  %~n0 -?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
 )
 CALL :ShowStatus "FINISHED"
                          