@ECHO OFF
 rem ==========================================================================
 rem === Send Job Status Messages to Central Syslog Servers
 rem ==========================================================================
 rem
 rem Created On: 19 Mar 2014
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Oct 2023 / 16 Feb 2022
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2003 and later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    SysLogGen .............. http://www.snmpsoft.com/freetools/sysloggen.html
 rem
 rem    Resource Kit ........... https://en.wikipedia.org/wiki/Resource_Kit
 rem ==========================================================================

 :::  This script, derived from TestSysLog.BAT, is designed to send syslog
 :::  status messages on behalf of other scripts. The primary usage is with
 :::  startup/shutdown/logon/logoff scripts, but it is not limited to this. It
 :::  is only necessary to have one of the above mentioned syslog generation
 :::  utilities.  SyslogGen is preferred.
 :::
 :::  (Feb 2018) v1.4 -- Changed Script Documentation to www.BrainWaveCC.com
 :::
 :::  (Apr 2018) v1.5 -- Added filtered CMD Title Routine;
 :::  -- Support for NeoLogger has been eliminated from this script;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com;
 :::
 :::  (Jan 2019) v1.6 -- Updated Debug Routine; Improved Logging
 :::
 :::  (Oct 2019) v1.7 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (Jan 2020) v2.0 -- Refactored the :Prepare4Syslog routine;
 :::  -- Support for NeoLogger has been eliminated from this script;
 :::
 :::  (May 2020) v2.1 -- Updated :DebugMode routine to support the STEP option;
 :::
 :::  (Jul 2020) v2.2 -- Add Version Information to the History Log;
 :::
 :::  (Oct 2023) v2.3 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 31 May 2020 / 17 May 2018
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Enable Debug Mode If 'Debug' Variable is Set to TRUE/LOG/STEP/VERBOSE
 rem === Updated On: 31 May 2020 / 04 Feb 2019
 rem ==========================================================================
:DebugMode
 SET #DEBUG=& FOR %%d IN (LOG STEP TRUE VERBOSE) DO IF /I "%Debug%"=="%%~d" SET #DEBUG=TRUE
 IF DEFINED #DEBUG (
	 SET @DIAGMODE=TRUE
	 SET @DEBUG_MODE=[DEBUG MODE ON]
	 SET @PAUSE_IF_DEBUG=TIMEOUT 5
	 SET @PRINT_IF_DEBUG=ECHO:^& SET @^& ECHO:^& TIMEOUT 5
	 SET @END_DEBUG_MODE=ECHO:^& ECHO *** DEBUG MODE FINISHED ***^& ECHO:^& SET @^& TIMEOUT 30
	 IF /I "%Debug%"=="LOG" (SET @PAUSE_IF_DEBUG=& SET @PRINT_IF_DEBUG=ECHO:^& SET @^& ECHO:)
	 IF /I "%Debug%"=="STEP" (SET @PAUSE_IF_DEBUG=ECHO ##PAUSE##& SET @PRINT_IF_DEBUG=ECHO ##PRINT##)
	 IF /I "%Debug%"=="VERBOSE" (SET @VERBOSE_OFF=ECHO ON) ELSE (FOR %%t IN (ON OFF) DO SET @VERBOSE_%%t=ECHO %%t)
	 ECHO *** DEBUG MODE = ON ***
	 ECHO ON
 ) ELSE ( FOR %%d IN (DEBUG_MODE PAUSE_IF_DEBUG PRINT_IF_DEBUG END_DEBUG_MODE VERBOSE_ON VERBOSE_OFF) DO SET @%%d=)


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 13 Jul 2020 / 28 Jan 2020
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v2.3.0.230

 rem -- Obtain Date/Time and Supporting Core Variables
 FOR /F "TOKENS=1-2 DELIMS=;" %%P IN ('DATEINFO --Script-Variables 2^>NUL') DO SET %%P=%%~Q

 rem -- Main Variables
 SET @QUIET=-q& IF /I "%Debug%"=="TRUE" SET @QUIET=
 SET @SERVERS=%~1& FOR %%P IN ("" ".") DO IF "%~1"=="%%~P" SET @SERVERS=syslog
 SET @PORTS=%~2& FOR %%P IN ("" ".") DO IF "%~2"=="%%~P" SET @PORTS=514
 SET @SEV=6& FOR %%P IN (0 1 2 3 4 5 6 7) DO IF "%~3"=="%%~P" SET @SEV=%~3
 SET @TAG=%~4& FOR %%P IN ("" ".") DO IF "%~4"=="%%~P" SET @TAG=BWSCRIPT
 SET @MSG=%*& FOR %%P IN ("" ".") DO IF "%~5"=="%%~P" SET @MSG=%1 %2 %3 %4 Completed execution of %@TAG% on %COMPUTERNAME%

 rem -- Set Variables for Syslog Output Utilities (i.e. SyslogGen)
 SET @S_UTILS="%~d0\Scripts\Utils\Misc" "%~d0\Scripts\Utils" "%SystemRoot%\System32"
 SET @SYSLOG_UTILS=SYSLOGGEN
 SET @SYSLOG_UTIL=
 SET @USE_SYSLOGS=FALSE
 SET @NO_SYSLOGGEN=Unable to recognize target address or hostname: No such host is known.
 FOR %%S IN (%@SYSLOG_UTILS%) DO (
	 SET @%%~S=
	 FOR %%P IN (%@S_UTILS%) DO IF EXIST "%%~P\%%~S.EXE" (
		 SET @%%~S=%%~P\%%~S.EXE
		 SET @USE_SYSLOGS=TRUE
		 SET @SYSLOG_UTIL=%%~S
	 )
 )
 CALL :Prepare4Syslog %@MSG%
 %@PRINT_IF_DEBUG%


 rem ==========================================================================
 rem === Cycle Through Syslog Servers on various ports, and attempt messages via TCP and UDP
 rem === Updated On: 28 Jan 2020 / 17 May 2018
 rem ==========================================================================
:SendMessages
 ECHO !@S_MSG!
 IF DEFINED @SYSLOG_UTIL IF /I "%@USE_SYSLOGS%"=="TRUE" FOR %%S IN (%@SERVERS%) DO FOR %%P IN (%@PORTS%) DO (
	 "%@SYSLOGGEN%" -t:%%~S -p:%%~P -f:19 -s:!@SEV! -tg:"%@TAG%" -m:"!@S_MSG2!" -q -hn:%COMPUTERNAME% | FIND /V "%@NO_SYSLOGGEN%"
 )
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 05 Oct 2019 / 25 Mar 2016
 rem ==========================================================================
:ExitBatch
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Execute Dynamic Command From Variable
 rem === Updated On: 19 Mar 2014
 rem ==========================================================================
:RunCMD
 rem %* = Command To Execute

 %*
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Prepare Messages for SyslogGen Syslog Utility
 rem === Updated On: 28 Jan 2020 / 17 May 2018
 rem ==========================================================================
:Prepare4Syslog
 rem %* = Syslog Message to be Cleaned Up

 SET @PREFIX=%1 %2 %3 %4
 SET @S_MSG=%*
 SET @S_MSG=!@S_MSG:%@PREFIX% =!

:TrimLeadingSpaces
 IF "!@S_MSG:~0,1!"==" " (
	 SET @S_MSG=!@S_MSG:~1,9999!
	 GOTO :TrimLeadingSpaces
 )

 SET @S_MSG2=!@S_MSG:\=\x5B!
 SET @S_MSG2=!@S_MSG2:"=\x22!
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 16 Feb 2022
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" SET @SCRIPT_BEG#="%DATE:~4,10% %TIME: =0%"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF
                                                                                   