@ECHO OFF
 rem ==========================================================================
 rem === Obtain Disk Space Stats via SRVINFO and Generate a CSV File
 rem ==========================================================================
 rem
 rem Created On: 19 May 2006
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Oct 2023 / 30 Nov 2020              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: NT4 or later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    Blat ................... http://www.blat.net/
 rem    Resource Kit ........... https://en.wikipedia.org/wiki/Resource_Kit
 rem
 rem  Input Files Used By Script (stored in %SystemDrive%\Scripts\Bat\Input):
 rem    CustomVariables.TXT .... Centralized Configuration Settings for Scripts
 rem
 rem  Required Scripts:
 rem    SetDrive.BAT ........... Set Global Variables for this Environment
 rem ==========================================================================

 :::  This script uses SRVINFO from the Windows Server Resource Kit to extract
 :::  hard disk information from remote systems, and save the output in a LOG
 :::  file that is later  renamed to .CSV (upon completion of the script).  The
 :::  PSINFO tool from SysInternals is also a viable utility for this script,
 :::  and has the significant advantage of being able to grab disk information
 :::  without relying on the drives to be shared, but it is much slower than
 :::  the SRVINFO utility.
 :::
 :::  The default input file is "C:\Scripts\Bat\Input\LowDisk.TXT" but you
 :::  select one at the command line.
 :::
 :::  The output is kept as .TXT until completion so that you can tell when
 :::  you can actually open the file with Excel (or other desired application).
 :::
 :::  This script was derived from the DriveInfo.BAT and OpsLogs.BAT scripts,
 :::  but modified for better readability in a spreadsheet app.
 :::
 :::  (Nov 2010) Changed the default log extension of the reports from .LOG to
 :::  .TXT so that the reports that are mailed to mobile devices can be read
 :::  automatically. (Not too many devices can natively handle .LOG files)
 :::
 :::  (Apr 2016) v2.6.4 -- Changed all instances of @RECIPIENTS to @RCPT-TO to
 :::  be consistent with all other scripts.
 :::
 :::  (Jul 2018) v3.0 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::  -- Added new Filtered CMD Title and Syntax Help processing routines;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com
 :::
 :::  (Oct 2018) v3.1 -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Oct 2019) v3.2 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (May 2020) v3.3 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::
 :::  (Nov 2020) v3.4 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Oct 2023) v3.5 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 30 Nov 2020 / 25 Feb 2018              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 30 Nov 2020 / 26 May 2020              
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v3.5.0.350

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Email Variables
 SET @MAIL=TRUE
 SET @MAILSERVER=smtp.mydomain.com
 SET @MAILSENDER=Alerts@MyDomain.com
 SET @RCPT-TO=-to Admin@MyDomain.com
 SET @SUBJECT=Disk Space Report [Total, Free, Used]

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- Main Variables
 SET /A @ROW=5
 SET @SERVERLIST=%~1& IF "%~1"=="" SET @SERVERLIST=%@LOWDISKLIST%

 rem -- Check for Test Variables
 FOR %%d IN ("" - /) DO FOR %%P IN (T TEST) DO IF /I "%~1"=="%%~d%%~P" (
	 SET @MAIL=TRUE
	 SET @SERVERLIST=%@TESTSRVLIST%
	 SET @RCPT-TO=%@TEST-RCPTS%
 )

 rem -- Standard Log Variables
 SET @LOGPATH=%@STORAGE%\Logs\LowDiskReports
 SET @LOGFILE=%@LOGPATH%\%~n0.%@FILESTAMP%
 SET @SUMMARY=%@LOGPATH%\%~n0_SUMMARY.TXT
 SET @NEWFILE=%@LOGFILE:.TXT=.CSV%


 rem ==========================================================================
 rem === Display Header Information To Console and Logfile
 rem === Updated On: 05 Feb 2015 / 23 Oct 2013              
 rem ==========================================================================
:HeaderInfo
 IF NOT EXIST "%@LOGPATH%" MD "%@LOGPATH%"
 ECHO %@TIMESTAMP% -- Generated By %@VER%
 ECHO Generating Disk Space Stats for "%@SERVERLIST%"
 ECHO:

 ( rem -- Write to Log
	 ECHO %@TIMESTAMP% -- Generated By %@VER%
	 ECHO Historical Low Disk Space Logs: \\%COMPUTERNAME%\LOGS$\LowDiskReports
	 ECHO:
	 ECHO %@MAJORDIV%
	 ECHO *** DISK SPACE STATS for Server List: "%@SERVERLIST%"
	 ECHO ***
	 ECHO ***
	 ECHO *** Additional Information Can Be Found Below:
	 ECHO *** -- http://Home.ASBzone.com/forums/post/1111.aspx
	 ECHO ***
	 ECHO %@MAJORDIV%
	 ECHO:
 ) >"%@SUMMARY%" 2>&1

 ( rem -- Write to Log
	 ECHO ,%@TIMESTAMP% -- Generated By %@VER%
	 ECHO ,Generating Disk Space Stats for Servers [%@SERVERLIST%]
	 ECHO:
	 ECHO ,,,---- CRITICAL DISK SPACE CONSUMPTION STATISTICS ----
	 ECHO SERVER NAME,DRIVE,FILESYS,TOTAL,USED,FREE,%% FREE,EXACT %% FREE,STATUS
 ) >>"%@LOGFILE%" 2>&1
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Get Server Information and Output to Logfile
 rem === Updated On: 21 Oct 2019 / 31 May 2006              
 rem ==========================================================================
:GetServerInfo
 FOR /F %%L IN ('TYPE "%@SERVERLIST%"') DO (
	 SET @FOUND=FALSE
	 SET @CONNECT=%%~L
	 SET @CONNECT=\\!@CONNECT:\=!

	 ECHO -- Getting Stats for !@CONNECT! >CON
	 FOR /F "TOKENS=1-5" %%S IN ('SRVINFO -NS !@CONNECT! ^| FIND "$" ^| FIND /V "\"') DO (
		 SET /A @ROW+=1
		 SET /A @FREE=%%V * 100 / %%U
		 SET @ALARM=& IF !@FREE! LEQ 10 SET @ALARM=---LOW---
		 SET @DRIVE=%%S
		 SET @FOUND=TRUE
		 ECHO !@CONNECT!,!@DRIVE:$=:!,%%T,%%U,%%W,%%V,!@FREE!%%,=F!@ROW!/D!@ROW!,!@ALARM!
	 )

	 IF "!@FOUND!"=="TRUE" ( rem -- If You Get Server Data, Add a Space/Row
		 SET /A @ROW+=1
		 ECHO:
	 )
 ) >>%@LOGFILE% 2>&1
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Display Completion Message to Console
 rem === Updated On: 29 May 2006                     
 rem ==========================================================================
:Completed
 REN "%@LOGFILE%" *.CSV
 ECHO:
 ECHO FINISHED Generating Disk Space Stats for Servers...
 ECHO:
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Mail Log File To NT Services On-Call Staff
 rem === Updated On: 25 Apr 2016 / 30 May 2006              
 rem ==========================================================================
:MailLogs
 IF "%@MAIL%"=="TRUE" (
	 ECHO:
	 ECHO Please Wait: Mailing Logs To %@RCPT-TO%
	 ECHO:
	 BLAT -install %@MAILSERVER% %@MAILSENDER% 5 %@MAILPORT%
	 BLAT "%@SUMMARY%" -attach "%@NEWFILE%" %@RCPT-TO% -s "%@SUBJECT%"
	 ECHO:
	 ECHO:
 )
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Generate Final Job Report
 rem === Updated On: 21 Apr 2015                     
 rem ==========================================================================
:FinalReport
 IF /I "%@CONSOLE_SESSION%"=="TRUE" DIR "%@NEWFILE%" & START "Low Diskspace Logs" "%@NEWFILE%"


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019              
 rem ==========================================================================
:ExitBatch
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Execute Dynamic Command From Variable
 rem === Updated On: 13 Jan 2007                     
 rem ==========================================================================
:RunCMD
 rem %1 = Command To Execute

 %*
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET @SCRIPT_BEG#="%%~d"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 30 Nov 2020 / 26 Jul 2018              
 rem ==========================================================================
:HelpMessage
 ECHO Obtain Disk Space Stats via SRVINFO and Generate a CSV File
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  [list of servers to check ^| OPTIONS]
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:
 ECHO -------------------
 ECHO OPTION DEFINITIONS:
 ECHO -------------------
 ECHO  -?,  /H ..... Display This Help Message  (also --HELP)
 ECHO  -??, /H2 .... Display Extended Help Message
 ECHO   /T, /TEST .. Use the Test List rather than a Full List
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0
	 ECHO  %~n0 /TEST
	 ECHO  %~n0 %~dp0Input\ServerList.TXT
	 ECHO  %~n0 D:\Input\ServerList.TXT
	 ECHO  %~n0 "E:\My Files\Low Servers.TXT"
	 ECHO  %~n0 -?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
	 ECHO:
	 ECHO  * Any parameters which contain spaces should be surrounded by quotes.
	 ECHO    Shortnames *can* be surrounded by quotes if you choose, but it is
	 ECHO    not mandatory by any means.
 )
 CALL :ShowStatus "FINISHED"
                  