@ECHO OFF
 rem =========================================================================
 rem === Generate MRTG Stats for Multiple Subnets
 rem =========================================================================
 rem
 rem Created On: 26 Aug 2001
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Oct 2023 / 30 Nov 2020              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2003 OR later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    MRTG ................... http://oss.oetiker.ch/mrtg/download.en.html
 rem    PERL ................... http://www.perl.org/get.html
 rem
 rem  Input Files Used By Script (stored in %SystemDrive%\Scripts\Bat\Input):
 rem    MRTG-Info.TXT .......... Map of Network IP Subnet Ranges to Pinpoint Suspect Systems
 rem
 rem  Required Scripts:
 rem    SetDrive.BAT ........... Set Global Variables for this Environment
 rem ==========================================================================

 :::  This is part of a family of scripts that facilitate the creation of MRTG
 :::  graphing of servers and networking devices.  You can target either a
 :::  single device, or a class C subnet of devices (xxx.xxx.xxx).  For even
 :::  more flexibility, it is possible to store the configuration info for
 :::  multiple subnets the input file MRTG-Info.TXT, rather than editing the
 :::  batch file for each change you need to make.
 :::
 :::  The input file is also used to storage the SNMP configuration info, the
 :::  location of the executables for PERL and MRTG, and your intended home
 :::  page for the MRTG graphing portal.
 :::
 :::  (Oct 2013) v2.6 -- The delimiter for all parameters can be either the
 :::  "-" or the "/" character.  This includes the help parameters.
 :::
 :::  (Apr 2015) v2.7 -- Updated :ShowStatus routine to track script duration.
 :::
 :::  (Jan 2017) v2.8 -- Explicitly set SNMP v2C options in CFGMAKER
 :::
 :::  PLEASE NOTE: The use of the MRTG-Info.TXT file is optional, but highly
 :::  recommended for flexibility.
 :::
 :::  (Jul 2018) v3.0 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::  -- Added new Filtered CMD Title and Syntax Help processing routines;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com
 :::
 :::  (Oct 2018) v3.1 -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Oct 2019) v3.2 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (May 2020) v3.3 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::
 :::  (Nov 2020) v3.4 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Oct 2023) v3.5 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 ::: --------
 :::  NOTES:
 ::: --------
 :::   * Using the MRTG-Info.TXT config file is optional, but highly recommended.
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 30 Nov 2020 / 30 Jul 2018              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem =========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 30 Nov 2020 / 26 May 2020              
 rem =========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v3.5.0.1340 %~2

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc --mp=1 -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- MRTG SNMP Variables -- Grab from MRTG-Info.TXT -OR- Set Manually
 SET @MRTGJOB=/TN "%@TASKFOLDER%MRTG Statistics Update"
 SET @SNMPOPT=--snmp-options=::5:2::2
 SET @SHOWALL=--show-op-down --no-down --zero-speed=100000000
 SET @INTDESC=--ifdesc=name,alias,descr,ip,eth,nr

 IF EXIST "%@MRTG-INFO%" (
	 FOR /F "USEBACKQ TOKENS=1* DELIMS=,; " %%S IN ("%@MRTG-INFO%") DO CALL :RunCMD SET %%S=%%T
 ) ELSE (
	 SET @MRTGDIR=%SystemDrive%\MRTG\Bin
	 SET @WORKDIR=%SystemDrive%\Inetpub\WWWroot\MRTG
	 SET @PERLBIN=%SystemDrive%\PERL\Bin
	 SET @SNMP=PRIVATE
	 SET @SUBNET1=10.10.10
	 SET @SUBNET2=172.16.1
	 SET @SUBNET3=192.168.1
	 SET @SUBNET4=
	 SET @ALLNETS=%@SUBNET1% %@SUBNET2% %@SUBNET3% %@SUBNET4%
 )

 rem -- Standard Log Variables
 SET @LOGPATH=%@STORAGE%\Logs\Monitoring
 SET @LOGFILE=%@LOGPATH%\%~n0.%@FILESTAMP%

 rem -- Key Variable Verification
 FOR %%V IN (WORKDIR MRTGDIR PERLBIN INDEX SNMP INTERVAL SUBNET1 ALLNETS ALLDEVICES) DO IF NOT DEFINED @%%~V (
	 ECHO:
	 ECHO *** WARNING: The @%%~V variable is undefined.
	 ECHO:
	 ECHO  Please ensure that all configuration info is provided correctly
	 ECHO  within the file: "%@MRTG-INFO%"
	 ECHO:
	 ECHO  Alternatively, this information can be entered directly into this script,
	 ECHO  although this is not recommended because it undermines portability.
	 ECHO %@DIVIDER*%
	 ECHO:
	 GOTO :HelpMessage
 )


 rem =========================================================================
 rem === Determine If Perl Bin Location is Valid
 rem === Updated On: 23 Sep 2011                     
 rem =========================================================================
:VerifyPerl
 IF NOT EXIST "%@PERLBIN%\Perl.exe" (
	 ECHO *** WARNING ***
	 ECHO:
	 ECHO Perl executable could not be found in: %@PERLBIN%
	 ECHO:
	 GOTO: ExitBatch
 )


 rem =========================================================================
 rem === Disable MRTG Scheduled Job Until Reconfig is Complete
 rem === Updated On: 10 Jan 2017 / 21 Apr 2015              
 rem =========================================================================
:DisableMRTG
 ECHO:
 ECHO *** Disabling the MRTG Scheduled Job ***
 SCHTASKS /CHANGE /DISABLE %@MRTGJOB%
 TIMEOUT 15
 ECHO:


 rem =========================================================================
 rem === Configure MRTG by All Listed Networks
 rem === Updated On: 27 Aug 2012 / 04 May 2011              
 rem =========================================================================
:ByAllNetworks
 IF /I "%1"=="ALL" (
	 FOR %%N IN (%@ALLNETS%) DO FOR /L %%I IN (1,1,254) DO (
		 ECHO Searching for %%~N.%%~I ...
		 FOR /F %%V IN ('PING -n 1 %%~N.%%~I ^| FIND /I "bytes="') DO CALL MRTGcfg %%~N.%%~I .
	 )
	 GOTO :ExitBatch
 )


 rem =========================================================================
 rem === Configure MRTG by Subnet
 rem === Updated On: 27 Aug 2012 / 04 May 2011              
 rem =========================================================================
:BySubnet
 IF /I "%1"=="SUBNET" (
	 FOR %%N IN (%~2 %~3 %~4 %~5 %~6 %~7 %~8 %~9) DO FOR /L %%I IN (1,1,254) DO (
		 ECHO Searching for %%~N.%%~I ...
		 FOR /F %%V IN ('PING -n 1 %%~N.%%~I ^| FIND /I "bytes="') DO CALL MRTGcfg %%~N.%%~I .
	 )
	 GOTO :ExitBatch
 )


 rem =========================================================================
 rem === Configure MRTG by Listed IPs
 rem === Updated On: 27 Aug 2012 / 04 May 2011              
 rem =========================================================================
:ByListedIPs
 IF /I "%1"=="LIST" (
	 FOR %%N IN (%@ALLDEVICES%) DO (
		 ECHO Searching for %%~N ...
		 FOR /F %%V IN ('PING -n 1 %%~N ^| FIND /I "bytes="') DO CALL MRTGcfg %%~N .
	 )
	 GOTO :ExitBatch
 )


 rem =========================================================================
 rem === Create MRTG Configuration File for One IP Address
 rem === Updated On: 10 Jan 2017 / 05 Feb 2015              
 rem =========================================================================
:ByIP
 ECHO Configuring MRTG for %~1 ...
 ECHO:
 SET @CFGFILE=MRTG-%~1.CFG
 SET @OKFILE=MRTG-%~1.OK
 SET @MRTGCMD=%@PERLBIN%\Perl cfgmaker --global "WorkDir: %@WORKDIR%" %@INTDESC% %@SNMPOPT% %@SHOWALL% --output %@CFGFILE%

 rem -- Go to MRTG Working Directory, and Create if Necessary
 IF NOT EXIST "%@WORKDIR%" MD "%@WORKDIR%" >NUL
 PUSHD "%@MRTGDIR%"

 rem -- Add Global and Custom MRTG Options to Config File
:SetGlobal
 IF EXIST %@CFGFILE%* DEL %@CFGFILE%*
 ( rem -- Write to Log
	 ECHO ## WorkDir: %@WORKDIR%
	 ECHO ## RunAsDaemon: No
	 FOR /F "TOKENS=3" %%V IN ('TYPE %@CFGFILE% ^| FIND "Interface"') DO ECHO ## Options[%%~V]: growright, bits
 ) >>%@CFGFILE%
 %@MRTGCMD% %@SNMP%@%~1


 rem -- Create MRTG Configuration File for PUBLIC community string
:Special
 FOR %%S IN (%@EXEMPT%) DO (
	 IF /I "%~1"=="%%S" %@MRTGCMD% %@ALTSNMP%@%~1
 )


 rem -- Process the Configuration via Perl
:ParseCFG
 %@PERLBIN%\Perl mrtg %@CFGFILE%
 FOR /F "TOKENS=*" %%F IN ('TYPE "%@CFGFILE%" ^| FIND "<"') DO SET @GOOD=%%~F
 IF /I "%@GOOD%"=="" (
	 DEL %@CFGFILE%*
	 DEL %@OKFILE%
 )
 POPD



 rem =========================================================================
 rem === [Re]Enable MRTG Scheduled Job Once Reconfig is Complete
 rem === Updated On: 21 Apr 2015                     
 rem =========================================================================
:EnableMRTG
 ECHO:
 ECHO *** Enabling the MRTG Scheduled Job ***
 SCHTASKS /CHANGE /ENABLE %@MRTGJOB%
 ECHO:


 rem =========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019              
 rem =========================================================================
:ExitBatch
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Execute Dynamic Command From Variable
 rem === Updated On: 16 Jul 2010                     
 rem ==========================================================================
:RunCMD
 rem %1 = Command To Execute

 %*
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version
 rem %3 = Flag for Hiding Status Message

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET @SCRIPT_BEG#="%%~d"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 IF "%~4"=="" (
	 ECHO:
	 ECHO *** %@SCRIPTSTATUS%: %@VER% [%DATE% at %TIME: =0%] *** %@CHCP_STATUS%
	 ECHO:
 )
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 30 Nov 2020 / 30 Jul 2018              
 rem ==========================================================================
:HelpMessage
 ECHO Creates the MRTG configuration files for a single device, OR for multiple devices [by list], OR for one or more class C subnets by IP.  Using the ALL or LIST parameters instead of an IP or subnet, will retrieve the list of subnets and other configuration data from the MRTG-Info.TXT input file.
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  [single IPv4 address ^| LIST ^| ALL]
 ECHO  %~n0  [SUBNET ^<Class C Subnet^>]
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:
 ECHO -------------------
 ECHO OPTION DEFINITIONS:
 ECHO -------------------
 ECHO  -?,  /H ..... Display This Help Message  (also --HELP)
 ECHO  -??, /H2 .... Display Extended Help Message
 ECHO  LIST ........ Interrogate Only the Devices in the MRTG-Info.TXT List
 ECHO  SUBNET ...... Interrogate All Devices Found in the Provided Class_C Subnet
 ECHO  ALL ......... Interrogate All Devices Found in ALL Subnets in the MRTG-Info.TXT List
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0 192.168.1.1
	 ECHO  %~n0 10.100.100.100
	 ECHO  %~n0 SUBNET 172.16.16
	 ECHO  %~n0 LIST
	 ECHO  %~n0 ALL
	 ECHO  %~n0 -?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
 )
 CALL :ShowStatus "FINISHED"
                                                                                    