@ECHO OFF
 rem =========================================================================
 rem === Generate MRTG Index
 rem =========================================================================
 rem
 rem Created On: 02 Sep 2002
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Oct 2023 / 30 Nov 2020              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2003 OR later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    MRTG ................... http://oss.oetiker.ch/mrtg/download.en.html
 rem    PERL ................... http://www.perl.org/get.html
 rem
 rem  Input Files Used By Script (stored in %SystemDrive%\Scripts\Bat\Input):
 rem    MRTG-Info.TXT .......... Map of Network IP Subnet Ranges to Pinpoint Suspect Systems
 rem
 rem  Required Scripts:
 rem    SetDrive.BAT ........... Set Global Variables for this Environment
 rem ==========================================================================

 :::  This is part of a family of scripts that facilitate the creation of MRTG
 :::  graphing of servers and networking devices.  This script requires no
 :::  parameters, and simply creates an HTML homepage in the root of your
 :::  web server folder specified in the MRTG-Info.TXT file.
 :::
 :::  (Oct 2013) v2.5 -- The delimiter for all parameters can be either the
 :::  "-" or the "/" character.  This includes the help parameters.
 :::
 :::  (Apr 2015) v2.6 -- Updated :ShowStatus routine to track script duration.
 :::
 :::  (Jul 2018) v3.0 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::  -- Added new Filtered CMD Title and Syntax Help processing routines;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com
 :::
 :::  (Oct 2018) v3.1 -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Oct 2019) v3.2 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (May 2020) v3.3 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::
 :::  (Nov 2020) v3.4 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Oct 2023) v3.5 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 ::: --------
 :::  NOTES:
 ::: --------
 :::   * Using the MRTG-Info.TXT config file is optional, but highly recommended.
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 30 Nov 2020 / 30 Jul 2018              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem =========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 30 Nov 2020 / 26 May 2020              
 rem =========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v3.5.0.350

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- MRTG SNMP Variables -- Grab from MRTG-Info.TXT -OR- Set Manually
 IF EXIST "%@MRTG-INFO%" (
	 FOR /F "USEBACKQ TOKENS=1* DELIMS=,; " %%S IN ("%@MRTG-INFO%") DO CALL :RunCMD SET %%S=%%T
 ) ELSE (
	 SET @MRTGDIR=%SystemDrive%\MRTG\Bin
	 SET @WORKDIR=%SystemDrive%\Inetpub\WWWroot\MRTG
	 SET @PERLBIN=%SystemDrive%\PERL\Bin
	 SET @INDEX=%@WORKDIR%\Index.html
	 SET @SNMP=PRIVATE
	 SET @ROUTER=192.168.1.2
	 SET @VPN=192.168.1.205
	 SET @DCs=192.168.1.203 192.168.1.207 192.168.1.209
	 SET @NAS=192.168.1.199
	 SET @MAIL=192.168.1.201
	 SET @SQL=192.168.1.205
	 SET @ALLDEVICES=%@ROUTER% %@NAS% %@DCs% %@VPN% %@MAIL% %@SQL%
 )

 rem -- Key Variable Verification
 FOR %%V IN (WORKDIR MRTGDIR PERLBIN INDEX SNMP INTERVAL SUBNET1 ALLNETS ALLDEVICES) DO IF NOT DEFINED @%%~V (
	 ECHO:
	 ECHO *** WARNING: The @%%~V variable is undefined.
	 ECHO:
	 ECHO  Please ensure that all configuration info is provided correctly
	 ECHO  within the file: "%@MRTG-INFO%"
	 ECHO:
	 ECHO  Alternatively, this information can be entered directly into this script,
	 ECHO  although this is not recommended because it undermines portability.
	 ECHO %@DIVIDER*%
	 ECHO:
	 GOTO :HelpMessage
 )


 rem =========================================================================
 rem === Update MRTG Stats
 rem === Updated On: 23 Sep 2011 / 23 Mar 2011              
 rem =========================================================================
:Main
 PUSHD "%@MRTGDIR%"
 FOR %%V IN (%@ALLDEVICES%) DO CALL :AddStr %%~V

 IF NOT EXIST "%@PERLBIN%\Perl.exe" (
	 ECHO *** WARNING ***
	 ECHO:
	 ECHO Perl executable could not be found in: %@PERLBIN%
	 ECHO:
 ) ELSE (
	 ECHO Setting the MRTG Index for %@MRTG%
	 %@PERLBIN%\Perl indexmaker %@MRTG% -output %@INDEX%
 )
 POPD


 rem =========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019              
 rem =========================================================================
:ExitBatch
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Execute Dynamic Command From Variable
 rem === Updated On: 16 Jul 2010                     
 rem ==========================================================================
:RunCMD
 rem %1 = Command To Execute

 %*
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Build Concatenation String for Index script
 rem === Updated On: 16 Jul 2010 / 26 Aug 2001              
 rem ==========================================================================
:AddStr
 rem %1 = New String to Add

 SET @CFILE="%@MRTGDIR%\MRTG-%~1.CFG"
 SET @TOKEN=
 IF NOT EXIST %@CFILE% GOTO :EOF
 FOR /F "TOKENS=*" %%T IN ('TYPE %@CFILE% ^| FIND /I "%@SNMP%"') DO SET @TOKEN=%%~T
 IF /I NOT "%@TOKEN%"=="" SET @MRTG=%@MRTG% %@CFILE%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET @SCRIPT_BEG#="%%~d"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 30 Nov 2020 / 30 Jul 2018              
 rem ==========================================================================
:HelpMessage
 ECHO Creates the HTML index page for your MRTG graphing portal after running MRTGcfg.BAT
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  {no parameters required}
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:
 ECHO -------------------
 ECHO OPTION DEFINITIONS:
 ECHO -------------------
 ECHO  -?,  /H ..... Display This Help Message  (also --HELP)
 ECHO  -??, /H2 .... Display Extended Help Message
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0
	 ECHO  %~n0 -?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
	 ECHO:
	 ECHO  * Options are case insensitive, and can be prefaced with "-" or "/".
 )
 CALL :ShowStatus "FINISHED"
                          