@ECHO OFF
 rem ==========================================================================
 rem === Generate Server Lists for Script Repository from SystemList.CSV
 rem ==========================================================================
 rem
 rem Created On: 09 Jan 2014
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 12 Jan 2024 / 16 Oct 2023              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2003 and later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    FileDate ............... http://www.ABFsoftware.com/
 rem    Resource Kit ........... https://en.wikipedia.org/wiki/Resource_Kit
 rem
 rem  Input Files Used By Script (stored in %SystemDrive%\Scripts\Bat\Input):
 rem    CustomVariables.TXT .... Centralized Configuration Settings for Scripts
 rem    SystemList.CSV ......... Source File for Systems and System Lists
 rem
 rem  Required Scripts:
 rem    SetDrive.BAT ........... Set Global Variables for this Environment
 rem ==========================================================================

 :::  This script generates the following system lists used by various scripts
 :::  in the repository based upon the information found in SystemList.CSV
 :::  (in the INPUT folder by default):
 :::   --- AD-DCs.TXT
 :::   --- ExchangeList.TXT
 :::   --- ExchangeTest.TXT
 :::   --- FileIO.TXT
 :::   --- FolderDefaults.TXT
 :::   --- LowDisk.TXT
 :::   --- MyServers.TXT
 :::   --- SecurityList.TXT
 :::   --- ServerList.TXT
 :::   --- SmallList.TXT
 :::   --- SUS_Servers.TXT
 :::   --- SUS_Clients.TXT
 :::   --- UptimeList.TXT
 :::
 :::  For additional information about the interaction between this script and
 :::  the SystemList.* files (xlsm and CSV), be sure to review the most recent
 :::  edition of the "Configuring the Script Repository.RTF" document found in
 :::  the INPUT folder.
 :::
 :::  (Dec 2014) v2.0 -- Updated to support v3.0 of the Excel spreadsheet,
 :::  with its enhanced columns for machine type and OS version.
 :::
 :::  (Nov 2015) v2.3 -- Eliminated the generation of Patch_Group_#.TXT files,
 :::  but maintained the existing spreadsheet layout for compatibility.
 :::
 :::  (Aug 2017) v2.4 -- Streamlined the commenting out of systems from all
 :::  INPUT files by placing a "C" in the "IN_DEFAULT" spreadsheet column;
 :::  Updated valid OS entries.
 :::
 :::  (May 2018) v2.5 -- Changed Script Documentation to www.BrainWaveCC.com;
 :::  -- Added new Filtered CMD Title and Syntax Help processing routines
 :::
 :::  (Jul 2018) v2.6 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::
 :::  (Oct 2018) v2.7 -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Oct 2019) v2.8 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (Oct 2019) v2.9 -- Update :PadLeft and :PadRight routines;
 :::
 :::  (Dec 2019) v2.10 -- Check for status of server entry before creating
 :::  custom lists;
 :::
 :::  (Jan 2020) v2.11 -- Consolidate :PadLeft and :PadRight routines;
 :::
 :::  (Mar 2020) v2.12 -- Added ";" to the Main Header of FolderDefaults;
 :::
 :::  (May 2020) v2.13 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::
 :::  (Nov 2020) v2.14 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Feb 2021) v2.15 -- Generate a ";" before all "------------";
 :::
 :::  (Mar 2021) v2.16 -- Timestamp the XLSM and CSV source files;
 :::
 :::  (Feb 2022) v3.0 -- Updated the list of Windows Server/Desktop editions;
 :::  -- Updated to support v3.2 of the Excel spreadsheet, with FileIO*.TXT;
 :::
 :::  (Oct 2023) v3.1 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::  (Jan 2024) v3.2 -- Place :CheckEssentials routine after :SyntaxHelp routine;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 30 Nov 2020 / 07 May 2018              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 12 Jan 2024 / 23 Feb 2022              
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v3.2.0.420

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Check for Essential Utilities and/or Scripts
 PATH %PATH%;%~dp0
 FINDFILES -w -m -f "NOW.EXE" "DATEINFO.EXE" "CHECKPARAMS.EXE" "MAKESTRING.EXE" "READABLE.EXE" "RemoveLine.BAT" "FIND.EXE" "RESOLVE.EXE" "FILEDATE.EXE" "HEAD.EXE"
 IF ERRORLEVEL 64 GOTO :ExitBatch

 rem -- Filename and Configuration Name Variables
 SET @LISTNAMES=AD-DCs ExchangeList ExchangeTest FileIO FileIO.Test FolderDefaults LowDisk MyServers SecurityList ServerList SmallList SUS_Servers SUS_Clients UptimeList
 SET @LISTNAME_VARS=FOLDER_DEF DC_LIST MGMT_LIST SERVER_LIST UPTIME_LIST SEC_LIST SMALL_LIST EXCH_LIST EXCH_TEST LOW_DISK SUS_SERVERS SUS_CLIENTS
 SET @WILDCARDS=FolderDefaults LowDisk SecurityList ServerList SUS_Servers SUS_Clients UptimeList
 SET @WILDCARD_VARS=FOLDER_DEF LOW_DISK SEC_LIST SERVER_LIST SUS_SERVERS SUS_CLIENTS UPTIME_LIST

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- Main Variables
 SET @FILTER=FINDSTR "Setting.date Total:"
 SET @RESOLVE=%@SCRIPTS%\Utils\Resolve.EXE
 SET @NEWTIME=%~2& IF "%~2"=="" FOR /F "DELIMS=:" %%D IN ('ECHO %TIME%') DO SET @NEWTIME=%%D-00-00
 SET @NEWTIME=%@NEWTIME::=-%
 SET @SOURCE=%~1& IF "%~1"=="" SET @SOURCE=%~dp0Input\SystemList.CSV
 SET @DEST= & FOR %%F IN ("%@SOURCE%") DO SET @DEST=%%~dpF
 SET @DEST=!@DEST:\\=!
 SET @DEST=!@DEST::\=_!
 SET @DEST=!@DEST::=_!
 SET @DEST=!@DEST:\=_!
 SET @DEST=!@DEST:__=_!& IF "!@DEST:~-1!"=="_" SET @DEST=!@DEST:~0,-1!
 SET @DESTDIR=%SystemDrive%\Temp\%~n0\!@DEST!
 SET @TEMPFILE=%TEMP%\Temp-%~n0-%@RANDOM%.TEMP
 SET @OS_EDITIONS="Linux " "Workstation" "Desktop" "Server" "Starter" "Home" "Basic" "Premium" "Professional" "Pro"  "Ultimate" "Standard" "Advanced" "Enterprise" "DataCenter" "Storage"
 SET @WINDOWS_DESKTOPS="XP" "Vista" "7" "8" "8.1" "10" "11"
 SET @WINDOWS_SERVERS="NT" "2000" "2003" "2008" "2012" "2016" "2019" "2022"
 SET @WINDOWS_OS=%@WINDOWS_SERVERS% %@WINDOWS_DESKTOPS%

 rem File Header Variables
 SET @MAIN_HEADER=;SERVER_NAME       	LOGS	ROOT 	BACKUPS   	MACHINE_CLASS	MACHINE_TYPE	OPERATING_SYSTEM	DESCRIPTION
 SET @FILE_IO_HDR=;SERVER_NAME,                     SHARE_NAME (Delimiter = ,)

 rem -- Output Files
 SET @DC_LIST=%@DESTDIR%\AD-DCs.TXT
 SET @EXCH_LIST=%@DESTDIR%\ExchangeList.TXT
 SET @EXCH_TEST=%@DESTDIR%\ExchangeTest.TXT
 SET @FOLDER_DEF=%@DESTDIR%\FolderDefaults.TXT
 SET @FILE_IO=%@DESTDIR%\FileIO.TXT
 SET @FILE_IO_TEST=%@DESTDIR%\FileIO.Test.TXT
 SET @LOW_DISK=%@DESTDIR%\LowDisk.TXT
 SET @MGMT_LIST=%@DESTDIR%\MyServers.TXT
 SET @SEC_LIST=%@DESTDIR%\SecurityList.TXT
 SET @SERVER_LIST=%@DESTDIR%\ServerList.TXT
 SET @SMALL_LIST=%@DESTDIR%\SmallList.TXT
 SET @SUS_SERVERS=%@DESTDIR%\SUS_Servers.TXT
 SET @SUS_CLIENTS=%@DESTDIR%\SUS_Clients.TXT
 SET @UPTIME_LIST=%@DESTDIR%\UptimeList.TXT
 SET @ALL_FILES=& FOR %%A IN (%@LISTNAMES%) DO SET @ALL_FILES=!@ALL_FILES! "%@DESTDIR%\%%~A.TXT"
 SET @WHICH_FILES=

 rem -- Standard Log Variables
 SET @LOGPATH=%@STORAGE%\Logs\Maint
 SET @LOGFILE=%@LOGPATH%\%~n0.%@FILESTAMP%
 SET @LOGLIST="%@LOGFILE%"
 %@PRINT_IF_DEBUG%


 rem ==========================================================================
 rem === Display Input/Output Files and Confirm Execution
 rem === Updated On: 30 Mar 2014 / 21 Jan 2014              
 rem ==========================================================================
:ConfirmExecution
 ECHO Preparing to Generate System Lists...
 ECHO  SOURCE CSV ............ %@SOURCE%
 ECHO  DESTINATION FOLDER .... %@DESTDIR%
 ECHO  TIME STAMP ............ %@NEWTIME:-=:%
 ECHO:
 IF /I "%@CONSOLE_SESSION%"=="TRUE" (
	 CHOICE /C YNQEX /T 20 /D Y /N /M "Continue running this script? (Y/N/Q) "
	 IF !ERRORLEVEL! GTR 1 GOTO :ExitBatch
 )
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Check for Valid and Accessible SystemList.CSV file
 rem === Updated On: 23 Feb 2022 / 11 Mar 2021              
 rem ==========================================================================
:CheckSystemList
 ECHO:
 ECHO Checking for Valid CSV File...
 ECHO %@DIVIDER%

 rem -- Check for Existence
 IF NOT EXIST "%@SOURCE%" (
	 ECHO:
	 ECHO *** SOURCE CSV FILE NOT FOUND ***
	 GOTO :ExitBatch
 )

 rem -- Check for Valid Version (v3.2+)
 SET @VALID_SOURCE=TRUE
 FOR /F "TOKENS=1-27 DELIMS=, " %%a IN ('HEAD -1 "%@SOURCE%" 2^>^&1') DO (
	 IF "%%~a"=="" SET @VALID_SOURCE=FALSE
	 IF "%%~b"=="" SET @VALID_SOURCE=FALSE
	 IF "%%~c"=="" SET @VALID_SOURCE=FALSE
	 IF "%%~d"=="" SET @VALID_SOURCE=FALSE
	 IF "%%~e"=="" SET @VALID_SOURCE=FALSE
	 IF "%%~f"=="" SET @VALID_SOURCE=FALSE
	 IF "%%~g"=="" SET @VALID_SOURCE=FALSE
	 IF "%%~h"=="" SET @VALID_SOURCE=FALSE
	 IF "%%~i"=="" SET @VALID_SOURCE=FALSE
	 IF "%%~j"=="" SET @VALID_SOURCE=FALSE
	 IF "%%~k"=="" SET @VALID_SOURCE=FALSE
	 IF "%%~l"=="" SET @VALID_SOURCE=FALSE
	 IF "%%~m"=="" SET @VALID_SOURCE=FALSE
	 IF "%%~n"=="" SET @VALID_SOURCE=FALSE
	 IF "%%~o"=="" SET @VALID_SOURCE=FALSE
	 IF "%%~p"=="" SET @VALID_SOURCE=FALSE
	 IF "%%~q"=="" SET @VALID_SOURCE=FALSE
	 IF "%%~r"=="" SET @VALID_SOURCE=FALSE
	 IF "%%~s"=="" SET @VALID_SOURCE=FALSE
	 IF "%%~t"=="" SET @VALID_SOURCE=FALSE
	 IF "%%~u"=="" SET @VALID_SOURCE=FALSE
	 IF "%%~v"=="" SET @VALID_SOURCE=FALSE
	 IF "%%~w"=="" SET @VALID_SOURCE=FALSE
	 IF "%%~x"=="" SET @VALID_SOURCE=FALSE
	 IF "%%~y"=="" SET @VALID_SOURCE=FALSE
	 IF "%%~z"=="" SET @VALID_SOURCE=FALSE
	 IF "%%~{"=="" SET @VALID_SOURCE=FALSE

	 ECHO  Column #01 = "%%~a"
	 ECHO  Column #02 = "%%~b"
	 ECHO  Column #03 = "%%~c"
	 ECHO  Column #04 = "%%~d"
	 ECHO  Column #05 = "%%~e"
	 ECHO  Column #06 = "%%~f"
	 ECHO  Column #07 = "%%~g"
	 ECHO  Column #08 = "%%~h"
	 ECHO  Column #09 = "%%~i"
	 ECHO  Column #10 = "%%~j"
	 ECHO  Column #11 = "%%~k"
	 ECHO  Column #12 = "%%~l"
	 ECHO  Column #13 = "%%~m"
	 ECHO  Column #14 = "%%~n"
	 ECHO  Column #15 = "%%~o"
	 ECHO  Column #16 = "%%~p"
	 ECHO  Column #17 = "%%~q"
	 ECHO  Column #18 = "%%~r"
	 ECHO  Column #19 = "%%~s"
	 ECHO  Column #20 = "%%~t"
	 ECHO  Column #21 = "%%~u"
	 ECHO  Column #22 = "%%~v"
	 ECHO  Column #23 = "%%~w"
	 ECHO  Column #24 = "%%~x"
	 ECHO  Column #25 = "%%~y"
	 ECHO  Column #26 = "%%~z"
	 ECHO  Column #27 = "%%~{"
	 ECHO:
 )

 IF /I "%@VALID_SOURCE%"=="FALSE" (
	 ECHO:
	 ECHO *** SELECTED CSV FILE DOES NOT MATCH BWCC v3.2 FORMAT ***
	 GOTO :ExitBatch
 )

 CALL "%~dp0RemoveLine.BAT" "%@SOURCE%" ",,,,,,,,,,,,,,,,,,,,,," | FIND "# of Lines Removed: "
 FOR %%F IN ("%@SOURCE%" "%@SOURCE:CSV=XLSM%") DO IF EXIST "%%~F" FILEDATE "%%~F" %@NEWTIME% | %@FILTER%
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Prepare Destination Folders
 rem === Updated On: 23 Feb 2022 / 21 Jan 2014              
 rem ==========================================================================
:CleanDestination
 FOR %%V IN ("%@LOGPATH%" "%@DESTDIR%") DO IF NOT EXIST "%%~V" MD "%%~V"

 IF EXIST "%@DESTDIR%\*.TXT" (
	 ECHO:
	 ECHO Moving Older Configuration Files...
	 ECHO %@DIVIDER%
	 MD "%@DESTDIR%\%@TODAY%-%@NOW%" >NUL 2>&1
	 MOVE "%@DESTDIR%\*.TXT" "%@DESTDIR%\%@TODAY%-%@NOW%"
	 ECHO:
 )
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Cycle Through SystemList.CSV and Process Each Line
 rem === Updated On: 23 Feb 2022 / 11 Mar 2021              
 rem ==========================================================================
:GenerateSystemLists
 CALL :GenerateHeader @MAIN_HEADER 175 >"%@FOLDER_DEF%"   2>&1
 CALL :GenerateHeader @FILE_IO_HDR 60  >"%@FILE_IO%"      2>&1
 CALL :GenerateHeader @FILE_IO_HDR 60  >"%@FILE_IO_TEST%" 2>&1

 ECHO Generating New Output Files...
 ECHO %@DIVIDER%
 FOR /F "SKIP=1 TOKENS=*" %%A IN (%@SOURCE%) DO (
	 SET @THISLINE=%%~A
	 SET @THISLINE=!@THISLINE:^(=[!
	 SET @THISLINE=!@THISLINE:^)=]!
	 SET @THISLINE=!@THISLINE:^&=+!
	 CALL :WriteOneLine !@THISLINE!
 )
 FILEDATE "%@DESTDIR%\*.TXT" %@NEWTIME% | %@FILTER%
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Generate Activity Report
 rem === Updated On: 23 Feb 2022 / 07 May 2018              
 rem ==========================================================================
:GenerateReport
 ( rem -- Write to Log
	 ECHO:
	 ECHO New System Lists Generated - [!DATE! at !TIME: =0!] - %@VER%
	 ECHO:
	 FOR %%V IN (%@ALL_FILES% %@WHICH_FILES%) DO IF EXIST "%%~V" (
		 ECHO %@DIVIDER*%
		 ECHO %%~V
		 ECHO %@DIVIDER*%
		 READABLE "%%~V" --NL 2
	 )
	 %@PRINT_IF_DEBUG%
	 DIR /OGD "%@DESTDIR%\*.TXT"
	 ECHO:
 ) >"%@LOGFILE%" 2>&1
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Perform Log Management/Maintenance (Or Copy Them for Debug Purposes)
 rem === Updated On: 11 Mar 2021 / 11 Jan 2015              
 rem ==========================================================================
:LogMaint
 ECHO:
 ECHO PERFORMING FINAL LOG MAINTENANCE...

 FOR %%L IN (%@LOGLIST%) DO IF EXIST %%L (
	 SET @THISLOG=%%L
	 COPY %%L !@THISLOG:%@FILESTAMP%=TXT! /Y
	 IF /I "%DEBUG%"=="COPY" COPY %%L "%TEMP%" /Y
	 IF %%L=="%@LOGFILE%" (ECHO: & TYPE %%L & DIR %%L) ELSE (DEL %%L >NUL)
 )
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019              
 rem ==========================================================================
:ExitBatch
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Pad Variable Content to the Left/Right for Display in Logs
 rem === Updated On: 22 Jan 2020 / 14 Oct 2019              
 rem ==========================================================================
:PadLeft
:PadRight
 rem %1 = Variable to Store New Content
 rem %2 = Existing Value to Pad
 rem %3 = Character to use as padding [typically " ", "." or "0"]
 rem %4 = New String Length

 SET @STR=%~2
 SET @LEN=& FOR /L %%c IN (0,1,199) DO IF "!@STR:~%%c,1!"=="" IF NOT DEFINED @LEN SET @LEN=%%c
 IF 1%@LEN% GEQ 1%~4 (
	 SET %~1=%~2
 ) ELSE (
	 IF /I "%~0"==":PadLeft" (
		 SET @TEMPVAR=& FOR /L %%c IN (1,1,%~4) DO SET @TEMPVAR=!@TEMPVAR!%~3
		 SET @TEMPVAR=!@TEMPVAR!%~2
		 SET %~1=!@TEMPVAR:~-%~4!
	 )
	 IF /I "%~0"==":PadRight" (
		 SET @TEMPVAR=%~2& FOR /L %%c IN (1,1,%~4) DO SET @TEMPVAR=!@TEMPVAR!%~3
		 SET %~1=!@TEMPVAR:~0,%~4!
	 )
 )
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Generate Header for FolderDefaults.TXT
 rem === Updated On: 23 Feb 2022 / 16 Feb 2021              
 rem ==========================================================================
:GenerateHeader
 rem %1 = Variable With Header Info
 rem %2 = Size of Separator

 SET @LEN=%~2& IF "%~2"=="" SET @LEN=80

 IF NOT "%~1"=="" ECHO !%~1!
 FOR /F %%L IN ('MAKESTRING "-" !@LEN!') DO ECHO ;%%L
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Obtain Variables from SystemList.CSV and Write Files
 rem === Updated On: 23 Feb 2022 / 24 Dec 2019              
 rem ==========================================================================
:WriteOneLine
 rem %* = Full Line with Variables to be Separated

 SET @THISLINE=%*
 SET @THISLINE=!@THISLINE:"=!
 SET @THISLINE=!@THISLINE:,=!
 SET #SERVER_NAME=

 FOR /F "TOKENS=1-26* DELIMS=" %%a IN ('ECHO !@THISLINE!') DO (
	 SET #SERVER_NAME=%%~a&    IF "%%~a"=="" (SET #SERVER_NAME= )
	 SET #IPADDR=%%~b&         IF "%%~b"=="" (SET #IPADDR=0.0.0.0)
	 SET #ENV=%%~c&            IF "%%~c"=="" (SET #ENV= )
	 SET #LOGS=%%~d&           IF "%%~d"=="" (SET #LOGS= )
	 SET #ROOT=%%~e&           IF "%%~e"=="" (SET #ROOT= )
	 SET #BACKUPS=%%~f&        IF "%%~f"=="" (SET #BACKUPS= )
	 SET #FILE_IO=%%~g&        IF "%%~g"=="" (SET #FILE_IO= )
	 SET #CLASS=%%~h&          IF "%%~h"=="" (SET #CLASS= )
	 SET #TYPE=%%~i&           IF "%%~i"=="" (SET #TYPE= )
	 SET #OS_VERSION=%%~j&     IF "%%~j"=="" (SET #OS_VERSION= )
	 SET #DEVICE_PURPOSE=%%~k& IF "%%~k"=="" (SET #DEVICE_PURPOSE= )
	 SET #PATCH_GROUP=%%~l&    IF "%%~l"=="" (SET #PATCH_GROUP=0)
	 SET #FOLDER_DEF=%%~m&     IF "%%~m"=="" (SET #FOLDER_DEF= )
	 SET #DC_LIST=%%~n&        IF "%%~n"=="" (SET #DC_LIST= )
	 SET #MGMT_LIST=%%~o&      IF "%%~o"=="" (SET #MGMT_LIST= )
	 SET #SERVER_LIST=%%~p&    IF "%%~p"=="" (SET #SERVER_LIST= )
	 SET #UPTIME_LIST=%%~q&    IF "%%~q"=="" (SET #UPTIME_LIST= )
	 SET #SEC_LIST=%%~r&       IF "%%~r"=="" (SET #SEC_LIST= )
	 SET #SMALL_LIST=%%~s&     IF "%%~s"=="" (SET #SMALL_LIST= )
	 SET #EXCH_LIST=%%~t&      IF "%%~t"=="" (SET #EXCH_LIST= )
	 SET #EXCH_TEST=%%~u&      IF "%%~u"=="" (SET #EXCH_TEST= )
	 SET #LOW_DISK=%%~v&       IF "%%~v"=="" (SET #LOW_DISK= )
	 SET #FILE_IO_TEST=%%~w&   IF "%%~w"=="" (SET #FILE_IO_TEST= )
	 SET #SUS_SERVERS=%%~x&    IF "%%~x"=="" (SET #SUS_SERVERS= )
	 SET #SUS_CLIENTS=%%~y&    IF "%%~y"=="" (SET #SUS_CLIENTS= )
	 SET #IN_DEFAULT=%%~z&     IF "%%~z"=="" (SET #IN_DEFAULT= )
	 SET #CUSTOM_LISTS=%%~{&   IF "%%~{"=="" (SET #CUSTOM_LISTS=)
 )

 rem -- Skip Processing if Blank Server Entry
 IF NOT DEFINED #SERVER_NAME GOTO :EOF

 rem -- Format OS Version Information
 FOR %%O IN (%@OS_EDITIONS%) DO    SET #OS_VERSION=!#OS_VERSION:%%~O=!
 FOR %%O IN (%@WINDOWS_OS%) DO     SET #OS_VERSION=!#OS_VERSION:Win %%~O=Windows %%~O!
 FOR %%O IN (1 2 3 4 5 6 7 8 9) DO SET #OS_VERSION=!#OS_VERSION:  = !
                                   SET #OS_VERSION=!#OS_VERSION: prietary= Proprietary!

 rem -- Format Output Data for Valid Server Entries
 SET #SERVER_NAME=!#SERVER_NAME:\=!
 CALL :PadRight @SN  "\\!#SERVER_NAME!"   " " 18
 CALL :PadRight @SN2 "!#SERVER_NAME: =!," " " 32
 CALL :PadRight @LG  "!#LOGS!"            " " 4
 CALL :PadRight @RT  "!#ROOT!"            " " 5
 CALL :PadRight @BK  "!#BACKUPS!"         " " 10
 CALL :PadRight @CL  "!#CLASS!"           " " 12
 CALL :PadRight @TY  "!#TYPE!:"           " " 12
 CALL :PadRight @OS  "!#OS_VERSION!"      " " 22
 CALL :PadRight @DP  "!#DEVICE_PURPOSE!"  " " 50
 IF "%#TYPE%%#OS_VERSION%"=="  " (SET @VER_INFO=-	Unknown) ELSE (SET @VER_INFO=%@TY%	%@OS%)
 IF "%#ENV%"==" "  (SET @INFO=%@VER_INFO%	%@DP%) ELSE (SET @INFO=%@VER_INFO%	%#ENV:~0,10%: %@DP%)

 rem -- Determine Content for Special INPUT Files [FileIO, FileIO.Test]
 IF /I NOT "!#FILE_IO!"=="-" (
	 FOR %%W IN (Y F) DO IF /I "!#IN_DEFAULT!"=="%%~W" (
		 ECHO  %@SN2:\=% !#FILE_IO! >>"%@FILE_IO%"
		 IF /I "!#FILE_IO_TEST!"=="Y" ECHO  %@SN2:\=% !#FILE_IO! >>"%@FILE_IO_TEST%"
		 IF /I "!#FILE_IO_TEST!"=="C" ECHO ;%@SN2:\=% !#FILE_IO! >>"%@FILE_IO_TEST%"
	 )
 )

 rem -- Determine Content for Standard INPUT Files [Y]
 IF /I "!#IN_DEFAULT!"=="Y" (
	 FOR %%V IN (%@LISTNAME_VARS%) DO (
		 IF /I "%%~V"=="FOLDER_DEF" (
			 IF /I "!#%%~V!"=="Y" ECHO  %@SN:\=%	%@LG%	%@RT%	%@BK%	%@CL%	%@INFO% >>"!@%%~V!"
			 IF /I "!#%%~V!"=="C" ECHO ;%@SN:\=%	%@LG%	%@RT%	%@BK%	%@CL%	%@INFO% >>"!@%%~V!"
		 ) ELSE (
			 IF /I "!#%%~V!"=="Y" ECHO  %@SN%	%@CL%	%@INFO% >>"!@%%~V!"
			 IF /I "!#%%~V!"=="C" ECHO ;%@SN%	%@CL%	%@INFO% >>"!@%%~V!"
		 )
	 )
 )

 rem -- Determine Content for Standard INPUT Files [C or F]
 FOR %%W IN (C F) DO IF /I "!#IN_DEFAULT!"=="%%~W" (
	 FOR %%V IN (%@LISTNAME_VARS%) DO (
		 IF /I "%%~V"=="FOLDER_DEF" (
			 FOR %%W IN (Y C F) DO IF /I "!#%%~V!"=="%%~W" ECHO ;%@SN:\=%	%@LG%	%@RT%	%@BK%	%@CL%	%@INFO% >>"!@%%~V!"
		 ) ELSE (
			 FOR %%W IN (Y C F) DO IF /I "!#%%~V!"=="%%~W" ECHO ;%@SN%	%@CL%	%@INFO% >>"!@%%~V!"
		 )
	 )
 )

 rem -- Determine Content for Custom INPUT Files
 IF /I "!#IN_DEFAULT!"=="Y" (
	 IF DEFINED #CUSTOM_LISTS (
		 SET @CUSTOM_LISTS=!#CUSTOM_LISTS!
		 SET @CUSTOM_LISTS=!@CUSTOM_LISTS:=,!

		 FOR %%L IN (!@CUSTOM_LISTS!) DO (
			 SET @C_LIST=& FOR /F "TOKENS=1-2 DELIMS=." %%C IN ('ECHO %%~L') DO (
				 IF "%%~C"=="+" (SET @C_LIST=%@WILDCARDS%) ELSE (SET @C_LIST=%%~C)
				 SET @C_SERVER=%%~D
			 )

			 FOR %%S IN (!@C_LIST!) DO FOR %%V IN (%@LISTNAMES%) DO IF /I "%%~S"=="%%~V" (
				 SET @IS_FOLDERDEF=FALSE& IF /I "%%~S"=="FolderDefaults" SET @IS_FOLDERDEF=TRUE
				 SET @THISLIST=%@DESTDIR%\%%~S.!@C_SERVER!.TXT
				 IF NOT DEFINED @%%~S.!@C_SERVER! (
					 BREAK ON >"!@THISLIST!"
					 SET @WHICH_FILES=!@WHICH_FILES! "!@THISLIST!"
					 SET @%%~S.!@C_SERVER!=TRUE
				 )

				 ( rem -- Write to Log
					 IF /I "!@IS_FOLDERDEF!"=="TRUE" (
						 IF NOT DEFINED @HEADER-%%~L ( SET @HEADER-%%~L=TRUE& CALL :GenerateHeader @MAIN_HEADER 175 )
						 IF /I "!@C_SERVER!"=="!#SERVER_NAME!" (SET #FD_TEMP=!#FOLDER_DEF!& SET #FOLDER_DEF=Y)
						 FOR %%W IN (C F) DO IF /I "!#IN_DEFAULT!"=="%%~W" SET #FOLDER_DEF=C
						 IF /I "!#FOLDER_DEF!"=="Y" ECHO  %@SN:\=%	%@LG%	%@RT%	%@BK%	%@CL%	%@INFO%
						 IF /I "!#FOLDER_DEF!"=="C" ECHO ;%@SN:\=%	%@LG%	%@RT%	%@BK%	%@CL%	%@INFO%
						 IF /I "!@C_SERVER!"=="!#SERVER_NAME!" SET #FOLDER_DEF=!#FD_TEMP!
					 ) ELSE (
						 FOR %%W IN (C F) DO IF /I "!#IN_DEFAULT!"=="%%~W" SET #FOLDER_DEF=C
						 IF /I "!#FOLDER_DEF!"=="Y" ECHO  %@SN%	%@CL%	%@INFO%
						 IF /I "!#FOLDER_DEF!"=="C" ECHO ;%@SN%	%@CL%	%@INFO%
					 )
				 ) >>"!@THISLIST!" 2>&1
			 )
		 )
	 )
 )
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET @SCRIPT_BEG#="%%~d"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 11 Mar 2021 / 30 Nov 2020              
 rem ==========================================================================
:HelpMessage
 ECHO Generate System Lists for Script Repository from SystemList.CSV
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  [source_csv] [hh:mm]
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:
 ECHO -------------------
 ECHO OPTION DEFINITIONS:
 ECHO -------------------
 ECHO  -?,  /H ..... Display This Help Message  (also --HELP)
 ECHO  -??, /H2 .... Display Extended Help Message
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0
	 ECHO  %~n0 %~dp0Input\SystemList.CSV
	 ECHO  %~n0 "D:\MyScripts\MyServerList.CSV" 08:00
	 ECHO  %~n0 "%SystemRoot%\Temp\ServerList.CSV" 20:00
	 ECHO  %~n0 /?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
	 ECHO:
	 ECHO  * Long filenames which contain spaces must be surrounded by quotes.
 )
 CALL :ShowStatus "FINISHED"
  