@ECHO OFF
 rem =========================================================================
 rem === Generate MRTG Stats
 rem =========================================================================
 rem
 rem Created On: 26 Aug 2001
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Oct 2023 / 22 Jan 2022              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2003 OR later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    MRTG ................... http://oss.oetiker.ch/mrtg/download.en.html
 rem    PERL ................... http://www.perl.org/get.html
 rem
 rem  Input Files Used By Script (stored in %SystemDrive%\Scripts\Bat\Input):
 rem    MRTG-Info.TXT .......... Map of Network IP Subnet Ranges to Pinpoint Suspect Systems
 rem
 rem  Required Scripts:
 rem    SetDrive.BAT ........... Set Global Variables for this Environment
 rem ==========================================================================

 :::  This is part of a family of scripts that facilitate the creation of MRTG
 :::  graphing of servers and networking devices.  This script requires no
 :::  parameters, and simply creates the schedule job that updates the MRTG
 :::  portal with current SNMP info.
 :::
 :::  (Oct 2013) v2.7 -- The delimiter for all parameters can be either the
 :::  "-" or the "/" character.  This includes the help parameters.
 :::
 :::  (Apr 2015) v2.8 -- Updated :ShowStatus routine to track script duration.
 :::
 :::  (Jul 2018) v3.0 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::  -- Added new Filtered CMD Title and Syntax Help processing routines;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com
 :::
 :::  (Aug 2018) v3.1 -- Removed AT command support;
 :::
 :::  (Oct 2018) v3.2 -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Mar 2019) v3.3 -- Enabled Code Page 1252 Support;
 :::
 :::  (Oct 2019) v3.4 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (May 2020) v3.5 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::
 :::  (Nov 2020) v3.6 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Jan 2022) v3.7 -- Modify the @NAMECHK verification to include \ prefix;
 :::
 :::  (Oct 2023) v3.8 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 ::: --------
 :::  NOTES:
 ::: --------
 :::   * Using the MRTG-Info.TXT config file is optional, but highly recommended.
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 30 Nov 2020 / 30 Jul 2018              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem =========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 30 Nov 2020 / 26 May 2020              
 rem =========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v3.7.0.370

 rem -- Default Variables (Override via CustomVariables.*.TXT)
 SET @INTERVAL=4

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- MRTG SNMP Variables -- Grab from MRTG-Info.TXT -OR- Set Manually
 IF EXIST "%@MRTG-INFO%" (
	 FOR /F "USEBACKQ TOKENS=1* DELIMS=,; " %%S IN ("%@MRTG-INFO%") DO CALL :RunCMD SET %%S=%%T
 ) ELSE (
	 SET @MRTGDIR=%SystemDrive%\MRTG\Bin
	 SET @PERLBIN=%SystemDrive%\PERL\Bin
	 SET @INTERVAL=3
	 SET @SUBNET1=10.10.10
	 SET @SUBNET2=172.16.1
	 SET @SUBNET3=192.168.1
	 SET @SUBNET4=
	 SET @ALLNETS=%@SUBNET1% %@SUBNET2% %@SUBNET3% %@SUBNET4%
 )

 rem -- Job Scheduling Variables
 SET @JOBNAME=MRTG Statistics Update
 SET @JOBUSER=SYSTEM
 SET @JOBFREQ=MINUTE /MO %@INTERVAL%
 SET @JOBTIME=00:00:00
 SET @JOBDATE=
 SET @JOBPATH=%~f0

 rem -- Standard Log Variables
 SET @LOGPATH=%@STORAGE%\Logs\Monitoring
 SET @LOGFILE=%@LOGPATH%\%~n0.%@FILESTAMP%

 rem -- Key Variable Verification
 FOR %%V IN (WORKDIR MRTGDIR PERLBIN INDEX SNMP INTERVAL SUBNET1 ALLNETS ALLDEVICES) DO IF NOT DEFINED @%%~V (
	 ECHO:
	 ECHO *** WARNING: The @%%~V variable is undefined.
	 ECHO:
	 ECHO  Please ensure that all configuration info is provided correctly
	 ECHO  within the file: "%@MRTG-INFO%"
	 ECHO:
	 ECHO  Alternatively, this information can be entered directly into this script,
	 ECHO  although this is not recommended because it undermines portability.
	 ECHO %@DIVIDER*%
	 ECHO:
	 GOTO :HelpMessage
 )


 rem ==========================================================================
 rem === Create Scheduled Job using SCHTASKS (XP/2003/Vista/+)
 rem === Updated On: 22 Jan 2022 / 09 Mar 2019              
 rem ==========================================================================
:ScheduleJobs
 IF /I "%@NOSCHED%"=="TRUE" IF /I NOT "%@UNSCHED%"=="TRUE" (
	 SET @JOBTYPE= // ONE-TIME
	 ECHO:
	 ECHO *** Job Scheduling Disabled ***
	 ECHO:
	 GOTO :ScheduleJobs_END
 )

 ECHO:
 ECHO Scheduling "%@JOBNAME%" on %COMPUTERNAME% at %@INTERVAL% minute Intervals...
 ECHO:

 IF %@OSBUILD% LSS 6000 SET @TASKFOLDER=
 SET @NAMECHK=%@JOBNAME:[=\[%
 SET @NAMECHK=!@NAMECHK:]=\]!
 SET @CHK_SCHED=SCHTASKS /QUERY /V /FO LIST
 SET @SET_SCHED=SCHTASKS /CREATE /TN "!@TASKFOLDER!%@JOBNAME%" /TR "%@JOBPATH%" /RU "%@JOBUSER%" /SC %@JOBFREQ% %@JOBDATE% /ST %@JOBTIME% %@JOBLEVEL% %@FORCE%

 rem -- Delete Scheduled Task from Old Custom Folders on Vista/2008+
 IF %@OSBUILD% GEQ 6000 (
	 FOR /F "TOKENS=1* DELIMS=\ " %%s IN ('%@CHK_SCHED% 2^>NUL ^| FIND "TaskName:" ^| FINDSTR /I /R /C:"\\%@NAMECHK%$"') DO (
		 IF /I NOT "%@TASKFOLDER%%@JOBNAME%"=="%%t" SCHTASKS /DELETE /TN "%%t" /F >NUL 2>&1
	 )
 )

 rem -- Check for Schedule Update Parameters
 IF /I "%@UPDATESCHED%"=="TRUE" (!@SET_SCHED!) ELSE (CALL :CheckSched "%~f0")
 ECHO %@DIVIDER-%
 ECHO:
 IF /I "%@SCHEDULEONLY%"=="TRUE" (
	 ECHO Task has been scheduled.  Now exiting script...
	 IF %@OSBUILD% LSS 6000 (SCHTASKS) ELSE (%@CHK_SCHED% /TN "!@TASKFOLDER!%@JOBNAME%")
	 GOTO :ExitBatch
 )
:ScheduleJobs_END
 %@PAUSE_IF_DEBUG%


 rem =========================================================================
 rem === Update MRTG Stats
 rem === Updated On: 23 Sep 2011 / 04 May 2011              
 rem =========================================================================
:Update
 IF NOT EXIST "%@PERLBIN%\Perl.exe" (
	 ECHO *** WARNING ***
	 ECHO:
	 ECHO Perl executable could not be found in: %@PERLBIN%
	 ECHO:
	 GOTO: HelpMessage
 ) ELSE (
	 NOW Updating MRTG Stats...
	 PUSHD "%@MRTGDIR%"
	 FOR %%N IN (%@ALLNETS%) DO FOR /L %%I IN (1,1,254) DO (
		 IF EXIST "MRTG-%%N.%%I.cfg" (
			 ECHO:
			 ECHO:
			 ECHO Processing MRTG-%%~N.%%~I.CFG
			 %@PERLBIN%\Perl mrtg MRTG-%%~N.%%~I.cfg
		 )
	 )
 )
 POPD
 %@PAUSE_IF_DEBUG%


 rem =========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019              
 rem =========================================================================
:ExitBatch
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Execute Dynamic Command From Variable
 rem === Updated On: 16 Jul 2010                     
 rem ==========================================================================
:RunCMD
 rem %1 = Command To Execute

 %*
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Check if Scheduled Job is Already Running
 rem === Updated On: 30 Jul 2018 / 05 Feb 2015              
 rem ==========================================================================
:CheckSched
 rem %1 = Standard Search Criteria
 rem %2 = Advanced Search Criteria

 SET @CHK=%2
 IF DEFINED @CHK (
	 %@CHK_SCHED% 2>NUL | FIND /I "%~1" | FINDSTR /I /R /C:%2 >NUL
 ) ELSE (
	 %@CHK_SCHED% 2>NUL | FIND /I "%~1" >NUL
 )
 IF ERRORLEVEL 1 %@SET_SCHED%
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET @SCRIPT_BEG#="%%~d"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 30 Nov 2020 / 30 Jul 2018              
 rem ==========================================================================
:HelpMessage
 ECHO Creates scheduled job to obtain SNMP info and update MRTG data every %@INTERVAL% minutes
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  {no parameters required}
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:
 ECHO -------------------
 ECHO OPTION DEFINITIONS:
 ECHO -------------------
 ECHO  -?,  /H ..... Display This Help Message  (also --HELP)
 ECHO  -??, /H2 .... Display Extended Help Message
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0
	 ECHO  %~n0 -?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
 )
 CALL :ShowStatus "FINISHED"
                                  