@ECHO OFF
 rem ==========================================================================
 rem === Change Password For Service Account on Remote Systems
 rem ==========================================================================
 rem
 rem Created On: 05 Jul 2001
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Oct 2023 / 30 Nov 2020              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2003 OR later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    Resource Kit ........... https://en.wikipedia.org/wiki/Resource_Kit
 rem    Alternate Tools ........ https://www.BrainWaveCC.com/knowledge-base/changingmanaging-service-passwords/
 rem ==========================================================================

 :::  This script uses SC from the ResKit to change the password (and account,
 :::  if necessary) of any particular service. The Service and Account info can
 :::  be stored in environment vars or passed as command line parameters.  If
 :::  no file list is provided at the command line, then all of the machines in
 :::  the domain (obtained via the NET VIEW command) will be used.
 :::
 :::  Machines names, as found in any list of files, must be prefaced with the
 :::  \\ (UNC) characters (e.g. \\SERVER1)
 :::
 :::  Command line parameters can be entered in any order. If an asterisk is
 :::  provided for the password, you will be prompted to enter the desired
 :::  password.
 :::
 :::  (Oct 2013) As of version 2.3, the delimiter for all parameters can be
 :::  either the "-" or the "/" character.  This includes the help parameters.
 :::
 :::  (Jul 2018) v2.0 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::  -- Added new Filtered CMD Title and Syntax Help processing routines;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com
 :::
 :::  (Oct 2018) v2.1 -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Feb 2019) v2.2 -- Added the --Script-Variables command for obtaining
 :::  core variables (including @FILESTAMP%);
 :::
 :::  (May 2019) v3.0 -- The new BrainWave GetInput utility is now used to
 :::  obtain password input without displaying it to the console.  Inpired by
 :::  Bill Stewart's nifty READLINE utility (www.westmesatech.com/?page_id=49);
 :::  -- Refactored the :GetParams routine using the CheckParams.exe utility;
 :::
 :::  (Jun 2019) v3.1 -- Another Refactoring of the :GetParams routine;
 :::
 :::  (Sep 2019) v3.2 -- Minor Refactoring of the :GetParams routine;
 :::
 :::  (Oct 2019) v3.3 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (Nov 2019) v3.4 -- Call SetDrive.BAT to initialize key variables;
 :::
 :::  (May 2020) v3.5 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::
 :::  (Nov 2020) v3.6 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Oct 2023) v3.7 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 30 Nov 2020 / 30 Jul 2018              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 30 Nov 2020 / 26 May 2020              
 rem ==========================================================================
:Variables
 SET $$=!
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v3.7.0.370

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc --mp=1 -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- Main Variables
 SET @SERV=Schedule
 SET @ACCT=%USERDOMAIN%\Service
 SET @NEWPWD=NewPassword
 SET @COMPLIST='NET VIEW'
 SET @LOGPATH=%@STORAGE%\Logs\Maint
 SET @LOGFILE=%@LOGPATH%\%~n0.%@FILESTAMP%

 rem -- Process Special Command-line Parameters
 SET @ALL_PARAMS=%*
 CALL :GetParams "S SERVICE"  @SERV & IF DEFINED @OK IF NOT DEFINED @SERV     GOTO :HelpMessage
 CALL :GetParams "A ACCT"     @ACCT & IF DEFINED @OK IF NOT DEFINED @ACCT     GOTO :HelpMessage
 CALL :GetParams "L LIST" @COMPLIST & IF DEFINED @OK IF NOT DEFINED @COMPLIST GOTO :HelpMessage
 CALL :GetParams "P PWD"    @NEWPWD & IF DEFINED @OK (
	 IF NOT DEFINED @NEWPWD SET @GETPWD=TRUE
	 IF {!@NEWPWD!}=={""}   SET @GETPWD=TRUE
	 IF {!@NEWPWD!}=={"*"}  SET @GETPWD=TRUE
	 IF DEFINED @GETPWD (SET @NEWPWD=""& FOR /F "TOKENS=*" %%P IN ('GetInput -p "Please Enter New Password: " -q') DO SET @NEWPWD="%%P")
	 SET @NEWPWD="!@NEWPWD:"=!"
	 IF !@NEWPWD!=="" (
		 ECHO The password cannot be blank%$$%
		 ECHO:
		 GOTO :HelpMessage
	 )
 )


 rem ==========================================================================
 rem === Cycle Through Domain (or File List) And Update Passwords
 rem === Updated On: 20 Jun 2019 / 11 Feb 2004              
 rem ==========================================================================
:GetComputers
 IF NOT EXIST "%@LOGPATH%" MD "%@LOGPATH%"

 NOW Updating Service Account/Password >"%@LOGFILE%"
 ECHO Getting a List of Computers via %@COMPLIST%...
 ECHO:
 FOR /F "USEBACKQ" %%V IN (%@COMPLIST%) DO CALL :ChangeAcct %%V
 ECHO Finished Updating Service Accounts...
 ECHO:
 GOTO :EOF


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019              
 rem ==========================================================================
:ExitBatch
 DIR "%@LOGFILE%" & TYPE "%@LOGFILE%"
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Change Password on Each Machine and Log Errors
 rem === Updated On: 20 Jun 2019 / 09 Feb 2004              
 rem ==========================================================================
:ChangeAcct
 rem %1 = Current System To Be Updated

 SET @PARAM=%1
 IF /I NOT "%@PARAM:~0,2%"=="\\" GOTO :EOF
 ECHO:
 ECHO Updating the "%@SERV%" service on "%1"...
 SC %1 CONFIG %@SERV% type= own obj= %@ACCT% password= %@NEWPWD% | FIND /I "SUCCESS"
 IF ERRORLEVEL 1 (
   ECHO There were problems updating "%1"
   ECHO %1 - Could Not Update >>"%@LOGFILE%"
   ECHO:
 ) ELSE (
	 ECHO %1 - Successfully Updated >>"%@LOGFILE%"
 )
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Use CheckParams.exe to Capture Selected Parameters
 rem === Updated On: 21 Jan 2020 / 24 Sep 2019              
 rem ==========================================================================
:GetParams
 rem %1 = Parameters to Check For
 rem %2 = Variable to Set

 SET @OK=& IF "%~1"=="" GOTO :EOF
 FOR /F "TOKENS=2-3*" %%v IN ('CHECKPARAMS -q -a -c "%~1" -s %@ALL_PARAMS% 2^>NUL') DO IF /I "%%~v"=="TRUE" (
	 SET @OK=T
	 SET @REQ=T
	 IF NOT "%~2"=="" SET %~2=%%~x
 )
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET @SCRIPT_BEG#="%%~d"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 30 Nov 2020 / 17 May 2020              
 rem ==========================================================================
:HelpMessage
 ECHO Change Password For Service Accounts on Remote Systems
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  ^</S:service_name^> [/A:account] ^</P:new_password^> [/L:list_of_servers.txt]
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:
 ECHO -------------------
 ECHO OPTION DEFINITIONS:
 ECHO -------------------
 ECHO  -?,  /H ..... Display This Help Message  (also --HELP)
 ECHO  -??, /H2 .... Display Extended Help Message
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0 /S:Schedule /P:newpass
	 ECHO  %~n0 /S:Schedule /A:ThisAcct /P:hard2guess
	 ECHO  %~n0 /S:MyService /A:ThisAcct /P:*
	 ECHO  %~n0 /S:NewService /A:DOMAIN\ServiceAcct /P:v3ryE@zY /L:C:\TEMP\SERVERS.TXT
	 ECHO  %~n0 -?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
	 ECHO:
	 ECHO  * This script cannot process passwords containing the following:
	 ECHO    ? * \ / ^" %% ^( ^) ^< ^> ^| or the exclamation point
 )
 CALL :ShowStatus "FINISHED"
              