@ECHO OFF
 rem ==========================================================================
 rem === Enable Windows File/Process Auditing (Windows NT to the Present)
 rem ==========================================================================
 rem
 rem Created On: 28 Apr 2001
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Oct 2023 / 17 Jan 2022              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: NT4 or later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    NLTEST ................. Resource Kit in NT4-2003 (Native in Vista+)
 rem    Resource Kit ........... https://en.wikipedia.org/wiki/Resource_Kit
 rem
 rem  Input Files Used By Script (stored in %SystemDrive%\Scripts\Bat\Input):
 rem    AuditSettings.TXT ...... List of Audit Categories and Settings
 rem
 rem  Required Scripts:
 rem    SetDrive.BAT ........... Set Global Variables for this Environment
 rem ==========================================================================

 :::  This script uses AUDITPOL from the NT/2000 ResKit to enable File Auditing
 :::  on the current machine or a remote system. It can be scripting for
 :::  multiple machines with a FOR command.
 :::
 :::  The new version of AUDITPOL that comes with Vista requires some changes
 :::  to the script so that the correct parameters are used.
 :::
 :::  (Jul 2010) v3.0 -- The script has been modified to support the version
 :::  of AUDITPOL that is native to Vista/Win7/2008+.  The script will
 :::  determine which paramters to use based on the version of the OS detected.
 :::  It will also make a backup of the current state, using the /Backup
 :::  parameter which is new to AUDITPOL.  The script also makes use of the
 :::  AuditSettings.TXT input file to configure the audit categories in
 :::  Vista/2008 and beyond.
 :::
 :::  (Oct 2013) v3.3 -- The delimiter for all parameters can be either the "-"
 :::  or the "/" character.  This includes the help parameters.
 :::
 :::  (Aug 2014) v4.0 -- Significantly modified AuditSettings.TXT configuration
 :::  file to support text-based configuration of both the old and new AUDITPOL
 :::  utilities.
 :::
 :::  (May 2016) v5.0 -- Now supports AUDITPOL on remote systems for Vista+;
 :::  Uses SYSTEMINFO to determine remote system and executes a remote shell
 :::  using PSEXEC or BeyondExecV2; Script versioning now includes build number
 :::
 :::  (May 2016) v5.1 -- Added verification of remote system using NLTEST
 :::
 :::  (Feb 2018) v5.2 -- Updated Pre-Vista AUDITPOL backups;
 :::  -- Changed the HELP routine to use FINDSTR;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com
 :::
 :::  (May 2018) v5.3 -- Generate Filtered CMD Title;
 :::  -- The SYNTAX HELP routine has been updated;
 :::
 :::  (Jul 2018) v5.4 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::
 :::  (Oct 2018) v5.5 -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Oct 2019) v5.6 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (May 2020) v5.7 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::
 :::  (Jul 2020) v5.8 -- Expand Support for Remote Console Tools;
 :::  -- Changed purge function to sort old files by date, in reverse order;
 :::  -- Ensure selection of proper Local Backups folder;
 :::
 :::  (Nov 2020) v5.9 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Jan 2022) v6.0 -- Use BrainWave Readable to format AuditPol reports;
 :::  -- Expand the processing of backups and their archives;
 :::  -- Added alternative methods for obtaining remote OS version info via
 :::  WMIC, DATEINFO, and PSINFO (especially for Windows 11 systems);
 :::
 :::  (Oct 2023) v6.1 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 25 Jul 2020 / 29 May 2018              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 17 Jan 2022 / 14 Jan 2022 / 30 Nov 2020              
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v6.1.0.710

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- Obtain Default Audit Variables for Remote Systems
 SET @MACHINE=\\%~1& IF "%~1"=="" SET @MACHINE=%COMPUTERNAME%
 SET @MACHINE=\\!@MACHINE:\=!
 SET @MACHINE=!@MACHINE:/=!
 SET @ISLOCAL=& IF /I "%@MACHINE%"=="\\%COMPUTERNAME%" SET @ISLOCAL=TRUE
 SET @BADSYS=ERROR: COULD NOT ACCESS !@MACHINE!

 rem -- Validate Access to Target System
 CALL :VerifyTarget !@MACHINE!
 IF NOT DEFINED @ALIVE (ECHO %@BADSYS%& GOTO :ExitBatch)

 rem -- Obtain Target OS Version Info
 CALL :GetRemoteOSVersion !@MACHINE!

 rem -- Standard Backup Variables
 SET @L_ARCHIVE=%@BACKUPS%\Audit
 SET @R_ARCHIVE=!@MACHINE!\Backups$\Audit
 SET @T_ARCHIVE=!@MACHINE!\Admin$\Temp
 SET @BCKUPFILE=%~n0-%@MACHINE:\=%.%@TODAY%-%@NOW%.CSV
	 IF DEFINED @ISLOCAL (
		 SET @ARCHIVES=!@L_ARCHIVE!
		 SET @SAVEFILE=!@ARCHIVES!\!@BCKUPFILE!
		 SET @SAVEHERE=
	 ) ELSE (
		 IF EXIST "!@MACHINE!\Backups$" (SET @ARCHIVES=!@R_ARCHIVE!) ELSE (SET @ARCHIVES=!@T_ARCHIVE!)
		 SET @L_ARCHIVE=%@BACKUPS%\Audit\Remote
		 SET @SAVEFILE=!@ARCHIVES!\!@BCKUPFILE!
		 SET @SAVEHERE=!@L_ARCHIVE!\!@BCKUPFILE!
	 )

 rem -- Obtain Target AuditPOL Info
 FOR %%a IN (%@SCRIPTS%\Utils;%@SCRIPTS%\Utils\Misc) DO IF EXIST "%%~a\AUDITPOL.EXE" SET @AUDITPOL=%%~a\AUDITPOL.EXE

 rem -- Check for "." or "/S" or "-S" to Enable Changes
 SET @GETCONFIG= + Reading Audit Values from Configuration File...
 SET @SETCONFIG= + Setting Audit Configuration for "%@MACHINE%"
 SET @READONLY=*** NO CHANGES MADE TO CONFIGURATION ***
 SET @NOTICE=Reviewing
 SET @ENABLE=FALSE& FOR %%P IN (/S -S .) DO IF /I "%~2"=="%%~P" (SET @ENABLE=TRUE& SET @NOTICE=Configuring)

 rem -- Obtain Default Audit Variables [former settings]
 SET @OBJ=FAILURE
 SET @PRIV=FAILURE
 SET @PROCESS=FAILURE

 rem -- Obtain Default Audit Variables [only used if AuditSettings.TXT does not exist]
 SET @SYSTEM=ALL
 SET @LOGON=ALL
 SET @OBJ=NONE
 SET @PRIV=NONE
 SET @PROCESS=NONE
 SET @POLICY=ALL
 SET @SAM=ALL
 SET @DIR=ALL
 SET @ACCT=ALL
 SET @ADSTUFF=

 rem -- Readable Parameters
 SET @FORMAT_OLD_AUDITPOL=-X -E --LC "*" --DL 1 --SK "\nRunning;\n(X) Audit"
 SET @FORMAT_NEW_AUDITPOL=-X -E --LC "*" --DL 1 --SK "\nSystem audit policy" --LA "\nCategory" --SB "\nCategory;\nLogon;\nObject;\nPrivilege;\nDetailed;\nPolicy;\nAccount;\nDS"

 rem -- Backup Archives to Keep (# of audit backups to keep per system)
 SET @KEEP_HISTORICAL=4

 rem -- Standard Log Variables
 SET @LOGPATH=%@STORAGE%\Logs\Maint
 SET @TEMPLOG=%@LOGPATH%\%~n0-%@MACHINE:\=%.TEMP.%@FILESTAMP%
 SET @LOGFILE=%@TEMPLOG:.TEMP.=.%
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Initialize Transaction Log File
 rem === Updated On: 14 Jan 2022 / 15 May 2016              
 rem ==========================================================================
:InitializeLog
 FOR %%F IN ("%@LOGPATH%" "%@ARCHIVES%" "%@L_ARCHIVE%" "%@R_ARCHIVE%") DO IF NOT EXIST "%%~F" MD "%%~F"

 ( rem -- Write to Log
	 ECHO %@DIVIDER#%
	 ECHO ### !DATE! on !TIME! -- Generate Current Audit Policy Info on !@MACHINE!" [v!@RMT_OSVER!]
	 ECHO %@DIVIDER#%
	 ECHO:
 ) >"%@LOGFILE%"
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Backup Existing Audit Configuration
 rem === Updated On: 14 Jan 2022 / 12 Feb 2018              
 rem ==========================================================================
:BackupConfig
 ECHO:
 ECHO %@NOTICE% Windows Audit Settings On: "!@MACHINE!" [v!@RMT_OSVER!]
 ECHO %@DIVIDER*%
 CALL :SectionHeader + Backing Up Current Audit Configuration...

 IF DEFINED @RMT_OSMAJ IF %@RMT_OSMAJ% GEQ 6 (
	 %@RMTCONSOLE% %SystemRoot%\System32\AUDITPOL /BACKUP /FILE:"%@SAVEFILE%"
	 SET @JUMPTO=NEW
 ) ELSE (
	 %@AUDITPOL% !@MACHINE! >"%@SAVEFILE%"
	 SET @JUMPTO=OLD
 )

 rem -- Consolidate Archive
 IF DEFINED @SAVEHERE (
	 DIR "%@SAVEFILE%" >NUL
	 SLEEP 5
	 IF EXIST "%@SAVEFILE%" COPY "%@SAVEFILE%" "%@SAVEHERE%" /V
 )
 IF DEFINED @JUMPTO GOTO :SetAuditing-%@JUMPTO%
 %@PAUSE_IF_DEBUG%
 GOTO :ExitBatch


 rem ==========================================================================
 rem === Display OR Configure Auditing on Local or Remote NT-2003 System
 rem === Updated On: 12 Feb 2018 / 13 May 2016              
 rem ==========================================================================
:SetAuditing-Old
 IF EXIST "%@AUDIT-INFO%" ( rem -- Get Audit Config or Use Defaults
	 CALL :SectionHeader %@GETCONFIG%
	 FOR /F "USEBACKQ EOL=; TOKENS=1-3 DELIMS=, " %%A IN ("%@AUDIT-INFO%") DO IF /I "%%~A"=="#OLD" SET %%~B=%%~C
 ) ELSE (
	 ECHO Input File "%@AUDIT-INFO%" Not Found.
	 ECHO Using Default Configuration Values.
	 ECHO:
 )

 IF "%@ENABLE%"=="TRUE" ( rem -- If ENABLE = TRUE, Make Audit Changes
	 CALL :SectionHeader %@SETCONFIG%
	 FOR /F %%S IN ('%@AUDITPOL% !@MACHINE! ^| FIND /I "Directory"') DO SET @ADSTUFF=/directory:%@DIR% /account:%@ACCT%
	 %@AUDITPOL% !@MACHINE! /ENABLE /system:%@SYSTEM% /logon:%@LOGON% /object:%@OBJ% /privilege:%@PRIV% /process:%@PROCESS% /policy:%@POLICY% /sam:%@SAM% !@ADSTUFF!
 ) ELSE (
	 ECHO %@READONLY%
 )
 %@PAUSE_IF_DEBUG%
 GOTO :DisplayCurrentConfig


 rem ==========================================================================
 rem === Display OR Configure Auditing on Local or Remote Vista/2008+ System
 rem === Updated On: 13 May 2016 / 05 Feb 2015              
 rem ==========================================================================
:SetAuditing-New
 IF EXIST "%@AUDIT-INFO%" ( rem -- Get Audit Config
	 CALL :SectionHeader %@GETCONFIG%
	 IF "%@ENABLE%"=="TRUE" ( rem -- If ENABLE = TRUE, Make Audit Changes
		 CALL :SectionHeader %@SETCONFIG%
		 FOR /F "USEBACKQ SKIP=2 EOL=; TOKENS=* DELIMS=, " %%A IN ("%@AUDIT-INFO%") DO CALL :ParseCategory %%A
	 ) ELSE (
		 ECHO %@READONLY%
	 )
 ) ELSE (
	 ECHO Input File "%@AUDIT-INFO%" Not Found.
	 ECHO Operation Halted.
	 ECHO:
	 %@PAUSE_IF_DEBUG%
	 GOTO :ExitBatch
 )
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Display Current Audit Configuration
 rem === Updated On: 13 Jan 2022 / 12 Feb 2018              
 rem ==========================================================================
:DisplayCurrentConfig
 ( rem -- Write to Log
	 CALL :SectionHeader + Displaying Current Audit Configuration...
	 ECHO -- !@MACHINE!
	 IF DEFINED @RMT_OSMAJ IF %@RMT_OSMAJ% GEQ 6 (
		 SET @READABLE_PARAMS=%@FORMAT_NEW_AUDITPOL%
		 %@RMTCONSOLE% "%SystemRoot%\System32\AUDITPOL" /GET /CATEGORY:*
	 ) ELSE (
		 SET @READABLE_PARAMS=%@FORMAT_OLD_AUDITPOL%
		 %@AUDITPOL% !@MACHINE!
	 )
 ) >"%@TEMPLOG%"

 FOR %%O IN (CON "%@LOGFILE%") DO READABLE "%@TEMPLOG%" !@READABLE_PARAMS! >>%%O 2>&1
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Keep Only The Last %@KEEP_HISTORICAL% Audit Backups (Default=30)
 rem === Updated On: 17 Jan 2022 / 16 Jan 2022 / 26 Jun 2017              
 rem ==========================================================================
:PruneOldBackups
 SET @FILES_DELETED=
 ( rem -- Write to Log
	 CALL :SectionHeader + Archiving Old Audit Backups [Keep only the last %@KEEP_HISTORICAL% copies for this system]
	 SET @THESE=*%@MACHINE:\=%*.CSV
	 IF /I "%@ARCHIVES%"=="%@L_ARCHIVE%" (
		 SET @THE_FOLDERS="%@ARCHIVES%\!@THESE!"
	 ) ELSE (
		 SET @THE_FOLDERS="%@L_ARCHIVE%\!@THESE!" "%@ARCHIVES%\!@THESE!"
	 )
	 FOR /F "SKIP=%@KEEP_HISTORICAL% TOKENS=*" %%V IN ('DIR !@THE_FOLDERS! /A-D /O-D /B /S 2^>NUL') DO (
		 SET @SRC=%%~dV
		 SET @SRC=!@SRC:\\=?:!
		 ECHO  - Deleting "!@SRC!:%%~nxV" from the Audit Backup Folder
		 DEL "%%~V" 2>NUL
		 SET @FILES_DELETED=TRUE
	 )
	 IF NOT DEFINED @FILES_DELETED ECHO  --- NO "!@THESE!" FILES PRUNED
	 %@PAUSE_IF_DEBUG%
 ) >"%@TEMPLOG%"
 FOR %%O IN (CON "%@LOGFILE%") DO TYPE "%@TEMPLOG%" >>%%O
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Generate Final Job Report
 rem === Updated On: 15 May 2016 / 21 Apr 2015              
 rem ==========================================================================
:FinalReport
 CALL :SectionHeader + Generate Final Report...
 DIR /OGD "%@LOGFILE%" "%@ARCHIVES%\%~n0*.CSV" | FINDSTR /I /V /R /C:"^ Volume "
 IF DEFINED @SAVEHERE ECHO: & DIR /OGD "%@L_ARCHIVE%\%~n0*.CSV" | FINDSTR /I /V /R /C:"^ Volume "
 IF EXIST "%@TEMPLOG%" DEL "%@TEMPLOG%"
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019              
 rem ==========================================================================
:ExitBatch
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Section Header
 rem === Updated On: 13 Jan 2022 / 11 Aug 2014              
 rem ==========================================================================
:SectionHeader
 rem %* = Section Header Info

 (ECHO:  )
 (ECHO:  )
 ECHO  %*
 ECHO %@DIVIDER-%
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Parse Input File for Audit Categories
 rem === Updated On: 17 Jan 2022 / 15 May 2016              
 rem ==========================================================================
:ParseCategory
 rem %1 = AuditPol Version
 rem %2 = Audit Category
 rem %3 = Audit Subcategory
 rem %4 = Configuration Setting

 SET @CATEGORY=& IF NOT "%~2"=="" SET @CATEGORY=/CATEGORY:"%~2"
 SET @SUBCATEGORY=& IF NOT "%~3"=="" SET @SUBCATEGORY=/SUBCATEGORY:"%~3"
 SET @CONFIGURE=%~4

 IF /I "%~1"=="#NEW" %@RMTCONSOLE% "%SystemRoot%\System32\AUDITPOL" /SET %@CATEGORY% %@SUBCATEGORY% %~4 2>NUL
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Determine Which Remote Console Tool to Use (PSEXEC, etc)
 rem === Updated On: 25 Jul 2020 / 20 May 2016              
 rem ==========================================================================
:VerifyTarget
 rem %1 = Current System for Which Info Will be Obtained

 ECHO Validating Connection To: %~1

 IF /I "%~1"=="\\%COMPUTERNAME%" (SET @ALIVE=LOCAL& SET @RMTCONSOLE=& GOTO :EOF)
 FOR /F "TOKENS=*" %%R IN ('WHERE BeyondExecV2 2^>NUL') DO SET @B_REMOTE=%%~nxR
 FOR /F "TOKENS=*" %%R IN ('WHERE PAEXEC       2^>NUL') DO SET @A_REMOTE=%%~nxR
 FOR /F "TOKENS=*" %%R IN ('WHERE PSEXEC       2^>NUL') DO SET @P_REMOTE=%%~nxR %@PSPARAMS% -s
 SET @RMTCONSOLE=&               IF DEFINED @B_REMOTE SET @RMTCONSOLE=%@B_REMOTE%
 IF /I NOT "%@NOPAEXEC%"=="TRUE" IF DEFINED @A_REMOTE SET @RMTCONSOLE=%@A_REMOTE%
 IF /I NOT "%@NOPSEXEC%"=="TRUE" IF DEFINED @P_REMOTE SET @RMTCONSOLE=%@P_REMOTE%
 SET @RMTCONSOLE=%@RMTCONSOLE% 2^>NUL %~1
 SET @ALIVE=& FOR /F "TOKENS=*" %%A IN ('NLTEST /QUERY /SERVER:%~1 2^>NUL ^| FIND /I "The command completed successfully"') DO SET @ALIVE=%%~A
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Obtain Target OS Version (SYSTEMINFO, WMIC, DATEINFO, etc)
 rem === Updated On: 17 Jan 2022                     
 rem ==========================================================================
:GetRemoteOSVersion
 rem %1 = Current System for Which Info Will be Obtained

 rem --------------------------------------------------------------------------
 rem -- Primary Method via SYSTEMINFO
 rem --------------------------------------------------------------------------
 FOR %%v IN (VER MAJ MIN BUILD) DO SET @RMT_OS%%~v=0
 IF DEFINED @ISLOCAL (
	 FOR %%L IN (VER MAJ MIN BUILD) DO SET @RMT_OS%%~L=!@OS%%~L!
 ) ELSE (
	 FOR /F "TOKENS=3-5 DELIMS=. " %%O IN ('SYSTEMINFO /S %@MACHINE% 2^>NUL ^| FIND /I "OS Version:                "') DO (
		 SET @RMT_OSVER=%%~O.%%~P.%%~Q
		 SET @RMT_OSMAJ=%%~O
		 SET @RMT_OSMIN=%%~P
		 SET @RMT_OSBUILD=%%~Q
	 )
 )
 IF NOT "!@RMT_OSMAJ!"=="0" GOTO :EOF

 rem
 rem --------------------------------------------------------------------------
 rem -- Alternate Method via WMIC with Remote Registry Enabled
 rem --------------------------------------------------------------------------
 ECHO  OBTAINING REMOTE VERSION INFO via WMIC...
 FOR /F "TOKENS=4" %%S IN ('SC %@MACHINE% QUERY "RemoteRegistry" ^| FIND "STATE"') DO (
	 IF NOT "%%~S"=="RUNNING" (
		 ECHO Starting Remote Registry Service on %@MACHINE%...
		 SC %@MACHINE% CONFIG RemoteRegistry start= AUTO
		 SC %@MACHINE% START RemoteRegistry
		 ECHO:
	 )
 )
 FOR /F "TOKENS=2-4 DELIMS=.=" %%O IN ('WMIC /NODE:"%@MACHINE:\=%" OS GET Version /FORMAT:LIST 2^>NUL ^| FIND "="') DO (
	 SET @RMT_OSVER=%%~O.%%~P.%%~Q
	 SET @RMT_OSMAJ=%%~O
	 SET @RMT_OSMIN=%%~P
	 SET @RMT_OSBUILD=%%~Q
 )
 IF NOT "!@RMT_OSMAJ!"=="0" GOTO :EOF

 rem
 rem --------------------------------------------------------------------------
 rem -- Alternate Method via DATEINFO with PSEXEC
 rem --------------------------------------------------------------------------
 ECHO  OBTAINING REMOTE VERSION INFO via DATEINFO...
 FOR /F "TOKENS=1-2 DELIMS=@;" %%P IN ('PSEXEC %@MACHINE% DATEINFO --Script-Variables 2^>NUL ^| FINDSTR /I "@OSB @OSM @OSV"') DO SET @RMT_%%P=%%~Q
 IF NOT "!@RMT_OSMAJ!"=="0" GOTO :EOF

 rem
 rem --------------------------------------------------------------------------
 rem -- Alternate Method via PSINFO
 rem --------------------------------------------------------------------------
 ECHO  OBTAINING REMOTE VERSION INFO via PSINFO...
 SET @RMT_OSBUILD=*
 FOR /F "TOKENS=2 DELIMS=:" %%V IN ('PSINFO %@PSPARAMS% %@MACHINE% 2^>NUL ^| FINDSTR /I /R "Product.version: Kernel.build.number:"') DO SET @RMT_OSBUILD=!@RMT_OSBUILD!.%%V
 SET @RMT_OSBUILD=!@RMT_OSBUILD:*.=!
 SET @RMT_OSBUILD=!@RMT_OSBUILD: =!
 FOR /F "DELIMS=." %%V IN ('ECHO !@RMT_OSBUILD!') DO SET @RMT_OSMAJ=%%V
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET @SCRIPT_BEG#="%%~d"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 13 Jan 2022 / 30 Nov 2020              
 rem ==========================================================================
:HelpMessage
 ECHO Configure Windows File/Process Auditing (NT to the Present)
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  [remote_system] [OPTIONS]
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:
 ECHO -------------------
 ECHO OPTION DEFINITIONS:
 ECHO -------------------
 ECHO  -?,  /H ..... Display This Help Message  (also --HELP)
 ECHO  -??, /H2 .... Display Extended Help Message
 ECHO       /S  .... Set Audit Policy
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0 \\your_system
	 ECHO  %~n0 \\remote_server /S
	 ECHO  %~n0 \\%%COMPUTERNAME%% /S
	 ECHO  %~n0 \\%COMPUTERNAME%
	 ECHO  %~n0 -?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO  * Options are case insensitive, and can be prefaced with "-" or "/".
	 ECHO:
	 ECHO  * For reasons known only to Microsoft, the version of AuditPol.exe that
	 ECHO    comes with Vista and later versions of Windows OSes does not natively
	 ECHO    support connecting to remote systems.  If a remote system is selected,
	 ECHO    the PSEXEC or BeyondExecV2 utilities will be automatically employed to
	 ECHO    make the remote connection.
 )
 CALL :ShowStatus "FINISHED"
                                                                                             