@ECHO OFF
 rem ==========================================================================
 rem === Set File Permissions on Local Hard Drives
 rem ==========================================================================
 rem
 rem Created On: 11 Jan 1999
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Oct 2023 / 30 Nov 2020              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2003 OR later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    CHOICE ................. Native in Windows 2003+ / Reskit in NT/2000
 rem    Resource Kit ........... https://en.wikipedia.org/wiki/Resource_Kit
 rem    Alternate Tools ........ http://KB.UltraTech-llc.com/?File=Perms.TXT
 rem
 rem  Required Scripts:
 rem    SetDrive.BAT ........... Set Global Variables for this Environment
 rem ==========================================================================

 :::  This script will remove the EVERYONE group from the local file system and
 :::  explicitly add the SYSTEM account with full control.  The functionality
 :::  of the LocalDrives script is incorporated into this script in order to
 :::  accommodate all local hard drives.  SRVINFO is used to determine all local
 :::  drives to run the script against. If you know which drives you want to
 :::  run this against, you could eliminate that section of the code.
 :::
 :::  (Nov 2010) Changed the default log extension of the reports from .LOG to
 :::  .TXT so that the reports that are mailed to mobile devices can be read
 :::  automatically. (Not too many devices can natively handle .LOG files)
 :::
 :::  (Jul 2018) v3.0 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::  -- Added new Filtered CMD Title and Syntax Help processing routines;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com
 :::
 :::  (Oct 2018) v3.1 -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Oct 2019) v3.2 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (May 2020) v3.3 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::
 :::  (Nov 2020) v3.4 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Oct 2023) v3.5 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 30 Nov 2020 / 30 Jul 2018              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 30 Nov 2020 / 26 May 2020              
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v3.5.0.270

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 (
	 ECHO Set File Permissions on Local Hard Drives
	 ECHO:
	 ECHO    YOU TYPED: %0 %*
	 ECHO:
	 ECHO   CMD SYNTAX: %~n0
	 ECHO               %~n0  [-h^|-?]
	 ECHO:
	 ECHO  USAGE NOTES:
	 ECHO    * Parameters surrounded by ^<^> are mandatory.
	 ECHO    * Parameters surrounded by [] are optional.
	 ECHO    * Options are case-insensitive, and can be prefaced by "-" or "/".
	 GOTO :ExitBatch
 )

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- Main Variables
 SET @CMDLINE=%CMDCMDLINE% %0 %*
 SET @NEWPERMS=/P "ADMINISTRATORS":F "SYSTEM":F /R "Everyone"
 SET @LOGPATH=%@STORAGE%\Logs\Maint
 SET @LOGFILE=%@LOGPATH%\%~n0.%@FILESTAMP%
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Allow User To Abort Operation
 rem === Updated On: 30 Mar 2011 / 28 Jan 1999              
 rem ==========================================================================
:DblCheck
 FOR %%P IN (%*) DO IF "%%P"=="." GOTO :DblCheck
 ECHO === Prepare to Set File Permissions for Local Drives ===
 ECHO:
 CHOICE /C YNQEX /T 20 /D Y /N /M "Continue running this script? (Y/N/Q) "
 IF %ERRORLEVEL% GTR 1 GOTO :ExitBatch
:DblCheck
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Set File Permissions for Local Drives
 rem === Updated On: 04 Mar 2003                     
 rem ==========================================================================
:SetPerms
 FOR /F "TOKENS=1 DELIMS=$" %%D IN ('SRVINFO -NS ^| FIND "$"') DO CALL :GetDrives %%D
 IF "%@DRIVES%"=="" (
   ECHO:
   ECHO No Local Drives Were Found....
   ECHO:
   GOTO :ExitBatch
 )

 IF NOT EXIST "%@LOGPATH%" MD "%@LOGPATH%"
 NOW Setting File Permissions for All Local Drives... >>"%@LOGFILE%"
 ECHO:
 ECHO Setting File Permissions for All Local Drives...
 ECHO:
 FOR %%D IN (%@DRIVES%) DO (
   XCACLS %%D\ /E /C %@NEWPERMS%
   FOR /F "TOKENS=* DELIMS=\" %%F IN ('DIR %%D\ /B /AD') DO XCACLS "%%D\%%F"        /E /C %@NEWPERMS%
   FOR /F "TOKENS=* DELIMS=\" %%F IN ('DIR %%D\ /B /AD') DO XCACLS "%%D\%%F\*.*" /T /E /C %@NEWPERMS%
 )
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Log Pertinent Info (If Any)
 rem === Updated On: 05 Feb 2015 / 25 Oct 2005              
 rem ==========================================================================
:DebugLog
 ( rem -- Write to Log
	 NOW %@CMDLINE%
	 SET
	 ECHO:
	 ECHO:
 ) >>"%@LOGFILE%"
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019              
 rem ==========================================================================
:ExitBatch
 ECHO:
 ECHO FINISHED Setting File Permissions on All Local Drives!
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Determine All Local Hard Drives
 rem === Updated On: 26 Jun 2001                     
 rem ==========================================================================
:GetDrives
 rem %1 = Current Drive

 SET @DRIVES=%@DRIVES% %1:
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET @SCRIPT_BEG#="%%~d"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF
                                                                             