@ECHO OFF
 rem ==========================================================================
 rem === Set Time Synchronization for Local NT/AD domains
 rem ==========================================================================
 rem
 rem Created On: 29 Jan 2008
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Oct 2023 / 30 Nov 2020              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2003 or later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    Resource Kit ........... https://en.wikipedia.org/wiki/Resource_Kit
 rem
 rem  Input Files Used By Script (stored in %SystemDrive%\Scripts\Bat\Input):
 rem    CustomVariables.TXT .... Centralized Configuration Settings for Scripts
 rem
 rem  Required Scripts:
 rem    SetDrive.BAT ........... Set Global Variables for this Environment
 rem ==========================================================================

 :::  This script uses the NET TIME and the Win32tm commands to view and
 :::  configure the NTP settings of the machines in your domain.  By default,
 :::  the servers listed as Domain Controllers are configured to the external
 :::  time source, and all other systems are configured to get their time
 :::  from a domain controller.
 :::
 :::  Prior to version 4.0, the list of systems to enable for external NTP
 :::  synchronization was derived from the "AD-DCs.TXT" input file.  This has
 :::  been changed to the "CustomVariables.TXT" input file via the @TIMESERVER
 :::  variable.
 :::
 :::  (Dec 2014) v5.0 -- Ensure that the W32Time service is set to auto as
 :::  a part of the configuration; Added a new logfile management routine.
 :::
 :::  (Nov 2017) v6.0 -- Ensure that Remote Regsitry Service is running on
 :::  remote systems before executing registry updates.
 :::
 :::  (Jul 2018) v7.0 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::  -- Added new Filtered CMD Title and Syntax Help processing routines;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com
 :::
 :::  (Oct 2018) v7.1 -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Dec 2018) v7.2 -- Lowered Retry Limit and made it customizable;
 :::
 :::  (Oct 2019) v7.3 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (May 2020) v7.4 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::
 :::  (Nov 2020) v7.5 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Oct 2023) v7.6 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 ::: --------
 :::  NOTES:
 ::: --------
 :::  The settings enabled by this script were derived from the following
 :::  Microsoft Knowledgebase articles:
 :::  ---- http://support.microsoft.com/kb/816042
 :::  ---- http://technet.microsoft.com/en-us/library/cc773013(WS.10).aspx
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 30 Nov 2020 / 26 Jul 2018              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 30 Nov 2020 / 26 May 2020              
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v7.5.0.610

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc --mp=1 -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Default Variables (Override via CustomVariables.*.TXT)
 SET @TIMESOURCE=1.us.pool.ntp.org  nist.time.gov  1.north-america.pool.ntp.org
 SET @TIMESERVER=%COMPUTERNAME%
 SET @SPECIAL_POLL=1800
 SET @NORMAL_POLL=15
 SET @POS_CORRECT=3700
 SET @NEG_CORRECT=3700
 SET @MAX_OFFSET=5
 SET @RETRY_LIMIT=10

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- Standard W32Time Variables
 SET @NTP_CLIENT_KEY=HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\W32Time\TimeProviders\NtpClient
 SET @NTP_PARAM_KEY=HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\W32Time\Parameters
 SET @NTP_CONFIG_KEY=HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\W32Time\Config
 SET @USEW32TM=TRUE& IF %@OSBUILD% LEQ 3790 SET @USEW32TM=FALSE
 SET @TIMESVC=W32Time
 SET @SYSTEMLIST=
 SET @OLD_TIMESOURCE=
 SET @NEW_TIMESOURCE=

 rem -- Determine Input Source For Script
 SET @SOURCE='TYPE "%~1" '& IF "%~1"=="." SET @SOURCE=%@SERVERLIST%

 rem -- Ensure Correct Time Source Format for NET TIME vs W32TM Commands
 FOR %%T IN (%@TIMESOURCE%) DO (
	 SET @OLD_TIMESOURCE=!@OLD_TIMESOURCE!,%%~T
	 SET @NEW_TIMESOURCE=!@NEW_TIMESOURCE! %%~T,0x1
	 SET @BAK_TIMESOURCE=!@BAK_TIMESOURCE! %%~T,0x2
 )

 rem -- Generate System List for Time Sync, Including Current Machine and Designated Time Servers
 FOR %%S IN (%@TIMESERVER%) DO SET @SYSTEMLIST=!@SYSTEMLIST! %%~S
 FOR /F "TOKENS=1 EOL=;" %%S IN (%@SOURCE%) DO SET @SYSTEMLIST=!@SYSTEMLIST! %%~S
 SET @TIMESERVER=%@TIMESERVER:\=%
 SET @SYSTEMLIST=%@SYSTEMLIST:\=%

 rem -- Main Variables
 SET @PARAMS=%1 %2 %3 %4 %5 %6 %7 %8 %9
 SET @LOGPATH=%@STORAGE%\Logs\Maint
 SET @LOGFILE=%@LOGPATH%\%~n0.%@FILESTAMP%
 SET @LOGLIST="%@LOGFILE%"
 %@PRINT_IF_DEBUG%


 rem ==========================================================================
 rem === Update Time Server Settings for Windows Systems (Determine Method)
 rem === Updated On: 22 Nov 2017 / 16 Nov 2017/24 Dec 2014              
 rem ==========================================================================
:SetTimeSync
 ECHO Configuring Time Servers...
 ECHO:

 rem -- Cycle through list of systems.
 FOR %%S IN (%@SYSTEMLIST%) DO (
	 IF NOT EXIST "\\%%~S\ADMIN$" SET @%%~S-SET=TRUE

	 IF /I NOT "!@%%~S-SET!"=="TRUE" (
		 ECHO [!TIME: =0!] Evaluating %%~S Settings... >CON
		 ECHO [!TIME: =0!] Evaluating %%~S Settings...
		 ECHO %@MINORDIV%
		 SET @EXTERNALSYNC=FALSE

		 rem -- Start "Remote Registry" service on %%~S if not running
		 FOR /F "TOKENS=4" %%Q IN ('SC \\%%~S QUERY "RemoteRegistry" ^| FIND "STATE"') DO (
			 IF NOT "%%~Q"=="RUNNING" (
				 ECHO [!TIME: =0!] Starting Remote Registry Service on %%~S ... >CON
				 ECHO [!TIME: =0!] Starting Remote Registry Service on %%~S ...
				 SC \\%%~S CONFIG RemoteRegistry start= AUTO
				 SC \\%%~S START RemoteRegistry
				 ECHO:
				 SLEEP 5
			 )
		 )

		 rem -- Make Registry Configuration Changes to the W32Time Service on the Remote System
		 NET TIME \\%%~S | FIND "Current time"
		 SC config %@TIMESVC% start= auto
		 REG ADD "\\%%~S\%@NTP_CLIENT_KEY%" /v "SpecialPollInterval"   /t REG_DWORD /d %@SPECIAL_POLL% /f
		 REG ADD "\\%%~S\%@NTP_CONFIG_KEY%" /v "MaxPollInterval"       /t REG_DWORD /d %@NORMAL_POLL%  /f
		 REG ADD "\\%%~S\%@NTP_CONFIG_KEY%" /v "MaxPosPhaseCorrection" /t REG_DWORD /d %@POS_CORRECT%  /f
		 REG ADD "\\%%~S\%@NTP_CONFIG_KEY%" /v "MaxNegPhaseCorrection" /t REG_DWORD /d %@NEG_CORRECT%  /f
		 REG ADD "\\%%~S\%@NTP_CONFIG_KEY%" /v "MaxAllowedPhaseOffset" /t REG_DWORD /d %@MAX_OFFSET%   /f

		 rem -- Identify Local Time Servers to Set Against External NTP Servers
		 FOR %%T IN (%@TIMESERVER%) DO IF /I "%%~S"=="%%~T" SET @EXTERNALSYNC=TRUE

		 IF /I "!@EXTERNALSYNC!"=="TRUE" (
			 rem ----------------------------------------
			 rem -- Configure Authoritative NTP Sources
			 rem ----------------------------------------
			 ECHO [!TIME: =0!] Configuring %%~S as an Authoritative NTP Source >CON
			 ECHO [!TIME: =0!] Configuring %%~S as an Authoritative NTP Source
			 REG ADD "\\%%~S\%@NTP_CONFIG_KEY%" /v "AnnounceFlags" /t REG_DWORD /d 5 /f >NUL
			 IF /I "%@USEW32TM%"=="TRUE" (
				 W32TM /CONFIG /COMPUTER:%%~S "/ManualPeerList:!@NEW_TIMESOURCE:~1,999!" /SyncFromFlags:MANUAL /RELIABLE:YES /UPDATE
			 ) ELSE (
				 NET TIME \\%%~S /SETSNTP:"!@OLD_TIMESOURCE:~1,999!,0x1"
			 )
		 ) ELSE (
			 rem ----------------------------------------
			 rem -- Configure NTP Client Systems
			 rem ----------------------------------------
			 ECHO [!TIME: =0!] Configuring %%~S as an NTP Client System >CON
			 ECHO [!TIME: =0!] Configuring %%~S as an NTP Client System
			 REG ADD "\\%%~S\%@NTP_CONFIG_KEY%" /v "AnnounceFlags" /t REG_DWORD /d 10 /f >NUL
			 REG ADD "\\%%~S\%@NTP_PARAM_KEY%" /v "NtpServer" /t REG_SZ /d "!@BAK_TIMESOURCE:~1,999!" /f >NUL
			 IF /I "%@USEW32TM%"=="TRUE" (
				 W32TM /CONFIG /COMPUTER:%%~S "/ManualPeerList:!@BAK_TIMESOURCE:~1,999!" /SyncFromFlags:DOMHIER /RELIABLE:NO /UPDATE
			 ) ELSE (
				 NET TIME \\%%~S /SETSNTP:""
			 )
		 )

		 rem -- Set Variable Based on Computer Name for Each Successful Machine Configuration
		 SET @%%~S-SET=TRUE
		 CALL :WaitForRestart \\%%~S "%@TIMESVC%"

		 rem -- Display Current NTP Setting
		 IF /I "%@USEW32TM%"=="TRUE" (
			 W32TM /RESYNC /COMPUTER:%%~S /REDISCOVER
			 W32TM /QUERY /COMPUTER:%%~S /PEERS
			 ECHO:
			 W32TM /TZ
		 ) ELSE (
			 NET TIME \\%%S /QUERYSNTP
		 )

		 NET TIME \\%%~S | FIND "Current time"
		 ECHO %@MINORDIV%
		 ECHO:
		 ECHO:
		 ECHO: >CON
	 )
 ) >>"%@LOGFILE%" 2>&1
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Perform Log Management/Maintenance (Or Copy Them for Debug Purposes)
 rem === Updated On: 11 Jan 2015 / 24 Dec 2014              
 rem ==========================================================================
:LogMaint
 ECHO PERFORMING FINAL LOG MAINTENANCE...

 FOR %%L IN (%@LOGLIST%) DO IF EXIST %%L (
	 SET @THISLOG=%%L
	 COPY %%L !@THISLOG:%@FILESTAMP%=TXT! /Y
	 IF /I "%DEBUG%"=="COPY" COPY %%L "%TEMP%" /Y
	 IF %%L=="%@LOGFILE%" (ECHO: & TYPE %%L & DIR %%L) ELSE (DEL %%L >NUL)
 )
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019              
 rem ==========================================================================
:ExitBatch
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Execute Dynamic Command From Variable
 rem === Updated On: 03 Aug 2010                     
 rem ==========================================================================
:RunCMD
 rem %1 = Command To Execute

 %*
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Wait For Service Restart To Fully Process
 rem === Updated On: 02 Dec 2018 / 24 Dec 2011              
 rem ==========================================================================
:WaitForRestart
 rem %1 = Name of System
 rem %2 = Name of Service

 SET /A @LOOP=0
 SET /A @LIMIT=%@RETRY_LIMIT%
 ECHO:
 ECHO Restarting "%~2" Service on %~1
 SC %~1 STOP "%~2" | FIND "STATE"

:QueryStop_Loop
 FOR /F "TOKENS=4" %%Q IN ('SC %~1 QUERY "%~2" ^| FIND "STATE"') DO SET @STATE=%%~Q
 IF DEFINED DEBUG ECHO  -@LOOP=!@LOOP!
 SET /A @LOOP+=1& IF !@LOOP! GTR !@LIMIT! GOTO :QueryStart
 IF /I NOT "!@STATE!"=="STOPPED" GOTO :QueryStop_Loop

 rem -- Start the W32Time Service
:QueryStart
 SET /A @LOOP=0
 SC %~1 START "%~2" | FIND "STATE"

:QueryStart_Loop
 FOR /F "TOKENS=4" %%Q IN ('SC %~1 QUERY "%~2" ^| FIND "STATE"') DO SET @STATE=%%~Q
 IF DEFINED DEBUG ECHO  -@LOOP=!@LOOP!
 SET /A @LOOP+=1& IF !@LOOP! GTR !@LIMIT! GOTO :EOF
 IF /I NOT "!@STATE!"=="RUNNING" GOTO :QueryStart_Loop
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET @SCRIPT_BEG#="%%~d"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 30 Nov 2020 / 02 Dec 2018              
 rem ==========================================================================
:HelpMessage
 ECHO Configures the NTP settings for a set of machines in a domain or workgroup
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  ^<. ^| list_of_machines^>
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:
 ECHO -------------------
 ECHO OPTION DEFINITIONS:
 ECHO -------------------
 ECHO  -?,  /H ..... Display This Help Message  (also --HELP)
 ECHO  -??, /H2 .... Display Extended Help Message
 ECHO       .  ..... Use default server list
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0
	 ECHO  %~n0 .
	 ECHO  %~n0 %SystemRoot%\Temp\Servers.TXT
	 ECHO  %~n0 %~dp0Input\ServerList.TXT
	 ECHO  %~n0 "D:\Lists\My Server List.TXT"
	 ECHO  %~n0 "%TEMP%\Lists\My Workgroup.TXT"
	 ECHO  %~n0 -?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
	 ECHO:
	 ECHO  * Any parameters which contain spaces should be surrounded by quotes.
	 ECHO       Shortnames *can* be surrounded by quotes if you choose, but it is
	 ECHO       not mandatory by any means.
 )
 CALL :ShowStatus "FINISHED"
                                                                            