@ECHO OFF
 rem ==========================================================================
 rem === Startup Script -- Install SysLog Agent / Log System Diagnostics Info
 rem ==========================================================================
 rem
 rem Created On: 07 Dec 2003
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Oct 2023 / 07 Dec 2022
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2003 and later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    Event-to-SysLog ........ https://code.google.com/archive/p/eventlog-to-syslog/
 rem ==========================================================================

 :::  This is a generic startup script which allows you to update various
 :::  scripts and utilities on your client machines, and it also installs
 :::  an EventLog-to-SysLog service.
 :::
 :::  Be sure to add all of the files/folders into the appropriate GPO folder
 :::  in order to have them copied to your client systems.
 :::
 :::  Example:
 :::        -- Logon.Settings
 :::        -- NOW.EXE
 :::        -- DateInfo.EXE
 :::        -- CheckParams.EXE
 :::        -- PrintFileInfo.EXE
 :::        -- ReadConfig.EXE
 :::        -- SyslogStatus.BAT
 :::        -- Startup.BAT
 :::        ----- Scripts\Util1.bat
 :::        ----- Scripts\Util2.exe
 :::        ----- Scripts\Util3.exe
 :::        ----- System32\File1.bat
 :::        ----- System32\File2.exe
 :::        ----- System32\File3.sys
 :::
 :::  There's a lot more you can do with a script like this...
 :::
 :::  (Feb 2019) v3.0 -- Updated DEBUG routine; Changed Logging methodology;
 :::  -- Added new DATEINFO --Script-Variables routine to set default variables;
 :::  -- Replaced FILEVER.EXE with PRINTFILEINFO.EXE;
 :::
 :::  (Oct 2019) v3.1 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::  -- Moved the CMD filtering to its own routine, and added {/?} filtering;
 :::
 :::  (Dec 2019) v3.2 -- Change FC to FILEHASH in File Verification Routine;
 :::  -- Corrected some issues with the Syslog Agent Service routines;
 :::
 :::  (Jan 2020) v4.0 -- The logon scripts are being modified to pull custom
 :::  settings from the "Logon.Settings.TXT" file to ensure that the scripts
 :::  themselves are consistent across various deployments;
 :::
 :::  (Mar 2020) v4.2 -- Create LogonData folder, and set visibility;
 :::
 :::  (Mar 2020) v4.3 -- Added routine to upload StartUp report to network;
 :::
 :::  (Apr 2020) v4.4 -- Updated FILEHASH to support the BLAKE3 and XXHASH64 ciphers;
 :::
 :::  (May 2020) v4.5 -- Updated :DebugMode routine to support the STEP option;
 :::  -- Use ReadConfig.exe to process the Logon.Settings config file;
 :::
 :::  (Jul 2020) v4.6 -- Add Version Information to the History Log;
 :::
 :::  (Jul 2020) v4.7 -- Added extra version info for the logon/logoff family
 :::  of scripts in the format of yyyy.mm.dd.hh.nn (@VERDATE);
 :::
 :::  (Aug 2020) v4.8 -- Minor update to consolidated status report;
 :::
 :::  (Aug 2020) v5.0 -- Alter sequence of Debug Verification routine;
 :::  -- Read additional debug related variables from Logon.Setting.TXT;
 :::  -- Make Header and Footer entries consistent across all scripts;
 :::
 :::  (Dec 2022) v5.1 -- Add subfolders to repository copy jobs;
 :::
 :::  (Oct 2023) v5.2 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::  NOTE: The logon/logoff/etc scripts are mostly self-contained, and make
 :::        use of their own central config file (Logon.Settings) rather than
 :::        the common config files of all the other scripts.
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/tools-for-managing-the-windows-eventlog/
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 31 May 2020 / 05 Oct 2019
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 17 Aug 2020 / 11 Aug 2020
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v5.2.0.920  v2023.09.17.1500

 rem -- Obtain Date/Time and Supporting Core Variables
 FOR /F "TOKENS=1-2 DELIMS=;" %%P IN ('DATEINFO --Script-Variables 2^>NUL') DO SET %%P=%%~Q

 rem -- Search Logon.Settings.*.TXT for Custom Variables
 SET @SETTINGS=%~dp0Logon.Settings.
 SET @CONFIG=& FOR %%F IN ("TXT" "%COMPUTERNAME%.TXT") DO IF EXIST "!@SETTINGS!%%~F" SET @CONFIG="!@SETTINGS!%%~F"
 IF DEFINED @CONFIG (
	 FOR /F "TOKENS=*" %%P IN ('READCONFIG %@CONFIG% 2^>NUL') DO CALL :RunCMD %%P
 ) ELSE (
	 rem -- IP Address OR DNS Name for SysLog Server
	 FOR %%n IN ("" 1 2 3) DO SET @SYSLOG%%~n=192.168.255.22%%~n
 )
 SET @SYSLOG_SERVERS=& FOR %%X IN ("" 1 2 3 4 5 6 7 8 9 0) DO IF DEFINED @SYSLOG%%~X SET @SYSLOG_SERVERS=!@SYSLOG_SERVERS!;!@SYSLOG%%~X!

 rem -- Standard Variables
 SET @NOTFOUND=The specified service does not exist as an installed service.
 SET @UPTODATE=FALSE
 SET @SVCFOUND=TRUE
 SET @SVCSTATE=*****

 rem -- Standard Folder Variables
 SET @FOLDERS=Bat Utils
 SET @SOURCE1=%~dp0Scripts
 SET @SOURCE2=%~dp0System32& IF /I "%@OS_ARCH%"=="x64" SET @SOURCE2=%~dp0System32\x64
 SET @DEST1=%SystemDrive%\Scripts
 SET @DEST2=%SystemRoot%\System32

 rem -- Syslog Agent Info
 SET @AGENT_NAME=EvtSys
 SET @AGENT_TITLE=EventLog-to-SysLog [%@AGENT_NAME% %@OS_ARCH%]
 SET @AGENT_STORE=%@DEST1%\Bat\Services\%@AGENT_NAME%

 rem -- Standard Log Variables
 SET @LOGPATH=%SystemDrive%\Storage\LogonData
 SET @LOGBASE=%@LOGPATH%\Logon Info
 SET @LOGFILE=%@LOGBASE%.%@FILESTAMP_R%
 SET @HISTORY=%@LOGBASE%.RECORDING.TXT
 SET @NETLOG=%@FILESERVER%\Maint\LogonData\%COMPUTERNAME%

 rem -- Version Info Messages
 SET @INITIATION=Startup Script .... %@LVERSION% ... STARTED on %COMPUTERNAME%
 SET @COMPLETION=%@INITIATION:STARTED=COMPLETED%


 rem ==========================================================================
 rem === Enable Debug Mode If 'Debug' Variable is Set to TRUE/LOG/STEP/VERBOSE
 rem === Updated On: 16 Aug 2020 / 31 May 2020
 rem ==========================================================================
:DebugMode
 SET #DEBUG=& FOR %%d IN (LOG STEP TRUE VERBOSE) DO IF /I "%Debug%"=="%%~d" SET #DEBUG=TRUE
 FOR %%d IN (%@DEBUG_SYS%) DO IF /I "%COMPUTERNAME%"=="%%~d" (SET #DEBUG=TRUE& SET DEBUG=STEP)
 IF EXIST "%@DEBUG_FLAG%" (SET #DEBUG=TRUE& SET DEBUG=STEP)
 IF DEFINED #DEBUG (
	 SET @DIAGMODE=TRUE
	 SET @DEBUG_MODE=[DEBUG MODE ON]
	 SET @PAUSE_IF_DEBUG=TIMEOUT 5
	 SET @PRINT_IF_DEBUG=ECHO:^& SET @^& ECHO:^& TIMEOUT 5
	 SET @END_DEBUG_MODE=ECHO:^& ECHO *** DEBUG MODE FINISHED ***^& ECHO:^& SET @^& TIMEOUT 30
	 SET @COMPLETION=%@COMPLETION% !@DEBUG_MODE!
	 IF /I "%Debug%"=="LOG" (SET @PAUSE_IF_DEBUG=& SET @PRINT_IF_DEBUG=ECHO:^& SET @^& ECHO:)
	 IF /I "%Debug%"=="STEP" (SET @PAUSE_IF_DEBUG=ECHO ##PAUSE##& SET @PRINT_IF_DEBUG=ECHO ##PRINT##)
	 IF /I "%Debug%"=="VERBOSE" (SET @VERBOSE_OFF=ECHO ON) ELSE (FOR %%t IN (ON OFF) DO SET @VERBOSE_%%t=ECHO %%t)
	 ECHO *** DEBUG MODE = ON ***
	 ECHO ON
 ) ELSE ( FOR %%d IN (DEBUG_MODE PAUSE_IF_DEBUG PRINT_IF_DEBUG END_DEBUG_MODE VERBOSE_ON VERBOSE_OFF) DO SET @%%d=)


 rem ==========================================================================
 rem === Record Startup Info locally, for diagnostics purposes
 rem === Updated On: 17 Aug 2020 / 05 Mar 2020
 rem ==========================================================================
:TrackStartup
 FOR %%D IN ("%@LOGPATH%" "%@NETLOG%") DO IF NOT EXIST "%%~D" MD "%%~D"
 ATTRIB +H "%@LOGPATH%"

 ( rem -- Write Log to File
	 NOW \*\v -- %@INITIATION%\n\*\n
	 SYSTEMINFO
	 COMPINFO
	 %@PRINT_IF_DEBUG%
 ) >"%@LOGFILE%" 2>&1


 rem ==========================================================================
 rem === Update Local Repository of Scripts and Utilities
 rem === Updated On: 07 Dec 2022 / 24 Apr 2020
 rem ==========================================================================
:UpdateFiles
 ( rem -- Write Log to File
	 NOW \n\v -- Updating System Scripts...\n
	 FOR %%V IN (%@FOLDERS%) DO XCOPY "%@SOURCE1%\%%~V" "%@DEST1%\%%~V" /C /I /D /F /H /K /Y /S
	                            XCOPY "%@SOURCE2%"      "%@DEST2%"      /C /I /D /F /H /K /Y /S

	 rem -- Validate Syslog Agent Service and Binaries
	 NOW \n\v -- Validating %@AGENT_TITLE% Installation...\n
	 IF EXIST "%@AGENT_STORE%\x32" REN "%@AGENT_STORE%\x32" x86
	 FOR /F "TOKENS=1" %%C IN ('FILEHASH --XXHash64 -K "%@SOURCE2%\%@AGENT_NAME%.EXE" "%@DEST2%\%@AGENT_NAME%.EXE" 2^>NUL') DO SET @UPTODATE=%%C
	 FOR /F "TOKENS=*" %%S IN ('SC QUERY "%@AGENT_NAME%" ^| FIND /I "%@NOTFOUND%"') DO SET @SVCFOUND=FALSE
	 FOR /F "TOKENS=4" %%S IN ('SC QUERY "%@AGENT_NAME%" ^| FIND "STATE"') DO SET @SVCSTATE=%%~S

	 rem -- Update or Install Syslog Agent Binaries if Files Aren't Current
	 IF /I "!@UPTODATE!!@SVCSTATE!"=="TRUERUNNING" (
		 ECHO %@AGENT_TITLE% service: *** UP-TO-DATE ***

	 ) ELSE (
		 rem -- Update Syslog Agent Binaries If Necessary
		 IF /I "!@UPTODATE!"=="FALSE" (
			 ECHO Updating %@AGENT_TITLE% service...
			 IF "!@SVCFOUND!"=="TRUE" (
				 IF /I "!@SVCSTATE!"=="RUNNING" (
					 NET STOP %@AGENT_NAME% /Y
					 SLEEP 2
				 )
				 %@DEST2%\%@AGENT_NAME% -u
				 SLEEP 2
			 )
			 XCOPY "%@SOURCE2%\%@AGENT_NAME%.*" "%@DEST2%" /C /I /F /H /K /Y
			 SLEEP 2
		 )

		 rem -- Install Syslog Agent Service If Necessary
		 IF /I NOT "!@SVCSTATE!"=="RUNNING" (
			 ECHO Installing %@AGENT_TITLE% service...
			 CALL :FindVersion "%@SOURCE2%\%@AGENT_NAME%.EXE"
			 IF DEFINED @FILEVER (
				 IF "!@FILEVER!" GEQ "4.5" (
					 %@DEST2%\%@AGENT_NAME% -i -h %@SYSLOG_SERVERS% -l 0 -f Local2
				 ) ELSE (
					 %@DEST2%\%@AGENT_NAME% -i -h %@SYSLOG% -b %@SYSLOG2% -l 0 -f Local2
				 )
			 ) ELSE (
				 ECHO Could Not Determine Version of %@AGENT_TITLE% Binaries
			 )
		 )

		 rem -- Start Syslog Agent Service if not running
		 FOR /F "TOKENS=4" %%S IN ('SC QUERY "%@AGENT_NAME%" ^| FIND "STATE"') DO IF NOT "%%~S"=="RUNNING" (
			 ECHO Starting %@AGENT_TITLE% service...
			 NET START %@AGENT_NAME%
		 )
	 )
 ) >>"%@LOGFILE%" 2>&1
 IF EXIST "%@LOGFILE%" TYPE "%@LOGFILE%"
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Set Machine-Level Registry Settings
 rem === Updated On: 17 Aug 2020 / 05 Mar 2020
 rem ==========================================================================
:SetRegistry
 ( rem -- Write Log to File
	 NOW \n\v -- Configuring Registry...\n
	 KIX32 "%~dp0Configure.KIX"
	 NOW \n\-\v -- %@COMPLETION%\n\-
	 %@PRINT_IF_DEBUG%
 ) >>"%@LOGFILE%" 2>&1
 IF EXIST "%@LOGFILE%" TYPE "%@LOGFILE%"
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Send Syslog Message When Completed (if available)
 rem === Updated On: 17 Aug 2020 / 13 Jul 2020
 rem ==========================================================================
:SendSyslogMessage
 NOW %@COMPLETION% >>"%@HISTORY%"
 IF EXIST "%~dp0SyslogStatus.BAT" CALL "%~dp0SyslogStatus.BAT" "%@SYSLOG_SERVERS%" . "6" "%~n0" %@COMPLETION%
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Copy Results to Network
 rem === Updated On: 12 Mar 2020
 rem ==========================================================================
:CopyToNetwork
 IF EXIST "%@LOGBASE%*.TXT" (
	 COPY "%@LOGFILE%" "%@NETLOG%.LOG" /Y
	 COPY "%@HISTORY%" "%@NETLOG%.TXT" /Y
	 COPY "%@LOGBASE%*.TXT" "%@NETLOG%" /Y
	 TYPE "%@LOGFILE%"
 )
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 05 Oct 2019 / 25 Mar 2016
 rem ==========================================================================
:ExitBatch
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Execute Dynamic Command From Variable
 rem === Updated On: 28 Jan 2020
 rem ==========================================================================
:RunCMD
 rem %* = Command To Execute

 %*
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Determine the version of the Agent Binary to be installed
 rem === Updated On: 17 Feb 2019 / 14 Apr 2014
 rem ==========================================================================
:FindVersion
 rem %1 = File to Check for Version Info

 SET @FILEVER=n/a
 FOR /F %%f IN ('PRINTFILEINFO "%~1" -q -f 2^>NUL') DO SET @FILEVER=%%~f
 IF /I "%@FILEVER%"=="n/a" SET @FILEVER=0.0.0.0
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 16 Feb 2022
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version
 rem %3 = Date-based Extended Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF NOT "%~3"=="" (SET @VERDATE=%~n0 %~3& SET @LVERSION=%~3)
 IF /I "%~1"=="STARTED" SET @SCRIPT_BEG#="%DATE:~4,10% %TIME: =0%"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF
                                                              