@ECHO OFF
 rem ==========================================================================
 rem === Capture Local Storage Usage using DriveInfo.BAT, DIRUSE and DISKSPD
 rem ==========================================================================
 rem
 rem Created On: 18 May 2009
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 24 Feb 2024 / 06 Feb 2024
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2003 or later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    InfoZip Utilities ...... http://sourceforge.net/projects/infozip/files/
 rem    SysLogGen .............. http://www.snmpsoft.com/freetools/sysloggen.html
 rem
 rem    DiskUsage .............. Native in Windows 11 Pro
 rem    DU ..................... https://technet.microsoft.com/en-us/sysinternals/bb896651
 rem    PSTOOLS ................ https://technet.microsoft.com/en-us/sysinternals/bb896649
 rem    XDIR ................... https://blogs.technet.microsoft.com/gbordier/tag/xdir-long-directories-folder-size/
 rem    DISKSPD ................ https://github.com/microsoft/diskspd
 rem    Resource Kit ........... https://en.wikipedia.org/wiki/Resource_Kit
 rem
 rem  Required Scripts:
 rem    DriveInfo.BAT .......... Script for reporting local and remote disk space
 rem    SetDrive.BAT ........... Set Global Variables for this Environment
 rem ==========================================================================

 :::  This script calls DriveInfo.BAT to obtain current freespace info, and
 :::  then enumerates the full disk consumption across all local drives.
 :::  It is currently scheduled as a daily job that runs at 2:30am local time.
 :::
 :::  (Nov 2010) Changed the default log extension of the reports from .LOG to
 :::  .TXT so that the reports that are mailed to mobile devices can be read
 :::  automatically. (Not too many devices can natively handle .LOG files)
 :::
 :::  (Jun 2011) Added support from the SysInternals DU.EXE utility to provide
 :::  detailed storage info for each folder on a drive.
 :::
 :::  (Oct 2011) As of version 2.8, support for additional parameters was
 :::  added so that the full directory and tree info can be captured if desired.
 :::
 :::  (Dec 2011) Simplified the job scheduling section by way of environment
 :::  variables for verifying whether to use SCHTASKS or the AT command.
 :::
 :::  (Jan 2012) Push the DIRUSE output to a temp file before executing the
 :::  FIND command to reduce the time for the processing and avoid aborted
 :::  jobs. Also eliminated all "Application Data" folders from the report.
 :::
 :::  (Dec 2012) Made it possible for the scheduled tasks to be stored in their
 :::  own folder, rather than the default root folder of the Task Scheduler.
 :::
 :::  (Oct 2014) v2.14 -- Ensured that the log folder is created on first run,
 :::  and that the names of temp folders are randomized to allow multiple
 :::  instances of the script to run concurrently.
 :::
 :::  (Dec 2014) v3.0 -- Used timestamped logfiles (instead of random logs) and
 :::  consolidate with new logfile maintenance routine.
 :::
 :::  (Dec 2015) v3.3 -- Capture the DIR and TREE output for the local Backups$
 :::  share, for use with the OpsLogs.BAT script.
 :::
 :::  (Nov 2017) v3.5 -- Add XDIR as one of the directory reporting tools
 :::
 :::  (May 2018) v4.0 -- Changed the HELP routine to use FINDSTR;
 :::  -- Added new Filtered CMD Title and Syntax Help processing routines;
 :::  -- Support for SysLog and JobHistory have been added to this script;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com
 :::
 :::  (Jul 2018) v4.1 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::
 :::  (Aug 2018) v4.2 -- Removed AT command support;
 :::
 :::  (Oct 2018) v4.3 -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Nov 2018) v4.4 -- Allow Schedule to be changed via CustomVariables*.TXT;
 :::
 :::  (Mar 2019) v4.5 -- Enabled Code Page 1252 Support;
 :::  -- Record additional attributes in the Job History log;
 :::  -- Prevent DEBUG MODE data from being stored in the .RECORDING.TXT file;
 :::  -- Change scheduled time from 2:30am to 3:15am to avoid DST issues;
 :::
 :::  (Oct 2019) v4.6 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (Oct 2019) v4.7 -- Update :PadLeft and :PadRight routines;
 :::
 :::  (Dec 2019) v4.8 -- Developed a drop-in replacement for Microsoft's
 :::  NOW.EXE utility, taking back that function from DATEINFO.EXE;
 :::
 :::  (Jan 2020) v5.0 -- Use CHECKPARAMS to evaluate parameters besides /? /H;
 :::  -- Added a routine for validating integer parameters;
 :::  -- Refactored the :Prepare4Syslog routine;
 :::  -- Consolidate :PadLeft and :PadRight routines;
 :::  -- Expanded Help Syntax Descriptions;
 :::
 :::  (May 2020) v5.1 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::
 :::  (Jul 2020) v5.2 -- Add a search for .DMP files as part of the storage scan;
 :::  -- Add support for the /M option to search for Memory Dump files only;
 :::
 :::  (Nov 2020) v5.3 -- Moved backups to SystemState folder under Backups$ share;
 :::  -- Streamlined SystemState reporting via both DIR and TREE commands;
 :::
 :::  (Nov 2020) v5.4 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Jan 2021) v6.0 -- Evaluate and record Disk I/O on one or more drives
 :::  using Microsoft DiskSpd and WinSat;
 :::
 :::  (Apr 2021) v6.1 -- Rename @SKIPDRIVE variable to @SPECIAL_DRV;
 :::  -- Add @SKIP_DRIVES variable to eliminate certain drives from evaluation;
 :::
 :::  (Jun 2021) v6.2 -- Rename @SPECIAL_DRV variable to @LABELS2SKIP;
 :::  -- Refactored the drive exclusion functionality to avoid all evaluations;
 :::
 :::  (Jul 2021) v6.3 -- Replace COMPINFO with PSINFO to avoid intermittent crashes;
 :::
 :::  (Jan 2022) v6.4 -- Modify the @NAMECHK verification to include \ prefix;
 :::  -- Use WMIC to capture the local drives that need to be checked for DiskSpeed;
 :::
 :::  (Oct 2023) v6.5 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::  (Feb 2024) v6.6 -- Add routine using the new Microsoft Disk Usage from
 :::  Windows 11 and later, and added Windows folder to the list;
 :::
 :::  (Feb 2024) v7.0 -- Run I/O tests on all local (FIXED) drives;
 :::  -- Provide Completion Time Info in Main Log;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 23 Jul 2020 / 14 May 2018
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 24 Feb 2024 / 06 Feb 2024
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v7.0.0.700

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Default Variables (Override via CustomVariables.*.TXT)
 SET @SKIP_DRIVES=A: B:
 SET @LABELS2SKIP=Secure.Cloud
 SET @BK_ROOT=\\%COMPUTERNAME%\Backups$
 SET @SYSSTATE=%@BK_ROOT%\SystemState
 SET @MINSIZE=250
 SET @DIR_REGEX="\[..\...\...\] \\....-..-.."
 SET @TREE_REGEX="|...........| |...........+ |...........\\ |...............\\ Folder.PATH Volume.serial ---InstallRoot ---Logs ---Database _vti_"

 rem -- Disk I/O Benchmarking Variables
 FOR /F "DELIMS=: " %%D IN ('WMIC LogicalDisk GET DeviceID^,DriveType 2^>NUL ^| FIND "3"') DO (
	 SET @TMP=%%~D:\Temp
	 IF NOT EXIST "!@TMP!" MD "!@TMP!" >NUL 2>NUL
	 IF EXIST "!@TMP!" SET @FILE-%%~D=!@TMP!\TestFile-%%~D.dat
	 SET @DRIVES=!@DRIVES! %%~D:
	 SET @FILES=!@FILES! !@FILE-%%~D!
 )
 SET @IO_MSG=Using DISKSPD ON %@DRIVES%
 SET @IO_OPTIONS=-c1G -b4K -t2 -d30 -w30 -si
 SET @WINSAT=& FOR /F %%V IN ('WHERE WINSAT 2^>NUL') DO SET @WINSAT=TRUE

 rem -- Job Scheduling Variables
 SET @JOBNAME=Track Disk Space Usage
 SET @JOBUSER=SYSTEM
 SET @JOBFREQ=DAILY
 SET @JOBTIME=03:15:00
 SET @JOBDATE=
 SET @JOBPATH="%~f0"
 SET @JOBTYPE=

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 CALL :SaveJobHistory ABORTED - NO %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- Standard OS Dependent Variables
 IF %@OSBUILD% GEQ 6000 (
	 SET @DIRCMD=/A-L
	 SET @APPDATA="\Application Data\Application Data\" "\Local Settings\"
	 SET @USERDATA="%SystemDrive%\Users"
 ) ELSE (
	 SET @DIRCMD=
	 SET @APPDATA="\\\\\\"
	 SET @USERDATA="%SystemDrive%\Documents and Settings"
 )

 rem -- Does DISKUSAGE Exist?
 FOR /F %%F IN ('FINDFILES -f "DISKUSAGE.EXE" 2^>NUL') DO SET @DISKUSAGE=%%~F

 rem -- Folder Math
 FOR /F %%M IN ('CCALC !@MINSIZE! * 1024 * 1024 2^>NUL') DO SET @MINSIZE-MB=%%~M

 rem -- Main Variables
 SET @BASEDRIVES= C:\ D:\ M:\ N:\
 SET @DIAGMODE=FALSE
 SET @PROGFILES="%ProgramFiles%" "%ProgramFiles(x86)%" "%SystemDrive%\Programs"

 rem -- Standard Log Variables
 SET @LOGPATH=%@STORAGE%\Logs\Maint
 SET @LOGFILE=%@LOGPATH%\%~n0.%@FILESTAMP%
 SET @TIMINGS=%@LOGPATH%\%~n0.RECORDING.TXT
 SET @OSDUMPS=%@LOGPATH%\OS_DUMPS.%@FILESTAMP%
 SET @DISK_IO=%@LOGPATH%\Disk-IO.%@FILESTAMP%
 FOR %%V IN (DIR TREE) DO SET @BACKUPS_%%~V=%@LOGPATH%\BackupsInfo-%%~V.%@FILESTAMP%
 SET @TEMPLOG=%@LOGFILE%.TEMP
 SET @FINDSTR=%@LOGFILE%.FIND
 SET @LOGLIST="%@LOGFILE%" "%@BACKUPS_DIR%" "%@BACKUPS_TREE%" "%@OSDUMPS%" "%@DISK_IO%"
 SET @CLEANUP="%@TEMPLOG%" "%@FINDSTR%" "!@TEMPLOG:TXT=%@FILESTAMP%!" "!@FINDSTR:TXT=%@FILESTAMP%!" %@FILES%

 rem -- Ensure Valid Log Processing Values
 CALL :GetInteger @MINROWS "%@MIN_LOG_ROWS%" 005 001 010
 CALL :GetInteger @MAXROWS "%@MAX_LOG_ROWS%" 199 030 399

 rem -- Process Special Command-line Parameters
 SET @ALL_PARAMS=%*
 CALL :GetParams "E * EXCLUDE" @EXCLUDE_DRV & IF NOT DEFINED @EXCLUDE_DRV SET @EXCLUDE_DRV=%@SKIP_DRIVES%

 CALL :GetParams "D DIAG"        & IF DEFINED @OK (SET @DIAGMODE=TRUE)
 CALL :GetParams "S ."           & IF DEFINED @OK (SET @UPDATESCHED=TRUE& SET @SCHEDULEONLY=TRUE)
 CALL :GetParams "U -"           & IF DEFINED @OK (SET @UPDATESCHED=TRUE& SET @SCHEDULEONLY=TRUE& SET @UNSCHED=TRUE)
 CALL :GetParams "N @"           & IF DEFINED @OK (SET @NOSCHED=TRUE)
 CALL :GetParams "M # MEMORY"    & IF DEFINED @OK (SET @DUMPONLY=TRUE) ELSE (SET @DUMPONLY=)
 CALL :GetParams "I $ IO DISKIO" & IF DEFINED @OK (SET @IO_ONLY=TRUE)  ELSE (SET @IO_ONLY=)

 rem -- Create Data Input File for FINDSTR Commands
 FOR %%V IN (%@APPDATA%) DO ECHO %%~V>>"%@FINDSTR%"
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Create Scheduled Job using SCHTASKS (XP/2003/Vista/+)
 rem === Updated On: 22 Jan 2022 / 09 Mar 2019
 rem ==========================================================================
:ScheduleJobs
 IF /I "%@NOSCHED%"=="TRUE" IF /I NOT "%@UNSCHED%"=="TRUE" (
	 SET @JOBTYPE= // ONE-TIME
	 ECHO:
	 ECHO *** Job Scheduling Disabled ***
	 ECHO:
	 GOTO :ScheduleJobs_END
 )

 ECHO:
 ECHO Scheduling "%@JOBNAME%" on %COMPUTERNAME%...
 ECHO:

 IF %@OSBUILD% LSS 6000 SET @TASKFOLDER=
 SET @NAMECHK=%@JOBNAME:[=\[%
 SET @NAMECHK=!@NAMECHK:]=\]!
 SET @CHK_SCHED=SCHTASKS /QUERY /V /FO LIST
 SET @SET_SCHED=SCHTASKS /CREATE /TN "!@TASKFOLDER!%@JOBNAME%" /TR "%@JOBPATH%" /RU "%@JOBUSER%" /SC %@JOBFREQ% %@JOBDATE% /ST %@JOBTIME% %@JOBLEVEL% %@FORCE%

 rem -- Delete Scheduled Task from Old Custom Folders on Vista/2008+
 IF %@OSBUILD% GEQ 6000 (
	 FOR /F "TOKENS=1* DELIMS=\ " %%s IN ('%@CHK_SCHED% 2^>NUL ^| FIND "TaskName:" ^| FINDSTR /I /R /C:"\\%@NAMECHK%$"') DO (
		 IF /I NOT "%@TASKFOLDER%%@JOBNAME%"=="%%t" SCHTASKS /DELETE /TN "%%t" /F >NUL 2>&1
	 )
 )

 rem -- Check for Schedule Update Parameters
 IF /I "%@UPDATESCHED%"=="TRUE" (!@SET_SCHED!) ELSE (CALL :CheckSched "%~f0")
 ECHO %@DIVIDER-%
 ECHO:
 IF /I "%@SCHEDULEONLY%"=="TRUE" (
	 ECHO Task has been scheduled.  Now exiting script...
	 IF %@OSBUILD% LSS 6000 (SCHTASKS) ELSE (%@CHK_SCHED% /TN "!@TASKFOLDER!%@JOBNAME%")
	 GOTO :ExitBatch
 )
:ScheduleJobs_END
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Display Details Disk Usage (via PSINFO, DISKUSAGE, XDIR, DIRUSE and DU)
 rem === Updated On: 06 Feb 2024 / 28 Nov 2021
 rem ==========================================================================
:DetailedDiskSpace
 IF NOT EXIST "%@LOGPATH%" MD "%@LOGPATH%"

 IF NOT DEFINED @EXCLUDE_DRV SET @EXCLUDE_DRV=~NONE~
 FOR /F "TOKENS=1" %%L IN ('PSINFO %@PSPARAMS% -d 2^>NUL ^| FIND ": Fixed" ^| FINDSTR /I /V "%@LABELS2SKIP% %@EXCLUDE_DRV%"') DO SET @LOCALDRIVES=!@LOCALDRIVES! %%~L
 IF NOT DEFINED @LOCALDRIVES SET @LOCALDRIVES=%@BASEDRIVES%
 SET @DRIVESONLY=%@LOCALDRIVES%& FOR %%F IN (%@PROGFILES% %@USERDATA% %SystemRoot%) DO IF EXIST "%%~F" SET @LOCALDRIVES=!@LOCALDRIVES! "%%~F"

 ( rem -- Write to Log
	 NOW "%@VER% [\v] -- DIAGMODE=%@DIAGMODE%\n"
	 ECHO DRIVES/FOLDERS TO BE CHECKED .....%@LOCALDRIVES%
	 ECHO:
	 ECHO DRIVES TO BE EXCLUDED ............ %@EXCLUDE_DRV%
	 ECHO:
	 ECHO MINIMUM FOLDER SIZE TO REPORT .... %@MINSIZE%MB
	 ECHO %@DIVIDER*%
	 ECHO:
 ) >>"%@LOGFILE%" 2>&1

 rem -- Check for DumpOnly or IO_Only options
 IF DEFINED @IO_ONLY  GOTO :DiskPerf
 IF DEFINED @DUMPONLY GOTO :MemoryDumps

 rem -- Continue Normal Sequence
 ECHO:
 ECHO Searching local drives for files larger than %@MINSIZE%MB
 ECHO ---------------------------------------------------------
 ECHO:
 ECHO [!TIME: =0!]: BASIC REPORT ----- Obtaining Free Disk Space
 CALL "%~dp0DriveInfo.BAT" >>"%@LOGFILE%" 2>&1

 rem -- Obtain Detailed Report
 ECHO [!TIME: =0!]: DETAILED REPORT -- Obtaining Disk Space Consumption
 ( rem -- Write to Log
	 ECHO:
	 FOR %%V IN (%@LOCALDRIVES%) DO (
		 ECHO [!TIME: =0!]: Directory Usage [DU] Report for "%%~V"
		 CALL :GetFolderName "%%~V"
		 DU %@PSPARAMS% -L 1 !@THISDIR!\ 2>NUL
		 ECHO:
	 )
	 CALL :ShowDivider "%@DIVIDER-%" 3

	 rem -- Provide Disk Consumption Data via DISKUSAGE [Windows 11 Only]
	 IF DEFINED @DISKUSAGE FOR %%V IN (%@LOCALDRIVES%) DO (
		 ECHO [!TIME: =0!]: Directory Usage [DISKUSAGE] Report for "%%~V" [^>%@MINSIZE% MB]
		 CALL :GetFolderName "%%~V"
		 DISKUSAGE /D:1 /E:%@MINSIZE-MB% !@THISDIR!\
		 ECHO:
		 ECHO %@DIVIDER-%
		 ECHO:
		 ECHO:
	 )
	 IF DEFINED @DISKUSAGE CALL :ShowDivider "%@DIVIDER-%" 3

	 rem -- Provide Disk Consumption Data via XDIR
	 FOR %%V IN (%@LOCALDRIVES%) DO (
		 ECHO [!TIME: =0!]: Directory Usage [XDIR] Report for "%%~V" [^>%@MINSIZE% MB]
		 CALL :GetFolderName "%%~V"
		 XDIR !@THISDIR!\ /MIN:%@MINSIZE%MB /LEV:2 /FORCE
		 ECHO:
		 ECHO %@DIVIDER-%
		 ECHO:
		 ECHO:
	 )
	 CALL :ShowDivider "%@DIVIDER-%" 3

	 rem -- Provide Disk Consumption Data via DIRUSE
	 ECHO [!TIME: =0!]: Directory Usage [DIRUSE] Report for %@LOCALDRIVES% [^>%@MINSIZE% MB]
	 DIRUSE /M /, /* /S /Q:%@MINSIZE% /D %@LOCALDRIVES% >"%@TEMPLOG%"
	 FINDSTR /I /V /G:"%@FINDSTR%" "%@TEMPLOG%"
	 CALL :ShowDivider "%@DIVIDER-%" 3

	 rem -- Provide List of 15 Largest Directories via DU
	 FOR %%V IN (%@DRIVESONLY%) DO (
		 ECHO:
		 ECHO [!TIME: =0!]: Top 15 Directories Report [DU] on %%~V
		 ECHO %@DIVIDER-%
		 DU %@PSPARAMS% -L 10 %%V | FIND ":\" | SORT >"%@TEMPLOG%" 2>NUL
		 TAIL -15 "%@TEMPLOG%" | SORT /R
		 ECHO %@DIVIDER-%
		 ECHO:
	 )
	 CALL :ShowDivider "%@DIVIDER*%" 3

	 rem -- Provide Extended Directory Info for Diagnostics Purposes
	 IF /I "%@DIAGMODE%"=="TRUE" (
		 FOR %%V IN (%@DRIVESONLY%) DO (
			 ECHO [!TIME: =0!]: EXTENDED REPORT -- Obtaining Folders and Files on %%V >CON
			 TREE %%V /A
			 CALL :ShowDivider "%@DIVIDER-%" 2
		 )

		 FOR %%V IN (%@DRIVESONLY%) DO (
			 DIR %%V /OGD %@DIRCMD% /S | FINDSTR /I /V /G:"%@FINDSTR%"
			 CALL :ShowDivider "%@DIVIDER-%" 2
		 )
	 )
	 NOW STORAGE REPORT COMPLETED
	 CALL :ShowDivider "%@DIVIDER*%" 3
 ) >>"%@LOGFILE%" 2>NUL
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Obtain DIR and TREE of Backup$ Share
 rem === Updated On: 05 Nov 2020 / 13 Dec 2019
 rem ==========================================================================
:Backups$-Info
 ECHO [!TIME: =0!]: BACKUPS$ REPORT -- Obtaining DIR and TREE Information for Backup$ Share

 FOR %%V IN (DIR TREE) DO ( rem -- Write to Log
	 NOW %@VER% [\v] -- %%~V\n\-
 ) >>"!@BACKUPS_%%~V!" 2>&1

 IF EXIST "%@BK_ROOT%" (
	 TREE "%@BK_ROOT%" /A | FINDSTR /I /R /V %@TREE_REGEX% >>"%@BACKUPS_TREE%" 2>&1

	 ( rem -- Write to Log
		 DIR /OGN /B /S "%@BK_ROOT%" | FINDSTR /I /V /R %@DIR_REGEX% | SORT
		 ECHO %@DIVIDER~%
		 DIR /OGD "%@SYSSTATE%" | FIND /I "SystemState-"
		 ECHO %@DIVIDER~%
		 DIR /OGN /S "%@BK_ROOT%" | FINDSTR /I /R "\.ZIP \.7Z"
	 ) >>"%@BACKUPS_DIR%" 2>&1
 )

 FOR %%V IN (TREE DIR) DO (
	 TYPE "!@BACKUPS_%%~V!"
	 CALL :ShowDivider "%@DIVIDER-%" 2
 ) >>"%@LOGFILE%" 2>&1
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Check for MEMORY.DMP and other .DMP files
 rem === Updated On: 10 Jan 2021 / 05 Nov 2020 / 23 Jul 2020
 rem ==========================================================================
:MemoryDumps
 ECHO [!TIME: =0!]: MEMORY DUMP REPORT -- Check for MEMORY.DMP and other .DMP files

 ( rem -- Write to Log
	 NOW %@VER% [\v] -- MEMORY DUMPS\n\-
	 DIR "%WINDIR%\*.DMP" /OGD /S 2>NUL | FINDSTR /R ":[0-9]"
 ) >>"!@OSDUMPS!" 2>&1

 ( rem -- Write to Log
	 TYPE "!@OSDUMPS!"
	 CALL :ShowDivider "%@DIVIDER-%" 2
 ) >>"%@LOGFILE%" 2>&1
 %@PRINT_IF_DEBUG%
 IF DEFINED @DUMPONLY GOTO :LogMaint


 rem ==========================================================================
 rem === Use Microsoft DiskSpd to Evaluate Disk I/O Performance
 rem === Updated On: 24 Feb 2024 / 13 Jan 2021
 rem ==========================================================================
:DiskPerf
 ECHO [!TIME: =0!]: DISK I/O REPORT -- Using %@IO_MSG%

 ( rem -- Write to Log
	 NOW %@VER% [\v] -- DISK PERFORMANCE via %@IO_MSG% \n\-
	 DiskSpd %@IO_OPTIONS% %@FILES%

	 IF DEFINED WINSAT (
		 NOW \n\n[\v] -- SYSTEM PERFORMANCE via WINSAT\n\-
		 FOR %%C IN (FEATURES MEM) DO (WinSat %%C& ECHO:)
		 FOR %%D IN (%@DRIVES%) DO (WinSat DISK -drive %%~D& ECHO:)
	 )
 ) >>"!@DISK_IO!" 2>&1

 ( rem -- Write to Log
	 TYPE "!@DISK_IO!"
	 CALL :ShowDivider "%@DIVIDER-%" 2
 	 NOW \n\*%~n0 Job Completed - [\v] - %@VER%\n\*
 ) >>"%@LOGFILE%" 2>&1
 %@PRINT_IF_DEBUG%


 rem ==========================================================================
 rem === Perform Log Management/Maintenance (Or Copy Them for Debug Purposes)
 rem === Updated On: 05 Nov 2020 / 15 May 2018
 rem ==========================================================================
:LogMaint
 CALL :SaveJobHistory
 ECHO [!TIME: =0!]: PERFORMING FINAL LOG MAINTENANCE...

 IF /I "%DEBUG%"=="COPY" FOR %%L IN (%@LOGLIST% %@CLEANUP%) DO IF EXIST %%L COPY %%L "%TEMP%" /Y

 FOR %%L IN (%@LOGLIST%) DO IF EXIST %%L (
	 SET @THISLOG=%%L
	 COPY %%L !@THISLOG:%@FILESTAMP%=TXT! /Y
	 IF %%L=="%@LOGFILE%" (ECHO: & TYPE %%L & DIR %%L)
 )
 FOR %%L IN (%@CLEANUP%) DO IF EXIST %%L DEL %%L >NUL
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019
 rem ==========================================================================
:ExitBatch
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Get Valid Folder Name (with or without quotes)
 rem === Updated On: 23 Nov 2017      
 rem ==========================================================================
:GetFolderName
 rem %1 = Folder to Evaluate

 SET @THISDIR=%~1
 IF /I NOT "%~s1"=="%~1" SET @THISDIR="%~1"
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Output Divider
 rem === Updated On: 03 Oct 2011      
 rem ==========================================================================
:ShowDivider
 rem %1 = Divider to Display
 rem %2 = Number of Lines to Display
 FOR /L %%Z IN (1,1,%~2) DO ECHO %~1
 ECHO:
 ECHO:
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Store Current Date and Time in Variable
 rem === Updated On: 16 Oct 2023 / 14 May 2018
 rem ==========================================================================
:GetTime
 rem %1 = Variable to Store Date/Time

 SET %~1=!DATE! at !TIME: =0!
 FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET %~1#="%%~d"
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Determine if Parameter is a Valid Integer, Or Set to Zero
 rem === Updated On: 21 Jan 2020      
 rem ==========================================================================
:GetInteger
 rem %1 = Variable to Store Integer
 rem %2 = Parameter to Evaluate
 rem %3 = Value to Default to, If Zero
 rem %4 = Optional: Lowest Value
 rem %5 = Optional: Highest Value

 SET @V1=%2& IF DEFINED @V1 SET @V1=!@V1:"=!
 SET @V2=%3& IF DEFINED @V2 SET @V2=!@V2:"=!
 SET @V3=%4& IF DEFINED @V3 SET @V3=!@V3:"=!
 SET @V4=%5& IF DEFINED @V4 SET @V4=!@V4:"=!
 FOR /F "TOKENS=3" %%i IN ('CHECKPARAMS -o --min "!@V3!" --max "!@V4!" -n "!@V1!" "!@V2!" 2^>NUL') DO SET %~1=%%i
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Use CheckParams.exe to Capture Selected Parameters
 rem === Updated On: 21 Jan 2020      
 rem ==========================================================================
:GetParams
 rem %1 = Parameters to Search For
 rem %2 = Variable to Set

 SET @OK=& IF "%~1"=="" GOTO :EOF
 FOR /F "TOKENS=2-3*" %%v IN ('CHECKPARAMS -q -a -c "%~1" -s %@ALL_PARAMS% 2^>NUL') DO IF /I "%%~v"=="TRUE" (
	 SET @OK=T
	 SET @REQ=T
	 IF NOT "%~2"=="" SET %~2=%%~x
 )
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Pad Variable Content to the Left/Right for Display in Logs
 rem === Updated On: 22 Jan 2020 / 14 Oct 2019
 rem ==========================================================================
:PadLeft
:PadRight
 rem %1 = Variable to Store New Content
 rem %2 = Existing Value to Pad
 rem %3 = Character to use as padding [typically " ", "." or "0"]
 rem %4 = New String Length

 SET @STR=%~2
 SET @LEN=& FOR /L %%c IN (0,1,199) DO IF "!@STR:~%%c,1!"=="" IF NOT DEFINED @LEN SET @LEN=%%c
 IF 1%@LEN% GEQ 1%~4 (
	 SET %~1=%~2
 ) ELSE (
	 IF /I "%~0"==":PadLeft" (
		 SET @TEMPVAR=& FOR /L %%c IN (1,1,%~4) DO SET @TEMPVAR=!@TEMPVAR!%~3
		 SET @TEMPVAR=!@TEMPVAR!%~2
		 SET %~1=!@TEMPVAR:~-%~4!
	 )
	 IF /I "%~0"==":PadRight" (
		 SET @TEMPVAR=%~2& FOR /L %%c IN (1,1,%~4) DO SET @TEMPVAR=!@TEMPVAR!%~3
		 SET %~1=!@TEMPVAR:~0,%~4!
	 )
 )
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Update Job Execution Stats
 rem === Updated On: 15 Oct 2019 / 09 Mar 2019
 rem ==========================================================================
:SaveJobHistory
 rem %* = Optional Message to include in Job History

 ECHO UPDATING JOB HISTORY.....
 FOR /F "TOKENS=4" %%S IN ('DIR "%@LOGFILE%" ^| FIND /I ".TXT"') DO SET @LOGSIZE=%%S

 CALL :GetTime @SCRIPT_END
 CALL :PadLeft @LS "!@LOGSIZE!" "." 9
 SET @ROWCOUNT=0
 SET @TRUNCATED=
 SET @ELAPSED=%@NO_TIME%& FOR /F "DELIMS=" %%d IN ('DATEINFO -t !@SCRIPT_BEG#! -n !@SCRIPT_END#! -e "hr:min:sec.ms" -q 2^>NUL') DO SET @ELAPSED=%%d
 SET @SPECIAL=%@JOBTYPE%& IF NOT "%~1"=="" SET @SPECIAL= // %*
 SET @JOBHISTORY=%@JOBNAME% // LOG SIZE.%@LS% // DURATION: %@ELAPSED% // START: %@SCRIPT_BEG% // END: !@SCRIPT_END! // %~n0 %@VERSION%%@SPECIAL%

 %@VERBOSE_OFF%
 BREAK ON >>"%@TIMINGS%"
 FOR /F %%c IN ('TYPE "%@TIMINGS%" ^| FIND /C "/"') DO SET @ROWCOUNT=%%c
 IF !@ROWCOUNT! GTR %@MAXROWS% (
	 SET @TRUNCATED= // TRUNCATED
	 SET @OLDTIME=!@TIMINGS:TXT=%@FILESTAMP%!
	 COPY "%@TIMINGS%" "!@OLDTIME!" /Y >CON
	 TAIL -%@MINROWS% "!@OLDTIME!" >"%@TIMINGS%" 2>&1
	 ECHO:
	 CALL :ForwardToSyslog "5" %@JOBNAME%: Job History Logfile Truncated -- [!DATE! at !TIME!]
 ) >>"%@LOGFILE%" 2>&1

 CALL :ForwardToSyslog "6" %@JOBHISTORY%!@TRUNCATED! >>"%@TIMINGS%" 2>&1
 %@VERBOSE_ON%
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Forward Output to Available Syslog Server(s) as LOCAL3
 rem === Updated On: 13 Jan 2020 / 07 Mar 2019
 rem ==========================================================================
:ForwardToSyslog
 rem %1 = Syslog Severity Level (RFC 3164)
 rem %* = Message to Send to Display and Send to Syslog Server

 SET @S_MSG=%*& IF "%~2"=="" SET @S_MSG=%1 - This is a Syslog Test Message from %@VER%
 SET @SEV=6
 FOR %%a IN (0 EMERGENCY) DO IF /I "%~1"=="%%~a" SET @SEV=0
 FOR %%a IN (1 ALERT) DO IF /I "%~1"=="%%~a" SET @SEV=1
 FOR %%a IN (2 CRITICAL) DO IF /I "%~1"=="%%~a" SET @SEV=2
 FOR %%a IN (3 ERROR) DO IF /I "%~1"=="%%~a" SET @SEV=3
 FOR %%a IN (4 WARNING) DO IF /I "%~1"=="%%~a" SET @SEV=4
 FOR %%a IN (5 NOTICE) DO IF /I "%~1"=="%%~a" SET @SEV=5
 FOR %%a IN (6 INFORMATION) DO IF /I "%~1"=="%%~a" SET @SEV=6
 FOR %%a IN (7 DEBUG) DO IF /I "%~1"=="%%~a" SET @SEV=7

 %@VERBOSE_OFF%
 CALL :Prepare4Syslog %@S_MSG%

 ECHO !@S_MSG!
 IF DEFINED @SYSLOG_UTIL IF /I "%@USE_SYSLOGS%"=="TRUE" FOR %%s IN ("" 0 1 2 3 4) DO IF DEFINED @SYSLOG%%~s (
	 SET @S_PORT=514& IF DEFINED @SYSLOG%%s_PORT SET @S_PORT=!@SYSLOG%%~s_PORT!
	 "%@SYSLOGGEN%" -t:!@SYSLOG%%~s! -p:!@S_PORT! -f:19 -s:!@SEV! -tg:"%~n0" -m:"!@S_MSG2!" -q -hn:%@LOCALHOST% | FIND /V "%@NO_SYSLOGGEN%"
 )
 %@VERBOSE_ON%
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Prepare Messages for Syslog Utility (SyslogGen)
 rem === Updated On: 19 Jan 2020 / 14 May 2018
 rem ==========================================================================
:Prepare4Syslog
 rem %* = Syslog Message to be Cleaned Up

 SET @PREFIX=%1
 SET @S_MSG=%*
 SET @S_MSG=!@S_MSG:%@PREFIX% =!

:TrimLeadingSpaces
 IF "!@S_MSG:~0,1!"==" " (
	 SET @S_MSG=!@S_MSG:~1,9999!
	 GOTO :TrimLeadingSpaces
 )

 SET @S_MSG2=!@S_MSG:\=\x5B!
 SET @S_MSG2=!@S_MSG2:"=\x22!
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Check if Scheduled Job is Already Running
 rem === Updated On: 15 May 2018 / 19 Jan 2015
 rem ==========================================================================
:CheckSched
 rem %1 = Standard Search Criteria
 rem %2 = Advanced Search Criteria

 SET @CHK=%2
 IF DEFINED @CHK (
	 %@CHK_SCHED% 2>NUL | FIND /I "%~1" | FINDSTR /I /R /C:%2 >NUL
 ) ELSE (
	 %@CHK_SCHED% 2>NUL | FIND /I "%~1" >NUL
 )
 IF ERRORLEVEL 1 %@SET_SCHED%
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" CALL :GetTime @SCRIPT_BEG
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 20 Apr 2021 / 04 Jan 2021
 rem ==========================================================================
:HelpMessage
 ECHO Capture Local Storage Usage using DriveInfo.BAT, DIRUSE and DISKSPD...
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  [OPTIONS]
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:
 ECHO -------------------
 ECHO OPTION DEFINITIONS:
 ECHO -------------------
 ECHO  -?,  /H ...... Display This Help Message  (also --HELP)
 ECHO  -??, /H2 ..... Display Extended Help Message
 ECHO   . , /S ...... Schedule the Job, But Don't Execute It
 ECHO   @ , /N ...... Execute the Job without Scheduling It
 ECHO   - , /U ...... Unschedule the Job Only, Without Executing It First
 ECHO   # , /M ...... Capture Info on Memory Dump Files Only
 ECHO   $ , /I ...... Evaluate Current Disk I/O on at least the primary drive
 ECHO   * , /E ...... Exclude these drives from evaluation [overrides input file settings]
 ECHO       /DIAG ... Run additional Diagnostics
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0
	 ECHO  %~n0 -
	 ECHO  %~n0 /DIAG
	 ECHO  %~n0 /M
	 ECHO  %~n0 /I
	 ECHO  %~n0 /E:"A: B: F: Q:"
	 ECHO  %~n0 -?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * The /DIAG parameter will add DIR and TREE output for the drives in
	 ECHO    question.  This will result in a huge increase in the log, but this
	 ECHO    parameter is not saved by the job scheduler automatically.
	 ECHO:
	 ECHO  * The /I and /M options are mutually exclusive.  Whichever one is
	 ECHO    provided first in the parameter list will be the one that is run.
	 ECHO:
	 ECHO  * Values for the /E command should be surrounded with doublequotes.
	 ECHO:
	 ECHO  * If /E is provided with no values, then the defaults from the config
	 ECHO    file will be used.
 )
 CALL :ShowStatus "FINISHED"
                          