@ECHO OFF
 rem ==========================================================================
 rem === Update Script Repositories Across Enterprise Servers via Input Lists
 rem ==========================================================================
 rem
 rem Created On: 06 Mar 2003 (as SyncScripts.BAT)
 rem Created On: 01 Jan 2004 (as ReSyncScripts.BAT)
 rem         By: Andrew S. Baker
 rem
 rem Consolidated On: 25 Jul 2010
 rem
 rem Updated On: 04 Apr 2024 / 31 Mar 2024 / 29 Mar 2024 / 17 Oct 2023
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2000 and later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    Resource Kit ........... https://en.wikipedia.org/wiki/Resource_Kit
 rem
 rem  Input Files Used By Script (stored in %SystemDrive%\Scripts\Bat\Input):
 rem    CustomVariables.TXT .... Centralized Configuration Settings for Scripts
 rem    FolderDefaults.TXT ..... Determine Location for Scripts and Logs
 rem    ServerList.TXT ......... List of Servers to Deploy Updated Scripts
 rem    ScriptInput.TXT ........ List of Input Files to Copy to Remote Systems
 rem    ScriptSource.TXT ....... List of Valid Script Repository Shares
 rem    RepositoryList.TXT ..... List of Valid Script Repository Folders
 rem
 rem  Required Scripts:
 rem    SetDrive.BAT ........... Set Global Variables for this Environment
 rem ==========================================================================

 :::  This script updates the local and remote scripting repositories usually
 :::  found in the C:\Scripts and \\remote_server\scripts_source$ folder trees.
 :::  In order to prevent overwriting good files with bad copies, it checks to
 :::  see if it is being run from an authorized system which has previously
 :::  been designated as a repository source server.
 :::
 :::  (Jul 2010) v5.0 -- both SyncScripts.BAT and ReSyncScripts.BAT have been
 :::  merged to use the same code.  Previously, they performed almost identical
 :::  functions, but used different code which was (ironically) not fully
 :::  synchronized.  SyncScripts.BAT will not update remote servers, while
 :::  ReSyncScripts.BAT will handle both local and remote repositories.
 :::
 :::  (Jul 2010) v5.1 -- all custom text verification routines have been
 :::  replaces with ROBOCOPY commands.  Some of the XCOPY sections have also
 :::  been replaced with the very flexible ROBOCOPY command.  Requires the
 :::  version of ROBOCOPY found natively with Windows Vista and later.
 :::
 :::  (Nov 2010) v5.4 -- It is now possible to provide read-only servers in
 :::  the ScriptSource.TXT list that will only be used for getting the latest
 :::  repository install.  Simply add a 4th parameter in any given line which
 :::  is non-blank.  (Preferably, use "READONLY").  The servers in these lines
 :::  will not be published to, during execution of this script.
 :::
 :::  (Mar 2011) v5.7 -- Some additional logic has been added to the routine
 :::  which verifies that the input file is valid.  It can now search for the
 :::  file in the current directory, as well as the default input folder, even
 :::  if the full path has not been provided.
 :::
 :::  (Sep 2011) v6.0 -- Added a /TEST parameter to run the script without
 :::  copying data, and also made changes to how the script replicates data
 :::  for non-publisher systems, resulting in faster operation.
 :::
 :::  (Apr 2012) v6.5 -- Added support for ROBOCOPY options, such as the
 :::  number of retries and simultaneous execution threads.
 :::
 :::  (Oct 2013) v7.0 -- The delimiter for all parameters can be either the
 :::  "-" or the "/" character.  This includes the help parameters.
 :::
 :::  (Feb 2014) v7.2 -- The script now checks for more variables to determine
 :::  if the current session is running as a local OR RDP console, and if so,
 :::  offers more prompts.
 :::
 :::  (Nov 2014) v7.4 -- Introduced the mini custom variables INPUT file
 :::  (CustomVariables.MINI.%COMPUTERNAME%.TXT) to facilitate minor deviations
 :::  from the normal CustomVariables.TXT INPUT file.
 :::
 :::  (Feb 2015) v7.5 -- Renamed core input files from xxx.TXT to [xxx].TXT to
 :::  make them easier to identify during system updates.
 :::
 :::  (Mar 2015) v7.7 -- Track elapsed time of script sync jobs using a new
 :::  version of DATEINFO
 :::
 :::  (Apr 2015) v7.8 -- Update Install.BAT for the source repository from
 :::  Repository-Install.BAT
 :::
 :::  (Oct 2015) v7.9 -- Migrate Cloud Storage root location for all scripts
 :::  repositories from Dropbox (Q:\) to Synology CloudStation (C:\Cloud\)
 :::
 :::  (May 2016) v8.0 -- Added verification of remote system using NLTEST;
 :::  Script versioning now includes build number.
 :::
 :::  (Jun 2017) v9.0 -- Added routine for cleaning up obsolete/unneeded files
 :::  from the repository using the new config file SyncScripts.TXT (and also
 :::  SyncScripts.%COMPUTERNAME%.TXT)
 :::
 :::  (Nov 2017) v9.1 -- Synchronized cleanup list variable (@CU_LISTS) with
 :::  the one recently added to SetDrive.BAT (@CLEANUP_LIST)
 :::
 :::  (Jan 2018) v9.2 -- Changed the HELP routine to use FINDSTR;
 :::  Update Perl, VBS and PowerShell Automation Scripts from \BAT folder
 :::
 :::  (Apr 2018) v9.3 -- Improved the performance of reading variables from
 :::  the ReplicationInfo.*.TXT INPUT files, using FINDSTR;
 :::  -- The SYNTAX HELP routine has been updated;
 :::  -- Added filtered CMD Title Routine;
 :::  -- Support for NeoLogger has been eliminated from this script;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com
 :::
 :::  (Jul 2018) v9.4 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::
 :::  (Aug 2018) v9.5 -- Expanded the Cleanup of Old Files from the Repository;
 :::
 :::  (Oct 2018) v9.6 -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Feb 2019) v9.7 -- Copy x64 Utilities after x86 utilities;
 :::
 :::  (Mar 2019) v9.8 -- Enabled Code Page 1252 Support;
 :::  -- Changed @CLEANUP_LIST to @CLEANUP_MAIN and added @CLEANUP_CUSTOM
 :::
 :::  (Jun 2019) v9.9 -- Reduced the number of Cloud Sources;
 :::
 :::  (Jul 2019) v9.10 -- Set Max Threads for ROBOCOPY to max logical CPUs;
 :::
 :::  (Jul 2019) v10.0 -- Always Add All Current BrainWave Utilities;
 :::
 :::  (Sep 2019) v10.1 -- Streamline Cleanup of Obsolete Files;
 :::  -- Allow remote push using configs of an alternate server;
 :::  -- Add the # option (@SKIPEXEs) to skip update of executables;
 :::  -- Refactored the :GetParams routine using the CheckParams.exe utility;
 :::  -- The @TEMP_STORE update sequence was converted to a subroutine;
 :::  -- Refactored the Default Input File Selection Process;
 :::
 :::  (Dec 2019) v10.3 -- Developed a drop-in replacement for Microsoft's
 :::  NOW.EXE utility, taking back that function from DATEINFO.EXE;
 :::  -- Use the New FILEHASH -K option to for File Comparison;
 :::
 :::  (Jan 2020) v10.4 -- Use CHECKPARAMS to evaluate parameters besides /? /H;
 :::
 :::  (Apr 2020) v10.5 -- Updated FILEHASH to support the BLAKE3 cipher;
 :::  -- Updated the DebugMode routines to support the STEP option;
 :::
 :::  (May 2020) v10.6 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::
 :::  (Jul 2020) v10.7 -- Added more logging to :DeleteObsoleteFiles routine;
 :::
 :::  (Nov 2020) v11.0 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::  -- Delete personal config files from the archive;
 :::
 :::  (Dec 2020) v11.1 -- Update the NOW.EXE instance associated with the repository;
 :::
 :::  (Jan 2021) v11.2 -- Update :DeleteObsoleteFiles routine;
 :::
 :::  (Mar 2021) v12.0 -- Prevent Cloud Updates when @LOCALONLY=TRUE
 :::
 :::  (Dec 2021) v12.1 -- Update SysInternals x64 to match installed x32 utilities
 :::
 :::  (Oct 2023) v12.2 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::  -- Update DateInfo.exe along with Now.exe for the default script repository;
 :::
 :::  (Mar 2024) v12.3 -- Added custom BartPE and AMPE repositories to the updates;
 :::
 :::  (Apr 2024) v12.4 -- Check for essential utilities with FINDFILES;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 27 Jul 2020 / 25 Feb 2018
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 04 Apr 2024 / 31 Mar 2024 / 29 Mar 2024 / 30 Dec 2021
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v12.4.0.3640

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc --mp=1 -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Check for Essential Utilities and/or Scripts
 SET @ESSENTIALS="%~dp0SetDrive.BAT" CHECKPARAMS.EXE DATEINFO.EXE NOW.EXE READCONFIG.EXE FIND.EXE
 PATH %PATH%;%~dp0
 FINDFILES -w -m -f %@ESSENTIALS% CHOICE.EXE FINDSTR.EXE NLTEST.EXE ROBOCOPY.EXE XCOPY.EXE
 IF ERRORLEVEL 64 GOTO :ExitBatch

 rem -- Main Variables
 SET @BW_SOURCE=%SystemDrive%\Code.Source
 SET @LOCAL=BAT\Input
 SET @REMOTE=Scripts\%@LOCAL%
 SET @ISTEST=
 SET @LOCALONLY=
 SET @VERBOSE=FALSE
 SET @TEST_RUN=FALSE
 SET @NOSHARES=FALSE
 SET @NOREMOTE=FALSE
 SET @MASTERPUBLISHER=FALSE
 SET @DIR_EXEMPTION=/XD Obsolete Old Temp Zips ZipFiles
 SET @FILE_EXEMPTION=/XF TimeSlots.TXT *.BKS *.BAD *.CHM *.HLP *.ZIP *.RAR *.7z
 SET @PERSONAL_CONFIGS=S*.CFG W*.CFG A*.CFG B*.CFG

 rem -- Set Global Variables from Centralized Script
 CALL "%~dp0SetDrive.BAT" "%~n0"

 rem -- Set Variables Related to XCOPY/ROBOCOPY
 SET @TODAY-X=/D:%@MM%-%@DD%-%@YY%
 SET @TODAY-R=/MAXAGE:1
 SET @RETRIES=/R:2 /W:5

 rem -- Cycle Through Repository Paths in File List And Select Last Viable Path
 IF EXIST "%@SOURCELIST%" (
	 FOR /F "USEBACKQ SKIP=2 TOKENS=2   EOL=; DELIMS=\"  %%S IN ("%@SOURCELIST%") DO SET @SOURCESYS=!@SOURCESYS! %%~S
	 FOR /F "USEBACKQ SKIP=2 TOKENS=2   EOL=; DELIMS=, " %%D IN ("%@SOURCELIST%") DO SET @DESTDIRS=!@DESTDIRS! "%%~D"
	 FOR /F "USEBACKQ SKIP=2 TOKENS=1,3 EOL=; DELIMS=, " %%R IN ("%@SOURCELIST%") DO IF "%%~S"=="" SET @REMOTEDIRS=!@REMOTEDIRS! "%%~R"
 ) ELSE (
	 ECHO:
	 ECHO No Valid Source File Found: "%@SOURCELIST%" Missing...
	 ECHO:
	 GOTO :HelpMessage
 )

 rem -- Server Input Lists (Default and Customized)
 SET @ALL_PARAMS=%*
 SET @HERE=%~dp0
 SET @INPUT=%~dp0Input\
 SET @TESTSRVLIST=%@INPUT%FolderDefaults.Test.TXT
 SET @ALL_SERVERS=%~1
 SET @CFG_SOURCE=%COMPUTERNAME%
 SET @SVRLISTS="%@INPUT%FolderDefaults.%COMPUTERNAME%.TXT" "%@INPUT%FolderDefaults.TXT"
 SET @REPOLIST="%@INPUT%RepositoryList.%COMPUTERNAME%.TXT" "%@INPUT%RepositoryList.TXT"

 rem -- Establish Local Client Script Repository Locations
 SET @CUSTOM_REPOS=
 FOR %%F IN (H: G: F: E: D: C: %SystemDrive%) DO IF EXIST "%%~F\Storage\Work" SET @LOCAL_REPOSITORY=%%~F\Storage\Work
 FOR %%C IN (AMPE BARTPE) DO (
	 IF EXIST "%@LOCAL_REPOSITORY%\Custom_%%~C\Scripts" SET @CUSTOM1_%%~C=%@LOCAL_REPOSITORY%\Custom_%%~C\Scripts
	 IF EXIST "%@LOCAL_REPOSITORY%\Custom_%%~C\Script_Repository" (
		SET @CUSTOM2_%%~C=%@LOCAL_REPOSITORY%\Custom_%%~C\Script_Repository
		SET @CUSTOM_REPOS=!@CUSTOM_REPOS! "!@CUSTOM2_%%~C!"
	)
 )

 rem -- Establish Cloud Storage Locations (Dropbox/CloudStation)
 SET @MY_CLOUDS="%SystemDrive%\Dropbox" "%SystemDrive%\Storage\Dropbox" "Q:\Dropbox" "Q:\Storage\Dropbox" "%SystemDrive%\Cloud\CS.Work"
 FOR %%C IN (AMPE BARTPE PUBLIC KB CORE LEGACY REPO) DO SET @CLOUD_%%~C=*
 FOR %%D IN (%@MY_CLOUDS%) DO IF EXIST "%%~D\Scripts\*.*" (
	 SET @CLOUD_AMPE=%%~D\Scripts\AMPE
	 SET @CLOUD_BARTPE=%%~D\Scripts\BARTPE
	 SET @CLOUD_PUBLIC=%%~D\Scripts\KB-PUBLIC
	 SET @CLOUD_KB=%%~D\Scripts\KB
	 SET @CLOUD_CORE=%%~D\Scripts\[Core]
	 SET @CLOUD_LEGACY=%%~D\Scripts\[Legacy]
	 SET @CLOUD_REPO=%%~D\Scripts\BWCC PE Script Repository
 )

 rem -- Establish Script Utility Cleanup Locations
 SET @CLOUD_CLEAN="%@CLOUD_BARTPE%" "%@CLOUD_KB%"
 FOR %%C IN (REPO PUBLIC CORE LEGACY) DO SET @CLOUD_CLEAN=!@CLOUD_CLEAN! "!@CLOUD_%%C!\Scripts"
 FOR %%F IN (%@REPOLIST%) DO IF EXIST "%%~F" FOR /F "USEBACKQ TOKENS=* EOL=; DELIMS=\" %%C IN ("%%~F") DO (
	 FOR %%D IN (%@MY_CLOUDS%) DO IF EXIST "%%~D\Scripts\%%~C" SET @CLOUD_CLEAN=!@CLOUD_CLEAN! "%%~D\Scripts\%%~C"
 )

 rem -- Source (Root) of Script/Utility Archives
 SET @SOURCE1=%SystemDrive%\Scripts
 SET @NOTFOUND=*** NO DESTINATION FOLDER SET ***
 SET @DEST1=%@NOTFOUND%& FOR %%S IN (%@DESTDIRS%) DO IF EXIST "%%~S" SET @DEST1=%%~S
 IF "%@DEST1%"=="%@NOTFOUND%" GOTO :AbortExecution

 rem -- Source and Destination of BrainWave Utilities
 SET @SOURCE10=%@BW_SOURCE%
 SET @DEST10=%@SOURCE1%\Utils;%@SOURCE1%\BAT\LogonScripts\Scripts\Utils

 rem -- Source and Destination of Utility Archive (x86)
 SET @SOURCE2A=%@SOURCE1%\Bat
 SET @SOURCE2B=%@SOURCE1%\Unix
 SET @SOURCE2C=%ProgramFiles(x86)%\Support Tools
 SET @SOURCE2D=%ProgramFiles(x86)%\Windows Resource Kits\Tools
 SET @SOURCE2E=%@FILE_REPOSITORY%\File Transfer Software\SSH\Putty
 SET @SOURCE2F=%ProgramFiles(x86)%\Putty
 SET @SOURCE2G=%SystemDrive%\Programs\7-Zip\x86
 SET @SOURCE2H=%ProgramFiles(x86)%\7-Zip
 SET @SOURCE2I=%@SOURCE1%\BAT\Services\EvtSys\x86
 SET @SOURCE2J=%@SOURCE1%\SysInternals
 SET @SOURCE2K=%@SOURCE1%\Utils
 SET @DEST2=%@SOURCE1%\Utils;%@SOURCE1%\Utils\Misc;%@DEST1%\Utils;%@DEST1%\Utils\Misc

 rem -- Source and Destination of Utility Archive (x64)
 SET @SOURCE12A=%@SOURCE1%\BAT\Services\EvtSys\x64
 SET @SOURCE12B=%ProgramFiles%\Support Tools
 SET @SOURCE12C=%ProgramFiles%\Windows Resource Kits\Tools
 SET @SOURCE12D=%ProgramFiles%\Putty
 SET @SOURCE12E=%SystemDrive%\Programs\7-Zip\x64
 SET @SOURCE12F=%ProgramFiles%\7-Zip
 SET @DEST12=%@SOURCE1%\Utils\x64;%@SOURCE1%\Utils\Misc;%@DEST1%\Utils\x64;%@DEST1%\Utils\Misc

 rem -- Source and Destination of SysInternal x64 Updates
 SET @SOURCE22=%@SOURCE1%\SysInternals
 SET @COMP22=%@SOURCE1%\Utils
 SET @DEST22=%@COMP22%\x64

 rem -- Destinations for Local Repository Host
 SET @DEST3A=%@SOURCE1%\Utils
 SET @DEST3B=%@SOURCE1%\Utils\x64
 SET @DEST8=& FOR %%V IN (Kixtart Perl VBS PowerShell) DO SET @DEST8=%@SOURCE1%\%%~V;!@DEST8!

 rem -- Destination for Local Copy of Logon Scripts
 SET @DEST4A=%@SOURCE1%\BAT\LogonScripts
 SET @DEST4B=%@SOURCE1%\BAT\LogonScripts\Scripts\BAT
 SET @DEST4C=%@SOURCE1%\BAT\LogonScripts\Scripts\Utils
 SET @DEST4D=%@SOURCE1%\BAT\LogonScripts\System32

 rem -- Source and Destination of Local Copy of Logon Scripts Utilities (x86)
 SET @SOURCE5A=%@SOURCE1%\Unix
 SET @SOURCE5B=%@SOURCE1%\Utils
 SET @SOURCE5C=%@SOURCE1%\BAT\Services\EvtSys\x86
 SET @DEST5=%@SOURCE1%\BAT\LogonScripts\System32

 rem -- Source and Destination of Local Copy of Logon Scripts (x64)
 SET @SOURCE6A=%@SOURCE1%\Utils\x64
 SET @SOURCE6B=%@SOURCE1%\BAT\Services\EvtSys\x64
 SET @DEST6=%@SOURCE1%\BAT\LogonScripts\System32\x64

 rem -- Source and Destination of Central Repository of Client Utilities
 SET @SOURCE7=%@SOURCE1%\Utils
 SET @DEST7A=%@CLOUD_KB%\Scripts\Utils
 SET @DEST7B=%@CLOUD_CORE%\Scripts\Utils
 SET @DEST7C=%@CLOUD_LEGACY%\Scripts\Utils

 rem -- Source and Destination of DropBox Shared Scripts
 SET @SOURCE9A=%@CLOUD_KB%
 SET @SOURCE9B=%@SOURCE1%\Bat
 SET @DEST9=%@LOCAL_REPOSITORY%\Shared_Scripts

 rem -- Source for Peer Updates (Determine MASTER PUBLISHER location)
 SET @SOURCE00=%SystemDrive%\Scripts
 FOR %%S IN (%@SOURCESYS%) DO IF /I "%COMPUTERNAME%"=="%%~S" (
	 SET @SOURCE00=%@DEST1%
	 SET @MASTERPUBLISHER=TRUE
 )

 rem -- Process Special Commandline Parameters
 CALL :GetParams "A ALT" @CFG_SOURCE & IF DEFINED @OK IF NOT DEFINED @CFG_SOURCE GOTO :HelpMessage

 CALL :GetParams ". * @ ~ #" & IF DEFINED @OK (SET @ALL_SERVERS=%@STORAGELIST%& SET @NOREMOTE=TRUE)
 CALL :GetParams ". *"       & IF DEFINED @OK (SET @NOREMOTE=FALSE)
 CALL :GetParams "~"         & IF DEFINED @OK (SET @NOSHARES=TRUE& SET @LOCALONLY=TRUE)
 CALL :GetParams "#"         & IF DEFINED @OK (SET @SKIPEXEs=TRUE)

 CALL :GetParams "T TEST"    & IF DEFINED @OK (
	 SET @ISTEST=/L
	 SET @TEST_RUN=TRUE
	 SET @TODAY-R=/MAXAGE:7
	 SET @ALL_SERVERS=%@TESTSTORLIST%
 )

 CALL :GetParams "S SPECIAL" & IF DEFINED @OK (
	 SET @TODAY-R=/MAXAGE:3
	 SET @ALL_SERVERS=%@TESTSTORLIST%
 )

 CALL :GetParams "V VERBOSE" & IF DEFINED @OK (
	 SET @ALL_SERVERS=!@STORAGELIST!
	 SET @VERBOSE=TRUE
 )

 rem -- Create Temporary Staging Areas for Remote Repositories
 SET @TEMP_STORE=%TEMP%\%~n0.%@CFG_SOURCE%{%@RANDOM%}

 rem -- Determine if Custom Lists Will Be Used
 SET @ISCUSTOM=FALSE
 IF EXIST "%@CUSTOMINPUT%" SET @ISCUSTOM=TRUE
 IF /I NOT "%@CFG_SOURCE%"=="%COMPUTERNAME%" SET @ISCUSTOM=TRUE

 rem -- Key Variable Verification
 FOR %%V IN (ROBOCOPY ALPHABET ALL_SERVERS CORELIST CUSTOMLIST SOURCETXT SOURCESYS DESTDIRS REMOTEDIRS FILE_REPOSITORY TODAY-X TODAY-R) DO IF NOT DEFINED @%%~V (
	 ECHO:
	 ECHO *** WARNING: The @%%~V variable is undefined.
	 ECHO:
	 ECHO  Please ensure that all configuration info is provided correctly
	 ECHO  within the file: "%@SOURCELIST%"
	 ECHO:
	 ECHO  Alternatively, this information can be entered directly into this script,
	 ECHO  although this is not recommended because it undermines portability.
	 ECHO %@DIVIDER*%
	 ECHO:
	 GOTO :HelpMessage
 )
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Display Script Header Information and Destination Selected
 rem === Updated On: 04 Nov 2013 / 21 Dec 2011
 rem ==========================================================================
:DisplayHeader
 ECHO:
 ECHO %@DIVIDER*%
 ECHO *** REPOSITORY: %@DEST1%
 ECHO ***
 IF /I "%@TEST_RUN%"=="TRUE" (
	 ECHO *** JOB STATUS: TEST RUN -- No files will be copied
 ) ELSE (
	 ECHO *** JOB STATUS: NORMAL EXECUTION -- Older files will be updated
 )
 ECHO %@DIVIDER*%
 ECHO:
 ECHO:
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Verify That Script is Running With Correct Format
 rem === Updated On: 21 Dec 2011 / 30 Sep 2011
 rem ==========================================================================
:VerifyScript
 CALL :GetTime @STIME

 IF /I "%@TEST_RUN%"=="TRUE" GOTO :GetValidList

 IF /I "%~nx0"=="SyncScripts.BAT" (
	 SET @NOREMOTE=TRUE
	 SET @ALL_SERVERS=!@STORAGELIST!
	 GOTO :GetValidList
 )

 IF NOT "%~0"=="%SystemDrive%\Scripts\Bat\ReSyncScripts.BAT" (
	 ECHO Thanks for Running This Script...
	 GOTO :FinalReport
 )
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Identify and Select Valid System List
 rem === Updated On: 24 Sep 2019 / 27 Jun 2017
 rem ==========================================================================
:GetValidList
 SET @FINDLIST="%~dp0Input\FolderDefaults.*%@ALL_SERVERS%*" "%~dp0Input\*%@ALL_SERVERS%*" "*%@ALL_SERVERS%*" "%@ALL_SERVERS%" %@SVRLISTS%
 FOR %%E IN (".TXT" "") DO FOR %%F IN (%@FINDLIST%) DO IF EXIST "%%~F%%~E" (
	 SET @ALL_SERVERS=%%~F%%~E
	 IF /I "%@CONSOLE_SESSION%"=="TRUE" (
		 CHOICE /C NYQEX /T 10 /D Y /N /M "Use the following system list? - '!@ALL_SERVERS!' "
		 IF !ERRORLEVEL! EQU 2 GOTO :GetValidList_END
		 IF !ERRORLEVEL! GTR 2 GOTO :ExitBatch
	 ) ELSE ( GOTO :GetValidList_END )
 )

 rem -- Abort script if no system list has been provided/accepted
:AbortExecution
 ECHO:
 ECHO ERROR: No Valid System List Was Provided -- Script Execution Aborted...
 ECHO:
 %@PAUSE_IF_DEBUG%
 GOTO :HelpMessage
:GetValidList_END


 rem ==========================================================================
 rem === Update Local Input/Source Files (based on local CUSTOM files)
 rem === Updated On: 13 Dec 2019 / 24 Sep 2019
 rem ==========================================================================
:LocalUpdate
 CALL :CreateTempRepository
 ECHO:
 NOW "Updating Local Input Files [at: \v]\n\-"
 IF /I "%@ISCUSTOM%"=="TRUE" (
	 rem -- Only Custom Copy Input Files if this Flag Exists
	 FOR %%A IN ("%@SOURCE00%" "%@TEMP_STORE%") DO IF EXIST "%%~A\%@LOCAL%\*.TXT" ATTRIB -R -S -H "%%~A\%@LOCAL%\*.TXT"
	 FOR %%F IN (%@SOURCETXT%) DO (
		 IF /I "%@CFG_SOURCE%"=="%COMPUTERNAME%" (
			 SET @GROUP_DESTINATION="%@SOURCE00%\%@LOCAL%\%%~F.TXT" "%@INPUT%%%~F.TXT" "%@TEMP_STORE%\%@LOCAL%\%%~F.TXT"
		 ) ELSE (
			 SET @GROUP_DESTINATION="%@TEMP_STORE%\%@LOCAL%\%%~F.TXT"
		 )

		 IF EXIST "%@SOURCE00%\%@LOCAL%\%%~F.%@CFG_SOURCE%.TXT" (
			 ECHO:
			 ECHO -- Copying: "%%~F.%@CFG_SOURCE%.TXT" to "%%~F.TXT"
			 IF /I NOT "%@ISTEST%"=="/L" (
				 FOR %%c IN (!@GROUP_DESTINATION!) DO (
					 COPY "%@SOURCE00%\%@LOCAL%\%%~F.%@CFG_SOURCE%.TXT" "%%~c" /Y
				 )
			 )
			 ECHO:
		 )
	 )
 ) ELSE (
	 rem -- Do NOT Copy Any Custom Input Files
	 ECHO -- Customized Source Files Will Not Be Copied As Default...
	 ECHO:
 )
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Update Scripts and Utilities (from MASTER PUBLISHER Location)
 rem === Updated On: 29 Mar 2024 / 17 Oct 2023 / 26 Jan 2022
 rem ==========================================================================
:PrimaryUpdate
 IF /I "%@MASTERPUBLISHER%"=="FALSE" (
	 ECHO *** Not a Valid MASTER PUBLISHER system ***
	 ECHO:
	 GOTO :ExitBatch
 )

 rem -- Skip Update of Executables if ANY Parameter = "#"
 IF /I "!@SKIPEXEs!"=="TRUE" (
	 FOR %%C IN (2 4 5 6 7) DO FOR %%S IN ("" %@ALPHABET%) DO SET @DEST%%~C%%~S=
	 SET @DEST4A=%@SOURCE1%\BAT\LogonScripts\Scripts\BAT
	 GOTO :UpdateLogonScripts
 )


 CALL :DivideSection HEADER "Updating Local Repository Install.BAT file and other utilities"
 CALL :UpdateOneFile "%@SOURCE1%\BAT\Repository-Install.BAT" "%@SOURCE00%\..\Install.BAT"
 CALL :UpdateOneFile "%@SOURCE1%\Utils\Now.EXE"              "%@SOURCE00%\..\Now.EXE"
 CALL :UpdateOneFile "%@SOURCE1%\Utils\DateInfo.EXE"         "%@SOURCE00%\..\DateInfo.EXE"
 ECHO:

 CALL :DivideSection HEADER "Updating Existing Batch/Shell Scripts, Executables and Input Files"
 FOR %%D IN (%@DEST1%) DO IF EXIST "%%~D" (
	 PUSHD "%%~D"
	 ECHO COPY TO: %%~D
	 FOR %%P IN (%@TODAY-X% /D) DO (
		 FOR /F "DELIMS=^>" %%Z IN ('XCOPY "%@SOURCE1%" /U /R %%~P /Y /S %@ISTEST% ^| FIND /V "File(s)"') DO ECHO  + %%~Z
	 )
	 ECHO:
	 POPD
 )

 CALL :DivideSection SUBHEADER "Updating Shell/Automation Scripts... [for Repository]"
 FOR %%D IN (%@DEST8%) DO IF EXIST "%%~D" (
	 PUSHD "%%~D"
	 ECHO COPY TO: %%~D
	 FOR /F "TOKENS=2 DELIMS=^>" %%Z IN ('XCOPY "%@SOURCE1%\BAT" /U /F /R /D /Y %@ISTEST% ^| FIND /V "File(s)"') DO ECHO %%~Z
	 ECHO:
	 POPD
 )

 CALL :DivideSection SUBHEADER "Updating Standard and Custom Input Files"
 IF EXIST "%@DEST1%\%@LOCAL%" (
	 PUSHD "%@DEST1%\%@LOCAL%"
	 ECHO COPY TO: %@DEST1%\%@LOCAL%
	 FOR %%F IN (%@SOURCETXT%) DO IF EXIST "%@SOURCE1%\%@LOCAL%\%%~F.*TXT" (
		 FOR /F "DELIMS=^>" %%Z IN ('XCOPY "%@SOURCE1%\%@LOCAL%\%%~F.*TXT" /R /D /Y %@ISTEST% /EXCLUDE:%@COPYEXEMPT% ^| FIND /V "File(s)"') DO ECHO  %%~Z
	 )
	 POPD
 )

 CALL :DivideSection SUBHEADER "Updating BrainWave Utilities... [for Repository]"
 FOR %%D IN (%@DEST10%) DO IF EXIST "%%~D" (
	 PUSHD "%%~D"
	 ECHO COPY TO: %%~D
	 FOR %%S IN ("" %@ALPHABET%) DO IF EXIST "!@SOURCE10%%~S!" (
		 FOR /F "TOKENS=*" %%F IN ('DIR /B /A "!@SOURCE10%%~S!\*64.EXE"') DO (
			 SET @FNAME64=%%~nxF
			 SET @FNAME=!@FNAME64:64=!

			 IF /I NOT "!@FNAME!"=="Placeholder" (
				 SET @FILECOMP32="!@SOURCE10%%~S!\!@FNAME!" "%%~D\!@FNAME!"
				 SET @FILECOMP64="!@SOURCE10%%~S!\!@FNAME64!" "%%~D\x64\!@FNAME!"

				 FOR %%A IN (32 64) DO FOR /F %%C IN ('FILEHASH --XXHash64 -K !@FILECOMP%%~A! 2^>NUL') DO (
					 IF /I "%%~C"=="FALSE" FOR /F %%Z IN ('COPY !@FILECOMP%%~A! /Y') DO ECHO  + !@FNAME! [%%~A-bit Edition]
				 )
			 )
		 )
	 )
	 ECHO:
	 POPD
 )

 CALL :DivideSection SUBHEADER "Updating x86 Executables... [for Repository]"
 FOR %%D IN (%@DEST2%) DO IF EXIST "%%~D" (
	 PUSHD "%%~D"
	 ECHO COPY TO: %%~D
	 FOR %%S IN ("" %@ALPHABET%) DO IF EXIST "!@SOURCE2%%~S!" (
		 FOR /F "TOKENS=2 DELIMS=^>" %%Z IN ('XCOPY "!@SOURCE2%%~S!" /U /F /R /D /Y %@ISTEST% ^| FIND /V "File(s)"') DO ECHO %%~Z
	 )
	 ECHO:
	 POPD
 )

 CALL :DivideSection SUBHEADER "Updating x64 Executables... [for Repository]"
 FOR %%D IN (%@DEST12%) DO IF EXIST "%%~D" (
	 PUSHD "%%~D"
	 ECHO COPY TO: %%~D
	 FOR %%S IN ("" %@ALPHABET%) DO IF EXIST "!@SOURCE12%%~S!" (
		 FOR /F "TOKENS=2 DELIMS=^>" %%Z IN ('XCOPY "!@SOURCE12%%~S!" /U /F /R /D /Y %@ISTEST% ^| FIND /V "File(s)"') DO ECHO %%~Z
	 )
	 ECHO:
	 POPD
 )

 CALL :DivideSection SUBHEADER "Updating x86 Executables... [for Main System]"
 PUSHD "%@DEST3A%"
 FOR %%S IN ("" %@ALPHABET%) DO IF EXIST "!@SOURCE2%%~S!" IF NOT "!@SOURCE2%%~S!"=="%@DEST3A%" (
	 FOR /F "TOKENS=2 DELIMS=^>" %%Z IN ('XCOPY "!@SOURCE2%%~S!" /U /F /R /D /Y %@ISTEST% ^| FIND /V "File(s)"') DO ECHO %%~Z
 )
 POPD

 CALL :DivideSection SUBHEADER "Updating x64 Executables... [for Main System]"
 PUSHD "%@DEST3B%"
 FOR %%S IN ("" %@ALPHABET%) DO IF EXIST "!@SOURCE12%%~S!" IF NOT "!@SOURCE12%%~S!"=="%@DEST3B%" (
	 FOR /F "TOKENS=2 DELIMS=^>" %%Z IN ('XCOPY "!@SOURCE12%%~S!" /U /F /R /D /Y %@ISTEST% ^| FIND /V "File(s)"') DO ECHO %%~Z
 )
 POPD

 rem -- Potential Short Circuit
 IF DEFINED @LOCALONLY GOTO :LocalCleanup

 CALL :DivideSection SUBHEADER "Updating x86 Executables... [for All Repository Systems]"
 FOR %%D IN ("" %@ALPHABET%) DO IF EXIST "!@DEST7%%~D!" (
	 PUSHD "!@DEST7%%~D!"
	 IF EXIST "%@SOURCE7%" FOR /F "TOKENS=2 DELIMS=^>" %%Z IN ('XCOPY "%@SOURCE7%" /U /F /R /D /Y /S %@ISTEST% ^| FIND /V "File(s)"') DO ECHO %%~Z
	 CALL :DeletePersonalConfigFiles "!@DEST7%%~D!"
 )
 POPD

 CALL :DivideSection SUBHEADER "Updating x64 Executables... [for All Repository Systems]"
 FOR %%D IN ("" %@ALPHABET%) DO IF EXIST "!@DEST7%%~D!\x64" (
	 PUSHD "!@DEST7%%~D!\x64"
	 IF EXIST "%@SOURCE7%\x64" FOR /F "TOKENS=2 DELIMS=^>" %%Z IN ('XCOPY "%@SOURCE7%\x64" /U /F /R /D /Y /S %@ISTEST% ^| FIND /V "File(s)"') DO ECHO %%~Z
 )
 POPD

 CALL :DivideSection SUBHEADER "Updating x64 Executables... [for SysInternals Only]"
 PUSHD "%@DEST22%"
 FOR %%S IN (%@COMP22%\*.EXE) DO IF EXIST "!@SOURCE22!\%%~nS64.exe" (
	 IF NOT EXIST "!@DEST22!\%%~nxS" (
		 BREAK > "!@DEST22!\%%~nxS"
		 FILEDATE "!@DEST22!\%%~nxS" 01/01/1991 >NUL
	 )
	 FOR /F "TOKENS=2 DELIMS=^>" %%Z IN ('XCOPY "!@SOURCE22!\%%~nS64.exe" "!@DEST22!\%%~nxS" /F /R /D /Y %@ISTEST% ^| FIND /V "File(s)"') DO ECHO %%~Z
 )
 POPD

 rem -- Cleanup Obsolete Files from Local System Only
:LocalCleanup
 CALL :DeleteObsoleteFiles

 SET @BANNER=Updating Batch/Shell Scripts, Executables and Input Files
 CALL :DivideSection FOOTER
 IF DEFINED @LOCALONLY GOTO :FinalReport
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Update Shared Script Samples Repository for DropBox/CloudStation
 rem === Updated On: 21 Feb 2019 / 31 Jan 2018
 rem ==========================================================================
:UpdateScriptSamples
 IF EXIST "%@DEST9%" (
	 CALL :DivideSection HEADER "Updating Shared Scripts... [for DropBox/CloudStation Repository]"
	 FOR /R "%@DEST9%" %%D IN (.) DO (
		 SET @SHOWHEADER=TRUE
		 CALL :UpdateFiles "%@SOURCE9A%\Input"    "%%~D" SHOW_DEST EXCLUSION_OFF
		 CALL :UpdateFiles "%@SOURCE9B%\Services" "%%~D" SHOW_DEST EXCLUSION_OFF
		 CALL :UpdateFiles "%@SOURCE9B%"          "%%~D" SHOW_DEST EXCLUSION_OFF RECURSION_OFF
		 CALL :UpdateFiles "%@SOURCE2A%"          "%%~D" SHOW_DEST EXCLUSION_OFF RECURSION_OFF

		 FOR %%A IN (B C D E F G H I J K L M N) DO IF DEFINED @SOURCE2%%A (
			 CALL :UpdateFiles "!@SOURCE2%%A!"      "%%~D" SHOW_DEST EXCLUSION_OFF
		 )
	 )

	 FOR /R "%@DEST9%" %%D IN (x64) DO IF EXIST "%%~D" (
		 PUSHD "%%~D"
		 FOR %%S IN ("" %@ALPHABET%) DO (
			 IF EXIST "!@SOURCE6%%~S!" (
				 FOR /F "TOKENS=2 DELIMS=^>" %%Z IN ('XCOPY "!@SOURCE6%%~S!" /U /F /R /Y %@ISTEST% ^| FIND /V "File(s)"') DO ECHO %%~Z
			 )
		 )
		 POPD
	 )
	 CALL :DivideSection FOOTER
 )
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Update Scripts and Utilities for Logon Scripts, If Folder Exists
 rem === Updated On: 31 Jan 2018 / 25 Apr 2013
 rem ==========================================================================
:UpdateLogonScripts
 IF EXIST "%@DEST4A%" (
	 CALL :DivideSection HEADER "Updating Logon Scripts Archive"

	 FOR %%D IN ("" %@ALPHABET%) DO IF EXIST "!@DEST4%%~D!" (
		 PUSHD "!@DEST4%%~D!"
		 ECHO:
		 ECHO TO: !@DEST4%%~D!
		 FOR %%S IN (%@ALPHABET%) DO IF EXIST "!@SOURCE2%%~S!" CALL :UpdateFiles "!@SOURCE2%%~S!" "!@DEST4%%~D!" SHOW_NONE EXCLUSION_OFF
		 POPD
	 )

	 PUSHD "%@DEST5%"
	 FOR %%S IN ("" %@ALPHABET%) DO IF EXIST "!@SOURCE5%%~S!" (
		 FOR %%P IN (%@TODAY-X% /D) DO (
			 FOR /F "TOKENS=1 DELIMS=^>" %%Z IN ('XCOPY "!@SOURCE5%%~S!" /U /R %%~P /Y %@ISTEST% ^| FIND /V "File(s)"') DO ECHO  + %%~Z
		 )
	 )
	 POPD

	 PUSHD "%@DEST6%"
	 FOR %%S IN ("" %@ALPHABET%) DO IF EXIST "!@SOURCE6%%~S!" (
		 FOR %%P IN (%@TODAY-X% /D) DO (
			 FOR /F "TOKENS=1 DELIMS=^>" %%Z IN ('XCOPY "!@SOURCE6%%~S!" /U /R %%~P /Y %@ISTEST% ^| FIND /V "File(s)"') DO ECHO  + %%~Z
		 )
	 )
	 POPD
	 CALL :DivideSection FOOTER
 )
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Update Local AND Remote Source File Repositories
 rem === Updated On: 31 Mar 2024 / 29 Mar 2024 / 23 Sep 2019
 rem ==========================================================================
:UpdateLocalRepository
 CALL :UpdateTempRepository

 CALL :DivideSection HEADER "Updating Local Source File Repository..."
 PUSHD "%@DEST1%"
 FOR %%P IN (%@TODAY-X% /D) DO (
	 FOR /F "TOKENS=1 DELIMS=^>" %%Z IN ('XCOPY "%@SOURCE1%" /U /R %%~P /Y /S %@ISTEST% ^| FIND /V "File(s)"') DO ECHO  + %%~Z
 )
 POPD
 CALL :DivideSection FOOTER


:UpdateCustomRepositories
 IF NOT "!@CLOUD_REPO!"=="*" (
	 CALL :DivideSection HEADER "Updating Local Custom Source File Repositories..."
	 PUSHD "%@SOURCE1%"
	 ECHO  ...attempting to update "%@CLOUD_REPO%"
	 CALL :UpdateOneFile "%@SOURCE1%\BAT\Repository-Install.BAT" "%@CLOUD_REPO%\Install.BAT" " * Install.BAT"
	 CALL :UpdateOneFile "%@SOURCE1%\Utils\Now.EXE"              "%@CLOUD_REPO%\Now.EXE"
	 CALL :UpdateOneFile "%@SOURCE1%\Utils\DateInfo.EXE"         "%@CLOUD_REPO%\DateInfo.EXE"
	 ECHO:
	 POPD

	 PUSHD "%@CLOUD_REPO%"
	 FOR %%C IN (%@CUSTOM_REPOS%) DO (
		 ECHO  ...attempting to update "%%~C"
		 IF EXIST "%%~C" (
			 CALL :UpdateFiles         "%@CLOUD_REPO%"  "%%~C"  SHOW_NONE SET_EXCLUSIONS MIRROR
			 CALL :DeleteObsoleteFiles "%%~C"
			 ECHO:
		 )
	 )
	 POPD
	 CALL :DivideSection FOOTER
 )

 rem -- Skip Update of Remote Repositories if ANY Parameter = "~"
 IF /I "!@NOSHARES!"=="TRUE" GOTO :SecondaryUpdate


:UpdateRemoteRepositories
 CALL :DivideSection HEADER "Updating Remote Source File Repositories..."
 PUSHD "%@DEST1%"
 FOR %%R IN (%@REMOTEDIRS%) DO FOR /F %%C IN ('ECHO "%%~R" ^| FIND /V /I "%COMPUTERNAME%"') DO (
	 ECHO  ...attempting to update "%%~R"
	 IF EXIST "%%~R" (
		 CALL :UpdateOneFile "%@SOURCE1%\BAT\Repository-Install.BAT" "%%~R\..\Install.BAT" " * Install.BAT"
		 CALL :UpdateFiles "%@DEST1%" "%%~R" SHOW_NONE SET_EXCLUSIONS MIRROR
		 CALL :DeleteObsoleteFiles "%%~R"
		 ECHO:
	 )
 )
 POPD
 CALL :DivideSection FOOTER
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Update Scripts and Utilities (Source Location)
 rem === Updated On: 03 Dec 2020 / 30 Jul 2020
 rem ==========================================================================
:SecondaryUpdate
 rem -- Skip Update of Remote Servers if ANY Parameter = "*"
 IF /I "!@NOREMOTE!"=="TRUE" GOTO :FinalReport

 CALL :DivideSection HEADER "Updating Remote Systems from Source..."
 FOR /F "TOKENS=1,3 SKIP=2 EOL=;" %%S IN (%@ALL_SERVERS%) DO IF /I NOT "%COMPUTERNAME%"=="%%~S" (
	 SET @SYSTEMTYPE=SUBSCRIBER
	 SET @REPOSITORY=%@SOURCE00%
	 SET @SKIPDIRS=%@DIR_EXEMPTION% LogonScripts
	 SET @SKIPFILES=%@FILE_EXEMPTION% %@PERSONAL_CONFIGS% *.*.TXT

	 FOR %%B IN (%@SOURCESYS%) DO IF /I "%%~S"=="%%~B" (
		 SET @SYSTEMTYPE=PUBLISHER
		 SET @REPOSITORY=%@SOURCE00%
		 SET @SKIPDIRS=%@DIR_EXEMPTION%
		 SET @SKIPFILES=%@FILE_EXEMPTION%
	 )

	 NOW \z "Copying Files to \\%%~S\%%~T [at: \v ] -- !@SYSTEMTYPE! \n \-"
	 IF "%Debug%"=="TRUE" (
		 %@ROBOCOPY% "%@TEMP_STORE%" "\\%%~S\%%~T\Scripts" *.* %@THREADS% %@RETRIES% /S /NP /XX /NDL /FP /L
		 %@ROBOCOPY% "%@SOURCE00%" "\\%%~S\%%~T\Scripts" *.* %@THREADS% %@RETRIES% /S /NP /XX /NDL /FP /L !@SKIPDIRS! !@SKIPFILES!
	 ) ELSE (
		 CALL :CopyFiles "%@TEMP_STORE%" "\\%%~S\%%~T\Scripts" *.* "        /IS !@SKIPDIRS! !@SKIPFILES!"
		 CALL :CopyFiles "%@TEMP_STORE%" "\\%%~S\%%~T\Scripts" *.TEST.TXT " /IS"
		 CALL :CopyFiles "%@SOURCE00%"   "\\%%~S\%%~T\Scripts" *.* "/XO /XX /XL !@SKIPDIRS! !@SKIPFILES!"
		 CALL :CopyFiles "%@SOURCE00%"   "\\%%~S\%%~T\Scripts" *.TEST.TXT " /IS"
		 CALL :UpdateFiles "%@SOURCE00%" "\\%%~S\%%~T\Scripts" SHOW_NONE SET_EXCLUSIONS ADD_NEW
	 )

	 rem -- Obtain any Relevent Custom Input Files
	 IF DEFINED #EXISTS-%%~S (
		 FOR %%L IN (%@SOURCETXT%) DO FOR %%F IN ("%%~L" "%%~L.MINI" "%%~L.TEST") DO IF EXIST "%@SOURCE00%\%@LOCAL%\%%~F.%%~S.TXT" (
			 ECHO:
			 ECHO -- Copying: "%%~F.%%~S.TXT" to Remote System
			 IF EXIST "\\%%~S\%%~T\%@REMOTE%\%%~F.%%~S.TXT" ATTRIB -R -S -H "\\%%~S\%%~T\%@REMOTE%\%%~F.%%~S.TXT"
			 IF /I NOT "%@ISTEST%"=="/L" COPY "%@SOURCE00%\%@LOCAL%\%%~F.%%~S.TXT" "\\%%~S\%%~T\%@REMOTE%\%%~F.%%~S.TXT" /Y
			 ECHO:
		 )
	 )

	 rem -- Remove any Custom Input/Source Files from REMOTE machines for which they do not apply
	 IF DEFINED #EXISTS-%%~S (
		 IF /I "!@SYSTEMTYPE!"=="SUBSCRIBER" FOR /F "TOKENS=*" %%C IN ('DIR /B /A "\\%%~S\%%~T\%@REMOTE%\*.*.TXT" 2^>NUL ^| FINDSTR /I /V ".%%~S.TXT .TEST.TXT"  ^| FIND /I /V ".MINI.TXT" ') DO (
			 ECHO  - \\%%~S\%%~T\%@REMOTE%\%%~C
			 DEL "\\%%~S\%%~T\%@REMOTE%\%%~C"
		 )
	 )

	 rem -- Remove the Superflous Scripts subfolder for any System
	 IF DEFINED #EXISTS-%%~S (
		 IF EXIST "\\%%~S\%%~T\Scripts\Scripts" (
			 ECHO  - \\%%~S\%%~T\Scripts\Scripts
			 RD /S /Q "\\%%~S\%%~T\Scripts\Scripts"
		 )
	 )

	 rem -- Remove the LogonScripts folder for any Non-Publisher System
	 IF DEFINED #EXISTS-%%~S (
		 IF /I "!@SYSTEMTYPE!"=="SUBSCRIBER" IF EXIST "\\%%~S\%%~T\Scripts\BAT\LogonScripts" (
			 ECHO  - \\%%~S\%%~T\Scripts\BAT\LogonScripts
			 RD /S /Q "\\%%~S\%%~T\Scripts\BAT\LogonScripts"
		 )
	 )

	 rem -- Remove Obsolete Files
	 IF DEFINED #EXISTS-%%~S CALL :DeleteObsoleteFiles "\\%%~S" "%%~T"

	 rem -- Update Systems with the Custom Input/Source Files
	 IF DEFINED #EXISTS-%%~S (
		 IF /I "%@ISCUSTOM%"=="TRUE" (
			 FOR %%F IN (%@SOURCETXT%) DO IF EXIST "%@SOURCE00%\%@LOCAL%\%%~F.%%~S.TXT" (
				 ECHO:
				 ECHO -- Copying: "%%~F.%%~S.TXT" to "%%~F.TXT"
				 IF EXIST "%@SOURCE00%\%@LOCAL%\%%~F.TXT" ATTRIB -R -S -H "\\%%~S\%%~T\%@REMOTE%\%%~F.TXT"
				 IF /I NOT "%@ISTEST%"=="/L" COPY "%@SOURCE00%\%@LOCAL%\%%~F.%%~S.TXT" "\\%%~S\%%~T\%@REMOTE%\%%~F.TXT" /Y
				 ECHO:
			 )
		 ) ELSE (
			 rem -- Do NOT Copy Any Custom Input Files
			 ECHO -- Customized Source Files Will Not Be Copied As Default...
			 ECHO:
		 )
	 )
	 ECHO:
	 ECHO:
 )
 CALL :DivideSection FOOTER
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Generate Final Report Indicating Script Runtime Duration
 rem === Updated On: 30 Apr 2015 / 26 Apr 2015
 rem ==========================================================================
:FinalReport
 CALL :GetTime @ETIME
 ECHO:
 ECHO %@DIVIDER-%
 ECHO  *** COMPLETED UPDATE OF REMOTE SYSTEM SCRIPTS ***
 ECHO %@DIVIDER-%
 ECHO  Started On .................................... %@STIME%
 ECHO  Ended On ...................................... %@ETIME%
 DATEINFO -t !@STIME#! -n !@ETIME#! -e "dys hrs mins secs.ms" -o " Total Duration of Sync Jobs ................... " 2>NUL
 ECHO %@DIVIDER-%
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 03 Oct 2019 / 28 Feb 2019
 rem ==========================================================================
:ExitBatch
 IF /I "%@TEST_RUN%"=="FALSE" IF EXIST "%@TEMP_STORE%" RD /S /Q "%@TEMP_STORE%"
 FOR /L %%V IN (1,1,9) DO POPD
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Provide Clean Copy from ROBOCOPY Job
 rem === Updated On: 27 Jul 2020 / 27 Jun 2017
 rem ==========================================================================
:CopyFiles
 rem %1 = Source Folder
 rem %2 = Destination Folder
 rem %3 = Filespec to Copy
 rem %4 = Extra Robocopy Parameters

 FOR /F "TOKENS=*" %%Z IN ('%@ROBOCOPY% "%~1" "%~2" %~3 %@THREADS% %@RETRIES% /S /NJH /NJS /NDL /NS /NC /NP /XJ %~4 ^| FIND /I "%~1"') DO (
	 SET @OUTPUT=%%~Z
	 SET @OUTPUT=!@OUTPUT:100%%=!
	 SET @OUTPUT=!@OUTPUT:0%%=!
	 ECHO  + !@OUTPUT:%~1=!
 )
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Draw Divider for Section Header and/or Footer
 rem === Updated On: 04 Nov 2013 / 10 Oct 2013
 rem ==========================================================================
:DivideSection
 rem %1 = Header OR Footer OR SubHeader
 rem %2 = Description of Section

 IF NOT "%~2"=="" SET @BANNER=%~2& IF "!@BANNER!"=="" SET @BANNER=Unnamed Section

 FOR %%Z IN (H HEAD HEADER) DO IF /I "%~1"=="%%~Z" ( rem - Display Header Info
	 ECHO:
	 ECHO %@BANNER%
	 ECHO %@DIVIDER-%
 )

 FOR %%Z IN (S SUB SUBHEADER) DO IF /I "%~1"=="%%~Z" ( rem - Display Sub-Header Info
	 ECHO:
	 ECHO:
	 ECHO %@BANNER%
	 ECHO %@DIVIDER-:~-60%
 )

 FOR %%Z IN (F FOOT FOOTER) DO IF /I "%~1"=="%%~Z" ( rem - Display Footer Info
	 ECHO %@DIVIDER*%
	 ECHO FINISHED: %@BANNER%
	 ECHO:
	 ECHO:
 )
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Store Current Date and Time in Variable
 rem === Updated On: 16 Oct 2023 / 23 Mar 2015
 rem ==========================================================================
:GetTime
 rem %1 = Variable to Store Date/Time

 SET %~1=!DATE! at !TIME: =0!
 FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET %~1#="%%~d"
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Ensure that Files are Updated from the Source
 rem === Updated On: 20 May 2016 / 16 Sep 2012
 rem ==========================================================================
:UpdateFiles
 rem %1 = Source Folder for updated files (Scripts and Utilities)
 rem %2 = Destination Folder for updated files (Scripts and Utilities)
 rem %3 = Special Options: RECURSION_OFF EXCLUSION_OFF SHOW_SOURCE SHOW_DEST SHOW_NONE MIRROR ADD_NEW

 SET @S_DIR=%~1& SET @S_DIR=!@S_DIR:\.=!
 SET @D_DIR=%~2& SET @D_DIR=!@D_DIR:\.=!
 SET @S_LABEL=!@S_DIR:~0,73!
 SET @D_LABEL=!@D_DIR:~0,75!

 rem -- Determine Parameters for ROBOCOPY Actions
 SET @MIRROR=
 SET @SUBFOLDERS=/S
 SET @IGNORE=/XX /XL
 SET @DISPLAY=SOURCE
 SET @ROBOEXEMPT=%@SKIPDIRS% %@SKIPFILES%

 FOR %%P IN (%*) DO (
	 FOR %%Q IN (NO_R RECURSION_OFF) DO IF /I "%%~P"=="%%~Q" SET @SUBFOLDERS=
	 FOR %%Q IN (NO_E EXCLUSION_OFF) DO IF /I "%%~P"=="%%~Q" SET @ROBOEXEMPT=
	 FOR %%Q IN (TO SHOW_DEST) DO       IF /I "%%~P"=="%%~Q" SET @DISPLAY=SOURCE& SET @LABEL=TO: !@D_LABEL!
	 FOR %%Q IN (FROM SHOW_SOURCE) DO   IF /I "%%~P"=="%%~Q" SET @DISPLAY=DEST& SET @LABEL=FROM: !@S_LABEL!
	 FOR %%Q IN (NO_INFO SHOW_NONE) DO  IF /I "%%~P"=="%%~Q" SET @DISPLAY=SILENT& SET @LABEL=
	 FOR %%Q IN (MIR MIRROR) DO         IF /I "%%~P"=="%%~Q" SET @MIRROR=/MIR& SET @IGNORE=
	 FOR %%Q IN (ADD ADD_NEW) DO        IF /I "%%~P"=="%%~Q" SET @IGNORE=/XX
 )

 rem -- Exit Subroutine if Destination Cannot Be Reached
 CALL :VerifyTarget "!@D_DIR!"
 IF NOT DEFINED @ALIVE (
	 ECHO This Machine Cannot Be Accessed At This Time...
	 GOTO :EOF
 )

 IF EXIST "!@S_DIR!" (
	 FOR /F "TOKENS=1* DELIMS=:" %%E IN ('%@ROBOCOPY% "!@S_DIR!" "!@D_DIR!" %@THREADS% %@RETRIES% !@SUBFOLDERS! !@MIRROR! !@IGNORE! !@ROBOEXEMPT! /NP /NJH /NJS /NDL /FP !@ISTEST!') DO (
		 SET @DLETTER=%%~E
		 SET @FILENAME= + !@DLETTER:~-1!:%%~F
		 SET @FULLPATH=%%~E%%~F
		 SET @FULLPATH=!@FULLPATH:\\=?!& CALL :ObtainUNC "!@FULLPATH!"

		 rem -- If Destination Is Reachable, Then Determine What To Display...
		 IF /I NOT "!@DISPLAY!"=="SILENT" IF /I "!@SHOWHEADER!"=="TRUE" (
			 ECHO:
			 ECHO !@LABEL!
			 SET @SHOWHEADER=FALSE
		 )

		 IF /I "!@VERBOSE!"=="TRUE" ECHO %%~E%%~F
		 IF /I "!@DISPLAY!"=="SOURCE" (
			 ECHO  + !@S_DIR!\%%~nxF
		 ) ELSE (
			 ECHO !@FILENAME!
		 )
	 )
 )
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Ensure that a Single File is Updated from the Source
 rem === Updated On: 19 Sep 2019 / 17 Apr 2015
 rem ==========================================================================
:UpdateOneFile
 rem %1 = Source File
 rem %2 = Destination File
 rem %3 = Message to Display

 SET @MSG=%~3
 SET @FC=COPY& FOR /F %%c IN ('FC "%~1" "%~2" 2^>NUL ^| FIND "%@NODIFF%"') DO SET @FC=

 ATTRIB -R -S -H "%~2"
 IF DEFINED @FC FOR /F "TOKENS=*" %%z IN ('COPY "%~1" "%~2" /V /Y') DO (
	 IF DEFINED @MSG (ECHO %@MSG%) ELSE (ECHO  + "Install.BAT" has been updated... [%%z])
 )
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Execute Dynamic Command From Variable
 rem === Updated On: 23 Oct 2013 / 27 Jul 2010
 rem ==========================================================================
:RunCMD
 rem %* = Command To Execute

 %*
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Delete Obsolete Files from Local or Remote Repository
 rem === Updated On: 14 Jan 2021 / 27 Jul 2020
 rem ==========================================================================
:DeleteObsoleteFiles
 rem %1 = Current System and Optional Path
 rem %2 = Optional UNC Share

 ECHO:
 ECHO:
 ECHO  ...Deleting Obsolete Files...
 SET @CLEANUP_LOC="%~1" "%~1\%~2" "%~1\%~2\Scripts" "%~1\Scripts" "%~1\Scripts$" "%~1\Scripts_Source$\Scripts"
	 IF "%~2"=="" SET @CLEANUP_LOC="%~1" "%~1\Scripts" "%~1\Scripts$" "%~1\Scripts_Source$\Scripts"
	 IF "%~1"=="" SET @CLEANUP_LOC="\\%COMPUTERNAME%\Scripts$" "\\%COMPUTERNAME%\Scripts_Source$\Scripts"

 FOR %%P IN ("%@HERE%" !@CLEANUP_LOC! %@CLOUD_CLEAN%) DO IF NOT DEFINED $$_%%~P IF EXIST "%%~P\*.*" (
	 SET $$_%%~P=TRUE
	 FOR %%L IN ("%@CLEANUP_MAIN%" "%@CLEANUP_CUSTOM%") DO IF EXIST "%%~L" (
		 FOR /F "USEBACKQ TOKENS=* EOL=;" %%F IN ("%%~L") DO IF EXIST "%%~P\%%~F" (
			 IF /I "%DEBUG%"=="DELETE" (
				 ECHO  * %%~P\%%~F
			 ) ELSE (
				 ECHO  - %%~P\%%~F
				 ATTRIB -S -H -R "%%~P\%%~F"
				 DEL "%%~P\%%~F"
			 )
		 )
	 )
 )
 ECHO %@DIVIDER-%
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Delete Personal Config Files
 rem === Updated On: 25 Nov 2020      
 rem ==========================================================================
:DeletePersonalConfigFiles
 rem %1 = Current Folder or UNC Share Path

 IF NOT EXIST "%~1" GOTO :EOF
 ECHO:
 ECHO:
 ECHO  ...Deleting Personal Config Files...
 FOR %%C IN (S W A B) DO DEL "%~1\%%~C*.CFG" /S
 ECHO %@DIVIDER-%
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Obtain UNC Path From ROBOCOPY Output
 rem === Updated On: 27 Jul 2010      
 rem ==========================================================================
:ObtainUNC
 rem %1 = String From Which to Derive UNC Path

 SET @STRING=%~1
 SET @STRING=!@STRING:(=[!
 SET @STRING=!@STRING:)=]!
 FOR /F "TOKENS=1* DELIMS=?" %%V IN ('ECHO !@STRING!') DO SET @UNCPATH= \\%%~W
 IF /I NOT "!@UNCPATH!"==" \\" SET @FILENAME=!@UNCPATH!
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Validate Availability of Target System for Repository
 rem === Updated On: 12 Mar 2019 / 20 May 2016
 rem ==========================================================================
:VerifyTarget
 rem %1 = Path of Destination System to Verify

 SET @SYS=
 SET @ALIVE=
 SET @PREFIX=%~1
 SET @PREFIX=%@PREFIX:~0,2%
 FOR /F "TOKENS=1 DELIMS=\" %%S IN ('ECHO %~1') DO SET @SYS=%%~S
 IF NOT "!@PREFIX!"=="\\" SET @SYS=%COMPUTERNAME%
 SET #UNREACHABLE-!@SYS!=TRUE
 IF DEFINED #EXISTS-!@SYS! (SET @ALIVE=TRUE& SET #UNREACHABLE-!@SYS!=& GOTO :EOF)

 FOR /F "TOKENS=*" %%A IN ('NLTEST /QUERY /SERVER:%@SYS% 2^>NUL ^| FIND /I "The command completed successfully"') DO (
	 SET @ALIVE=!@SYS!
	 SET #UNREACHABLE-!@SYS!=
 )
 IF EXIST "%~1" SET @ALIVE=%~1
 SET #EXISTS-!@SYS!=!@SYS!
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Create/Update Temp Repository for Standard Subscribers
 rem === Updated On: 30 Jul 2020 / 24 Sep 2019
 rem ==========================================================================
:CreateTempRepository
:UpdateTempRepository
 rem %1 = Optional Marker for Folder Creation

 IF /I "%~0"==":CreateTempRepository" (
	 SET @PURGE=/PURGE
	 SET @JOBMSG=CREATING
	 RD /S /Q "%@TEMP_STORE%"    >NUL 2>NUL
	 MD "%@TEMP_STORE%\%@LOCAL%" >NUL 2>NUL
 ) ELSE (
	 SET @PURGE=
	 SET @JOBMSG=UPDATING
 )

 CALL :DivideSection HEADER "%@JOBMSG% Temporary Repository..."
 CALL :CopyFiles "%@DEST1%" "%@TEMP_STORE%" *.TEST.TXT "%@PURGE% %@TODAY-R% /IS"
 CALL :CopyFiles "%@DEST1%" "%@TEMP_STORE%" *.*        "%@PURGE% %@TODAY-R% /IS /XF %@PERSONAL_CONFIGS% *.*.TXT"
 IF DEFINED @DEBUG DIR "%@TEMP_STORE%" /OGD /S
 CALL :DivideSection FOOTER
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Use CheckParams.exe to Capture Selected Parameters
 rem === Updated On: 21 Jan 2020 / 24 Sep 2019
 rem ==========================================================================
:GetParams
 rem %1 = Parameters to Search For
 rem %2 = Variable to Set

 SET @OK=& IF "%~1"=="" GOTO :EOF
 FOR /F "TOKENS=2-3*" %%v IN ('CHECKPARAMS -q -a -c "%~1" -s %@ALL_PARAMS% 2^>NUL') DO IF /I "%%~v"=="TRUE" (
	 SET @OK=T
	 SET @REQ=T
	 IF NOT "%~2"=="" SET %~2=%%~x
 )
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET @SCRIPT_BEG#="%%~d"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 30 Nov 2020 / 20 Sep 2019
 rem ==========================================================================
:HelpMessage
 ECHO Update Script Repositories Across Enterprise Servers via Input Lists
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  ^<OPTIONS ^| serverlist^> [/ALT:^|/A:^<Alt_Server_Name^>] [/TEST^|/T] [/SPECIAL^|/S] [/VERBOSE^|/V]
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:
 ECHO -------------------
 ECHO OPTION DEFINITIONS:
 ECHO -------------------
 ECHO  -?,  /H ......... Display This Help Message  (also --HELP)
 ECHO  -??, /H2 ........ Display Extended Help Message
 ECHO       .  ......... Use default server list (mandatory for using XCOPY parameters)
 ECHO       @  ..........Update ALL script repositories, but skip active machine list
 ECHO       ~  ......... Update local script repository ONLY
 ECHO       #  ......... Skip Update of Executable files
 ECHO  /A,  /ALT ....... Provide an alternate server name to base the configs upon
 ECHO  /T,  /TEST ...... Perform a Test Run with normal ROBOCOPY commands in LIST mode
 ECHO  /S,  /SPECIAL ... Perform a Special Test with Restricted Parameters
 ECHO  /V,  /VERBOSE ... Generate additional output
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0 .
	 ECHO  %~n0 @
	 ECHO  %~n0 ~
	 ECHO  %~n0 %~dp0Input\ServerList.TXT
	 ECHO  %~n0 "%~dp0Input\ServerList-Complete.TXT" /TEST
	 ECHO  %~n0 "D:\Special Folder\Custom\Server List.TXT" /S
	 ECHO  %~n0 "E:\Lists\Selected Systems.TXT" /ALT:SERVER01
	 ECHO  %~n0 -?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
	 ECHO:
	 ECHO  * If the /A: option is used, a non-blank value must be provided.
	 ECHO:
	 ECHO  * Long filenames which contain spaces must be surrounded by quotes.
	 ECHO    Shortnames *can* be surrounded by quotes, but it is not mandatory.
 )
 CALL :ShowStatus "FINISHED"
                                                               