@ECHO OFF
 IF DEFINED @SANITIZED CLS
 rem ==========================================================================
 rem === View/Set the Credentials on Scheduled Jobs (Local or Remote)
 rem ==========================================================================
 rem
 rem Created On: 08 Jan 2015
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 04 Apr 2024 / 02 Apr 2024 / 02 Dec 2023 / 16 Oct 2023
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2003 and later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem ==========================================================================

 :::  This script displays all local/remote scheduled tasks which are using a
 :::  specific account. Additional options include changing the password for
 :::  the tasks, or changing both the username and password for all tasks in
 :::  question. It is also possible to filter on text within the description
 :::  of the scheduled job.
 :::
 :::  (Jan 2015) v2.0 -- Improved the processing of script parameters, and
 :::  added the ability to change username along with password.
 :::
 :::  (Jan 2015) v2.2 -- Major performance fixes; Also corrected a bug with
 :::  the optional search parameter (/F) functionality.
 :::
 :::  (Apr 2015) v2.3 -- Added a couple of defaults (using ".") and the
 :::  calculation of the script timing.
 :::
 :::  (Apr 2015) v2.4 -- Updated :ShowStatus routine to track script duration.
 :::
 :::  (May 2018) v2.9 -- Changed the HELP routine to use FINDSTR;
 :::  -- Added new Filtered CMD Title and Syntax Help processing routines;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com
 :::
 :::  (Jul 2018) v2.10 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::
 :::  (Oct 2018) v2.11 -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Mar 2019) v2.12 -- Enabled Code Page 1252 Support;
 :::
 :::  (May 2019) v3.0 -- The new BrainWave GetInput utility is now used to
 :::  obtain password input without displaying it to the console.  Inpired by
 :::  Bill Stewart's nifty READLINE utility (www.westmesatech.com/?page_id=49);
 :::  -- Refactored the :GetParams routine using the CheckParams.exe utility;
 :::
 :::  (Oct 2019) v3.1 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (Jan 2020) v3.2 -- Refactored the :GetParams routine as per other scripts;
 :::
 :::  (May 2020) v3.3 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::
 :::  (Nov 2020) v3.4 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Oct 2023) v3.5 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::  (Apr 2024) v3.6 -- Check for essential utilities with FINDFILES;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 22 Mar 2022 / 30 Nov 2020
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %1 %2
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 04 Apr 2024 / 02 Apr 2024 / 30 Nov 2020
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v3.6.0.421

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc --mp=2 -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Check for Essential Utilities and/or Scripts
 SET @ESSENTIALS="%~dp0SetDrive.BAT" CHECKPARAMS.EXE DATEINFO.EXE NOW.EXE READCONFIG.EXE FIND.EXE
 PATH %PATH%;%~dp0
 FINDFILES -w -m -f %@ESSENTIALS% GETINPUT.EXE FINDSTR.EXE SCHTASKS.EXE
 IF ERRORLEVEL 64 GOTO :ExitBatch

 rem -- Set Global Variables from Centralized Script
 CALL "%~dp0SetDrive.BAT" "%~n0"

 rem -- Main Variables
 SET @SYSTEM=%~1&  IF "%~1"=="." SET @SYSTEM=%COMPUTERNAME%
 SET @ACCOUNT=%~2& IF "%~2"=="." SET @ACCOUNT=SYSTEM
 SET @JOBACCT=%@ACCOUNT%
 SET @FILTER=
 SET @SEARCH=
 SET @READONLY=TRUE
 SET @CHANGES=NONE
 SET @EXCLUDE=\Microsoft\
 SET @NODATA=*** NO ENTRIES FOUND ***
 SET @SCHTASKS=SCHTASKS /S %@SYSTEM% /QUERY /FO LIST

 rem -- Standard Log Variables
 SET @LOGPATH=%@STORAGE%\Logs\Maint
 SET @LFORMAT=%@LOGPATH%\%~n0-%@SYSTEM%
 SET @LOGFILE=%@LFORMAT%.%@FILESTAMP%
 FOR %%L IN (SUMMARY ALLTASKS TEMPLIST TASKINFO) DO SET @%%L=%@LFORMAT%.%%L.%@FILESTAMP%
 SET @LOGLIST="%@LOGFILE%"
 SET @CLEANUP="%@SUMMARY%" "%@ALLTASKS%" "%@TEMPLIST%" "%@TASKINFO%"

 rem -- Process Special Command-line Parameters
 SET @ALL_PARAMS=%*
 CALL :GetParams "F FIND" @SEARCH & IF DEFINED @OK IF NOT DEFINED @SEARCH (GOTO :HelpMessage) ELSE (
	 SET @FILTER=^^^| FIND /I "!@SEARCH!"
 )

 CALL :GetParams "N NEW" @NEWACCT & IF DEFINED @OK IF NOT DEFINED @NEWACCT (GOTO :HelpMessage) ELSE (
	 SET @NEWACCT="!@NEWACCT!"
	 SET @JOBACCT=!@NEWACCT!
 )

 CALL :GetParams "P PWD" @PASSWORD & IF DEFINED @OK IF NOT DEFINED @PASSWORD (GOTO :HelpMessage) ELSE (
	 SET @READONLY=FALSE
	 IF /I "!@PASSWORD!"=="+" (SET @PASSWORD=*)
	 IF /I "!@PASSWORD!"=="." (FOR /F "TOKENS=*" %%P IN ('GetInput -p "Please Enter Password: " -q') DO SET @PASSWORD="%%P")
	 SET @PASSWORD="!@PASSWORD:"=!"
	 IF !@PASSWORD!=="" (
		 ECHO Password cannot be blank!
		 ECHO:
		 GOTO :HelpMessage
	 )
 )
 %@PRINT_IF_DEBUG%


 rem ==========================================================================
 rem === Display Job Header
 rem === Updated On: 30 Apr 2015 / 31 Jan 2015
 rem ==========================================================================
:Header
 TITLE View/Set the Credentials of Scheduled Jobs on %@SYSTEM%

 ( rem -- Write to Log
	 ECHO *** %COMPUTERNAME% - [%DATE% at %TIME%] - %@VER%
	 ECHO:
	 ECHO Looking for Scheduled Tasks Matching the Following:
	 ECHO %@DIVIDER-%
	 ECHO  SYSTEM NAME .............. %@SYSTEM%
	 ECHO  CURRENT USER ACCOUNT ..... %@ACCOUNT%
	 IF DEFINED @NEWACCT ECHO  CHANGE USER ACCOUNT TO ... %@NEWACCT%
	 IF DEFINED @SEARCH  ECHO  SEARCH IN TASKNAME FOR ... %@SEARCH%
	 ECHO  LOGFILE .................. %@LOGFILE%
	 ECHO %@DIVIDER-%
	 ECHO:
 ) >"%@SUMMARY%" 2>&1

 FOR %%O IN (CON "%@LOGFILE%") DO TYPE "%@SUMMARY%" >>%%O 2>&1
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Check or Set Scheduled Task Options
 rem === Updated On: 26 May 2019 / 13 Dec 2017
 rem ==========================================================================
:ReviewInfo
 BREAK ON  >"%@TEMPLIST%"
 BREAK ON >>"%@TASKINFO%"
 %@SCHTASKS% | FIND "TaskName:" | FIND /I /V "%@EXCLUDE%" >"%@ALLTASKS%" 2>&1

 ( rem -- Write to Log
	 FOR /F "USEBACKQ TOKENS=1* DELIMS=\" %%S IN ("%@ALLTASKS%") DO (
		 %@SCHTASKS% /V /TN "%%~T" >"%@TASKINFO%" 2>&1
		 SET @FINDIT=& FOR /F %%F IN ('TYPE "%@TASKINFO%" %@FILTER%') DO SET @FINDIT=TRUE
		 SET @HASACCT=& FOR /F %%F IN ('TYPE "%@TASKINFO%" ^| FIND "Run As User:" ^| FINDSTR /I /R "!@ACCOUNT!$"') DO SET @HASACCT=TRUE

		 IF /I "!@FINDIT!/!@HASACCT!"=="TRUE/TRUE" (
			 ( %@SCHTASKS% /V /TN "%%~T" | FINDSTR "Run: State: TaskName: User: Mode:" & ECHO: ) | SORT /R >>"%@TEMPLIST%" 2>&1

			 FOR /F "TOKENS=*" %%J IN ('TYPE "%@TASKINFO%" ^| FIND "Task To Run:"') DO (
				 SET @THISTASK=%%~J
				 SET @THISTASK=!@THISTASK:Task To Run:                          =!
				 SET @THISTASK=!@THISTASK:"=\"!
				 IF /I "!@READONLY!"=="TRUE" (
					 ECHO:
					 ECHO %@DIVIDER*%
					 ECHO Displaying Task Info for: %%~T
					 ECHO %@DIVIDER*%
					 TYPE "%@TASKINFO%"
				 ) ELSE (
					 ECHO Setting Credentials for: %%~T
					 IF !@PASSWORD!=="*" (
						 SCHTASKS /S %@SYSTEM% /CHANGE /RU %@JOBACCT% /RP %@PASSWORD% /TN "%%~T" /TR "!@THISTASK!" >CON 2>&1
					 ) ELSE (
						 SCHTASKS /S %@SYSTEM% /CHANGE /RU %@JOBACCT% /RP %@PASSWORD% /TN "%%~T" /TR "!@THISTASK!"
					 )
					 SET @CHANGES=YES
					 ECHO %@DIVIDER~%
				 )
				 ECHO:
			 )
		 )
	 )
 ) >"%@SUMMARY%" 2>&1

 FOR %%O IN (CON "%@LOGFILE%") DO TYPE "%@SUMMARY%" >>%%O 2>&1
 %@PAUSE_IF_DEBUG%

 rem -- Set Variables to View Updated Configuration (after change)
 IF /I "!@READONLY!"=="FALSE" (
	 SET @READONLY=TRUE
	 SET @ACCOUNT=!@ACCOUNT! %@JOBACCT%
	 %@PRINT_IF_DEBUG%
	 GOTO :ReviewInfo
 )


 rem ==========================================================================
 rem === Display Summary of All Relevant Scheduled Tasks
 rem === Updated On: 15 May 2015 / 27 Apr 2015
 rem ==========================================================================
:Summary
 SET @HASDATA=& FOR /F "USEBACKQ" %%L IN ("%@TEMPLIST%") DO SET @HASDATA=TRUE

 ( rem -- Write to Log
	 ECHO:
	 ECHO %@DIVIDER-%
	 ECHO --- SCHEDULED TASK SUMMARY LIST			[CHANGES MADE: !@CHANGES!]
	 ECHO %@DIVIDER-%
	 IF NOT DEFINED @HASDATA (
		 ECHO  %@NODATA%
	 ) ELSE (
		 FOR /F "USEBACKQ TOKENS=*" %%L IN ("%@TEMPLIST%") DO (
			 SET @THISLINE=%%L
			 SET @THISLINE=!@THISLINE:Scheduled Task State:                 =Task State:   !
			 IF "%%~L"=="" (ECHO:) ELSE (ECHO !@THISLINE:                        =!)
		 )
	 )
	 ECHO %@DIVIDER-%
	 CALL :GetTime @SCRIPT_END
	 DATEINFO -t !@SCRIPT_BEG#! -n !@SCRIPT_END#! -e "hr:min:sec.ms" -o "Script Processing Duration: "
	 ECHO %@DIVIDER-%
 ) >"%@SUMMARY%" 2>&1

 FOR %%O IN (CON "%@LOGFILE%") DO TYPE "%@SUMMARY%" >>%%O 2>&1
 TIMEOUT 5
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Perform Log Management/Maintenance (Or Copy Them for Debug Purposes)
 rem === Updated On: 12 Jan 2015 / 09 Jan 2015
 rem ==========================================================================
:LogMaint
 ECHO:
 ECHO PERFORMING FINAL LOG MAINTENANCE...

 IF /I "%DEBUG%"=="COPY" FOR %%L IN (%@LOGLIST% %@CLEANUP%) DO IF EXIST %%L COPY %%L "%TEMP%" /Y

 FOR %%L IN (%@LOGLIST%) DO IF EXIST %%L (
	 SET @THISLOG=%%L
	 COPY %%L !@THISLOG:%@FILESTAMP%=TXT! /Y
	 IF %%L=="%@LOGFILE%" (ECHO: & TYPE %%L & DIR %%L)
 )
 FOR %%L IN (%@CLEANUP%) DO IF EXIST %%L DEL %%L >NUL
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019
 rem ==========================================================================
:ExitBatch
 TITLE Command Prompt (%USERDOMAIN%\%USERNAME%)   %@DEBUG_MODE%
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Store Current Date and Time in Variable
 rem === Updated On: 16 Oct 2023 / 01 Apr 2015
 rem ==========================================================================
:GetTime
 rem %1 = Variable to Store Date/Time

 SET %~1=!DATE! at !TIME: =0!
 FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET %~1#="%%~d"
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Use CheckParams.exe to Capture Selected Parameters
 rem === Updated On: 21 Jan 2020      
 rem ==========================================================================
:GetParams
 rem %1 = Parameters to Search For
 rem %2 = Variable to Set

 SET @OK=& IF "%~1"=="" GOTO :EOF
 FOR /F "TOKENS=2-3*" %%v IN ('CHECKPARAMS -q -a -c "%~1" -s %@ALL_PARAMS% 2^>NUL') DO IF /I "%%~v"=="TRUE" (
	 SET @OK=T
	 SET @REQ=T
	 IF NOT "%~2"=="" SET %~2=%%~x
 )
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" CALL :GetTime @SCRIPT_BEG
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 02 Apr 2024 / 02 Dec 2023 / 30 Nov 2020
 rem ==========================================================================
:HelpMessage
 ECHO View/Set the Credentials on Scheduled Jobs (Local or Remote)
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  ^<system^> ^<username^>                                 [-f:TaskInfo]  [OPTIONS]
 ECHO  %~n0  ^<system^> ^<username^> [-p:[password^|+^|.]]             [-f:TaskInfo]  [OPTIONS]
 ECHO  %~n0  ^<system^> ^<username^> [-n:NEW_username ^<-p:password^>] [-f:TaskInfo]  [OPTIONS]
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:
 ECHO -------------------
 ECHO OPTION DEFINITIONS:
 ECHO -------------------
 ECHO  -?,  /H ..... Display This Help Message  (also --HELP)
 ECHO  -??, /H2 .... Display Extended Help Message
 ECHO  /P:xxx  ..... Password to set for the Selected Username, where xxx = actual password (less secure)
 ECHO  /P:.    ..... Password will be prompted via command shell once per machine           (more secure)
 ECHO  /P:+    ..... Password will be prompted securely at runtime once per scheduled job   (most secure, but tedious)
 ECHO  /N      ..... New Account to use for the Scheduled Task (password must match this account)
 ECHO  /F      ..... Find only tasks that contain this string
 ECHO   .      ..... Return %COMPUTERNAME% (1st parameter) or %COMPUTERNAME%\SYSTEM (2nd parameter)
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0 %COMPUTERNAME% MyService
	 ECHO  %~n0 SERVER1 MyDomain\MyService -p:.
	 ECHO  %~n0 MACHINE10 Prod\SpecialAcct -n:Prod\OtherAcct "-p:SOm3P@ssw0rd"
	 ECHO  %~n0 SYSTEM27A ServiceAcct -n:OURDOMAIN\ServiceAcct "-p:0th3rP@ssw0rd"
	 ECHO  %~n0 COMPUTER5 Dev\AdminAcct "-f:Bob's Tasks"
	 ECHO  %~n0 . AdminScripts -n:%USERDOMAIN%\AdminScripts -p:.
	 ECHO  %~n0 -?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
	 ECHO:
	 ECHO:
	 ECHO  * Any parameters that contain spaces should be surrounded by quotes,
	 ECHO    including the option prefix itself.
	 ECHO:
	 ECHO  * Now that we have a viable replacement for "SET /P" (GetInput.exe)
	 ECHO    which can be used to enter passwords at the command prompt, it is
	 ECHO    advised that you use /P:. as the preferred /P variant.
	 ECHO:
	 ECHO  * Usernames can be given in "Username" or "Domain\Username" format.
	 ECHO:
	 ECHO  * The script may fail to change scheduled jobs whose passwords are
	 ECHO    provided at the command prompt, and contain any of the following
	 ECHO    characters: ? * \ / ^" %% ^( ^) ^< ^> ^| or the exclamation point
	 ECHO:
	 ECHO  * An elevated CMD promt is required in Windows 2008 (and beyond) in
	 ECHO    order to set or display the list of scheduled tasks.
	 ECHO:
	 ECHO  * "." can be used ^(1st parameter^) to return the current machine
	 ECHO  * "." can be used ^(2nd parameter^) to return the SYSTEM account
 )
 CALL :ShowStatus "FINISHED"
                                                                                   