@ECHO OFF
 rem ==========================================================================
 rem === Show Results from Setting Variables via @Variables.BAT / SetDrive.BAT
 rem ==========================================================================
 rem
 rem Created On: 19 Feb 2006
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Oct 2023 / 24 May 2022              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2000 or later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem  Input Files Used By Script (stored in %SystemDrive%\Scripts\Bat\Input):
 rem    ArchiveInfo.TXT ........ List of source and destination folder(s)
 rem    CheckVPN.TXT ........... List of remote IPs to check for VPN Connectivity
 rem    CheckWAN.TXT ........... List of remote IPs to check for WAN Connectivity
 rem    CustomVariables.TXT .... Centralized Configuration Settings for Scripts
 rem    MRTG-Info.TXT .......... Map of Network IP Subnet Ranges to Pinpoint Suspect Systems
 rem    ReplicationInfo.TXT .... List of source and destination folder(s)
 rem    WinLibraryList.TXT ..... List of Windows Library Settings
 rem
 rem  Required Scripts:
 rem    SetDrive.BAT ........... Set Global Variables for this Environment
 rem ==========================================================================

 :::  This is nothing but a test script for seeing what variables will be set
 :::  by the @Variables.BAT file, and what were passed via the CLI.  As of Nov
 :::  2010, it also captures the values in the CustomVariables.TXT input file.
 :::
 :::  (Oct 2013) v2.0 -- The script also captures the values from the
 :::  following (and appropriate) INPUT files:
 :::   --- MRTG-Info.TXT
 :::   --- ArchiveInfo.TXT
 :::   --- ReplicationInfo.TXT
 :::   --- WinLibraryList.TXT
 :::
 :::  (Mar 2015) v2.2 -- Adds Support for CustomVariables.MINI files
 :::
 :::  (Apr 2015) v2.3 -- Configuration info is now derived from SetDrive.BAT
 :::  rather than from @Variables.BAT.
 :::
 :::  (Jan 2018) v3.0 -- Added an express mode to bypass the TIMEOUT settings
 :::  when running in a script or for debug purposes; Added variables from the
 :::  following INPUT files.
 :::   --- CheckVPN.TXT
 :::   --- CheckWAN.TXT
 :::
 :::  (Apr 2018) v3.1 -- Changed the HELP routine to use FINDSTR;
 :::  -- Added filtered CMD Title Routine;
 :::  -- The SYNTAX HELP routine has been updated;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com
 :::
 :::  (Jul 2018) v3.2 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::
 :::  (Oct 2018) v3.3 -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Dec 2018) v3.4 -- Redisplay any searched items at the end of the job;
 :::
 :::  (Oct 2019) v3.5 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (Mar 2020) v4.0 -- Read Configuration files using new BrainWave Utility;
 :::
 :::  (Apr 2020) v4.1 -- Updated ReadConfig.exe to the public release version;
 :::  -- Updated the DebugMode routines to support the STEP option;
 :::
 :::  (May 2020) v4.2 -- Renamed the variable for the MINI custom variables file;
 :::  -- Now support MINI editions of the ARCHIVE-INFO and REPL-INFO config files;
 :::  -- Removed the legacy edition of the OS build number parser;
 :::
 :::  (Nov 2020) v4.3 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (May 2022) v4.4 -- Use -A instead of -G for ReadConfig.exe parameters;
 :::
 :::  (Oct 2023) v4.5 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 30 Nov 2020 / 13 Apr 2018              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 24 May 2022 / 30 Nov 2020              
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v4.5.0.450

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Capture CommandLine at Time of Script Execution
 SET @CMDLINE=(%CMDCMDLINE%) %~f0 %*
 SET @TIMEOUT=TIMEOUT 30& IF DEFINED DEBUG SET @TIMEOUT=

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- Grab Appropriate Custom Variables from CustomVariables.*.TXT
 SET #CUSTOM=& FOR %%F IN ("%@CUSTOM-VARS%" "%@CUST-VARS-MINI%") DO IF EXIST "%%~F" (
	 SET #CUSTOM=PRESENT
	 FOR /F "TOKENS=*" %%R IN ('READCONFIG "%%~F" -F -A 2^>NUL') DO CALL :RunCMD %%R
 )
 CALL :Check4List CUSTOM

 rem -- Grab Appropriate Custom Variables from ArchiveInfo.*.TXT
 FOR %%C IN ("%@ARCHIVE-INFO%" "%@ARCH-INFO-MINI%") DO IF EXIST "%%~C" (
	 SET #ARCHIVE-INFO=PRESENT
	 FOR /F "TOKENS=*" %%R IN ('READCONFIG "%%~C" -F -A 2^>NUL') DO CALL :RunCMD %%R
	 FOR /F "TOKENS=1 DELIMS=;" %%J IN ('READCONFIG "%%~C" -N -S "#CREATEARCHIVES" -P 2^>NUL') DO SET @VALIDJOB-CREATE-%%~J=%%~J& SET @JOBLIST=TRUE
	 FOR /F "TOKENS=1 DELIMS=;" %%J IN ('READCONFIG "%%~C" -N -S "#RETAINBACKUPS"  -P 2^>NUL') DO SET @VALIDJOB-RETAIN-%%~J=%%~J& SET @JOBLIST=TRUE
	 FOR /F "TOKENS=1 DELIMS=;" %%J IN ('READCONFIG "%%~C" -N -S "#MOVEARCHIVES"   -P 2^>NUL') DO SET @VALIDJOB-MOVE-%%~J=%%~J& SET @JOBLIST=TRUE
 )
 CALL :Check4List ARCHIVE-INFO

 rem -- Grab Appropriate Custom Variables from MRTG-Info.*.TXT
 IF EXIST "%@MRTG-INFO%" (
	 SET #MRTG=PRESENT
	 FOR /F "TOKENS=*" %%R IN ('READCONFIG "%@MRTG-INFO%" -M 2^>NUL') DO CALL :RunCMD %%R
 )
 CALL :Check4List MRTG

 rem -- Grab Appropriate Custom Variables from ReplicationInfo.*.TXT
 FOR %%C IN ("%@REPL-INFO%" "%@REPL-INFO-MINI%") DO IF EXIST "%%~C" (
	 SET #REPLICATION-INFO=PRESENT
	 FOR /F "TOKENS=*" %%R IN ('READCONFIG "%%~C" -F -A 2^>NUL') DO CALL :RunCMD %%R
	 FOR /F "TOKENS=1 DELIMS=;" %%J IN ('READCONFIG "%%~C" -N -S "@SKIPFILE" -P 2^>NUL') DO SET @VALIDJOB-REPL-%%~J=%%~J& SET @JOBLIST=TRUE
 )
 CALL :Check4List REPLICATION-INFO

 rem -- Grab Appropriate Custom Variables from WinLibraryList.*.TXT
 IF EXIST "%@LIBRARYLIST%" (
	 SET #WINLIBRARY=PRESENT
	 SET @COUNT=100
	 FOR /F "TOKENS=*" %%L IN ('READCONFIG "%@LIBRARYLIST%" -N 2^>NUL') DO (
		 SET /A @COUNT+=1
		 SET @LIBRARY-!@COUNT:~-2!=%%L
	 )
 )
 CALL :Check4List WINLIBRARY

 rem -- Grab Appropriate Custom Variables from CheckVPN.*.TXT
 IF EXIST "%@VPNLIST%" (
	 SET #VPNLIST=PRESENT
	 SET @COUNT=100
	 FOR /F "TOKENS=*" %%L IN ('READCONFIG "%@VPNLIST%" -N 2^>NUL') DO (
		 SET /A @COUNT+=1
		 SET @VPNLIST-!@COUNT:~-2!=%%L
	 )
 )
 CALL :Check4List VPNLIST

 rem -- Grab Appropriate Custom Variables from CheckWAN.*.TXT
 IF EXIST "%@WANLIST%" (
	 SET #WANLIST=PRESENT
	 SET @COUNT=100
	 FOR /F "TOKENS=*" %%L IN ('READCONFIG "%@WANLIST%" -N 2^>NUL') DO (
		 SET /A @COUNT+=1
		 SET @WANLIST-!@COUNT:~-2!=%%L
	 )
 )
 CALL :Check4List WANLIST


 rem ==========================================================================
 rem === Main Script: Display All Variables
 rem === Updated On: 19 Dec 2018 / 01 Jan 2018              
 rem ==========================================================================
:Main
 ECHO %@DIVIDER-%
 ECHO --- ENVIRONMENT VARIABLES by %@VER% on %@TIMESTAMP%
 ECHO %@DIVIDER-%
 ECHO:
 ECHO  Path of CMD ..... %CMDCMDLINE%
 ECHO  %%0 full path .... %~f0
 ECHO  Full CMD line ... %@CMDLINE%
 ECHO %@DIVIDER-%

 FOR /L %%P IN (0,1,9) DO (
	 SET #N=%%~P
	 CALL :RunCMD SET #V=%%~!#N!
	 IF NOT "!#V!"=="" ECHO  %%%!#N! = !#V!
 )
 ECHO  %%* = %*
 ECHO %@DIVIDER-%
 FOR %%P IN (%*) DO SET %%~P 2>NUL
 ECHO %@DIVIDER-%
 %@TIMEOUT%

 FOR %%V IN (\ @ #) DO SET %%~V 2>NUL
 ECHO %@DIVIDER-%
 IF NOT "%~1"=="" (
	 FOR %%P IN (%*) DO SET %%~P 2>NUL
	 ECHO %@DIVIDER-%
 )
 ECHO:
 %@TIMEOUT%


 rem ==========================================================================
 rem === If ANY of the parameters is "." then make these variables permanent
 rem === Updated On: 24 May 2022 / 15 May 2020              
 rem ==========================================================================
:MakePermanent
 FOR %%P IN (%*) DO IF "%%~P"=="." (
	 ECHO:
	 ECHO Setting Variables Permanently [for current CMD session]...
	 ECHO:
	 ENDLOCAL
	 CALL "%~dp0\SetDrive.BAT" TestScript
	 FOR %%F IN ("%@CUSTOM-VARS%" "%@CUST-VARS-MINI%") DO IF EXIST "%%~F" (
		 FOR /F "TOKENS=*" %%R IN ('READCONFIG "%%~F" -F -A 2^>NUL') DO CALL :RunCMD %%R
	 )
 )


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019              
 rem ==========================================================================
:ExitBatch
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Execute Dynamic Command From Variable
 rem === Updated On: 05 Nov 2010                     
 rem ==========================================================================
:RunCMD
 rem %1 = Command To Execute

 %*
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Check for Variable Container
 rem === Updated On: 15 May 2020                     
 rem ==========================================================================
:Check4List
 rem %1 = Variable to Check

 IF NOT DEFINED #%~1 (
	 ECHO:
	 ECHO WARNING: MISSING %~1 VARIABLES...
	 ECHO:
 )
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET @SCRIPT_BEG#="%%~d"
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 30 Nov 2020 / 29 Apr 2018              
 rem ==========================================================================
:HelpMessage
 ECHO Show All Variables Set via SetDrive.BAT (formerly @Variables.BAT)...
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  [prefixes of variables to search for]
 ECHO  %~n0  [OPTIONS]
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:
 ECHO -------------------
 ECHO OPTION DEFINITIONS:
 ECHO -------------------
 ECHO  -?,  /H ..... Display This Help Message  (also --HELP)
 ECHO  -??, /H2 .... Display Extended Help Message
 ECHO       .  ..... Make the Variables Permanent for this Console Session
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO   %~n0
	 ECHO   %~n0 @TEST @VAR
	 ECHO   %~n0 @CHECK @SCRIPT
	 ECHO   %~n0 abcd %RANDOM% #NUM @DATE
	 ECHO   %~n0 .
	 ECHO   %~n0 -?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
	 ECHO:
	 ECHO  * To search for variables in any config file, select one or more of
	 ECHO    the initial characters of that variable as a parameter of this
	 ECHO    script.  This will cause all environment variables which begin with
	 ECHO    those characters to be shown first in the list. ^(Variables in this
	 ECHO    script repository typically begin with the @ character [e.g. @TEST]^).
 )
 CALL :ShowStatus "FINISHED"
                                          