@ECHO OFF
 rem ==========================================================================
 rem === Update Timestamp of Recently Changed Scripts and Input Files
 rem ==========================================================================
 rem
 rem Created On: 29 Mar 2006
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Oct 2023 / 30 Nov 2020              
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2000 and later
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    FileDate ............... http://www.ABFsoftware.com/
 rem    Resource Kit ........... https://en.wikipedia.org/wiki/Resource_Kit
 rem ==========================================================================

 :::  This script synchronizes the time on scripts and input files that have
 :::  been changed under the C:\Scripts\Bat tree within the current day.
 :::
 :::  (Jan 2012) v4.3 -- The source folder can be set as a command-line
 :::  parameter.
 :::
 :::  (Mar 2012) v5.0 -- The parameter evaluation routines have been changed
 :::  to provide more flexibility in execution of the script.
 :::
 :::  (Feb 2014) v5.3 -- The source folder, if none is provided via the
 :::  command-line, is dependent upon the starting location of this script.
 :::
 :::  (Mar 2015) v5.6 -- Track elapsed time of timestamp job using a new
 :::  version of DATEINFO
 :::
 :::  (Apr 2015) v5.7 -- The job duration routines are now handled by the
 :::  :ShowStatus subroutine only
 :::
 :::  (Apr 2017) v5.8 -- DATEINFO.exe has totally replaced DOFF.exe.
 :::
 :::  (Apr 2018) v5.9 -- The SYNTAX HELP routine has been updated;
 :::  -- Changed the HELP routine to use FINDSTR;
 :::  -- Added filtered CMD Title Routine;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com
 :::
 :::  (Jul 2018) v5.10 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::
 :::  (Oct 2018) v5.11 -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Feb 2019) v6.0 -- Enabled Code Page 1252 Support;
 :::
 :::  (Oct 2019) v6.1 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (Jan 2020) v6.2 -- Use CHECKPARAMS to evaluate parameters besides /? /H;
 :::
 :::  (May 2020) v6.3 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::
 :::  (Nov 2020) v6.4 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Oct 2023) v6.5 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 09 Nov 2020 / 23 Apr 2018              
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 30 Nov 2020 / 26 May 2020              
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v6.5.0.660

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- Default Script Variables
 SET @LIST=
 SET @FIXALL=TRUE
 SET @SHOWALL=FALSE
 SET @DATEFORMAT=-f "mm/dd/yyyy  hh:nn am/pm"

 rem -- Process Special Command-line Parameters
 SET @REQ=&
 SET @ALL_PARAMS=%*

 CALL :GetParams "P" @SOURCE
 CALL :GetParams "E" @FILTER
 CALL :GetParams "T" @NEWTIME
 CALL :GetParams "V" @SHOWALL
 IF NOT DEFINED @REQ GOTO :HelpMessage

 IF "%@SOURCE%"=="" SET @SOURCE=%~dp0..
 FOR %%F IN ("" ".") DO IF "%@FILTER%"=="%%~F" SET @FILTER=TXT BAT CMD KIX VBS PS PL INI INPUT CSV XL
 FOR %%L IN (%@FILTER%) DO SET @LIST=!@LIST! .%%~L

 IF "%@NEWTIME%"=="." (
	 FOR /F "TOKENS=1-2" %%T IN ('DATEINFO -z "1:00:00"  -f "mm/dd/yyyy hh-nn-00" -q') DO (
		 SET @NEWDATE=%%~T
		 SET @NEWTIME=%%~U
		 SET @FIXALL=FALSE
	 )
 )

 IF "%@NEWTIME%"=="@" (
	 FOR /F "TOKENS=1-2" %%T IN ('DATEINFO -z "1:00:00"  -f "mm/dd/yyyy hh-00-00" -q') DO (
		 SET @NEWDATE=%%~T
		 SET @NEWTIME=%%~U
		 SET @FIXALL=FALSE
	 )
 )

 IF "%@NEWTIME%"=="" (
	 SET @TIMECALC=
	 SET @DATEFORMAT=-f "mm/dd/yyyy"
	 SET @MESSAGE=The Following Files Were NOT Updated Because No New Time was Provided
 ) ELSE (
	 SET @NEWTIME=%@NEWTIME::=-%
	 SET @MESSAGE=The Time Stamps of the Following Files Have Been Updated
	 SET @TIMECALC=%@NEWTIME:-=:%
 )

 IF DEFINED @NEWDATE (SET @DATECALC=%@NEWDATE%) ELSE (SET @DATECALC=%@DATE%)
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Process All Files in the Directory Tree To Synchronize Times
 rem === Updated On: 02 Oct 2008                     
 rem ==========================================================================
:Main
 ECHO Updating TimeStamp of Recently Changed Scripts and Input Files...
 FOR %%F IN (%@FILTER%) DO CALL :UpdateTimeStamp "%%~F"
 SET @EXT=*.*
 CALL :ShowFiles "%@MESSAGE%"
 %@PAUSE_IF_DEBUG%
 ECHO:


 rem ==========================================================================
 rem === Generate Final Job Report
 rem === Updated On: 01 Mar 2019 / 28 Feb 2019 / 28 Apr 2015              
 rem ==========================================================================
:FinalReport
 IF DEFINED @NEWTIME (
	 FOR /F "TOKENS=*" %%D IN ('DATEINFO -s "%@DATECALC% %@TIMECALC%" %@DATEFORMAT% -q') DO SET @DATEFIND=%%~D
	 FOR /F "TOKENS=*" %%F IN ('DIR /S "%@SOURCE%\*.*" ^| FIND /I /C "!@DATEFIND!"') DO (
		 ECHO %@DIVIDER~%
		 ECHO ~~~ DATE STAMP WAS SET ON %%F FILE[S]
		 ECHO %@DIVIDER~%
		 ECHO:
	 )
 )
 IF /I "%@SHOWALL%"=="TRUE" FOR %%F IN (%@FILTER%) DO (
 	IF EXIST "%@SOURCE%\*.%%~F" DIR "%@SOURCE%\*.%%~F" /OGD /S | FIND "/"
 )


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019              
 rem ==========================================================================
:ExitBatch
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Store Current Date and Time in Variable
 rem === Updated On: 16 Oct 2023 / 23 Mar 2015              
 rem ==========================================================================
:GetTime
 rem %1 = Variable to Store Date/Time

 SET %~1=!DATE! at !TIME: =0!
 FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET %~1#="%%~d"
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Output for Selected Files
 rem === Updated On: 09 Mar 2012 / 23 Jun 2010              
 rem ==========================================================================
:ShowFiles
 rem %1 = Optional Header
 SET @HEADER=%~1& IF "%~1"=="" SET @HEADER=%@EXT% FILES TO BE UPDATED
 SET @BLANK=TRUE
 CALL :SetFORFILES %@EXT%

 ECHO:
 ECHO:
 ECHO %@MINORDIV%
 ECHO --- %@HEADER%
 ECHO %@MINORDIV%

 FOR /F "TOKENS=*" %%V IN ('%@FORFILES% 2^>NUL') DO IF NOT EXIST "%%~V\" (
	 SET @BLANK=FALSE
	 DIR "%%~V" | FIND /I "%%~nV" | FINDSTR /I "%@LIST%"
 )

 IF /I "%@BLANK%"=="TRUE" ECHO  *** NO %@EXT% FILES WILL BE UPDATED ***
 ECHO %@MINORDIV%
 ECHO:
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Synchronize Date/Time Stamp of Selected Files
 rem === Updated On: 19 Feb 2014 / 21 Apr 2011              
 rem ==========================================================================
:UpdateTimeStamp
 rem %1 = Current Filter to Apply to the Date/Time Stamp
 SET @EXT=*.%~1*

 IF "%@NEWTIME%"=="" GOTO :EOF
 CALL :ShowFiles "%@EXT% files that will be updated to: %@NEWTIME:-=:%"
 FOR /F "TOKENS=*" %%V IN ('%@FORFILES% 2^>NUL') DO (
	 SET @CHANGETIME=TRUE
	 IF "%@FIXALL%"=="TRUE" (
		 FILEDATE "%%~V" %@MM%/%@DD%/%@YYYY% %@NEWTIME:~0,5% | FIND /i """"
	 ) ELSE (
		 FOR /F "TOKENS=3 DELIMS=: " %%T IN ('ECHO %%~tV') DO FOR %%C IN (00 30) DO IF "%%~T"=="%%~C" SET @CHANGETIME=FALSE
		 IF "!@CHANGETIME!"=="TRUE" FILEDATE "%%~V" !@NEWDATE! !@NEWTIME! | FIND /i """"
	 )
 )
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Determine Which Version of FORFILES.EXE To Use
 rem === Updated On: 13 Jan 2012 / 02 Oct 2008              
 rem ==========================================================================
:SetFORFILES
 rem %1 = Apply The Correct Filter for FORFILES

 SET @FORFILES=FORFILES -p"%@SOURCE%" -s -m%~1 -d+0 -c"cmd /c ECHO @PATH\@FILE"
 IF EXIST "%SystemRoot%\System32\FORFILES.EXE" (
	 SET @FORFILES=%SystemRoot%\System32\FORFILES -p "%@SOURCE%" -s -m %~1 -d +0 -c "cmd /c ECHO @PATH"
 )
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Use CheckParams.exe to Capture Selected Parameters
 rem === Updated On: 21 Jan 2020 / 27 Feb 2017              
 rem ==========================================================================
:GetParams
 rem %1 = Parameters to Search For
 rem %2 = Variable to Set

 SET @OK=& IF "%~1"=="" GOTO :EOF
 FOR /F "TOKENS=2-3*" %%v IN ('CHECKPARAMS -q -a -c "%~1" -s %@ALL_PARAMS% 2^>NUL') DO IF /I "%%~v"=="TRUE" (
	 SET @OK=T
	 SET @REQ=T
	 IF NOT "%~2"=="" SET %~2=%%~x
 )
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019              
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" CALL :GetTime @SCRIPT_BEG
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 30 Nov 2020 / 29 Nov 2020              
 rem ==========================================================================
:HelpMessage
 ECHO Update Timestamp of Recently Changed Scripts and Input Files
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  [-p:^<root path for files^>] [-e:^<valid extension(s)^>] [-t:^<time^>] [-v:^<TRUE^|FALSE^>]
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:
 ECHO -----------------------------
 ECHO OPTION DEFINITIONS and VALUES:
 ECHO -----------------------------
 ECHO  -?,  /H .......... Display This Help Message  (also --HELP)
 ECHO  -??, /H2 ......... Display Extended Help Message
 ECHO       /P .......... Root Path to Search for files
 ECHO       /E .......... Valid File extension(s)
 ECHO             .  .... (valid extension value -- All Extensions)
 ECHO       /T .......... Valid Time in hh:mm format to set timestamp to
 ECHO             @  .... (valid time value -- Next Full Hour)
 ECHO       /V .......... Show all changed files (default = FALSE)
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0
	 ECHO  %~n0 -e:.
	 ECHO  %~n0 -e:BAT
	 ECHO  %~n0 /p:"D:\Scripts" /e:TXT /t:12:30
	 ECHO  %~n0 -t:12:30 -e:"TXT BAT"
	 ECHO  %~n0 -p:"E:\MyScripts" -e:. -t:15:45 -v:TRUE
	 ECHO  %~n0 /e:. /t:@
	 ECHO  %~n0 -e:"BAT KIX" /t:16:00 -p:"E:\MyScripts\Bat"
	 ECHO  %~n0 -?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
	 ECHO:
	 ECHO  * The options need not be entered in the order shown above. If "." is
	 ECHO    selected as the extension, then the default selection will be *.TXT,
	 ECHO    *.INI, *.BAT, *.CMD, *.KIX and *.VBS will be selected.
	 ECHO:
	 ECHO  * Multiple extensions MUST be enclosed in quotation marks.
 )
 CALL :ShowStatus "FINISHED"
                                                              