@ECHO OFF
 CLS
 rem ==========================================================================
 rem === Monthly Archive of Old Log Files in ZIP (Traverse Directory Tree)
 rem ==========================================================================
 rem
 rem Created On: 22 Jun 2003
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 26 Jul 2018  (Prior Update: 17 May 2018)
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2003 and later (Originally on NT4+)
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    InfoZip Utilities ...... http://sourceforge.net/projects/infozip/files/
 rem    SysLogGen .............. http://www.snmpsoft.com/freetools/sysloggen.html
 rem
 rem    CHOICE ................. Native in Windows 2003+ / Reskit in NT/2000
 rem    PSTOOLS ................ https://technet.microsoft.com/en-us/sysinternals/bb896649
 rem    Resource Kit ........... https://en.wikipedia.org/wiki/Resource_Kit
 rem
 rem  Input Files Used By Script (stored in %SystemDrive%\Scripts\Bat\Input):
 rem    CustomVariables.TXT .... Configure global and script specific variables, including Alert Recipients
 rem
 rem  Other Scripts Called:
 rem    SetDrive.BAT ........... Primary Script for Setting Common Variables
 rem ==========================================================================

 :::  This purpose of this script is to create a zip archive of all files in a
 :::  given directory tree which match a particular pattern such as *.LOG,
 :::  *.TXT, or LOG*.TXT.  The name of the root folder is the only parameter
 :::  which must be passed to the script at the command line.  All other
 :::  variables should be set within the script itself.
 :::
 :::  All files in the folder which match the filespec, but are older than the
 :::  1st day of the month in which the script is run (basically, all files
 :::  created in a previous month) will be zipped into a file containing the
 :::  current year+month in the name, and then deleted once the archive has
 :::  been validated. (If you don't want the originals deleted, remove the -m
 :::  parameter from the ZIP command). The script will attempt to schedule a
 :::  monthly job to run as SYSTEM, but the account can be changed via the
 :::  Task Scheduler or the environment variables.
 :::
 :::  Over time, you'll want to start pruning the zip archive folder, as it
 :::  will steadily consume diskspace.  The script now includes a self-purging
 :::  routine that will move files into an "Archive" folder after 100 days,
 :::  and then delete any files in this archive folder than are older than
 :::  550 days. These defaults can all be changed by altering the two %@KEEP_%
 :::  variables (@KEEP_SHORT-TERM and @KEEP_HISTORICAL), or by entering the
 :::  two parameters at the command prompt.  You might find it best to keep
 :::  the files around for a least 3 months for easy comparison.
 :::
 :::  The compression is courtesy of the free InfoZip utilities, and this
 :::  script was tested under XP, Vista, Win7, 2003 and 2008 (x32 and x64),
 :::  but should run just fine under NT4 and 2000. The previous limitation
 :::  of 4GB zip archive files was finally corrected by an update in late 2009.
 :::
 :::  The DOFF.EXE utility makes it much easier to determine the previous
 :::  day and month than in times past, and has been added to all DATE/TIME
 :::  scripts as a basic routine.
 :::
 :::  (Jul 2010) In order to support archiving multi-extension files, such as
 :::  *.TXT.LOG or *.LOG.###, the @FILESPEC variable will accept entries such
 :::  as "LOG.+" or "TXT.+".  This is because of the way the FOR command
 :::  handles the * character, and the fact that the @FILESPEC variable
 :::  contents are passed to the FOR comamnd for processing. Variable
 :::  substitution will turn all "+" characters into "*" characters at time
 :::  of execution.  No provision is currently made for the "?" wildcard.
 :::
 :::  (Jul 2010) Support has also been added for the "CustomVariables.TXT"
 :::  input file, so that changes to the @FILESPEC, @EXCLUDE and @BYPASS
 :::  variables can be supported without modifying the script each time.
 :::
 :::  (Jul 2010) Specific deviations from the otherwise global @FILESPEC
 :::  criteria can made on a folder by folder level, by creating a file
 :::  called "+ZipLogsOverride.INPUT" in the local folder.  The contents of
 :::  this folder are identical to the desired @FILESPEC.  The name of the
 :::  file can be set and modified in the "CustomVariables.TXT" input file.
 :::
 :::  (Nov 2010) Added the name of the current machine as the prefix to
 :::  the archive name.  This will make it easier to store multiple logs
 :::  of the same type from a farm of servers in a central location.
 :::
 :::  (Nov 2010) v4.3 -- Added the option of deleting or not deleting empty
 :::  folders during log processing by way of a fourth (4th) parameter on
 :::  the command-line. If the value is TRUE, then empty folders will be
 :::  removed during the pruning/archival section of the script.  If no value
 :::  is provided, the default setting is FALSE. If you would like to maintain
 :::  the old behavior without changing all your scheduled jobs, then add the
 :::  @DELEMPTY variable with a setting of TRUE to your "CustomVariables.TXT"
 :::  file.  Please note that this will override whatever setting you make
 :::  for EVERY ZipLogs-*.BAT job.
 :::
 :::  (Mar 2011) The DblCheck routine has been updated to use the CHOICE.EXE
 :::  command rather than the TIMEOUT command.  This allows more flexibility
 :::  to abort or continue with the script.
 :::
 :::  (Dec 2011) Simplified the job scheduling section by way of environment
 :::  variables for verifying whether to use SCHTASKS or the AT command.
 :::
 :::  (Jan 2012) v4.8 -- The script has been configured to preserve the entire
 :::  path name as part of the archive name so that when archives are moved,
 :::  the source location can be easily identified.
 :::
 :::  The issue of supporting long path names via SCHTASKS was a vexing one
 :::  until I came across a KB article that outlines the \ character as the
 :::  escape character for quotation marks.  (e.g. \" )
 :::  For more details, see: http://support.microsoft.com/kb/823093
 :::
 :::  (Nov 2012) Because of a bug in how certain versions of Windows parse
 :::  quotes in a scheduled task, the script was changed to use only the
 :::  short name of folders in the path, eliminating the need for quotes.
 :::  For more details, see: http://support.microsoft.com/kb/951246
 :::
 :::  (Dec 2012) Made it possible for the scheduled tasks to be stored in their
 :::  own folder, rather than the default root folder of the Task Scheduler.
 :::
 :::  (Jan 2013) v5.0 -- Now uses the normal path name unless it contains
 :::  spaces, in which case, use the short name.
 :::
 :::  (Feb 2014) v5.2 -- Added the capability to send log data to one or more
 :::  Syslog servers using the SyslogGen or NeoLog utilities in conjunction
 :::  with the @SYSLOGx environment variables.
 :::
 :::  (Mar 2014) v5.3 -- Upgraded the DebugMode options to add another
 :::  variable (@PRINT_IF_DEBUG). Also revamped the routines for logging to
 :::  Syslog servers.
 :::
 :::  (Apr 2014) v5.4 -- Documented two parameters which have been available
 :::  for some time. One is for scheduling an archive job without running it,
 :::  and the other is for running an archive job without scheduling it.
 :::
 :::  (Jun 2014) v5.5 -- The Task Scheduler history in Windows 2012 does not
 :::  appreciate the single quote character in the job name, so it has been
 :::  replaced by square brackets [].
 :::
 :::  (Feb 2015) v5.6 -- Modified the FINDSTR parameters used for validating
 :::  the existence of the scheduled job;  Removed the shortname adjustment
 :::  that was previously made for Windows Vista/2008 RTM (Nov 2012).
 :::
 :::  (Apr 2015) v5.8 -- Scheduled jobs on Windows 8/2012 which contain the []
 :::  characters, do not always execute. As a result, these have been removed
 :::  from the script.
 :::
 :::  (Apr 2015) v5.9 -- Updated :ShowStatus routine to track script duration.
 :::
 :::  (May 2015) v6.0 -- Added a job history file (*.RECORDING.TXT) to keep a
 :::  log of archival sessions.
 :::
 :::  (Nov 2015) v6.1 -- Removed DNS error output from the Syslog routines.
 :::
 :::  (Feb 2016) v6.2 -- Provided a parameter to calculate and use the default
 :::  log location.
 :::
 :::  (Jan 2018) v6.3 -- Changed the HELP routine to use FINDSTR;
 :::  Fixed a new issue with using FINDSTR /R /C in checking for scheduled jobs.
 :::
 :::  (Feb 2018) v6.4 -- Extended "NEVER PURGE" timeframe from 999 to 9999 days.
 :::
 :::  (May 2018) v7.0 -- Changed Script Documentation to www.BrainWaveCC.com;
 :::  -- Added new Filtered CMD Title and Syntax Help processing routines;
 :::  -- Migrated :SaveJobHistory to a subroutine, for additional flexibility;
 :::  -- Added the ability to delete folders older than 90 days via @MAX_AGE;
 :::  -- Support for NeoLogger has been eliminated from this script;
 :::  -- Filtered Out SCHTASKS Error Messages;
 :::
 :::  (Jul 2018) v7.1 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::
 ::: --------
 :::  NOTES:
 ::: --------
 :::  *	The ZIP parameters are case sensitive.
 :::  *	This script processes the main folder + subdirectories.
 :::  *	This script requires SETDRIVE.BAT for the DATE/TIME routines.
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Last Modified On: 15 May 2018  (Prior Update: n/a)
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL
 SET @TTT=


 rem ==========================================================================
 rem === Enable Debug Mode If 'Debug' Variable is Set to "TRUE"
 rem === Last Modified On: 05 May 2015  (Prior Update: 15 Feb 2015)
 rem ==========================================================================
:DebugMode
 FOR %%d IN (TRUE LOG) DO IF /I "%Debug%"=="%%~d" (
	 SET @DIAGMODE=TRUE
	 SET @DEBUG_MODE=[DEBUG MODE ON]
	 SET @PAUSE_IF_DEBUG=TIMEOUT 5
	 SET @PRINT_IF_DEBUG=ECHO:^& SET @^& ECHO:^& TIMEOUT 5
	 SET @END_DEBUG_MODE=ECHO:^& ECHO *** DEBUG MODE FINISHED ***^& ECHO:^& SET @^& TIMEOUT 30
	 IF /I "%Debug%"=="LOG" (SET @PAUSE_IF_DEBUG=& SET @PRINT_IF_DEBUG=ECHO:^& SET @^& ECHO:)
	 ECHO *** DEBUG MODE = ON ***
	 ECHO ON
 ) ELSE ( FOR %%d IN (DEBUG_MODE PAUSE_IF_DEBUG PRINT_IF_DEBUG END_DEBUG_MODE) DO SET @%%d=)


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Last Modified On: 26 Jul 2018  (Prior Update: 16 May 2018)
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v7.1.0.660

 rem -- Display Syntax Help if Required/Requested
 IF "%~1"=="" GOTO :HelpMessage
 FOR /F "TOKENS=2" %%H IN ('CHECKPARAMS -q -c "H HELP ?" -s %*') DO GOTO :HelpMessage

 rem -- Default Variables (Override via CustomVariables.*.TXT)
 SET @EXCLUDE=-x *.ZIP *.RAR *\zia* *Zips\* *ZipFiles\*
 SET @FILESPEC=LOG LOG+ LOG_+ LOG.+ +.LOG TXT TXT+ TXT_+ TXT.+ +.TXT INFO CSV BAK TRN PP2 EVT SAV TRC Old.+
 SET @BYPASS=+ZipLogsOverride.INPUT

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 CALL :SaveJobHistory
	 GOTO :ExitBatch
 )

 rem -- Standard Variables
 SET @PSPARAMS=-AcceptEula
 SET @PARAMS=%*
 SET @MINROWS=%@MIN_LOG_ROWS%&   IF "%@MIN_LOG_ROWS%"==""   SET @MINROWS=3
 SET @MAXROWS=%@MAX_LOG_ROWS%&   IF "%@MAX_LOG_ROWS%"==""   SET @MAXROWS=99
 SET @MAX_AGE=%@MAX_FOLDER_AGE%& IF "%@MAX_FOLDER_AGE%"=="" SET @MAX_AGE=90
 FOR %%V IN (FILES_ARCHIVED DIRS_PRUNED ARCHIVES_MOVED ARCHIVES_PRUNED) DO SET @%%V=0

 rem -- Get Folder Names
 SET @TARGET="%~1"& IF /I "%~1"=="*DEFAULT*LOGS*" SET @TARGET=%@LOGBASE%
 IF /I "%~s1"=="%~1" (SET @TARGET_S=) ELSE (SET @TARGET_S=%~s1)
 SET @TARGET=%@TARGET:"=%
 SET @DEST=%@TARGET%\ZipFiles
 SET @TARGET_D=%@TARGET:(=%
 SET @TARGET_D=%@TARGET_D:)=%

 rem -- Historical Archiving Variables (in Days)
 SET @GRAVEYARD=%@DEST%\Archives
 IF "%~2"=="" (SET @KEEP_SHORT-TERM=0) ELSE (SET @KEEP_SHORT-TERM=%~2)
 IF "%~3"=="" (SET @KEEP_HISTORICAL=0) ELSE (SET @KEEP_HISTORICAL=%~3)
 IF !@KEEP_SHORT-TERM! LEQ 0 SET @KEEP_SHORT-TERM=100
 IF !@KEEP_HISTORICAL! LEQ 0 SET @KEEP_HISTORICAL=550

 rem -- Determine Whether or Not to Delete Empty Folders During Processing
 SET @DELEMPTY=FALSE& IF /I "%~4"=="TRUE" SET @DELEMPTY=TRUE

 rem -- Job Scheduling Variables
 SET @JOBINFO=Log Archive of
 SET @JOBNAME=%@JOBINFO% %@TARGET%& CALL :SanitizeName @JOBNAME
 SET @JOBUSER=SYSTEM
 SET @JOBFREQ=MONTHLY
 SET @JOBTIME=00:15:00
 SET @JOBDATE=/D 1
 SET @AT_DATE=/EVERY:1
 SET @JOBPATH="%~f0" \"%@TARGET%\" %@KEEP_SHORT-TERM% %@KEEP_HISTORICAL% %@DELEMPTY%
 SET @JOBPATH=%@JOBPATH:(=^(%
 SET @JOBPATH=%@JOBPATH:)=^)%
 SET @AT_PATH=%~f0 """%@TARGET%""" %@KEEP_SHORT-TERM% %@KEEP_HISTORICAL% %@DELEMPTY%
 SET @JOBVRFY=%@TARGET%

 rem -- Obsolete/Former Job Scheduling Names
 SET @OLDJOB0=%@JOBINFO% '%@TARGET%'
 SET @OLDJOB1=%@JOBINFO% [%@TARGET%]
 SET @OLDJOB2=%@JOBINFO% {%@TARGET%}
 IF DEFINED @TARGET_S (
	 SET @OLDJOB7=%@JOBINFO% '%@TARGET_S%'
	 SET @OLDJOB8=%@JOBINFO% [%@TARGET_S%]
	 SET @OLDJOB9=%@JOBINFO% {%@TARGET_S%}
 )
 FOR /L %%I IN (0,1,9) DO IF DEFINED @OLDJOB%%I CALL :SanitizeName @OLDJOB%%I

 rem -- Grab Appropriate Custom Variables from CustomVariables.*.TXT
 FOR %%F IN ("%@CUSTOM-VARS%" "%@CUST-MINI-VARS%") DO IF EXIST "%%~C" (
	 FOR /F "USEBACKQ TOKENS=1-2* DELIMS=,; " %%R IN ("%%~C") DO FOR %%Z IN ("&" "%~n0") DO IF /I "%%~M"=="%%~Z" CALL :RunCMD SET %%S=%%T
 )

 rem -- Process Special Commandline Parameters
 SET @UPDATESCHED=FALSE
 FOR %%P IN (%*) DO (
	 FOR %%V IN (-D /D @) DO IF /I "%%~P"=="%%~V" (SET @NOSCHED=TRUE& SET @KEEP_SHORT-TERM=0& SET @KEEP_HISTORICAL=9999)
	 FOR %%V IN (-S /S .) DO IF /I "%%~P"=="%%~V" (SET @UPDATESCHED=TRUE& SET @SCHEDULEONLY=TRUE)
	 FOR %%V IN (-U /U -) DO IF /I "%%~P"=="%%~V" (SET @UPDATESCHED=TRUE& SET @UNSCHED=TRUE)
	 FOR %%V IN (-N /N $) DO IF /I "%%~P"=="%%~V" (SET @UPDATESCHED=TRUE)
 )

 rem -- Standard Log Variables
 SET @LOGPATH=%@STORAGE%\Logs\Archiving
 SET @LOGFILE=%@LOGPATH%\%@JOBNAME%.%@FILESTAMP%
 SET @TIMINGS=%@LOGPATH%\%@JOBNAME%.RECORDING.TXT

 rem -- Key Variable Verification
 FOR %%V IN (DEST BYPASS FILESPEC JOBTIME GRAVEYARD KEEP_SHORT-TERM KEEP_HISTORICAL DELEMPTY) DO IF NOT DEFINED @%%~V (
	 ECHO:
	 ECHO *** WARNING: The @%%~V variable is undefined.
	 ECHO:
	 ECHO  Please ensure that all configuration info is provided correctly
	 ECHO  within the file: "%@CUSTOM-VARS%"
	 ECHO:
	 ECHO  Alternatively, this information can be entered directly into this script,
	 ECHO  although this is not recommended because it undermines portability.
	 ECHO %@DIVIDER*%
	 ECHO:
	 GOTO :HelpMessage
 )
 %@PRINT_IF_DEBUG%


 rem ==========================================================================
 rem === Validate Source and Script Log Folders
 rem === Last Modified On: 01 May 2015  (Prior Update: 21 Apr 2015)
 rem ==========================================================================
:CheckFolders
 IF /I "%@UNSCHED%"=="TRUE" GOTO :ScheduleJobs
 SET @NOTFOUND=
 IF EXIST "%@TARGET%" FOR %%F IN ("%@DEST%" "%@LOGPATH%") DO IF NOT EXIST %%F MD %%F >NUL
 FOR %%F IN ("%@TARGET%" "%@DEST%" "%@LOGPATH%") DO IF NOT EXIST %%F SET @NOTFOUND=%%F !@NOTFOUND!

 IF DEFINED @NOTFOUND (
	 ECHO *** SCRIPT ERROR: ONE OR MORE KEY FOLDERS NOT FOUND ***
	 ECHO:
	 ECHO COULD NOT CREATE OR ACCESS THE FOLLOWING:
	 FOR %%F IN (%@NOTFOUND%) DO ECHO  -- %%F
	 ECHO:
	 GOTO :HelpMessage
 )
 %@PRINT_IF_DEBUG%


 rem ==========================================================================
 rem === Allow User To Abort Operation
 rem === Last Modified On: 05 May 2015  (Prior Update: 21 Apr 2015)
 rem ==========================================================================
:DblCheck
 ECHO === Prepare to Create %@JOBINFO% "%@TARGET%" and subfolders ===
 ECHO:
 ECHO  Main archives will be kept for %@KEEP_SHORT-TERM% days...
 ECHO  Historical archives will be kept for %@KEEP_HISTORICAL% days...
 ECHO:
 ECHO  Logfile: "%@LOGFILE%"
 ECHO:
 IF /I "%@CONSOLE_SESSION%"=="TRUE" (
	 CHOICE /C YNQEX /T 20 /D Y /N /M "Continue running this script? (Y/N/Q) "
	 IF !ERRORLEVEL! GTR 1 GOTO :ExitBatch
 )
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Create Scheduled Job using SCHTASKS (XP/2003/Vista/+) or AT (NT/2000)
 rem === Last Modified On: 23 Jan 2018  (Prior Update: 13 Mar 2016)
 rem ==========================================================================
:ScheduleJobs
 IF /I NOT "%@UNSCHED%"=="TRUE" IF /I "%@NOSCHED%"=="TRUE" (
	 ECHO:
	 ECHO *** Job Scheduling Disabled ***
	 ECHO:
	 GOTO :ScheduleJobs_END
 )

 ECHO:
 ECHO Scheduling "%@JOBNAME%" on %COMPUTERNAME%...
 ECHO:

 IF EXIST "%SystemRoot%\System32\SCHTASKS.EXE" (
	 IF %@OSBUILD% LSS 6000 SET @TASKFOLDER=
	 SET @NAMECHK=%@JOBNAME:[=\[%
	 SET @NAMECHK=!@NAMECHK:]=\]!
	 SET @CHK_SCHED=SCHTASKS /QUERY /V /FO LIST
	 SET @SET_SCHED=SCHTASKS /CREATE /TN "!@TASKFOLDER!%@JOBNAME%" /TR "%@JOBPATH%" /RU "%@JOBUSER%" /SC %@JOBFREQ% %@JOBDATE% /ST %@JOBTIME% %@JOBLEVEL% %@FORCE%
 ) ELSE (
	 SET @CHK_SCHED=AT
	 SET @SET_SCHED=AT %@JOBTIME% %@AT_DATE% %@AT_PATH%
 )

 rem -- Delete Scheduled Task from Old Custom Folders on Vista/2008+
 IF %@OSBUILD% GEQ 6000 (
	 FOR /F "TOKENS=1* DELIMS=\ " %%s IN ('%@CHK_SCHED% ^| FIND "TaskName:" ^| FINDSTR /I /R /C:"%@NAMECHK%$"') DO (
		 IF /I NOT "%@TASKFOLDER%%@JOBNAME%"=="%%t" SCHTASKS /DELETE /TN "%%t" /F >NUL 2>&1
	 )
 )

 rem -- Delete Obsolete Scheduled Tasks (Containing Single Quotes, Containing 8.3 Filenames, etc)
 FOR /L %%I IN (0,1,9) DO IF DEFINED @OLDJOB%%I FOR %%f IN (\ "%@TASKFOLDER%") DO (
	 FOR /F %%s IN ('SCHTASKS /DELETE /TN "%%~f!@OLDJOB%%I!" /F 2^>^&1 ^| FIND "SUCCESS: "') DO ECHO Obsolete Task Deleted...
 )
 ECHO:

 rem -- Check to Remove Scheduled Job
 IF /I "%@UNSCHED%"=="TRUE" (
	 SCHTASKS /DELETE /TN "!@TASKFOLDER!%@JOBNAME%" %@FORCE% 2>NUL
	 ECHO Task has been UNscheduled.  Now exiting script...
	 GOTO :ExitBatch
 )

 rem -- Check for Schedule Update Parameters
 IF /I "%@UPDATESCHED%"=="TRUE" (!@SET_SCHED!) ELSE (CALL :CheckSched "%~f0" " %@JOBVRFY%$")
 ECHO %@DIVIDER-%
 ECHO:
 IF /I "%@SCHEDULEONLY%"=="TRUE" (
	 ECHO Task has been scheduled.  Now exiting script...
	 IF %@OSBUILD% LSS 6000 (SCHTASKS) ELSE (%@CHK_SCHED% /TN "!@TASKFOLDER!%@JOBNAME%")
	 TIMEOUT 10
	 GOTO :ExitBatch
 )
:ScheduleJobs_END
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Start at Root Log Folder and Process All Subfolders
 rem === Last Modified On: 26 Jul 2018  (Prior Update: 15 May 2018)
 rem ==========================================================================
:ProcessFolders
 %@ECHO:NOW% Processing "%@TARGET%" and subfolders for log files to Zip... >"%@LOGFILE%"
 ECHO Processing "%@TARGET%" and subfolders for log files to Zip...
 ECHO:
 FOR /R "%@TARGET%" %%V IN (.) DO (
	 SET @EXTENSIONS=%@FILESPEC%
	 IF EXIST "%%~V\%@BYPASS%" FOR /F "EOL=; USEBACKQ TOKENS=*" %%B IN ("%%~V\%@BYPASS%") DO SET @EXTENSIONS=%%~B
	 FOR %%F IN (!@EXTENSIONS!) DO CALL :MakeArchive "%%~V" *.%%~F
 )

 rem -- Determine Total Number of Files That Were Archived
 FOR /F "TOKENS=1" %%A IN ('TYPE "%@LOGFILE%" ^| FIND /C " testing: "') DO SET @FILES_ARCHIVED=%%~A
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Move Zips Into GraveYard After %@KEEP_SHORT-TERM% Days (Default=100)
 rem === Last Modified On: 05 May 2015  (Prior Update: 05 Feb 2015)
 rem ==========================================================================
:MoveIntoGraveYard
 SET @FORFILES=FORFILES -p"%@DEST%" -m*.ZIP -d-%@KEEP_SHORT-TERM% -c"CMD /C IF @ISDIR==FALSE ECHO @PATH\@FILE"
 IF EXIST "%SystemRoot%\System32\FORFILES.EXE" (
	 SET @FORFILES=%SystemRoot%\System32\FORFILES -p "%@DEST%" -m *.ZIP -d -%@KEEP_SHORT-TERM% -c "CMD /C IF @ISDIR==FALSE ECHO @PATH"
 )

 ( rem -- Write to Log
	 IF NOT EXIST "%@GRAVEYARD%" MD "%@GRAVEYARD%"
	 ECHO:
	 ECHO:
	 ECHO Manage Primary Archives
	 ECHO  -- Move all archives older than %@KEEP_SHORT-TERM% days to the GraveYard --
	 FOR /F "TOKENS=*" %%V IN ('%@FORFILES% 2^>NUL') DO (
		 ECHO Moving "%%~V" into Archive Graveyard Folder
		 MOVE "%%~V" "%@GRAVEYARD%"
		 SET /A @ARCHIVES_MOVED+=1
	 )
 ) >>"%@LOGFILE%" 2>&1
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Delete All Zips Older Than %@KEEP_HISTORICAL% Days (Default=550)
 rem === Last Modified On: 05 May 2015  (Prior Update: 21 Apr 2015)
 rem ==========================================================================
:PruneArchive
 SET @FORFILES=FORFILES -p"%@GRAVEYARD%" -m*.ZIP -d-%@KEEP_HISTORICAL% -c"CMD /C IF @ISDIR==FALSE ECHO @PATH\@FILE"
 IF EXIST "%SystemRoot%\System32\FORFILES.EXE" (
	 SET @FORFILES=%SystemRoot%\System32\FORFILES -p "%@GRAVEYARD%" -m *.ZIP -d -%@KEEP_HISTORICAL% -c "CMD /C IF @ISDIR==FALSE ECHO @PATH"
 )

 ( rem -- Write to Log
	 ECHO:
	 ECHO:
	 ECHO Pruning Old Backups from Archive Folder
	 ECHO  -- Delete all archives older than %@KEEP_HISTORICAL% days old --
	 FOR /F "TOKENS=*" %%V IN ('%@FORFILES% 2^>NUL') DO (
		 ECHO Deleting "%%~V" from Archive Graveyard Folder
		 DEL "%%~V"
		 SET /A @ARCHIVES_PRUNED+=1
	 )
	 ECHO:
	 ECHO:
 ) >>"%@LOGFILE%" 2>&1
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Generate Final Job Report
 rem === Last Modified On: 16 May 2018  (Prior Update: 05 May 2015)
 rem ==========================================================================
:FinalReport
 CALL :SaveJobHistory

 ( rem -- Write to Log
	 CALL :ForwardToSyslog "6" FINISHED Creating %@JOBINFO% "%@TARGET_D%" in %@ELAPSED%
	 ECHO:
	 DIR /OGD /S "%@DEST%"
	 ECHO:
	 DIR /OGD "%@LOGPATH%"
	 ECHO:
	 ECHO %@DIVIDER~%
	 ECHO  Total Files Archived ...!@FA!
	 ECHO  Empty Folders Pruned ......!@DP!
	 ECHO  Old Archives Moved ........!@AM!
	 ECHO  Old Archives Pruned .......!@AP!
	 ECHO %@DIVIDER~%
 ) >>"%@LOGFILE%" 2>&1
 COPY "%@LOGFILE%" "!@LOGFILE:%@FILESTAMP%=TXT!" /V /Y
 IF /I "%@CONSOLE_SESSION%"=="TRUE" TYPE "%@LOGFILE%"
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Last Modified On: 15 Feb 2015  (Prior Update: 01 Nov 2011)
 rem ==========================================================================
:ExitBatch
 CALL :ShowStatus "FINISHED"
 TITLE Command Prompt (%USERDOMAIN%\%USERNAME%)   %@DEBUG_MODE%
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Create Zip Archive and Move All Log Files
 rem === Last Modified On: 16 May 2018  (Prior Update: 05 May 2015)
 rem ==========================================================================
:MakeArchive
 rem %1 = Current Folder (Full Path Name)
 rem %2 = Current FileSpec

 SET @EMPTYDIR=TRUE
 SET @CURRENTDIR=%~f1
 SET @CURRENTSPEC=%~2
 SET @CURRENTSPEC=%@CURRENTSPEC:+=*%

 SET @LONGNAME=%@CURRENTDIR::\=_%
 SET @LONGNAME=%@LONGNAME:\=_%
 SET @ARCHIVE=%@DEST%\%COMPUTERNAME%_%@LONGNAME%.%@LASTMONTH_EXP%.ZIP

 IF EXIST "%@CURRENTDIR%\%@CURRENTSPEC%" (
	 PUSHD "%@CURRENTDIR%"
	 ECHO Processing "%@CURRENTDIR%\%@CURRENTSPEC%" Files... >CON
	 ECHO:
	 ECHO %@DIVIDER-%
	 ECHO Processing "%@CURRENTDIR%\%@CURRENTSPEC%" Files...
	 ECHO:
	 DIR "%@CURRENTSPEC%"
	 ECHO:
	 ZIP %@EXCLUDE% -m -9 -v -dgds 1m -T -tt %@BEGMONTH% "%@ARCHIVE%" %@CURRENTSPEC%
	 ECHO:
	 DIR
	 POPD
 ) >>"%@LOGFILE%" 2>&1
 %@PAUSE_IF_DEBUG%

 rem -- Delete Empty and Unattached Folders After Archiving Logs...
 IF NOT EXIST "%@CURRENTDIR%" GOTO :EOF
 FOR /F %%E IN ('DIR /A /B "%@CURRENTDIR%"') DO SET @EMPTYDIR=FALSE
 FOR /F "TOKENS=1*" %%A IN ('PSFILE %@PSPARAMS% 2^>NUL ^| FIND "]"') DO IF /I "%@CURRENTDIR%"=="%%~B" SET @EMPTYDIR=FALSE

 IF /I "%@EMPTYDIR%"=="TRUE" (
	 CALL :CheckFolderAge "%@CURRENTDIR%"
	 FOR %%E IN (DELEMPTY OLD_FOLDER) DO IF "!@%%~E!"=="TRUE" SET @REMOVE_DIR=TRUE

	 IF /I "!@REMOVE_DIR!"=="TRUE" (
		 SET /A @DIRS_PRUNED+=1
		 ECHO Deleting Empty Folder: "%@CURRENTDIR%"... >CON
		 ECHO:
		 ECHO Deleting Empty Folder: "%@CURRENTDIR%"...
		 RD /Q "%@CURRENTDIR%"
		 ECHO %@DIVIDER-%
		 ECHO:
		 ECHO:
	 )
 ) >>"%@LOGFILE%" 2>&1
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Execute Dynamic Command From Variable
 rem === Last Modified On: 20 Dec 2011  (Prior Update: 12 Jan 2007)
 rem ==========================================================================
:RunCMD
 rem %* = Command To Execute

 %*
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Generate Safe Job Schedule Name
 rem === Last Modified On: 21 Apr 2015  (Prior Update: 01 Apr 2015)
 rem ==========================================================================
:SanitizeName
 rem %1 = Name of Variable to CleanUp

 IF NOT DEFINED %1 GOTO :EOF
 SET %~1=!%~1:(=[!
 SET %~1=!%~1:)=]!
 SET %~1=!%~1::\=_!
 SET %~1=!%~1:\=_!
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Store Current Date and Time in Variable
 rem === Last Modified On: 01 Apr 2015  (Prior Update: n/a)
 rem ==========================================================================
:GetTime
 rem %1 = Variable to Store Date/Time

 SET %~1=!DATE! at !TIME: =0!
 SET %~1#="!%~1:~4,10! !%~1:~18,99!"
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Check Folder Age
 rem === Last Modified On: 16 May 2018  (Prior Update: 15 May 2018)
 rem ==========================================================================
:CheckFolderAge
 rem %1 = Folder to Check

 SET @FOLDER_DATE=%@TODAY%
 FOR %%z IN (FOLDER_AGE OLD_FOLDER REMOVE_DIR) DO SET @%%z=

 IF NOT EXIST "%~1" GOTO :EOF
 FOR /F %%d IN ('DIR "%~1" ^| FIND " .."') DO SET @FOLDER_DATE=%%d
 FOR /F %%a IN ('DATEINFO -S -B "%@FOLDER_DATE%" -Q') DO SET @FOLDER_AGE=%%a
 IF %@FOLDER_AGE% GTR %@MAX_AGE% SET @OLD_FOLDER=TRUE
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Pad Variable Content to the Left for Display in Logfiles
 rem === Last Modified On: 05 May 2015  (Prior Update: n/a)
 rem ==========================================================================
:PadLeft
 rem %1 = Variable to Store New Content
 rem %2 = Existing Value to Pad
 rem %3 = Character to use as padding [typically "0" or " "]
 rem %4 = New Character Length

 SET @TEMPVAR=& FOR /L %%c IN (1,1,%~4) DO SET @TEMPVAR=!@TEMPVAR!%~3
 SET @TEMPVAR=!@TEMPVAR!%~2
 SET %~1=!@TEMPVAR:~-%~4!
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Pad Variable Content to the Right for Display in Logfiles
 rem === Last Modified On: 05 May 2015  (Prior Update: n/a)
 rem ==========================================================================
:PadRight
 rem %1 = Variable to Store New Content
 rem %2 = Existing Value to Pad
 rem %3 = Character to use as padding [typically "." or " "]
 rem %4 = New Character Length

 SET @TEMPVAR=%~2& FOR /L %%c IN (1,1,%~4) DO SET @TEMPVAR=!@TEMPVAR!%~3
 SET %~1=!@TEMPVAR:~0,%~4!
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Update Job Execution Stats
 rem === Last Modified On: 16 May 2018  (Prior Update: 05 May 2015)
 rem ==========================================================================
:SaveJobHistory
 rem %* = NO PARAMETERS NEEDED

 CALL :GetTime @SCRIPT_END
 CALL :PadLeft @FA "%@FILES_ARCHIVED%" "." 6
 CALL :PadLeft @DP "%@DIRS_PRUNED%" "." 3
 CALL :PadLeft @AM "%@ARCHIVES_MOVED%" "." 3
 CALL :PadLeft @AP "%@ARCHIVES_PRUNED%" "." 3
 SET @ROWCOUNT=0
 SET @TRUNCATED=
 SET @ELAPSED=%@NO_TIME%& FOR /F "DELIMS=" %%d IN ('DATEINFO -t !@SCRIPT_BEG#! -n !@SCRIPT_END#! -e "hr:min:sec.ms" -q 2^>NUL') DO SET @ELAPSED=%%d
 SET @JOBHISTORY="%@TARGET_D%" // FILES ZIPPED.%@FA% // EMPTY DIRS PRUNED.%@DP% // ARCHIVES [MOVED.%@AM% PRUNED.%@AP%] // DURATION: %@ELAPSED% // START: %@SCRIPT_BEG% // END: !@SCRIPT_END! // %~n0 %@VERSION%

 BREAK ON >>"%@TIMINGS%"
 FOR /F %%c IN ('TYPE "%@TIMINGS%" ^| FIND /C "/"') DO SET @ROWCOUNT=%%c
 IF !@ROWCOUNT! GTR %@MAXROWS% (
	 SET @TRUNCATED= // TRUNCATED
	 SET @OLDTIME=!@TIMINGS:TXT=%@FILESTAMP%!
	 COPY "%@TIMINGS%" "!@OLDTIME!" /V /Y >CON
	 TAIL -%@MINROWS% "!@OLDTIME!" >"%@TIMINGS%"
	 ECHO:
	 CALL :ForwardToSyslog "5" %@JOBNAME%: Job History Logfile Truncated -- [!DATE! at !TIME!]
 ) >>"%@LOGFILE%" 2>&1

 CALL :ForwardToSyslog "6" %@JOBHISTORY%!@TRUNCATED! >>"%@TIMINGS%" 2>>"%@LOGFILE%"
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Forward Output to Available Syslog Server(s) as LOCAL3
 rem === Last Modified On: 15 May 2018  (Prior Update: 23 Nov 2015)
 rem ==========================================================================
:ForwardToSyslog
 rem %1 = Syslog Severity Level (RFC 3164)
 rem %* = Message to Send to Display and Send to Syslog Server

 SET @S_MSG=%*& IF "%~2"=="" SET @S_MSG=%1 - This is a Syslog Test Message from %@VER%
 SET @SEV=6
 FOR %%a IN (0 EMERGENCY) DO IF /I "%~1"=="%%~a" SET @SEV=0
 FOR %%a IN (1 ALERT) DO IF /I "%~1"=="%%~a" SET @SEV=1
 FOR %%a IN (2 CRITICAL) DO IF /I "%~1"=="%%~a" SET @SEV=2
 FOR %%a IN (3 ERROR) DO IF /I "%~1"=="%%~a" SET @SEV=3
 FOR %%a IN (4 WARNING) DO IF /I "%~1"=="%%~a" SET @SEV=4
 FOR %%a IN (5 NOTICE) DO IF /I "%~1"=="%%~a" SET @SEV=5
 FOR %%a IN (6 INFORMATION) DO IF /I "%~1"=="%%~a" SET @SEV=6
 FOR %%a IN (7 DEBUG) DO IF /I "%~1"=="%%~a" SET @SEV=7

 CALL :Prepare4Syslog %@S_MSG%

 ECHO !@S_MSG!
 IF DEFINED @SYSLOG_UTIL IF /I "%@USE_SYSLOGS%"=="TRUE" FOR %%s IN ("" 0 1 2 3 4) DO IF DEFINED @SYSLOG%%~s (
	 SET @S_PORT=514& IF DEFINED @SYSLOG%%s_PORT SET @S_PORT=!@SYSLOG%%~s_PORT!
	 IF /I "%@SYSLOG_UTIL%"=="SYSLOGGEN" "%@SYSLOGGEN%" -t:!@SYSLOG%%~s! -p:!@S_PORT! -f:19 -s:!@SEV! -tg:"%~n0" -m:"!@S_MSG2!" -q -hn:%@LOCALHOST% | FIND /V "%@NO_SYSLOGGEN%"
 )
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Prepare Messages for Syslog Utils (via SyslogGen)
 rem === Last Modified On: 15 May 2018  (Prior Update: 16 Dec 2014)
 rem ==========================================================================
:Prepare4Syslog
 rem %* = Syslog Message to be Cleaned Up

 SET @PREFIX=%1
 SET @S_MSG=%*
 SET @S_MSG=!@S_MSG:%@PREFIX% =!
 CALL :TrimLeadingSpaces @S_MSG
 SET @S_MSG2=!@S_MSG:\=\x5B!
 SET @S_MSG2=!@S_MSG2:"=\x22!
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Remove Leading Spaces from String
 rem === Last Modified On: 16 Dec 2014  (Prior Update: n/a)
 rem ==========================================================================
:TrimLeadingSpaces
 rem %1 = Name of Variable to CleanUp

 SET @STR=!%~1!
 IF NOT "!@STR:~0,1!"==" " GOTO :EOF
 SET %1=!@STR:~1,9999!
 GOTO :TrimLeadingSpaces


 rem ==========================================================================
 rem === SUBROUTINE: Check if Scheduled Job is Already Running
 rem === Last Modified On: 15 May 2018  (Prior Update: 05 Feb 2015)
 rem ==========================================================================
:CheckSched
 rem %1 = Standard Search Criteria
 rem %2 = Advanced Search Criteria

 SET @CHK=%2
 IF DEFINED @CHK (
	 %@CHK_SCHED% 2>NUL | FIND /I "%~1" | FINDSTR /I /R /C:%2 >NUL
 ) ELSE (
	 %@CHK_SCHED% 2>NUL | FIND /I "%~1" >NUL
 )
 IF ERRORLEVEL 1 %@SET_SCHED%
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Last Modified On: 27 Mar 2016  (Prior Update: 05 May 2015)
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" CALL :GetTime @SCRIPT_BEG
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 ECHO:
 ECHO *** %@SCRIPTSTATUS%: %@VER% [%DATE% at %TIME: =0%] ***
 ECHO:
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Last Modified On: 17 May 2018  (Prior Update: 11 Feb 2016)
 rem ==========================================================================
:HelpMessage
 ECHO Create MONTHLY ZIP archives of .TXT, .LOG and other text files
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  ^<folder^> [days before archive] [days before graveyard purge]  [OPTIONS]
 ECHO  %~n0  ^<folder^> [OPTIONS]
 ECHO  %~n0  *DEFAULT*LOGS* [OPTIONS]
 ECHO  %~n0  [-h^|-?]
 ECHO:
 IF DEFINED @KEEP_SHORT-TERM IF DEFINED @KEEP_HISTORICAL (
	 ECHO 	Number of days before zip files are archived .....: %@KEEP_SHORT-TERM%
	 ECHO 	Number of days before archived zips are purged ...: %@KEEP_HISTORICAL%
	 ECHO:
 )
 ECHO -------------------
 ECHO OPTION DEFINITIONS:
 ECHO -------------------
 ECHO  /H, -? = Display This Help Message
 ECHO  /S,  . = Schedule the Job, But Don't Execute It
 ECHO  /D,  @ = Execute the Job without Scheduling It
 ECHO  /N,  $ = Execute the Job, and Update the Schedule
 ECHO  /U,  - = Unschedule the Job Only, Without Executing It First
 ECHO:
 ECHO ----------------
 ECHO SYNTAX EXAMPLES:
 ECHO ----------------
 ECHO  %~n0 C:\LOGS
 ECHO  %~n0 D:\Backup 150
 ECHO  %~n0 D:\TEST 200 900
 ECHO  %~n0 "F:\SYSTEM LOGS" %@KEEP_SHORT-TERM% %@KEEP_HISTORICAL%
 ECHO  %~n0 *DEFAULT*LOGS* 100 600 /S
 ECHO  %~n0 %SystemDrive%\TheseLogs /D
 ECHO  %~n0 -?
 ECHO:
 ECHO ------------
 ECHO USAGE NOTES:
 ECHO ------------
 ECHO  * Parameters surrounded by ^<^> are mandatory
 ECHO  * Parameters surrounded by [] are optional.
 ECHO:
 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
 ECHO:
 ECHO  * Any parameters which contain spaces should be surrounded by quotes.
 ECHO    Shortnames *can* be surrounded by quotes if you choose, but it is not
 ECHO    mandatory by any means.
 CALL :ShowStatus "FINISHED"
                                                                                                                                                                                  