@ECHO OFF
 rem ==========================================================================
 rem === Create Zip My File Repository in "My Documents"
 rem ==========================================================================
 rem
 rem Created On: 01 Jul 2003
 rem         By: Andrew S. Baker
 rem
 rem Updated On: 16 Oct 2023 / 30 Nov 2020
 rem         By: Andrew S. Baker
 rem


 rem ==========================================================================
 rem  OS Required: Windows 2003 and later (Originally on NT4+)
 rem
 rem  Non-Native Utilities Used/Required:
 rem    BrainWave Utilities .... https://www.BrainWaveCC.com/brainwave-utilities
 rem
 rem    InfoZip Utilities ...... http://sourceforge.net/projects/infozip/files/
 rem    SysLogGen .............. http://www.snmpsoft.com/freetools/sysloggen.html
 rem
 rem    CHOICE ................. Native in Windows 2003+ / Reskit in NT/2000
 rem    Resource Kit ........... https://en.wikipedia.org/wiki/Resource_Kit
 rem
 rem  Input Files Used By Script (stored in %SystemDrive%\Scripts\Bat\Input):
 rem    CustomVariables.TXT .... Centralized Configuration Settings for Scripts
 rem
 rem  Required Scripts:
 rem    SetDrive.BAT ........... Set Global Variables for this Environment
 rem ==========================================================================

 :::  This purpose of this script is to create a zip archive of all files in a
 :::  "My Documents" folder.  The name of the root folder is the only parameter
 :::  which must be passed  to the script at the command line.   If none is
 :::  passed, the default "My Documents" folder is selected.
 :::
 :::  Unlike the ZipLogs script, this script does not delete old files in the
 :::  source tree -- it simply creates a current zip file.  This is useful for
 :::  supplementing the backup process.
 :::
 :::  The zip compatible compression is provided courtesy of the free
 :::  InfoZip utilities, which support all versions of Windows since 2001,
 :::  (has been tested on XP and later) and can process files in excess of 4GB.
 :::
 :::  (Jul 2010) Support has also been added for the "CustomVariables.TXT"
 :::  input file, so that changes to the @FILESPEC, @EXCLUDE and @BYPASS
 :::  variables can be supported without modifying the script each time.
 :::
 :::  (Mar 2011) The DblCheck routine has been updated to use the CHOICE.EXE
 :::  command rather than the TIMEOUT command.  This allows more flexibility
 :::  to abort or continue with the script.
 :::
 :::  (Mar 2014) v2.10 -- Upgraded the DebugMode options to add another
 :::  variable (@PRINT_IF_DEBUG). Also revamped the routines for logging to
 :::  Syslog servers.
 :::
 :::  (Feb 2015) v3.0 -- Modified the FINDSTR parameters used for validating
 :::  the existence of the scheduled job;  Removed the shortname adjustment
 :::  that was previously made for Windows Vista/2008 RTM (Nov 2012).
 :::
 :::  (May 2015) v4.0 -- Added a job history file (*.RECORDING.TXT) to keep a
 :::  log of archival sessions.
 :::
 :::  (Nov 2015) v4.1 -- Removed DNS error output from the Syslog routines.
 :::
 :::  (May 2018) v5.0 -- Changed the HELP routine to use FINDSTR;
 :::  -- The SYNTAX HELP routine has been updated;
 :::  -- Added filtered CMD Title Routine;
 :::  -- Added new Filtered CMD Title and Syntax Help processing routines;
 :::  -- Support for NeoLogger has been eliminated from this script;
 :::  -- Changed Script Documentation to www.BrainWaveCC.com
 :::
 :::  (Jul 2018) v5.1 -- The new BrainWave CheckParams utility is now used to
 :::  check for the /H and /? parameters, replacing the use of FIND/FINDSTR;
 :::
 :::  (Oct 2018) v5.2 -- Corrected and expanded the DEBUG logging routine;
 :::
 :::  (Mar 2019) v5.3 -- Enabled Code Page 1252 Support;
 :::  -- Record additional attributes in the Job History log;
 :::  -- Prevent DEBUG MODE data from being stored in the .RECORDING.TXT file;
 :::
 :::  (Oct 2019) v5.4 -- Updated :ShowStatus routine to reset TITLE/CodePage;
 :::
 :::  (Oct 2019) v5.5 -- Update :PadLeft and :PadRight routines;
 :::
 :::  (Jan 2020) v5.6 -- Refactored the :Prepare4Syslog routine;
 :::  -- Consolidate :PadLeft and :PadRight routines;
 :::  -- Added a routine for validating integer parameters;
 :::
 :::  (May 2020) v5.7 -- Migrated :DebugMode routine to SetDrive.BAT;
 :::  -- Moved Custom Options and Parameter Processing to Subroutines, rather than
 :::  standard compound statements within parentheticals;
 :::
 :::  (Nov 2020) v5.8 -- Provided extended HELP options (--?? / --H2);
 :::  -- New CHECKPARAMS syntax for processing extended help options;
 :::
 :::  (Oct 2023) v5.9 -- Change the way the @SCRIPT_BEG# variable is
 :::  calculated in various subroutines by using DATEINFO instead of %DATE%;
 :::
 ::: --------
 :::  NOTES:
 ::: --------
 :::  *	The ZIP parameters ARE case sensitive.
 :::  *	This script processes the main folder + subdirectories.
 :::  *	This script requires SETDRIVE.BAT for the global variables.
 :::
 :::
 :::  ADDITIONAL SCRIPTING TOOLS/INFO:
 :::  -------------------------------------------------------------------------
 :::  https://www.BrainWaveCC.com/knowledge-base/scripting-and-automation-options-for-windows/
 :::  https://www.BrainWaveCC.com/knowledge-base/must-have-utilities-for-windows-usersadmins/
 :::  -------------------------------------------------------------------------


 rem ==========================================================================
 rem === Generate Filtered CMD Title
 rem === Updated On: 30 Nov 2020 / 15 May 2018
 rem ==========================================================================
:DisplayTitle
 SET @TTT= %*
 SET @TTT=%@TTT:/?={HELP}%
 TITLE [%USERDOMAIN%\%USERNAME%] - %~f0 %@TTT% >NUL 2>NUL
 SET @TTT=


 rem ==========================================================================
 rem === Initialize Environment Variables
 rem === Updated On: 30 Nov 2020 / 26 May 2020
 rem ==========================================================================
:Variables
 SETLOCAL ENABLEDELAYEDEXPANSION & CALL :ShowStatus "STARTED" v5.9.0.560

 rem -- Display Syntax Help if Required/Requested
 CHECKPARAMS --rc -c "H HELP ?" -x "?? H2" -s %*
 IF ERRORLEVEL 1 IF NOT ERRORLEVEL 16 GOTO :HelpMessage

 rem -- Default Variables (Override via CustomVariables.*.TXT)
 SET @EXCLUDE=-x \*.ZIP \zia* \*.BAK \*.TMP \*(Old* \MailAttachments \My*

 rem -- Job Scheduling Variables
 SET @JOBTYPE=

 rem -- Set Global Variables from Centralized Script
 SET @VARIABLES="%~dp0SetDrive.BAT"
 IF EXIST %@VARIABLES% (
	 CALL %@VARIABLES% "%~n0"
 ) ELSE (
	 ECHO *** WARNING: CRITICAL SCRIPT COMPONENT MISSING ***
	 ECHO:
	 ECHO Unable to initialize custom script variables. Could not find %@VARIABLES%
	 ECHO:
	 DIR %@VARIABLES%
	 CALL :SaveJobHistory ABORTED - NO %@VARIABLES%
	 GOTO :ExitBatch
 )

 rem -- Main Variables
 SET @FILES_ARCHIVED=0
 CALL :GetInteger @MINROWS "%@MIN_LOG_ROWS%" 003 001 010
 CALL :GetInteger @MAXROWS "%@MAX_LOG_ROWS%" 099 030 199

 rem -- Get Folder Names
 SET @TARGET="%~1"& IF "%~1"=="" SET @TARGET="%USERPROFILE%\Documents"
 SET @TARGET=%@TARGET:"=%
 SET @DEST=%@STORAGE%\Zips

 rem -- ZipName Variables
 SET @ZIPNAME=Documents of %USERNAME%
 SET @ARCHIVE=%@DEST%\%COMPUTERNAME%_%@ZIPNAME%.%@TODAY%-%@NOW%.ZIP

 rem -- Standard Log Variables
 SET @LOGPATH=%@STORAGE%\Logs\Archiving
 SET @LOGFILE=%@LOGPATH%\%@ZIPNAME%.%@FILESTAMP%
 SET @TIMINGS=%@LOGPATH%\%@ZIPNAME%.RECORDING.TXT
 %@PRINT_IF_DEBUG%


 rem ==========================================================================
 rem === Validate Source and Script Log Folders
 rem === Updated On: 07 Mar 2019 / 01 May 2015
 rem ==========================================================================
:CheckFolders
 SET @NOTFOUND=
 IF EXIST "%@TARGET%" FOR %%F IN ("%@DEST%" "%@LOGPATH%") DO IF NOT EXIST %%F MD %%F >NUL
 FOR %%F IN ("%@TARGET%" "%@DEST%" "%@LOGPATH%") DO IF NOT EXIST %%F SET @NOTFOUND=%%F !@NOTFOUND!

 IF DEFINED @NOTFOUND (
	 ECHO *** SCRIPT ERROR: ONE OR MORE KEY FOLDERS NOT FOUND ***
	 ECHO:
	 ECHO COULD NOT CREATE OR ACCESS THE FOLLOWING:
	 FOR %%F IN (%@NOTFOUND%) DO ECHO  -- %%F
	 ECHO:
	 CALL :SaveJobHistory ABORTED - MISSING FOLDER
	 GOTO :HelpMessage
 )
 %@PRINT_IF_DEBUG%


 rem ==========================================================================
 rem === Allow User To Abort Operation
 rem === Updated On: 05 May 2015 / 19 Feb 2014
 rem ==========================================================================
:DblCheck
 ECHO === Prepare to Create Zip Archive of the "My Documents" folder ===
 ECHO:
 ECHO  Source: "%@TARGET%"
 ECHO  Logfile: "%@LOGFILE%"
 ECHO:
 IF /I "%@CONSOLE_SESSION%"=="TRUE" (
	 CHOICE /C YNQEX /T 20 /D Y /N /M "Continue running this script? (Y/N/Q) "
	 IF !ERRORLEVEL! GTR 1 GOTO :ExitBatch
 )
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Create Zip Archives of "%@TARGET%"
 rem === Updated On: 29 Jul 2019 / 15 May 2018 / 05 May 2015
 rem ==========================================================================
:ZipFiles
 NOW Creating Archive of "%@TARGET%"... >"%@LOGFILE%"
 ECHO Creating Archive of "%@TARGET%"...
 ECHO:

 rem -- Zip 'My Documents' Archive
 PUSHD "%@TARGET%"
 ECHO:
 ECHO   Zipping 'My Documents' Archive of %USERNAME%
 ( rem -- Write to Log
	 ZIP !@EXCLUDE! -9 -v -dgds 1m -T -S -R -dt -b "%TEMP%" "%@ARCHIVE%" *.*
	 ECHO %@DIVIDER-%
	 ECHO:
 ) >>"%@LOGFILE%" 2>&1
 POPD

 rem -- Determine Total Number of Files That Were Archived
 FOR /F "TOKENS=1" %%A IN ('TYPE "%@LOGFILE%" ^| FIND /C " testing: "') DO SET @FILES_ARCHIVED=%%~A
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Generate Final Job Report
 rem === Updated On: 15 May 2018 / 05 May 2015
 rem ==========================================================================
:FinalReport
 CALL :SaveJobHistory

 ( rem -- Write to Log
	 CALL :ForwardToSyslog "6" FINISHED Creating Document Archive of "%@TARGET%!"
	 ECHO:
	 DIR /OGD /S "%@DEST%"
	 ECHO:
	 DIR /OGD "%@LOGPATH%"
	 ECHO:
	 ECHO %@DIVIDER~%
	 ECHO  Total Files Archived ...!@FA!
	 ECHO %@DIVIDER~%
 ) >>"%@LOGFILE%" 2>&1
 COPY "%@LOGFILE%" "!@LOGFILE:%@FILESTAMP%=TXT!" /V /Y
 IF /I "%@CONSOLE_SESSION%"=="TRUE" TYPE "%@LOGFILE%"
 %@PAUSE_IF_DEBUG%


 rem ==========================================================================
 rem === Reset Environment Variables and Exit Batch File
 rem === Updated On: 04 Oct 2019 / 28 Feb 2019
 rem ==========================================================================
:ExitBatch
 CALL :ShowStatus "FINISHED"
 ENDLOCAL
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Execute Dynamic Command From Variable
 rem === Updated On: 16 Jul 2010      
 rem ==========================================================================
:RunCMD
 rem %* = Command To Execute

 %*
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Store Current Date and Time in Variable
 rem === Updated On: 16 Oct 2023 / 01 May 2015
 rem ==========================================================================
:GetTime
 rem %1 = Variable to Store Date/Time

 SET %~1=!DATE! at !TIME: =0!
 FOR /F "TOKENS=*" %%d IN ('DATEINFO -S %@DATEFMT% -Q 2^>NUL') DO SET %~1#="%%~d"
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Determine if Parameter is a Valid Integer, Or Set to Zero
 rem === Updated On: 21 Jan 2020      
 rem ==========================================================================
:GetInteger
 rem %1 = Variable to Store Integer
 rem %2 = Parameter to Evaluate
 rem %3 = Value to Default to, If Zero
 rem %4 = Optional: Lowest Value
 rem %5 = Optional: Highest Value

 SET @V1=%2& IF DEFINED @V1 SET @V1=!@V1:"=!
 SET @V2=%3& IF DEFINED @V2 SET @V2=!@V2:"=!
 SET @V3=%4& IF DEFINED @V3 SET @V3=!@V3:"=!
 SET @V4=%5& IF DEFINED @V4 SET @V4=!@V4:"=!
 FOR /F "TOKENS=3" %%i IN ('CHECKPARAMS -o --min "!@V3!" --max "!@V4!" -n "!@V1!" "!@V2!" 2^>NUL') DO SET %~1=%%i
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Pad Variable Content to the Left/Right for Display in Logs
 rem === Updated On: 22 Jan 2020 / 14 Oct 2019
 rem ==========================================================================
:PadLeft
:PadRight
 rem %1 = Variable to Store New Content
 rem %2 = Existing Value to Pad
 rem %3 = Character to use as padding [typically " ", "." or "0"]
 rem %4 = New String Length

 SET @STR=%~2
 SET @LEN=& FOR /L %%c IN (0,1,199) DO IF "!@STR:~%%c,1!"=="" IF NOT DEFINED @LEN SET @LEN=%%c
 IF 1%@LEN% GEQ 1%~4 (
	 SET %~1=%~2
 ) ELSE (
	 IF /I "%~0"==":PadLeft" (
		 SET @TEMPVAR=& FOR /L %%c IN (1,1,%~4) DO SET @TEMPVAR=!@TEMPVAR!%~3
		 SET @TEMPVAR=!@TEMPVAR!%~2
		 SET %~1=!@TEMPVAR:~-%~4!
	 )
	 IF /I "%~0"==":PadRight" (
		 SET @TEMPVAR=%~2& FOR /L %%c IN (1,1,%~4) DO SET @TEMPVAR=!@TEMPVAR!%~3
		 SET %~1=!@TEMPVAR:~0,%~4!
	 )
 )
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Update Job Execution Stats
 rem === Updated On: 05 May 2020 / 27 Apr 2020 / 15 Oct 2019
 rem ==========================================================================
:SaveJobHistory
 rem %* = Optional Message to include in Job History

 CALL :GetTime @SCRIPT_END
 CALL :PadLeft @FA "%@FILES_ARCHIVED%" "." 6
 SET @ROWCOUNT=0
 SET @TRUNCATED=
 SET @ELAPSED=%@NO_TIME%& FOR /F "DELIMS=" %%d IN ('DATEINFO -t !@SCRIPT_BEG#! -n !@SCRIPT_END#! -e "hr:min:sec.ms" -q 2^>NUL') DO SET @ELAPSED=%%d
 SET @SPECIAL=%@JOBTYPE%& IF NOT "%~1"=="" SET @SPECIAL= // %*
 SET @JOBHISTORY="%@TARGET%" // FILES ZIPPED.%@FA% // DURATION: %@ELAPSED% // START: %@SCRIPT_BEG% // END: !@SCRIPT_END! // %~n0 %@VERSION%%@SPECIAL%

 %@VERBOSE_OFF%
 BREAK ON >>"%@TIMINGS%"
 FOR /F %%c IN ('TYPE "%@TIMINGS%" ^| FIND /C "/"') DO SET @ROWCOUNT=%%c
 IF !@ROWCOUNT! GTR %@MAXROWS% (
	 SET @TRUNCATED= // TRUNCATED
	 SET @OLDTIME=!@TIMINGS:TXT=%@FILESTAMP%!
	 COPY "%@TIMINGS%" "!@OLDTIME!" /V /Y >CON
	 TAIL -%@MINROWS% "!@OLDTIME!" >"%@TIMINGS%"
	 ECHO:
	 CALL :ForwardToSyslog "5" %@ZIPNAME%: Job History Logfile Truncated -- [!DATE! at !TIME!]
 ) >>"%@LOGFILE%" 2>&1

 CALL :ForwardToSyslog "6" %@JOBHISTORY%!@TRUNCATED! >>"%@TIMINGS%" 2>>"%@LOGFILE%"
 %@VERBOSE_ON%
 %@PAUSE_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Forward Output to Available Syslog Server(s) as LOCAL3
 rem === Updated On: 13 Jan 2020 / 07 Mar 2019
 rem ==========================================================================
:ForwardToSyslog
 rem %1 = Syslog Severity Level (RFC 3164)
 rem %* = Message to Send to Display and Send to Syslog Server

 SET @S_MSG=%*& IF "%~2"=="" SET @S_MSG=%1 - This is a Syslog Test Message from %@VER%
 SET @SEV=6
 FOR %%a IN (0 EMERGENCY) DO IF /I "%~1"=="%%~a" SET @SEV=0
 FOR %%a IN (1 ALERT) DO IF /I "%~1"=="%%~a" SET @SEV=1
 FOR %%a IN (2 CRITICAL) DO IF /I "%~1"=="%%~a" SET @SEV=2
 FOR %%a IN (3 ERROR) DO IF /I "%~1"=="%%~a" SET @SEV=3
 FOR %%a IN (4 WARNING) DO IF /I "%~1"=="%%~a" SET @SEV=4
 FOR %%a IN (5 NOTICE) DO IF /I "%~1"=="%%~a" SET @SEV=5
 FOR %%a IN (6 INFORMATION) DO IF /I "%~1"=="%%~a" SET @SEV=6
 FOR %%a IN (7 DEBUG) DO IF /I "%~1"=="%%~a" SET @SEV=7

 %@VERBOSE_OFF%
 CALL :Prepare4Syslog %@S_MSG%

 ECHO !@S_MSG!
 IF DEFINED @SYSLOG_UTIL IF /I "%@USE_SYSLOGS%"=="TRUE" FOR %%s IN ("" 0 1 2 3 4) DO IF DEFINED @SYSLOG%%~s (
	 SET @S_PORT=514& IF DEFINED @SYSLOG%%s_PORT SET @S_PORT=!@SYSLOG%%~s_PORT!
	 "%@SYSLOGGEN%" -t:!@SYSLOG%%~s! -p:!@S_PORT! -f:19 -s:!@SEV! -tg:"%~n0" -m:"!@S_MSG2!" -q -hn:%@LOCALHOST% | FIND /V "%@NO_SYSLOGGEN%"
 )
 %@VERBOSE_ON%
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Prepare Messages for SyslogGen Syslog Utility
 rem === Updated On: 14 Jan 2020 / 15 May 2018
 rem ==========================================================================
:Prepare4Syslog
 rem %* = Syslog Message to be Cleaned Up

 SET @PREFIX=%1
 SET @S_MSG=%*
 SET @S_MSG=!@S_MSG:%@PREFIX% =!

:TrimLeadingSpaces
 IF "!@S_MSG:~0,1!"==" " (
	 SET @S_MSG=!@S_MSG:~1,9999!
	 GOTO :TrimLeadingSpaces
 )

 SET @S_MSG2=!@S_MSG:\=\x5B!
 SET @S_MSG2=!@S_MSG2:"=\x22!
 %@PRINT_IF_DEBUG%
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Set/Display Script Version and Execution Status
 rem === Updated On: 16 Oct 2023 / 24 Dec 2019
 rem ==========================================================================
:ShowStatus
 rem %1 = Run Status of Script
 rem %2 = Current Application Version

 IF NOT DEFINED @DATEFMT SET @DATEFMT=-F "mm/dd/yyyy hh:nn:ss.zzz"
 SET @SCRIPTSTATUS=%~1& IF "%~1"=="" SET @SCRIPTSTATUS=RUNNING
 IF NOT "%~2"=="" (SET @VER=%~nx0 %~2& SET @VERSION=%~2)
 IF /I "%~1"=="STARTED" CALL :GetTime @SCRIPT_BEG
 IF /I "%~1"=="FINISHED" (
	 IF DEFINED $CODEPAGE FOR /F "TOKENS=1* DELIMS=:" %%B IN ('CHCP %$CODEPAGE%') DO SET @CHCP_STATUS= {Restoring Code Page:%%C}
	 IF DEFINED @END_DEBUG_MODE %@END_DEBUG_MODE:"=%
	 TITLE Command Prompt [%USERDOMAIN%\%USERNAME%]   !@DEBUG_MODE!
	 DATEINFO -t %@SCRIPT_BEG#% -e "hr:min:sec.ms" -o "\n*** DURATION: " 2>NUL
 )
 NOW \n*** %@SCRIPTSTATUS%: %@VER% [\v] *** %@CHCP_STATUS%\n!@CRLF-%~1!
 GOTO :EOF


 rem ==========================================================================
 rem === SUBROUTINE: Display Syntax Help if Required/Requested
 rem === Updated On: 30 Nov 2020 / 15 May 2018
 rem ==========================================================================
:HelpMessage
 ECHO Create On-Demand ZIP archives of the user profile or personal document folder.
 ECHO:
 ECHO ----------
 ECHO YOU TYPED:  %0 %*
 ECHO ----------
 ECHO:
 ECHO -----------
 ECHO CMD SYNTAX:
 ECHO -----------
 ECHO  %~n0  [folder]
 ECHO  %~n0  [-h^|-?^|-h2^|-??]
 ECHO:

 IF ERRORLEVEL 8 IF NOT ERRORLEVEL 16 (
	 ECHO ----------------
	 ECHO SYNTAX EXAMPLES:
	 ECHO ----------------
	 ECHO  %~n0
	 ECHO  %~n0 "D:\Files\Some Documents"
	 ECHO  %~n0 "%USERPROFILE%\My Documents"
	 ECHO  %~n0 -?
	 ECHO:
	 ECHO ------------
	 ECHO USAGE NOTES:
	 ECHO ------------
	 ECHO  * Parameters surrounded by ^<^> are mandatory.
	 ECHO  * Parameters surrounded by [] are optional.
	 ECHO:
	 ECHO  * Options are case-insensitive, and can be prefaced by "-" or "/".
	 ECHO:
	 ECHO  * Any parameters which contain spaces should be surrounded by quotes.
	 ECHO    Shortnames *can* be surrounded by quotes if you choose, but it is not
	 ECHO    mandatory by any means.
 )
 CALL :ShowStatus "FINISHED"
                                                                             